@@ -1,10 +1,10 @@
 require 'benchmark'
 
 def bench_class_definition(bm)
-  bm.report("10000 def/undef method") {
+  bm.report("1m class Foo; end") {
     class << self
-      10000.times {
-        def my_bogus_method; end; undef my_bogus_method
+      1_000_000.times {
+        class Foo; end
       }
     end
   }
@@ -12,4 +12,4 @@ end
 
 if $0 == __FILE__
   (ARGV[0] || 10).to_i.times { Benchmark.bm(40) {|bm| bench_class_definition(bm)} }
-end
\ No newline at end of file
+end
