@@ -268,7 +268,7 @@ Note: Inherited methods from class (({Object})) cannot be used as XML-RPC names,
 
 
 = History
-    $Id: client.rb 2062 2006-06-10 19:14:15Z headius $
+    $Id: client.rb 5479 2008-01-03 21:39:44Z headius $
 
 =end
 
@@ -558,7 +558,7 @@ module XMLRPC
       expected = resp["Content-Length"] || "<unknown>"
       if data.nil? or data.size == 0 
         raise "Wrong size. Was #{data.size}, should be #{expected}" 
-      elsif expected.to_i != data.size and resp["Transfer-Encoding"].nil?
+      elsif expected != "<unknown>" and expected.to_i != data.size and resp["Transfer-Encoding"].nil?
         raise "Wrong size. Was #{data.size}, should be #{expected}"
       end
 
