@@ -9,6 +9,8 @@ import "java_integration.fixtures.BeanLikeInterfaceHandler"
 import "java_integration.fixtures.ConstantHoldingInterface"
 import "java_integration.fixtures.ReturnsInterface"
 import "java_integration.fixtures.ReturnsInterfaceConsumer"
+import "java_integration.fixtures.AnotherRunnable"
+import "java.lang.Runnable"
 
 describe "Single-method Java interfaces implemented in Ruby" do
   before :all do
@@ -362,6 +364,24 @@ describe "A Ruby class including a descendant interface" do
     UsesSingleMethodInterface.callIt(dosmi).should == "foo"
     UsesDescendantOfSingleMethodInterface.callThat(dosmi).should == "bar"
   end
+
+  it "inherits implementation of super-interface methods from superclass" do
+    super_impl = Class.new do
+      include SingleMethodInterface
+      
+      def callIt; "foo"; end
+    end
+    
+    impl = Class.new(super_impl) do
+      include DescendantOfSingleMethodInterface
+      def callThat; "bar"; end
+    end
+    
+    dosmi = impl.new
+
+    UsesSingleMethodInterface.callIt(dosmi).should == "foo"
+    UsesDescendantOfSingleMethodInterface.callThat(dosmi).should == "bar"
+  end
 end
 
 describe "Single object implementing methods of interface" do 
@@ -521,7 +541,6 @@ describe "A child extending a Ruby class that includes Java interfaces" do
 end
 
 describe "Calling methods through interface on Ruby objects with methods defined on singleton class" do
-  
   before(:each) do
     @klass = Class.new do
       include SingleMethodInterface
@@ -545,25 +564,37 @@ describe "Calling methods through interface on Ruby objects with methods defined
   end
   
   it "should handle two object with combo of instance_eval and module extension" do
-    pending "Needs work in ifc impl method lookup logic" do
-      @obj1.instance_eval("def callIt; return :one; end;")
-      @module = Module.new { def callIt; return :two; end; }
-      @obj2.extend(@module)
+    @obj1.instance_eval("def callIt; return :one; end;")
+    @module = Module.new { def callIt; return :two; end; }
+    @obj2.extend(@module)
 
-      UsesSingleMethodInterface.callIt(@obj1).should == :one
-      UsesSingleMethodInterface.callIt(@obj2).should == :two
-    end
+    UsesSingleMethodInterface.callIt(@obj1).should == :one
+    UsesSingleMethodInterface.callIt(@obj2).should == :two
   end
   
   it "should handle two object with combo of instance_eval and module extension in opposite order" do
-    pending "Needs work in ifc impl method lookup logic" do
-      @obj1.instance_eval("def callIt; return :one; end;")
-      @module = Module.new { def callIt; return :two; end; }
-      @obj2.extend(@module)
+    @obj1.instance_eval("def callIt; return :one; end;")
+    @module = Module.new { def callIt; return :two; end; }
+    @obj2.extend(@module)
 
-      UsesSingleMethodInterface.callIt(@obj2).should == :two
-      UsesSingleMethodInterface.callIt(@obj1).should == :one
-    end
+    UsesSingleMethodInterface.callIt(@obj2).should == :two
+    UsesSingleMethodInterface.callIt(@obj1).should == :one
   end
 end
 
+# JRUBY-2999
+describe "A Ruby class implementing Java interfaces with overlapping methods" do
+  it "should implement without error" do
+    cls = Class.new do
+      include AnotherRunnable
+      include Runnable
+      attr_accessor :foo
+      def run
+        @foo = 'success'
+      end
+    end
+
+    obj = nil
+    lambda {obj = cls.new}.should_not raise_error
+  end
+end
