@@ -37,12 +37,16 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby;
 
+import java.io.BufferedReader;
+import java.io.IOException;
 import java.io.InputStream;
+import java.io.InputStreamReader;
 import java.io.PrintStream;
 
 import org.jruby.exceptions.MainExitException;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.exceptions.ThreadKill;
+import org.jruby.platform.Platform;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.SafePropertyAccessor;
@@ -78,6 +82,12 @@ public class Main {
     }
 
     public static void main(String[] args) {
+        // Ensure we're not running on GCJ, since it's not supported and leads to weird errors
+        if (Platform.IS_GCJ) {
+            System.err.println("Fatal: GCJ (GNU Compiler for Java) is not supported by JRuby.");
+            System.exit(1);
+        }
+        
         Main main = new Main();
         
         try {
@@ -181,6 +191,11 @@ public class Main {
 
         InputStream in   = config.getScriptSource();
         String filename  = config.displayedFileName();
+
+        String[] args = parseShebangOptions(in);
+        if (args.length > 0) {
+            config.processArguments(args);
+        }
         Ruby runtime     = Ruby.newInstance(config);
         
         // set thread context JRuby classloader here, for the main thread
@@ -258,4 +273,39 @@ public class Main {
     public void printProperties() {
         config.getOutput().print(config.getPropertyHelp());
     }
+    
+    private String[] parseShebangOptions(InputStream in) {
+        BufferedReader reader = null;
+        String[] result = new String[0];
+        if (in == null) return result;
+        try {
+            in.mark(1024);
+            reader = new BufferedReader(new InputStreamReader(in, "iso-8859-1"), 8192);
+            String firstLine = reader.readLine();
+            if (firstLine.length() > 2 && firstLine.charAt(0) == '#' && firstLine.charAt(1) == '!') {
+                int index = firstLine.indexOf("ruby", 2);
+
+                // JRUBY-3456: This was not considering that the executable
+                // name may actually be "jruby.bat" and the arg processing
+                // should happen after it.
+                if (firstLine.indexOf("ruby.bat", 2) == index) {
+                    index += 4;
+                }
+
+                if (firstLine.length() < index + 5) {
+                    in.reset();
+                    return result;
+                }
+                String option = firstLine.substring(index + 5);
+                result = option.split("\\s");
+            }
+        } catch (Exception ex) {
+            // ignore error
+        } finally {
+            try {
+                in.reset();
+            } catch (IOException ex) {}
+        }
+        return result;
+    }
 }
