@@ -33,6 +33,7 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicBoolean;
+
 import org.jruby.anno.JRubyMethod;
 import org.jruby.javasupport.JavaObject;
 import org.jruby.javasupport.util.RuntimeHelpers;
@@ -42,6 +43,7 @@ import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.runtime.builtin.InstanceVariableTable;
 import org.jruby.runtime.builtin.InstanceVariables;
 import org.jruby.runtime.builtin.InternalVariables;
 import org.jruby.runtime.builtin.Variable;
@@ -54,23 +56,13 @@ import org.jruby.util.TypeConverter;
  *
  * @author enebo
  */
-public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, CoreObjectType, InstanceVariables, InternalVariables {
+public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Comparable<IRubyObject>, CoreObjectType, InstanceVariables, InternalVariables {
 
     // The class of this object
     protected transient RubyClass metaClass;
 
     protected int flags; // zeroed by jvm
 
-    // The dataStruct is a place where custom information can be
-    // contained for core implementations that doesn't necessarily
-    // want to go to the trouble of creating a subclass of
-    // RubyObject. The OpenSSL implementation uses this heavily to
-    // save holder objects containing Java cryptography objects.
-    // Java integration uses this to store the Java object ref.
-    protected transient Object dataStruct;
-
-    private transient Finalizer finalizer;
-
     /**
      * The variableTable contains variables for an object, defined as:
      * <ul>
@@ -80,11 +72,8 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * </ul>
      *
      * Constants are stored separately, see {@link RubyModule}.
-     *
      */
-    protected transient volatile VariableTableEntry[] variableTable;
-    protected transient int variableTableSize;
-    protected transient int variableTableThreshold;
+    protected transient volatile InstanceVariableTable variables;
 
     /**
      * The error message used when some one tries to modify an
@@ -107,8 +96,9 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     public static final int NIL_F = 1 << 1;
     public static final int FROZEN_F = 1 << 2;
     public static final int TAINTED_F = 1 << 3;
+    public static final int UNTRUSTED_F = 1 << 4;
 
-    public static final int FL_USHIFT = 4;
+    public static final int FL_USHIFT = 5;
 
     public static final int USER0_F = (1<<(FL_USHIFT+0));
     public static final int USER1_F = (1<<(FL_USHIFT+1));
@@ -225,7 +215,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     protected void taint(Ruby runtime) {
         runtime.secure(4);
         if (!isTaint()) {
-        	testFrozen("object");
+        	testFrozen();
             setTaint(true);
         }
     }
@@ -237,7 +227,18 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      */
    protected final void testFrozen(String message) {
        if (isFrozen()) {
-           throw getRuntime().newFrozenError(message + " " + getMetaClass().getName());
+           throw getRuntime().newFrozenError(message);
+       }
+   }
+
+    /** rb_frozen_class_p
+     *
+     * Helper to test whether this object is frozen, and if it is will
+     * throw an exception based on the message.
+     */
+   protected final void testFrozen() {
+       if (isFrozen()) {
+           throw getRuntime().newFrozenError("object");
        }
    }
 
@@ -409,9 +410,19 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      */
     public IRubyObject infectBy(IRubyObject obj) {
         if (obj.isTaint()) setTaint(true);
+        if (obj.isUntrusted()) setUntrusted(true);
         return this;
     }
 
+    final RubyBasicObject infectBy(RubyBasicObject obj) {
+        flags |= (obj.flags & (TAINTED_F | UNTRUSTED_F));
+        return this;
+    }
+
+    final RubyBasicObject infectBy(int tuFlags) {
+        flags |= (tuFlags & (TAINTED_F | UNTRUSTED_F));
+        return this;
+    }
 
     /**
      * Is this value frozen or not? Shortcut for doing
@@ -437,6 +448,31 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         }
     }
 
+
+    /**
+     * Is this value untrusted or not? Shortcut for doing
+     * getFlag(UNTRUSTED_F).
+     *
+     * @return true if this object is frozen, false otherwise
+     */
+    public boolean isUntrusted() {
+        return (flags & UNTRUSTED_F) != 0;
+    }
+
+    /**
+     * Sets whether this object is frozen or not. Shortcut for doing
+     * setFlag(FROZEN_F, frozen).
+     *
+     * @param frozen should this object be frozen?
+     */
+    public void setUntrusted(boolean untrusted) {
+        if (untrusted) {
+            flags |= UNTRUSTED_F;
+        } else {
+            flags &= ~UNTRUSTED_F;
+        }
+    }
+
     /**
      *  Is object immediate (def: Fixnum, Symbol, true, false, nil?).
      */
@@ -664,11 +700,19 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         return TypeConverter.convertToTypeWithCheck(this, getRuntime().getArray(), "to_ary");
     }
 
+    // 1.9 rb_check_to_integer
+    IRubyObject checkIntegerType(Ruby runtime, IRubyObject obj, String method) {
+        if (obj instanceof RubyFixnum) return obj;
+        IRubyObject conv = TypeConverter.convertToType(obj, getRuntime().getInteger(), method, false);
+        return conv instanceof RubyInteger ? conv : obj.getRuntime().getNil();
+    }
+
     public IRubyObject dup() {
         if (isImmediate()) throw getRuntime().newTypeError("can't dup " + getMetaClass().getName());
 
         IRubyObject dup = getMetaClass().getRealClass().allocate();
         if (isTaint()) dup.setTaint(true);
+        if (isUntrusted()) dup.setUntrusted(true);
 
         initCopy(dup, this);
 
@@ -721,6 +765,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         initCopy(clone, this);
 
         if (isFrozen()) clone.setFrozen(true);
+        if (isUntrusted()) clone.setUntrusted(true);
         return clone;
     }
 
@@ -780,14 +825,25 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @see org.jruby.runtime.builtin.IRubyObject#dataWrapStruct()
      */
     public synchronized void dataWrapStruct(Object obj) {
-        this.dataStruct = obj;
+        if (obj == null) {
+            removeInternalVariable("__wrap_struct__");
+        } else {
+            fastSetInternalVariable("__wrap_struct__", obj);
+        }
     }
 
+    // The dataStruct is a place where custom information can be
+    // contained for core implementations that doesn't necessarily
+    // want to go to the trouble of creating a subclass of
+    // RubyObject. The OpenSSL implementation uses this heavily to
+    // save holder objects containing Java cryptography objects.
+    // Java integration uses this to store the Java object ref.
+    //protected transient Object dataStruct;
     /**
      * @see org.jruby.runtime.builtin.IRubyObject#dataGetStruct()
      */
     public synchronized Object dataGetStruct() {
-        return dataStruct;
+        return fastGetInternalVariable("__wrap_struct__");
     }
 
     /** rb_obj_id
@@ -869,6 +925,10 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         return context.getRuntime().newBoolean(!equalInternal(context, this, other));
     }
 
+    public int compareTo(IRubyObject other) {
+        return (int)callMethod(getRuntime().getCurrentContext(), "<=>", other).convertToInteger().getLongValue();
+    }
+
     /** rb_obj_equal
      *
      * Will by default use identity equality to compare objects. This
@@ -909,30 +969,36 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     /**
      * Adds the specified object as a finalizer for this object.
      */
-    public void addFinalizer(IRubyObject finalizer) {
-        if (this.finalizer == null) {
-            this.finalizer = new Finalizer(getRuntime().getObjectSpace().idOf(this));
-            getRuntime().addFinalizer(this.finalizer);
+    public void addFinalizer(IRubyObject f) {
+        Finalizer finalizer = (Finalizer)fastGetInternalVariable("__finalizer__");
+        if (finalizer == null) {
+            finalizer = new Finalizer(getRuntime().getObjectSpace().idOf(this));
+            fastSetInternalVariable("__finalizer__", finalizer);
+            getRuntime().addFinalizer(finalizer);
         }
-        this.finalizer.addFinalizer(finalizer);
+        finalizer.addFinalizer(f);
     }
 
     /**
      * Remove all the finalizers for this object.
      */
     public void removeFinalizers() {
+        Finalizer finalizer = (Finalizer)fastGetInternalVariable("__finalizer__");
         if (finalizer != null) {
             finalizer.removeFinalizers();
-            finalizer = null;
-            getRuntime().removeFinalizer(this.finalizer);
+            removeInternalVariable("__finalizer__");
+            getRuntime().removeFinalizer(finalizer);
         }
     }
 
-
     //
     // COMMON VARIABLE METHODS
     //
 
+    public InstanceVariableTable getVariables() {
+        return variables;
+    }
+
     /**
      * Returns true if object has any variables, defined as:
      * <ul>
@@ -960,14 +1026,14 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      */
     // TODO: must override in RubyModule to pick up constants
     public List<Variable<IRubyObject>> getVariableList() {
-        VariableTableEntry[] table = variableTableGetTable();
-        ArrayList<Variable<IRubyObject>> list = new ArrayList<Variable<IRubyObject>>();
-        IRubyObject readValue;
-        for (int i = table.length; --i >= 0; ) {
-            for (VariableTableEntry e = table[i]; e != null; e = e.next) {
-                if ((readValue = e.value) == null) readValue = variableTableReadLocked(e);
-                list.add(new VariableEntry<IRubyObject>(e.name, readValue));
-            }
+        final ArrayList<Variable<IRubyObject>> list = new ArrayList<Variable<IRubyObject>>();
+        InstanceVariableTable variables = getVariables();
+        if (variables != null) {
+            variables.visit(new InstanceVariableTable.TryLockVisitor(this) {
+                public void visit(String name, Object value) {
+                    list.add(new VariableEntry<IRubyObject>(name, (IRubyObject)value));
+                }
+            });
         }
         return list;
     }
@@ -977,14 +1043,16 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      */
    // TODO: must override in RubyModule to pick up constants
    public List<String> getVariableNameList() {
-        VariableTableEntry[] table = variableTableGetTable();
-        ArrayList<String> list = new ArrayList<String>();
-        for (int i = table.length; --i >= 0; ) {
-            for (VariableTableEntry e = table[i]; e != null; e = e.next) {
-                list.add(e.name);
-            }
-        }
-        return list;
+       final ArrayList<String> list = new ArrayList<String>();
+       InstanceVariableTable variables = getVariables();
+       if (variables != null) {
+           variables.visit(new InstanceVariableTable.Visitor() {
+               public void visit(String name, Object value) {
+                   list.add(name);
+               }
+           });
+       }
+       return list;
     }
 
     /**
@@ -1007,57 +1075,12 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         return name.length() > 0 && ((c = name.charAt(0)) == '@' || (c <= 'Z' && c >= 'A'));
     }
 
-    //
-    // VARIABLE TABLE METHODS, ETC.
-    //
-
-    protected static final int VARIABLE_TABLE_DEFAULT_CAPACITY = 8; // MUST be power of 2!
-    protected static final int VARIABLE_TABLE_MAXIMUM_CAPACITY = 1 << 30;
-    protected static final float VARIABLE_TABLE_LOAD_FACTOR = 0.75f;
-    protected static final VariableTableEntry[] VARIABLE_TABLE_EMPTY_TABLE = new VariableTableEntry[0];
-
-    /**
-     * Every entry in the variable map is represented by an instance
-     * of this class.
-     */
-    protected static final class VariableTableEntry {
-        final int hash;
-        final String name;
-        volatile IRubyObject value;
-        final VariableTableEntry next;
-
-        VariableTableEntry(int hash, String name, IRubyObject value, VariableTableEntry next) {
-            assert name == name.intern() : name + " is not interned";
-            this.hash = hash;
-            this.name = name;
-            this.value = value;
-            this.next = next;
-        }
-    }
-
-    /**
-     * Reads the value of the specified entry, locked on the current
-     * object.
-     */
-    protected synchronized IRubyObject variableTableReadLocked(VariableTableEntry entry) {
-        return entry.value;
-    }
-
     /**
      * Checks if the variable table contains a variable of the
      * specified name.
      */
     protected boolean variableTableContains(String name) {
-        VariableTableEntry[] table;
-        if ((table = variableTable) != null) {
-            int hash = name.hashCode();
-            for (VariableTableEntry e = table[hash & (table.length - 1)]; e != null; e = e.next) {
-                if (hash == e.hash && name.equals(e.name)) {
-                    return true;
-                }
-            }
-        }
-        return false;
+        return variables == null ? false : variables.contains(name);
     }
 
     /**
@@ -1066,16 +1089,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * an interned Java String.
      */
     protected boolean variableTableFastContains(String internedName) {
-        assert internedName == internedName.intern() : internedName + " not interned";
-        VariableTableEntry[] table;
-        if ((table = variableTable) != null) {
-            for (VariableTableEntry e = table[internedName.hashCode() & (table.length - 1)]; e != null; e = e.next) {
-                if (internedName == e.name) {
-                    return true;
-                }
-            }
-        }
-        return false;
+        return variables == null ? false : variables.fastContains(internedName);
     }
 
     /**
@@ -1084,18 +1098,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @return the object or null if not found
      */
     protected IRubyObject variableTableFetch(String name) {
-        VariableTableEntry[] table;
-        IRubyObject readValue;
-        if ((table = variableTable) != null) {
-            int hash = name.hashCode();
-            for (VariableTableEntry e = table[hash & (table.length - 1)]; e != null; e = e.next) {
-                if (hash == e.hash && name.equals(e.name)) {
-                    if ((readValue = e.value) != null) return readValue;
-                    return variableTableReadLocked(e);
-                }
-            }
-        }
-        return null;
+        return variables == null ? null : (IRubyObject)variables.fetch(name);
     }
 
     /**
@@ -1104,52 +1107,20 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      *
      * @return the object or null if not found
      */
-    protected IRubyObject variableTableFastFetch(String internedName) {
-        VariableTableEntry[] table;
-        IRubyObject readValue;
-        if ((table = variableTable) != null) {
-            for (VariableTableEntry e = table[internedName.hashCode() & (table.length - 1)]; e != null; e = e.next) {
-                if (internedName == e.name) {
-                    if ((readValue = e.value) != null) return readValue;
-                    return variableTableReadLocked(e);
-                }
-            }
-        }
-        return null;
+    protected Object variableTableFastFetch(String internedName) {
+        return variables == null ? null : variables.fastFetch(internedName);
     }
 
     /**
      * Store a value in the variable store under the specific name.
      */
-    protected IRubyObject variableTableStore(String name, IRubyObject value) {
-        int hash = name.hashCode();
+    protected Object variableTableStore(String name, Object value) {
         synchronized(this) {
-            VariableTableEntry[] table;
-            VariableTableEntry e;
-            if ((table = variableTable) == null) {
-                table =  new VariableTableEntry[VARIABLE_TABLE_DEFAULT_CAPACITY];
-                e = new VariableTableEntry(hash, name.intern(), value, null);
-                table[hash & (VARIABLE_TABLE_DEFAULT_CAPACITY - 1)] = e;
-                variableTableThreshold = (int)(VARIABLE_TABLE_DEFAULT_CAPACITY * VARIABLE_TABLE_LOAD_FACTOR);
-                variableTableSize = 1;
-                variableTable = table;
-                return value;
+            if (variables == null) {
+                variables = new InstanceVariableTable(name.intern(), value);
+            } else {
+                variables.store(name, value);
             }
-            int potentialNewSize;
-            if ((potentialNewSize = variableTableSize + 1) > variableTableThreshold) {
-                table = variableTableRehash();
-            }
-            int index;
-            for (e = table[index = hash & (table.length - 1)]; e != null; e = e.next) {
-                if (hash == e.hash && name.equals(e.name)) {
-                    e.value = value;
-                    return value;
-                }
-            }
-            e = new VariableTableEntry(hash, name.intern(), value, table[index]);
-            table[index] = e;
-            variableTableSize = potentialNewSize;
-            variableTable = table; // write-volatile
         }
         return value;
     }
@@ -1158,38 +1129,14 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * Will store the value under the specified name, where the name
      * needs to be an interned Java String.
      */
-    protected IRubyObject variableTableFastStore(String internedName, IRubyObject value) {
-        if (IdUtil.isConstant(internedName)) new Exception().printStackTrace();
-
+    protected Object variableTableFastStore(String internedName, Object value) {
         assert internedName == internedName.intern() : internedName + " not interned";
-        int hash = internedName.hashCode();
         synchronized(this) {
-            VariableTableEntry[] table;
-            VariableTableEntry e;
-            if ((table = variableTable) == null) {
-                table =  new VariableTableEntry[VARIABLE_TABLE_DEFAULT_CAPACITY];
-                e = new VariableTableEntry(hash, internedName, value, null);
-                table[hash & (VARIABLE_TABLE_DEFAULT_CAPACITY - 1)] = e;
-                variableTableThreshold = (int)(VARIABLE_TABLE_DEFAULT_CAPACITY * VARIABLE_TABLE_LOAD_FACTOR);
-                variableTableSize = 1;
-                variableTable = table;
-                return value;
+            if (variables == null) {
+                variables = new InstanceVariableTable(internedName, value);
+            } else {
+                variables.fastStore(internedName, value);
             }
-            int potentialNewSize;
-            if ((potentialNewSize = variableTableSize + 1) > variableTableThreshold) {
-                table = variableTableRehash();
-            }
-            int index;
-            for (e = table[index = hash & (table.length - 1)]; e != null; e = e.next) {
-                if (internedName == e.name) {
-                    e.value = value;
-                    return value;
-                }
-            }
-            e = new VariableTableEntry(hash, internedName, value, table[index]);
-            table[index] = e;
-            variableTableSize = potentialNewSize;
-            variableTable = table; // write-volatile
         }
         return value;
     }
@@ -1198,54 +1145,17 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * Removes the entry with the specified name from the variable
      * table, and returning the removed value.
      */
-    protected IRubyObject variableTableRemove(String name) {
+    protected Object variableTableRemove(String name) {
         synchronized(this) {
-            VariableTableEntry[] table;
-            if ((table = variableTable) != null) {
-                int hash = name.hashCode();
-                int index = hash & (table.length - 1);
-                VariableTableEntry first = table[index];
-                VariableTableEntry e;
-                for (e = first; e != null; e = e.next) {
-                    if (hash == e.hash && name.equals(e.name)) {
-                        IRubyObject oldValue = e.value;
-                        // All entries following removed node can stay
-                        // in list, but all preceding ones need to be
-                        // cloned.
-                        VariableTableEntry newFirst = e.next;
-                        for (VariableTableEntry p = first; p != e; p = p.next) {
-                            newFirst = new VariableTableEntry(p.hash, p.name, p.value, newFirst);
-                        }
-                        table[index] = newFirst;
-                        variableTableSize--;
-                        variableTable = table; // write-volatile
-                        return oldValue;
-                    }
-                }
-            }
+            return variables == null ? null : variables.remove(name);
         }
-        return null;
-    }
-
-    /**
-     * Get the actual table used to save variable entries.
-     */
-    protected VariableTableEntry[] variableTableGetTable() {
-        VariableTableEntry[] table;
-        if ((table = variableTable) != null) {
-            return table;
-        }
-        return VARIABLE_TABLE_EMPTY_TABLE;
     }
 
     /**
      * Get the size of the variable table.
      */
     protected int variableTableGetSize() {
-        if (variableTable != null) {
-            return variableTableSize;
-        }
-        return 0;
+        return variables == null ? 0 : variables.getSize(); 
     }
 
     /**
@@ -1254,91 +1164,17 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      */
     protected void variableTableSync(List<Variable<IRubyObject>> vars) {
         synchronized(this) {
-            variableTableSize = 0;
-            variableTableThreshold = (int)(VARIABLE_TABLE_DEFAULT_CAPACITY * VARIABLE_TABLE_LOAD_FACTOR);
-            variableTable =  new VariableTableEntry[VARIABLE_TABLE_DEFAULT_CAPACITY];
-            for (Variable<IRubyObject> var : vars) {
-                variableTableStore(var.getName(), var.getValue());
-            }
+            variables = new InstanceVariableTable(vars);
         }
     }
 
-    /**
-     * Rehashes the variable table. Must be called from a synchronized
-     * block.
-     */
-    // MUST be called from synchronized/locked block!
-    // should only be called by variableTableStore/variableTableFastStore
-    protected final VariableTableEntry[] variableTableRehash() {
-        VariableTableEntry[] oldTable = variableTable;
-        int oldCapacity;
-        if ((oldCapacity = oldTable.length) >= VARIABLE_TABLE_MAXIMUM_CAPACITY) {
-            return oldTable;
-        }
-
-        int newCapacity = oldCapacity << 1;
-        VariableTableEntry[] newTable = new VariableTableEntry[newCapacity];
-        variableTableThreshold = (int)(newCapacity * VARIABLE_TABLE_LOAD_FACTOR);
-        int sizeMask = newCapacity - 1;
-        VariableTableEntry e;
-        for (int i = oldCapacity; --i >= 0; ) {
-            // We need to guarantee that any existing reads of old Map can
-            //  proceed. So we cannot yet null out each bin.
-            e = oldTable[i];
-
-            if (e != null) {
-                VariableTableEntry next = e.next;
-                int idx = e.hash & sizeMask;
-
-                //  Single node on list
-                if (next == null)
-                    newTable[idx] = e;
-
-                else {
-                    // Reuse trailing consecutive sequence at same slot
-                    VariableTableEntry lastRun = e;
-                    int lastIdx = idx;
-                    for (VariableTableEntry last = next;
-                         last != null;
-                         last = last.next) {
-                        int k = last.hash & sizeMask;
-                        if (k != lastIdx) {
-                            lastIdx = k;
-                            lastRun = last;
-                        }
-                    }
-                    newTable[lastIdx] = lastRun;
-
-                    // Clone all remaining nodes
-                    for (VariableTableEntry p = e; p != lastRun; p = p.next) {
-                        int k = p.hash & sizeMask;
-                        VariableTableEntry m = new VariableTableEntry(p.hash, p.name, p.value, newTable[k]);
-                        newTable[k] = m;
-                    }
-                }
-            }
-        }
-        variableTable = newTable;
-        return newTable;
-    }
-
     /**
      * Method to help ease transition to new variables implementation.
      * Will likely be deprecated in the near future.
      */
-    @SuppressWarnings("unchecked")
     protected Map variableTableGetMap() {
         HashMap map = new HashMap();
-        VariableTableEntry[] table;
-        IRubyObject readValue;
-        if ((table = variableTable) != null) {
-            for (int i = table.length; --i >= 0; ) {
-                for (VariableTableEntry e = table[i]; e != null; e = e.next) {
-                    if ((readValue = e.value) == null) readValue = variableTableReadLocked(e);
-                    map.put(e.name, readValue);
-                }
-            }
-        }
+        if (variables != null) variables.getMap(this, map);
         return map;
     }
 
@@ -1346,22 +1182,11 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * Method to help ease transition to new variables implementation.
      * Will likely be deprecated in the near future.
      */
-    @SuppressWarnings("unchecked")
     protected Map variableTableGetMap(Map map) {
-        VariableTableEntry[] table;
-        IRubyObject readValue;
-        if ((table = variableTable) != null) {
-            for (int i = table.length; --i >= 0; ) {
-                for (VariableTableEntry e = table[i]; e != null; e = e.next) {
-                    if ((readValue = e.value) == null) readValue = variableTableReadLocked(e);
-                    map.put(e.name, readValue);
-                }
-            }
-        }
+        if (variables != null) variables.getMap(this, map);
         return map;
     }
 
-
     //
     // INTERNAL VARIABLE METHODS
     //
@@ -1402,7 +1227,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     /**
      * @see org.jruby.runtime.builtin.InternalVariables#fastGetInternalVariable
      */
-    public IRubyObject fastGetInternalVariable(String internedName) {
+    public Object fastGetInternalVariable(String internedName) {
         assert !isRubyVariable(internedName);
         return variableTableFastFetch(internedName);
     }
@@ -1410,7 +1235,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     /**
      * @see org.jruby.runtime.builtin.InternalVariables#setInternalVariable
      */
-    public void setInternalVariable(String name, IRubyObject value) {
+    public void setInternalVariable(String name, Object value) {
         assert !isRubyVariable(name);
         variableTableStore(name, value);
     }
@@ -1418,7 +1243,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     /**
      * @see org.jruby.runtime.builtin.InternalVariables#fastSetInternalVariable
      */
-    public void fastSetInternalVariable(String internedName, IRubyObject value) {
+    public void fastSetInternalVariable(String internedName, Object value) {
         assert !isRubyVariable(internedName);
         variableTableFastStore(internedName, value);
     }
@@ -1426,7 +1251,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     /**
      * @see org.jruby.runtime.builtin.InternalVariables#removeInternalVariable
      */
-    public IRubyObject removeInternalVariable(String name) {
+    public Object removeInternalVariable(String name) {
         assert !isRubyVariable(name);
         return variableTableRemove(name);
     }
@@ -1442,17 +1267,17 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     /**
      * @see org.jruby.runtime.builtin.InternalVariables#getInternalVariableList
      */
-    public List<Variable<IRubyObject>> getInternalVariableList() {
-        VariableTableEntry[] table = variableTableGetTable();
-        ArrayList<Variable<IRubyObject>> list = new ArrayList<Variable<IRubyObject>>();
-        IRubyObject readValue;
-        for (int i = table.length; --i >= 0; ) {
-            for (VariableTableEntry e = table[i]; e != null; e = e.next) {
-                if (!isRubyVariable(e.name)) {
-                    if ((readValue = e.value) == null) readValue = variableTableReadLocked(e);
-                    list.add(new VariableEntry<IRubyObject>(e.name, readValue));
+    public List<Variable<Object>> getInternalVariableList() {
+        final ArrayList<Variable<Object>> list = new ArrayList<Variable<Object>>();
+        InstanceVariableTable variables = getVariables();
+        if (variables != null) {
+            variables.visit(new InstanceVariableTable.TryLockVisitor(this) {
+                public void visit(String name, Object value) {
+                    if (!isRubyVariable(name)) {
+                        list.add(new VariableEntry<Object>(name, value));
+                    }
                 }
-            }
+            });
         }
         return list;
     }
@@ -1499,7 +1324,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      */
     public IRubyObject fastGetInstanceVariable(String internedName) {
         assert IdUtil.isInstanceVariable(internedName);
-        return variableTableFastFetch(internedName);
+        return (IRubyObject)variableTableFastFetch(internedName);
     }
 
     /** rb_iv_set / rb_ivar_set
@@ -1509,7 +1334,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     public IRubyObject setInstanceVariable(String name, IRubyObject value) {
         assert IdUtil.isInstanceVariable(name) && value != null;
         ensureInstanceVariablesSettable();
-        return variableTableStore(name, value);
+        return (IRubyObject)variableTableStore(name, value);
     }
 
     /**
@@ -1518,7 +1343,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     public IRubyObject fastSetInstanceVariable(String internedName, IRubyObject value) {
         assert IdUtil.isInstanceVariable(internedName) && value != null;
         ensureInstanceVariablesSettable();
-        return variableTableFastStore(internedName, value);
+        return (IRubyObject)variableTableFastStore(internedName, value);
      }
 
     /**
@@ -1527,23 +1352,23 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     public IRubyObject removeInstanceVariable(String name) {
         assert IdUtil.isInstanceVariable(name);
         ensureInstanceVariablesSettable();
-        return variableTableRemove(name);
+        return (IRubyObject)variableTableRemove(name);
     }
 
     /**
      * @see org.jruby.runtime.builtin.InstanceVariables#getInstanceVariableList
      */
     public List<Variable<IRubyObject>> getInstanceVariableList() {
-        VariableTableEntry[] table = variableTableGetTable();
-        ArrayList<Variable<IRubyObject>> list = new ArrayList<Variable<IRubyObject>>();
-        IRubyObject readValue;
-        for (int i = table.length; --i >= 0; ) {
-            for (VariableTableEntry e = table[i]; e != null; e = e.next) {
-                if (IdUtil.isInstanceVariable(e.name)) {
-                    if ((readValue = e.value) == null) readValue = variableTableReadLocked(e);
-                    list.add(new VariableEntry<IRubyObject>(e.name, readValue));
+        final ArrayList<Variable<IRubyObject>> list = new ArrayList<Variable<IRubyObject>>();
+        InstanceVariableTable variables = getVariables();
+        if (variables != null) {
+            variables.visit(new InstanceVariableTable.TryLockVisitor(this) {
+                public void visit(String name, Object value) {
+                    if (IdUtil.isInstanceVariable(name)) {
+                        list.add(new VariableEntry<IRubyObject>(name, (IRubyObject)value));
+                    }                    
                 }
-            }
+            });
         }
         return list;
     }
@@ -1552,14 +1377,16 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * @see org.jruby.runtime.builtin.InstanceVariables#getInstanceVariableNameList
      */
     public List<String> getInstanceVariableNameList() {
-        VariableTableEntry[] table = variableTableGetTable();
-        ArrayList<String> list = new ArrayList<String>();
-        for (int i = table.length; --i >= 0; ) {
-            for (VariableTableEntry e = table[i]; e != null; e = e.next) {
-                if (IdUtil.isInstanceVariable(e.name)) {
-                    list.add(e.name);
+        final ArrayList<String> list = new ArrayList<String>();
+        InstanceVariableTable variables = getVariables();
+        if (variables != null) {
+            variables.visit(new InstanceVariableTable.Visitor() {
+                public void visit(String name, Object value) {
+                    if (IdUtil.isInstanceVariable(name)) {
+                        list.add(name);
+                    }
                 }
-            }
+            });
         }
         return list;
     }
@@ -1567,14 +1394,16 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
     /**
      * @see org.jruby.runtime.builtin.InstanceVariables#getInstanceVariableNameList
      */
-    public void copyInstanceVariablesInto(InstanceVariables other) {
-        VariableTableEntry[] table = variableTableGetTable();
-        for (int i = table.length; --i >= 0; ) {
-            for (VariableTableEntry e = table[i]; e != null; e = e.next) {
-                if (IdUtil.isInstanceVariable(e.name)) {
-                    other.setInstanceVariable(e.name, e.value);
+    public void copyInstanceVariablesInto(final InstanceVariables other) {
+        InstanceVariableTable variables = getVariables();
+        if (variables != null) {
+            variables.visit(new InstanceVariableTable.Visitor() {
+                public void visit(String name, Object value) {
+                    if (IdUtil.isInstanceVariable(name)) { 
+                        other.setInstanceVariable(name, (IRubyObject)value);
+                    }
                 }
-            }
+            });
         }
     }
 
@@ -1583,7 +1412,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      * including information about whether this object is frozen, or
      * tainted. Will throw a suitable exception in that case.
      */
-    protected void ensureInstanceVariablesSettable() {
+    protected final void ensureInstanceVariablesSettable() {
         if (!isFrozen() && (getRuntime().getSafeLevel() < 4 || isTaint())) {
             return;
         }
@@ -1610,6 +1439,7 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
      */
     public class Finalizer implements Finalizable {
         private long id;
+        private IRubyObject firstFinalizer;
         private List<IRubyObject> finalizers;
         private AtomicBoolean finalized;
 
@@ -1619,28 +1449,35 @@ public class RubyBasicObject implements Cloneable, IRubyObject, Serializable, Co
         }
 
         public void addFinalizer(IRubyObject finalizer) {
-            if (finalizers == null) {
-                finalizers = new ArrayList<IRubyObject>();
+            if (firstFinalizer == null) {
+                firstFinalizer = finalizer;
+            } else {
+                if (finalizers == null) finalizers = new ArrayList<IRubyObject>(4);
+                finalizers.add(finalizer);
             }
-            finalizers.add(finalizer);
         }
 
         public void removeFinalizers() {
+            firstFinalizer = null;
             finalizers = null;
         }
 
         @Override
         public void finalize() {
             if (finalized.compareAndSet(false, true)) {
+                if (firstFinalizer != null) callFinalizer(firstFinalizer);
                 if (finalizers != null) {
                     for (int i = 0; i < finalizers.size(); i++) {
-                        IRubyObject finalizer = finalizers.get(i);
-                        RuntimeHelpers.invoke(
-                                finalizer.getRuntime().getCurrentContext(),
-                                finalizer, "call", RubyBasicObject.this.id());
+                        callFinalizer(finalizers.get(i));
                     }
                 }
             }
         }
+        
+        private void callFinalizer(IRubyObject finalizer) {
+            RuntimeHelpers.invoke(
+                    finalizer.getRuntime().getCurrentContext(),
+                    finalizer, "call", RubyBasicObject.this.id());
+        }
     }
 }
