@@ -167,17 +167,21 @@ public class RubyBignum extends RubyInteger {
      * This is faster, and mostly correct (?)
      */
     static double convertToDouble(BigInteger bigint) {
+        boolean negative = bigint.signum() == -1;
+        if (negative) bigint = bigint.abs();
         byte[] arr = bigint.toByteArray();
         double res = 0;
-        double acc = 1;
-        for (int i = arr.length - 1; i > 0 ; i--)
-        {
+        double acc = 1d;
+        for (int i = arr.length - 1; i > 0 ; i--) {
             res += (double) (arr[i] & 0xff) * acc;
             acc *= 256;
         }
-        res += (double) arr[0] * acc; // final byte sign is significant
-        return res;
+        if (arr[0] != 0) {
+            res += (double) (arr[0] & 0xff) * acc;
+        }
+        return (negative) ? -res : res ;
     }
+
     
     /** rb_int2big
      * 
@@ -231,27 +235,31 @@ public class RubyBignum extends RubyInteger {
     @JRubyMethod(name = "+", required = 1)
     public IRubyObject op_plus(ThreadContext context, IRubyObject other) {
         if (other instanceof RubyFixnum) {
-            return addFixnum((RubyFixnum)other);
+            return addFixnum(((RubyFixnum)other).getLongValue());
         } else if (other instanceof RubyBignum) {
-            return addBignum((RubyBignum)other);
+            return addBignum(((RubyBignum)other).value);
         } else if (other instanceof RubyFloat) {
             return addFloat((RubyFloat)other);
         }
         return addOther(context, other);
     }
-    
-    private IRubyObject addFixnum(RubyFixnum other) {
-        return bignorm(getRuntime(), value.add(fix2big(other)));
+
+    private IRubyObject addFixnum(long other) {
+        BigInteger result = value.add(BigInteger.valueOf(other));
+        if (other > 0 && value.signum() > 0) return new RubyBignum(getRuntime(), result);
+        return bignorm(getRuntime(), result);
     }
-    
-    private IRubyObject addBignum(RubyBignum other) {
-        return bignorm(getRuntime(), value.add(other.value));
+
+    private IRubyObject addBignum(BigInteger other) {
+        BigInteger result = value.add(other);
+        if (value.signum() > 0 && other.signum() > 0) return new RubyBignum(getRuntime(), result);
+        return bignorm(getRuntime(), result);
     }
-    
+
     private IRubyObject addFloat(RubyFloat other) {
         return RubyFloat.newFloat(getRuntime(), big2dbl(this) + other.getDoubleValue());
     }
-    
+
     private IRubyObject addOther(ThreadContext context, IRubyObject other) {
         return coerceBin(context, "+", other);
     }
@@ -262,27 +270,31 @@ public class RubyBignum extends RubyInteger {
     @JRubyMethod(name = "-", required = 1)
     public IRubyObject op_minus(ThreadContext context, IRubyObject other) {
         if (other instanceof RubyFixnum) {
-            return subtractFixnum((RubyFixnum)other);
+            return subtractFixnum(((RubyFixnum)other).getLongValue());
         } else if (other instanceof RubyBignum) {
-            return subtractBignum((RubyBignum)other);
+            return subtractBignum(((RubyBignum)other).value);
         } else if (other instanceof RubyFloat) {
             return subtractFloat((RubyFloat)other);
         }
         return subtractOther(context, other);
     }
-    
-    private IRubyObject subtractFixnum(RubyFixnum other) {
-        return bignorm(getRuntime(), value.subtract(fix2big(((RubyFixnum) other))));
+
+    private IRubyObject subtractFixnum(long other) {
+        BigInteger result = value.subtract(BigInteger.valueOf(other));
+        if (value.signum() < 0 && other > 0) return new RubyBignum(getRuntime(), result);
+        return bignorm(getRuntime(), result);
     }
-    
-    private IRubyObject subtractBignum(RubyBignum other) {
-        return bignorm(getRuntime(), value.subtract(((RubyBignum) other).value));
+
+    private IRubyObject subtractBignum(BigInteger other) {
+        BigInteger result = value.subtract(other);
+        if (value.signum() < 0 && other.signum() > 0) return new RubyBignum(getRuntime(), result);
+        return bignorm(getRuntime(), result);
     }
-    
+
     private IRubyObject subtractFloat(RubyFloat other) {
-        return RubyFloat.newFloat(getRuntime(), big2dbl(this) - ((RubyFloat) other).getDoubleValue());
+        return RubyFloat.newFloat(getRuntime(), big2dbl(this) - other.getDoubleValue());
     }
-    
+
     private IRubyObject subtractOther(ThreadContext context, IRubyObject other) {
         return coerceBin(context, "-", other);
     }
@@ -292,11 +304,14 @@ public class RubyBignum extends RubyInteger {
      */
     @JRubyMethod(name = "*", required = 1)
     public IRubyObject op_mul(ThreadContext context, IRubyObject other) {
+        Ruby runtime = context.getRuntime();
         if (other instanceof RubyFixnum) {
-            return bignorm(getRuntime(), value.multiply(fix2big(((RubyFixnum) other))));
+            BigInteger result = value.multiply(fix2big(((RubyFixnum) other)));
+            return result.signum() == 0 ? RubyFixnum.zero(runtime) : new RubyBignum(runtime, result);
         }
         if (other instanceof RubyBignum) {
-            return bignorm(getRuntime(), value.multiply(((RubyBignum) other).value));
+            BigInteger result = value.multiply(((RubyBignum)other).value);
+            return result.signum() == 0 ? RubyFixnum.zero(runtime) : new RubyBignum(runtime, result);
         } else if (other instanceof RubyFloat) {
             return RubyFloat.newFloat(getRuntime(), big2dbl(this) * ((RubyFloat) other).getDoubleValue());
         }
@@ -306,9 +321,8 @@ public class RubyBignum extends RubyInteger {
     /**
      * rb_big_divide. Shared part for both "/" and "div" operations.
      */
-    private IRubyObject op_divide(ThreadContext context, IRubyObject other, String op) {
-        assert ("/".equals(op) || "div".equals(op));
-
+    private IRubyObject op_divide(ThreadContext context, IRubyObject other, boolean slash) {
+        Ruby runtime = context.getRuntime();
         final BigInteger otherValue;
         if (other instanceof RubyFixnum) {
             otherValue = fix2big((RubyFixnum) other);
@@ -316,26 +330,25 @@ public class RubyBignum extends RubyInteger {
             otherValue = ((RubyBignum) other).value;
         } else if (other instanceof RubyFloat) {
             double div = big2dbl(this) / ((RubyFloat) other).getDoubleValue();
-            if ("/".equals(op)) {
-                return RubyFloat.newFloat(getRuntime(),
-                        big2dbl(this) / ((RubyFloat) other).getDoubleValue());
+            if (slash) {
+                return RubyFloat.newFloat(runtime, div);
             } else {
-                return RubyNumeric.dbl2num(getRuntime(), div);
+                return RubyNumeric.dbl2num(runtime, div);
             }
         } else {
-            return coerceBin(context, op, other);
+            return coerceBin(context, slash ? "/" : "div", other);
         }
 
-        if (otherValue.equals(BigInteger.ZERO)) {
-            throw getRuntime().newZeroDivisionError();
-        }
+        if (otherValue.signum() == 0) throw runtime.newZeroDivisionError();
 
-        BigInteger[] results = value.divideAndRemainder(otherValue);
-
-        if ((value.signum() * otherValue.signum()) == -1 && results[1].signum() != 0) {
-            return bignorm(getRuntime(), results[0].subtract(BigInteger.ONE));
+        final BigInteger result;
+        if (value.signum() * otherValue.signum() == -1) {
+            BigInteger[] results = value.divideAndRemainder(otherValue);
+            result = results[1].signum() != 0 ? results[0].subtract(BigInteger.ONE) : results[0];
+        } else {
+            result = value.divide(otherValue); 
         }
-        return bignorm(getRuntime(), results[0]);
+        return bignorm(getRuntime(), result);
     }
 
     /** rb_big_div
@@ -343,7 +356,7 @@ public class RubyBignum extends RubyInteger {
      */
     @JRubyMethod(name = {"/"}, required = 1)
     public IRubyObject op_div(ThreadContext context, IRubyObject other) {
-        return op_divide(context, other, "/");
+        return op_divide(context, other, true);
     }
 
     /** rb_big_idiv
@@ -351,7 +364,7 @@ public class RubyBignum extends RubyInteger {
      */
     @JRubyMethod(name = {"div"}, required = 1)
     public IRubyObject op_idiv(ThreadContext context, IRubyObject other) {
-        return op_divide(context, other, "div");
+        return op_divide(context, other, false);
     }
 
     /** rb_big_divmod
@@ -359,6 +372,7 @@ public class RubyBignum extends RubyInteger {
      */
     @JRubyMethod(name = "divmod", required = 1)
     public IRubyObject divmod(ThreadContext context, IRubyObject other) {
+        Ruby runtime = context.getRuntime();
         final BigInteger otherValue;
         if (other instanceof RubyFixnum) {
             otherValue = fix2big((RubyFixnum) other);
@@ -368,18 +382,14 @@ public class RubyBignum extends RubyInteger {
             return coerceBin(context, "divmod", other);
         }
 
-        if (otherValue.equals(BigInteger.ZERO)) {
-            throw getRuntime().newZeroDivisionError();
-        }
+        if (otherValue.signum() == 0) throw runtime.newZeroDivisionError();
 
         BigInteger[] results = value.divideAndRemainder(otherValue);
-
         if ((value.signum() * otherValue.signum()) == -1 && results[1].signum() != 0) {
             results[0] = results[0].subtract(BigInteger.ONE);
             results[1] = otherValue.add(results[1]);
     	}
-        final Ruby runtime = getRuntime();
-        return RubyArray.newArray(getRuntime(), bignorm(runtime, results[0]), bignorm(runtime, results[1]));
+        return RubyArray.newArray(runtime, bignorm(runtime, results[0]), bignorm(runtime, results[1]));
     }
 
     /** rb_big_modulo
@@ -395,16 +405,14 @@ public class RubyBignum extends RubyInteger {
         } else {
             return coerceBin(context, "%", other);
         }
-        if (otherValue.equals(BigInteger.ZERO)) {
-            throw getRuntime().newZeroDivisionError();
-        }
-        BigInteger result = value.mod(otherValue.abs());
-        if (otherValue.signum() == -1 && result.signum() != 0) {
-            result = otherValue.add(result);
-        }
-        return bignorm(getRuntime(), result);
+        Ruby runtime = context.getRuntime();
+        if (otherValue.signum() == 0) throw runtime.newZeroDivisionError();
 
-            }
+        BigInteger result = value.mod(otherValue.abs());
+        if (otherValue.signum() == -1 && result.signum() != 0) result = otherValue.add(result);
+        return bignorm(runtime, result);
+
+    }
 
     /** rb_big_remainder
      * 
@@ -419,10 +427,9 @@ public class RubyBignum extends RubyInteger {
         } else {
             return coerceBin(context, "remainder", other);
         }
-        if (otherValue.equals(BigInteger.ZERO)) {
-            throw getRuntime().newZeroDivisionError();
-    }
-        return bignorm(getRuntime(), value.remainder(otherValue));
+        Ruby runtime = context.getRuntime();
+        if (otherValue.signum() == 0) throw runtime.newZeroDivisionError();
+        return bignorm(runtime, value.remainder(otherValue));
     }
 
     /** rb_big_quo
@@ -639,6 +646,14 @@ public class RubyBignum extends RubyInteger {
         return value.testBit((int)position) ? RubyFixnum.one(getRuntime()) : RubyFixnum.zero(getRuntime());
     }
 
+    @Override
+    public final int compareTo(IRubyObject other) {
+        if (other instanceof RubyBignum) {
+            return value.compareTo(((RubyBignum)other).value);
+        }
+        return (int)coerceCmp(getRuntime().getCurrentContext(), "<=>", other).convertToInteger().getLongValue();
+    }
+
     /** rb_big_cmp     
      * 
      */
