@@ -57,6 +57,7 @@ import static org.jruby.util.Numeric.f_zero_p;
 import static org.jruby.util.Numeric.k_exact_p;
 import static org.jruby.util.Numeric.k_inexact_p;
 
+import org.jcodings.specific.ASCIIEncoding;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.runtime.Arity;
@@ -417,7 +418,7 @@ public class RubyComplex extends RubyNumeric {
     private static IRubyObject convertCommon(ThreadContext context, IRubyObject recv, IRubyObject a1, IRubyObject a2) {
         Frame frame = context.getCurrentFrame();
         IRubyObject backref = frame.getBackRef();
-        if (backref != null && backref instanceof RubyMatchData) ((RubyMatchData)backref).use();
+        if (backref instanceof RubyMatchData) ((RubyMatchData)backref).use();
 
         if (a1 instanceof RubyString) a1 = str_to_c_strict(context, a1);
         if (a2 instanceof RubyString) a2 = str_to_c_strict(context, a2);
@@ -794,7 +795,8 @@ public class RubyComplex extends RubyNumeric {
      */
     private static boolean signbit(ThreadContext context, IRubyObject x) {
         if (x instanceof RubyFloat) {
-            return Double.doubleToLongBits(((RubyFloat)x).getDoubleValue()) < 0;
+            double value = ((RubyFloat)x).getDoubleValue();
+            return !Double.isNaN(value) && Double.doubleToLongBits(value) < 0;
         }
         return f_negative_p(context, x);
     }
@@ -816,10 +818,11 @@ public class RubyComplex extends RubyNumeric {
         RubyString str = f_to_s(context, real).convertToString();
         str.cat(impos ? (byte)'+' : (byte)'-');
         str.cat(f_to_s(context, f_abs(context, image)).convertToString().getByteList());
+        if (!lastCharDigit(str)) str.cat((byte)'*');
         str.cat((byte)'i');
         return str;
     }
-    
+
     /** nucomp_inspect
      * 
      */
@@ -831,11 +834,17 @@ public class RubyComplex extends RubyNumeric {
         str.cat(f_inspect(context, real).convertToString().getByteList());
         str.cat(impos ? (byte)'+' : (byte)'-');
         str.cat(f_inspect(context, f_abs(context, image)).convertToString().getByteList());
+        if (!lastCharDigit(str)) str.cat((byte)'*');
         str.cat((byte)'i');
         str.cat((byte)')');
         return str;
     }
 
+    private static boolean lastCharDigit(RubyString str) {
+        ByteList bytes = str.getByteList();
+        return ASCIIEncoding.INSTANCE.isDigit(bytes.bytes[bytes.begin + bytes.realSize - 1]);
+    }
+
     /** nucomp_marshal_dump
      * 
      */
@@ -902,7 +911,7 @@ public class RubyComplex extends RubyNumeric {
         IRubyObject sr, si, re;
         sr = si = re = runtime.getNil();
         boolean po = false;
-        IRubyObject m = RubyRegexp.newRegexp(runtime, Numeric.ComplexPatterns.comp_pat0).callMethod(context, "match", s);
+        IRubyObject m = RubyRegexp.newDummyRegexp(runtime, Numeric.ComplexPatterns.comp_pat0).callMethod(context, "match", s);
 
         if (!m.isNil()) {
             sr = m.callMethod(context, "[]", RubyFixnum.one(runtime));
@@ -912,7 +921,7 @@ public class RubyComplex extends RubyNumeric {
         }
 
         if (m.isNil()) {
-            m = RubyRegexp.newRegexp(runtime, Numeric.ComplexPatterns.comp_pat1).callMethod(context, "match", s);
+            m = RubyRegexp.newDummyRegexp(runtime, Numeric.ComplexPatterns.comp_pat1).callMethod(context, "match", s);
 
             if (!m.isNil()) {
                 sr = runtime.getNil();
@@ -927,7 +936,7 @@ public class RubyComplex extends RubyNumeric {
         }
 
         if (m.isNil()) {
-            m = RubyRegexp.newRegexp(runtime, Numeric.ComplexPatterns.comp_pat2).callMethod(context, "match", s);
+            m = RubyRegexp.newDummyRegexp(runtime, Numeric.ComplexPatterns.comp_pat2).callMethod(context, "match", s);
             if (m.isNil()) return runtime.newArray(runtime.getNil(), recv);
             sr = m.callMethod(context, "[]", RubyFixnum.one(runtime));
             if (m.callMethod(context, "[]", RubyFixnum.two(runtime)).isNil()) {
@@ -971,7 +980,7 @@ public class RubyComplex extends RubyNumeric {
         RubyArray a = str_to_c_internal(context, recv);
         if (a.eltInternal(0).isNil() || a.eltInternal(1).convertToString().getByteList().length() > 0) {
             IRubyObject s = recv.callMethod(context, "inspect");
-            throw context.getRuntime().newArgumentError("invalid value for Complex: " + s.convertToString());
+            throw context.getRuntime().newArgumentError("invalid value for convert(): " + s.convertToString());
         }
         return a.eltInternal(0);
     }    
