@@ -109,8 +109,8 @@ public class RubyException extends RubyObject {
             unmarshalStream.registerLinkTarget(exc);
             unmarshalStream.defaultVariablesUnmarshal(exc);
             
-            exc.message = exc.removeInternalVariable("mesg");
-            exc.set_backtrace(exc.removeInternalVariable("bt"));
+            exc.message = (IRubyObject)exc.removeInternalVariable("mesg");
+            exc.set_backtrace((IRubyObject)exc.removeInternalVariable("bt"));
             
             return exc;
         }
@@ -149,8 +149,10 @@ public class RubyException extends RubyObject {
     public static final int RUBY_FRAMED = 2;
     public static final int RUBY_COMPILED = 3;
     public static final int RUBY_HYBRID = 4;
+
+    public static final int RAW_FRAME_CROP_COUNT = 10;
     
-    public static final int TRACE_TYPE;
+    public static int TRACE_TYPE;
     
     static {
         String style = SafePropertyAccessor.getProperty("jruby.backtrace.style", "ruby_framed").toLowerCase();
@@ -288,7 +290,7 @@ public class RubyException extends RubyObject {
                     printStackTraceLine(errorStream, stackTraceLine);
                 }
 
-                if (!debug && i == RubyException.TRACE_HEAD && elements.length > RubyException.TRACE_MAX) {
+                if (!debug && TRACE_TYPE != RAW && i == RubyException.TRACE_HEAD && elements.length > RubyException.TRACE_MAX) {
                     int hiddenLevels = elements.length - RubyException.TRACE_HEAD - RubyException.TRACE_TAIL;
                             errorStream.print("\t ... " + hiddenLevels + " levels...\n");
                     i = elements.length - RubyException.TRACE_TAIL;
@@ -312,4 +314,9 @@ public class RubyException extends RubyObject {
             
         return true;
     }
+
+    // rb_exc_new3
+    public static IRubyObject newException(ThreadContext context, RubyClass exceptionClass, IRubyObject message) {
+        return exceptionClass.callMethod(context, "new", message.convertToString());
+    }
 }
