@@ -183,6 +183,10 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         }
     };
 
+    public String getPath() {
+        return path;
+    }
+
     @JRubyModule(name="File::Constants")
     public static class Constants {}
     
@@ -306,92 +310,97 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         
         // null channel always succeeds for all locking operations
         if (descriptor.isNull()) return RubyFixnum.zero(context.getRuntime());
-        
-        FileChannel fileChannel = (FileChannel)descriptor.getChannel();
-        int lockMode = RubyNumeric.num2int(lockingConstant);
 
-        // Exclusive locks in Java require the channel to be writable, otherwise
-        // an exception is thrown (terminating JRuby execution).
-        // But flock behavior of MRI is that it allows
-        // exclusive locks even on non-writable file. So we convert exclusive
-        // lock to shared lock if the channel is not writable, to better match
-        // the MRI behavior.
-        if (!openFile.isWritable() && (lockMode & LOCK_EX) > 0) {
-            lockMode = (lockMode ^ LOCK_EX) | LOCK_SH;
+        if (descriptor.getChannel() instanceof FileChannel) {
+            FileChannel fileChannel = (FileChannel)descriptor.getChannel();
+            int lockMode = RubyNumeric.num2int(lockingConstant);
+
+            // Exclusive locks in Java require the channel to be writable, otherwise
+            // an exception is thrown (terminating JRuby execution).
+            // But flock behavior of MRI is that it allows
+            // exclusive locks even on non-writable file. So we convert exclusive
+            // lock to shared lock if the channel is not writable, to better match
+            // the MRI behavior.
+            if (!openFile.isWritable() && (lockMode & LOCK_EX) > 0) {
+                lockMode = (lockMode ^ LOCK_EX) | LOCK_SH;
+            }
+
+            try {
+                switch (lockMode) {
+                    case LOCK_UN:
+                    case LOCK_UN | LOCK_NB:
+                        if (currentLock != null) {
+                            currentLock.release();
+                            currentLock = null;
+
+                            return RubyFixnum.zero(context.getRuntime());
+                        }
+                        break;
+                    case LOCK_EX:
+                        if (currentLock != null) {
+                            currentLock.release();
+                            currentLock = null;
+                        }
+                        currentLock = fileChannel.lock();
+                        if (currentLock != null) {
+                            return RubyFixnum.zero(context.getRuntime());
+                        }
+
+                        break;
+                    case LOCK_EX | LOCK_NB:
+                        if (currentLock != null) {
+                            currentLock.release();
+                            currentLock = null;
+                        }
+                        currentLock = fileChannel.tryLock();
+                        if (currentLock != null) {
+                            return RubyFixnum.zero(context.getRuntime());
+                        }
+
+                        break;
+                    case LOCK_SH:
+                        if (currentLock != null) {
+                            currentLock.release();
+                            currentLock = null;
+                        }
+
+                        currentLock = fileChannel.lock(0L, Long.MAX_VALUE, true);
+                        if (currentLock != null) {
+                            return RubyFixnum.zero(context.getRuntime());
+                        }
+
+                        break;
+                    case LOCK_SH | LOCK_NB:
+                        if (currentLock != null) {
+                            currentLock.release();
+                            currentLock = null;
+                        }
+
+                        currentLock = fileChannel.tryLock(0L, Long.MAX_VALUE, true);
+                        if (currentLock != null) {
+                            return RubyFixnum.zero(context.getRuntime());
+                        }
+
+                        break;
+                    default:
+                }
+            } catch (IOException ioe) {
+                if (context.getRuntime().getDebug().isTrue()) {
+                    ioe.printStackTrace(System.err);
+                }
+                // Return false here
+            } catch (java.nio.channels.OverlappingFileLockException ioe) {
+                if (context.getRuntime().getDebug().isTrue()) {
+                    ioe.printStackTrace(System.err);
+                }
+                // Return false here
+            }
+
+            return context.getRuntime().getFalse();
+        } else {
+            // We're not actually a real file, so we can't flock
+            return context.getRuntime().getFalse();
         }
-
-        try {
-            switch (lockMode) {
-                case LOCK_UN:
-                case LOCK_UN | LOCK_NB:
-                    if (currentLock != null) {
-                        currentLock.release();
-                        currentLock = null;
-
-                        return RubyFixnum.zero(context.getRuntime());
-                    }
-                    break;
-                case LOCK_EX:
-                    if (currentLock != null) {
-                        currentLock.release();
-                        currentLock = null;
-                    }
-                    currentLock = fileChannel.lock();
-                    if (currentLock != null) {
-                        return RubyFixnum.zero(context.getRuntime());
-                    }
-
-                    break;
-                case LOCK_EX | LOCK_NB:
-                    if (currentLock != null) {
-                        currentLock.release();
-                        currentLock = null;
-                    }
-                    currentLock = fileChannel.tryLock();
-                    if (currentLock != null) {
-                        return RubyFixnum.zero(context.getRuntime());
-                    }
-
-                    break;
-                case LOCK_SH:
-                    if (currentLock != null) {
-                        currentLock.release();
-                        currentLock = null;
-                    }
-
-                    currentLock = fileChannel.lock(0L, Long.MAX_VALUE, true);
-                    if (currentLock != null) {
-                        return RubyFixnum.zero(context.getRuntime());
-                    }
-
-                    break;
-                case LOCK_SH | LOCK_NB:
-                    if (currentLock != null) {
-                        currentLock.release();
-                        currentLock = null;
-                    }
-
-                    currentLock = fileChannel.tryLock(0L, Long.MAX_VALUE, true);
-                    if (currentLock != null) {
-                        return RubyFixnum.zero(context.getRuntime());
-                    }
-
-                    break;
-                default:
-            }
-        } catch (IOException ioe) {
-            if (context.getRuntime().getDebug().isTrue()) {
-                ioe.printStackTrace(System.err);
-            }
-            // Return false here
-        } catch (java.nio.channels.OverlappingFileLockException ioe) {
-            if (context.getRuntime().getDebug().isTrue()) {
-                ioe.printStackTrace(System.err);
-            }
-            // Return false here
-        }
-
-        return context.getRuntime().getFalse();
     }
 
     @JRubyMethod(required = 1, optional = 2, frame = true, visibility = Visibility.PRIVATE)
@@ -453,8 +462,8 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         
         return this;
     }
-    
-    private void sysopenInternal(String path, ModeFlags modes, int perm) throws InvalidValueException {
+
+    protected void sysopenInternal(String path, ModeFlags modes, int perm) throws InvalidValueException {
         openFile = new OpenFile();
         
         openFile.setPath(path);
@@ -466,7 +475,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         registerDescriptor(descriptor);
     }
     
-    private void openInternal(String path, String modeString) throws InvalidValueException {
+    protected void openInternal(String path, String modeString) throws InvalidValueException {
         openFile = new OpenFile();
 
         openFile.setMode(getIOModes(getRuntime(), modeString).getOpenFileFlags());
@@ -1125,8 +1134,13 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         }
 
         if (child.equals(".")) {
-            // skip it
-            if (canonicalPath != null && canonicalPath.length() == 0 ) canonicalPath += "/";
+            // no canonical path yet or length is zero, and we have a / followed by a dot...
+            if (slash == -1) {
+                // we don't have another slash after this, so replace /. with /
+                if (canonicalPath != null && canonicalPath.length() == 0 && slash == -1) canonicalPath += "/";
+            } else {
+                // we do have another slash; omit both / and . (JRUBY-1606)
+            }
         } else if (child.equals("..")) {
             if (canonicalPath == null) throw new IllegalArgumentException("Cannot have .. at the start of an absolute path");
             int lastDir = canonicalPath.lastIndexOf('/');
@@ -1491,27 +1505,17 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         return runtime.newFixnum(oldMask);
     }
 
-    /**
-     * This method does NOT set atime, only mtime, since Java doesn't support anything else.
-     */
     @JRubyMethod(required = 2, rest = true, meta = true)
     public static IRubyObject utime(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         Ruby runtime = context.getRuntime();
-        
-        // Ignore access_time argument since Java does not support it.
-        
-        long mtime;
-        if (args[1] instanceof RubyTime) {
-            mtime = ((RubyTime) args[1]).getJavaDate().getTime();
-        } else if (args[1] instanceof RubyNumeric) {
-            mtime = RubyNumeric.num2long(args[1]);
-        } else if (args[1] == runtime.getNil()) {
-            mtime = System.currentTimeMillis();
-        } else {
-            RubyTime time = (RubyTime) TypeConverter.convertToType(args[1], runtime.getTime(),"to_time", true);
-            mtime = time.getJavaDate().getTime();
+        long[] atimeval = null;
+        long[] mtimeval = null;
+
+        if (args[0] != runtime.getNil() || args[1] != runtime.getNil()) {
+            atimeval = extractTimeval(runtime, args[0]);
+            mtimeval = extractTimeval(runtime, args[1]);
         }
-        
+
         for (int i = 2, j = args.length; i < j; i++) {
             RubyString filename = RubyString.stringValue(args[i]);
             runtime.checkSafeString(filename);
@@ -1521,8 +1525,8 @@ public class RubyFile extends RubyIO implements EncodingCapable {
             if (!fileToTouch.exists()) {
                 throw runtime.newErrnoENOENTError(" No such file or directory - \"" + filename + "\"");
             }
-            
-            fileToTouch.setLastModified(mtime);
+
+            runtime.getPosix().utimes(fileToTouch.getAbsolutePath(), atimeval, mtimeval);
         }
         
         return runtime.newFixnum(args.length - 2);
@@ -1552,6 +1556,34 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         return runtime.newFixnum(args.length);
     }
 
+    /**
+     * Extract a timeval (an array of 2 longs: seconds and microseconds from epoch) from
+     * an IRubyObject.
+     */
+    private static long[] extractTimeval(Ruby runtime, IRubyObject value) {
+        long[] timeval = new long[2];
+
+        if (value instanceof RubyFloat) {
+            timeval[0] = Platform.IS_32_BIT ? RubyNumeric.num2int(value) : RubyNumeric.num2long(value);
+            double fraction = ((RubyFloat) value).getDoubleValue() % 1.0;
+            timeval[1] = (long)(fraction * 1e6 + 0.5);
+        } else if (value instanceof RubyNumeric) {
+            timeval[0] = Platform.IS_32_BIT ? RubyNumeric.num2int(value) : RubyNumeric.num2long(value);
+            timeval[1] = 0;
+        } else {
+            RubyTime time;
+            if (value instanceof RubyTime) {
+                time = ((RubyTime) value);
+            } else {
+                time = (RubyTime) TypeConverter.convertToType(value, runtime.getTime(), "to_time", true);
+            }
+            timeval[0] = Platform.IS_32_BIT ? RubyNumeric.num2int(time.to_i()) : RubyNumeric.num2long(time.to_i());
+            timeval[1] = Platform.IS_32_BIT ? RubyNumeric.num2int(time.usec()) : RubyNumeric.num2long(time.usec());
+        }
+
+        return timeval;
+    }
+
     // Fast path since JNA stat is about 10x slower than this
     private static IRubyObject getLastModified(Ruby runtime, String path) {
         JRubyFile file = JRubyFile.create(runtime.getCurrentDirectory(), path);
