@@ -43,6 +43,8 @@ import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.common.IRubyWarnings.ID;
 import org.jruby.java.MiniJava;
+import org.jruby.runtime.Block;
+import org.jruby.runtime.BlockBody;
 import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
@@ -70,7 +72,7 @@ public class RubyFixnum extends RubyInteger {
             }
         };
 
-        if (runtime.getInstanceConfig().getCompatVersion() == CompatVersion.RUBY1_8) {
+        if (!runtime.is1_9()) {
             fixnum.includeModule(runtime.getPrecision());
         }
 
@@ -238,6 +240,22 @@ public class RubyFixnum extends RubyInteger {
      *  Instance Methods
      *  ================ 
      */
+    @Override
+    public IRubyObject times(ThreadContext context, Block block) {
+        Ruby runtime = context.getRuntime();
+        long value = this.value;
+        if (block.getBody().getArgumentType() == BlockBody.ZERO_ARGS) {
+            IRubyObject nil = runtime.getNil();
+            for (long i = 0; i < value; i++) {
+                block.yield(context, nil);
+            }
+        } else {
+            for (long i = 0; i < value; i++) {
+                block.yield(context, RubyFixnum.newFixnum(runtime, i));
+            }
+        }
+        return this;
+    }
 
     /** fix_to_s
      * 
@@ -382,13 +400,9 @@ public class RubyFixnum extends RubyInteger {
         Ruby runtime = context.getRuntime();
         if (other instanceof RubyFixnum) {
             long otherValue = ((RubyFixnum) other).value;
-            if (value == 0) {
-                return RubyFixnum.zero(runtime);
-            }
             long result = value * otherValue;
-            if (result / value != otherValue) {
-                return RubyBignum.newBignum(runtime, value).op_mul(context, other);
-            }
+            if (result == 0) return RubyFixnum.zero(runtime);
+            if (result / value != otherValue) return RubyBignum.newBignum(runtime, value).op_mul(context, other);
             return newFixnum(runtime, result);
         } else if (other instanceof RubyBignum) {
             return ((RubyBignum) other).op_mul(context, this);
@@ -622,80 +636,153 @@ public class RubyFixnum extends RubyInteger {
     /** fix_equal
      * 
      */
-    @JRubyMethod(name = "==")
+    @JRubyMethod(name = "==", compat = CompatVersion.RUBY1_8)
     @Override
     public IRubyObject op_equal(ThreadContext context, IRubyObject other) {
-        if (other instanceof RubyFixnum) {
-            return RubyBoolean.newBoolean(context.getRuntime(), value == ((RubyFixnum) other).value);
-        }
+        if (other instanceof RubyFixnum) return RubyBoolean.newBoolean(context.getRuntime(), value == ((RubyFixnum) other).value);
         return super.op_num_equal(context, other);
     }
 
+    @JRubyMethod(name = "==", compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_equal19(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyFixnum) return RubyBoolean.newBoolean(context.getRuntime(), value == ((RubyFixnum) other).value);
+        return op_equalOther(context, other);
+    }
+
+    private IRubyObject op_equalOther(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyBignum) return RubyBoolean.newBoolean(context.getRuntime(),
+                BigInteger.valueOf(value).compareTo(((RubyBignum)other).getValue()) == 0);
+        if (other instanceof RubyFloat) return RubyBoolean.newBoolean(context.getRuntime(), (double)value == ((RubyFloat)other).getDoubleValue());
+        return super.op_num_equal(context, other);
+    }
+
+    @Override
+    public final int compareTo(IRubyObject other) {
+        if (other instanceof RubyFixnum) {
+            long otherValue = ((RubyFixnum)other).value;
+            return value == otherValue ? 0 : value > otherValue ? 1 : -1; 
+        }
+        return compareToOther(other);
+    }
+
+    private int compareToOther(IRubyObject other) {
+        if (other instanceof RubyBignum) return BigInteger.valueOf(value).compareTo(((RubyBignum)other).getValue());
+        if (other instanceof RubyFloat) return Double.compare((double)value, ((RubyFloat)other).getDoubleValue());
+        return (int)coerceCmp(getRuntime().getCurrentContext(), "<=>", other).convertToInteger().getLongValue();
+    }
+
     /** fix_cmp
      * 
      */
-    @JRubyMethod(name = "<=>")
+    @JRubyMethod(name = "<=>", compat = CompatVersion.RUBY1_8)
     public IRubyObject op_cmp(ThreadContext context, IRubyObject other) {
-        if (other instanceof RubyFixnum) {
-            return compareFixnum(context, (RubyFixnum)other);
-        }
+        if (other instanceof RubyFixnum) return compareFixnum(context.getRuntime(), (RubyFixnum)other);
         return coerceCmp(context, "<=>", other);
     }
-    
-    private IRubyObject compareFixnum(ThreadContext context, RubyFixnum other) {
+
+    private IRubyObject compareFixnum(Ruby runtime, RubyFixnum other) {
         long otherValue = ((RubyFixnum) other).value;
-        if (value == otherValue) {
-            return RubyFixnum.zero(context.getRuntime());
-        }
-        if (value > otherValue) {
-            return RubyFixnum.one(context.getRuntime());
-        }
-        return RubyFixnum.minus_one(context.getRuntime());
+        return value == otherValue ? RubyFixnum.zero(runtime) : value > otherValue ?
+                RubyFixnum.one(runtime) : RubyFixnum.minus_one(runtime);
+    }
+
+    @JRubyMethod(name = "<=>", compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_cmp19(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyFixnum) return compareFixnum(context.getRuntime(), (RubyFixnum)other);
+        return compareOther(context, other);
+    }
+
+    private IRubyObject compareOther(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyBignum) return newFixnum(context.getRuntime(), BigInteger.valueOf(value).compareTo(((RubyBignum)other).getValue()));
+        if (other instanceof RubyFloat) return dbl_cmp(context.getRuntime(), (double)value, ((RubyFloat)other).getDoubleValue());
+        return coerceCmp(context, "<=>", other);
     }
 
     /** fix_gt
      * 
      */
-    @JRubyMethod(name = ">")
+    @JRubyMethod(name = ">", compat = CompatVersion.RUBY1_8)
     public IRubyObject op_gt(ThreadContext context, IRubyObject other) {
-        if (other instanceof RubyFixnum) {
-            return RubyBoolean.newBoolean(context.getRuntime(), value > ((RubyFixnum) other).value);
+        if (other instanceof RubyFixnum) return RubyBoolean.newBoolean(context.getRuntime(), value > ((RubyFixnum) other).value);
+        return coerceRelOp(context, ">", other);
     }
+
+    @JRubyMethod(name = ">", compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_gt19(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyFixnum) return RubyBoolean.newBoolean(context.getRuntime(), value > ((RubyFixnum) other).value);
+        return op_gtOther(context, other);
+    }
+
+    private IRubyObject op_gtOther(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyBignum) return RubyBoolean.newBoolean(context.getRuntime(),
+                BigInteger.valueOf(value).compareTo(((RubyBignum)other).getValue()) > 0);
+        if (other instanceof RubyFloat) return RubyBoolean.newBoolean(context.getRuntime(), (double)value > ((RubyFloat) other).getDoubleValue());
         return coerceRelOp(context, ">", other);
     }
 
     /** fix_ge
      * 
      */
-    @JRubyMethod(name = ">=")
+    @JRubyMethod(name = ">=", compat = CompatVersion.RUBY1_8)
     public IRubyObject op_ge(ThreadContext context, IRubyObject other) {
-        if (other instanceof RubyFixnum) {
-            return RubyBoolean.newBoolean(context.getRuntime(), value >= ((RubyFixnum) other).value);
-            }
+        if (other instanceof RubyFixnum) return RubyBoolean.newBoolean(context.getRuntime(), value >= ((RubyFixnum) other).value);
+        return coerceRelOp(context, ">=", other);
+    }
+
+    @JRubyMethod(name = ">=", compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_ge19(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyFixnum) return RubyBoolean.newBoolean(context.getRuntime(), value >= ((RubyFixnum) other).value);
+        return op_geOther(context, other);
+    }
+
+    private IRubyObject op_geOther(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyBignum) return RubyBoolean.newBoolean(context.getRuntime(),
+                BigInteger.valueOf(value).compareTo(((RubyBignum)other).getValue()) >= 0);
+        if (other instanceof RubyFloat) return RubyBoolean.newBoolean(context.getRuntime(), (double)value >= ((RubyFloat) other).getDoubleValue());
         return coerceRelOp(context, ">=", other);
     }
 
     /** fix_lt
      * 
      */
-    @JRubyMethod(name = "<")
+    @JRubyMethod(name = "<", compat = CompatVersion.RUBY1_8)
     public IRubyObject op_lt(ThreadContext context, IRubyObject other) {
-        if (other instanceof RubyFixnum) {
-            return RubyBoolean.newBoolean(context.getRuntime(), value < ((RubyFixnum) other).value);
-        }
-        
+        if (other instanceof RubyFixnum) return RubyBoolean.newBoolean(context.getRuntime(), value < ((RubyFixnum) other).value);
         return coerceRelOp(context, "<", other);
     }
-        
+
+    @JRubyMethod(name = "<", compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_lt19(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyFixnum) return RubyBoolean.newBoolean(context.getRuntime(), value < ((RubyFixnum) other).value);
+        return op_ltOther(context, other);
+    }
+
+    private IRubyObject op_ltOther(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyBignum) return RubyBoolean.newBoolean(context.getRuntime(),
+                BigInteger.valueOf(value).compareTo(((RubyBignum)other).getValue()) < 0);
+        if (other instanceof RubyFloat) return RubyBoolean.newBoolean(context.getRuntime(), (double)value < ((RubyFloat) other).getDoubleValue());
+        return coerceRelOp(context, "<", other);
+    }
+
     /** fix_le
      * 
      */
-    @JRubyMethod(name = "<=")
+    @JRubyMethod(name = "<=", compat = CompatVersion.RUBY1_8)
     public IRubyObject op_le(ThreadContext context, IRubyObject other) {
-        if (other instanceof RubyFixnum) {
-            return RubyBoolean.newBoolean(context.getRuntime(), value <= ((RubyFixnum) other).value);
-        }
-        
+        if (other instanceof RubyFixnum) return RubyBoolean.newBoolean(context.getRuntime(), value <= ((RubyFixnum) other).value);
+        return coerceRelOp(context, "<=", other);
+    }
+
+    @JRubyMethod(name = "<=", compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_le19(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyFixnum) return RubyBoolean.newBoolean(context.getRuntime(), value <= ((RubyFixnum) other).value);
+        return op_leOther(context, other);
+    }
+
+    private IRubyObject op_leOther(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyBignum) return RubyBoolean.newBoolean(context.getRuntime(),
+                BigInteger.valueOf(value).compareTo(((RubyBignum)other).getValue()) <= 0);
+        if (other instanceof RubyFloat) return RubyBoolean.newBoolean(context.getRuntime(), (double)value <= ((RubyFloat) other).getDoubleValue());
         return coerceRelOp(context, "<=", other);
     }
 
@@ -843,11 +930,6 @@ public class RubyFixnum extends RubyInteger {
         return this;
     }
 
-    @Override
-    public IRubyObject freeze(ThreadContext context) {
-        return this;
-    }
-    
     // Piece of mri rb_to_id
     @Override
     public String asJavaString() {
