@@ -42,7 +42,6 @@ import java.util.Map;
 
 import org.jruby.anno.JRubyMethod;
 import org.jruby.common.IRubyWarnings.ID;
-import org.jruby.environment.OSEnvironmentReaderExcepton;
 import org.jruby.environment.OSEnvironment;
 import org.jruby.internal.runtime.ValueAccessor;
 import org.jruby.javasupport.JavaUtil;
@@ -131,13 +130,24 @@ public class RubyGlobal {
         runtime.getGlobalVariables().define("$0", d);
 
         // Version information:
-        IRubyObject version = runtime.newString(Constants.RUBY_VERSION).freeze(context);
+        IRubyObject version = null;
+        IRubyObject patchlevel = null;
         IRubyObject release = runtime.newString(Constants.COMPILE_DATE).freeze(context);
         IRubyObject platform = runtime.newString(Constants.PLATFORM).freeze(context);
         IRubyObject engine = runtime.newString(Constants.ENGINE).freeze(context);
 
+        switch (runtime.getInstanceConfig().getCompatVersion()) {
+        case RUBY1_8:
+            version = runtime.newString(Constants.RUBY_VERSION).freeze(context);
+            patchlevel = runtime.newString(Constants.RUBY_PATCHLEVEL).freeze(context);
+            break;
+        case RUBY1_9:
+            version = runtime.newString(Constants.RUBY1_9_VERSION).freeze(context);
+            patchlevel = runtime.newString(Constants.RUBY1_9_PATCHLEVEL).freeze(context);
+            break;
+        }
         runtime.defineGlobalConstant("RUBY_VERSION", version);
-        runtime.defineGlobalConstant("RUBY_PATCHLEVEL", runtime.newString(Constants.RUBY_PATCHLEVEL).freeze(context));
+        runtime.defineGlobalConstant("RUBY_PATCHLEVEL", patchlevel);
         runtime.defineGlobalConstant("RUBY_RELEASE_DATE", release);
         runtime.defineGlobalConstant("RUBY_PLATFORM", platform);
         runtime.defineGlobalConstant("RUBY_ENGINE", engine);
@@ -249,15 +259,9 @@ public class RubyGlobal {
     }
 
     private static void defineGlobalEnvConstants(Ruby runtime) {
-
     	Map environmentVariableMap = null;
     	OSEnvironment environment = new OSEnvironment();
-    	try {
-    		environmentVariableMap = environment.getEnvironmentVariableMap(runtime);
-    	} catch (OSEnvironmentReaderExcepton e) {
-    		// If the environment variables are not accessible shouldn't terminate 
-    		runtime.getWarnings().warn(ID.MISCELLANEOUS, e.getMessage());
-    	}
+        environmentVariableMap = environment.getEnvironmentVariableMap(runtime);
 		
     	if (environmentVariableMap == null) {
             // if the environment variables can't be obtained, define an empty ENV
@@ -273,7 +277,6 @@ public class RubyGlobal {
         Map systemProps = environment.getSystemPropertiesMap(runtime);
         runtime.defineGlobalConstant("ENV_JAVA", new StringOnlyRubyHash(
                 runtime, systemProps, runtime.getNil()));
-        
     }
 
     private static class NonEffectiveGlobalVariable extends GlobalVariable {
