@@ -109,6 +109,7 @@ public class RubyHash extends RubyObject implements Map {
         runtime.setHash(hashc);
         hashc.index = ClassIndex.HASH;
         hashc.kindOf = new RubyModule.KindOf() {
+            @Override
             public boolean isKindOf(IRubyObject obj, RubyModule type) {
                 return obj instanceof RubyHash;
             }
@@ -127,6 +128,7 @@ public class RubyHash extends RubyObject implements Map {
         }
     };
 
+    @Override
     public int getNativeTypeIndex() {
         return ClassIndex.HASH;
     }
@@ -321,6 +323,7 @@ public class RubyHash extends RubyObject implements Map {
             return oldValue;
         }
 
+        @Override
         public boolean equals(Object other){
             if(!(other instanceof RubyHashEntry)) return false;
             RubyHashEntry otherEntry = (RubyHashEntry)other;
@@ -330,6 +333,7 @@ public class RubyHash extends RubyObject implements Map {
             return false;
         }
 
+        @Override
         public int hashCode(){
             return key.hashCode() ^ value.hashCode();
         }
@@ -687,6 +691,7 @@ public class RubyHash extends RubyObject implements Map {
      *
      */
     @JRubyMethod(name = "to_a")
+    @Override
     public RubyArray to_a() {
         final Ruby runtime = getRuntime();
         final RubyArray result = RubyArray.newArray(runtime, size);
@@ -753,6 +758,7 @@ public class RubyHash extends RubyObject implements Map {
         return this;
     }
 
+    @Override
     public RubyHash convertToHash() {
         return this;
     }
@@ -761,6 +767,11 @@ public class RubyHash extends RubyObject implements Map {
         internalPut(key, value);
     }
 
+    public final RubyHash fastASetChained(IRubyObject key, IRubyObject value) {
+        internalPut(key, value);
+        return this;
+    }
+
     @Deprecated
     public IRubyObject op_aset(IRubyObject key, IRubyObject value) {
         return op_aset(getRuntime().getCurrentContext(), key, value);
@@ -769,31 +780,46 @@ public class RubyHash extends RubyObject implements Map {
     /** rb_hash_aset
      *
      */
-    @JRubyMethod(name = {"[]=", "store"}, required = 2)
+    @JRubyMethod(name = {"[]=", "store"}, required = 2, compat = CompatVersion.RUBY1_8)
     public IRubyObject op_aset(ThreadContext context, IRubyObject key, IRubyObject value) {
         modify();
 
-        if (!(key instanceof RubyString)) {
-            internalPut(key, value);
+        if (key instanceof RubyString) {
+            op_asetForString(context.getRuntime(), (RubyString)key, value);
         } else {
-            final RubyHashEntry entry = internalGetEntry(key);
-            if (entry != NO_ENTRY) {
-                entry.value = value;
-            } else {
-                RubyString realKey = (RubyString)key;
-                
-                if (!realKey.isFrozen()) {
-                    realKey = realKey.strDup(context.getRuntime(), realKey.getMetaClass().getRealClass());;
-                    realKey.setFrozen(true);
-                }
-                
-                internalPut(realKey, value, false);
-            }
+            internalPut(key, value);
         }
 
         return value;
     }
 
+    @JRubyMethod(name = {"[]=", "store"}, required = 2, compat = CompatVersion.RUBY1_9)
+    public IRubyObject op_aset19(ThreadContext context, IRubyObject key, IRubyObject value) {
+        modify();
+
+        Ruby runtime = context.getRuntime();
+        if (key.getMetaClass().getRealClass() == runtime.getString()) {
+            op_asetForString(runtime, (RubyString)key, value);
+        } else {
+            internalPut(key, value);
+        }
+
+        return value;
+    }
+
+    private void op_asetForString(Ruby runtime, RubyString key, IRubyObject value) {
+        final RubyHashEntry entry = internalGetEntry(key);
+        if (entry != NO_ENTRY) {
+            entry.value = value;
+        } else {
+            if (!key.isFrozen()) {
+                key = key.strDup(runtime, key.getMetaClass().getRealClass());
+                key.setFrozen(true);
+            }
+            internalPut(key, value, false);
+        }
+    }
+
     /**
      * Note: this is included as a compatibility measure for AR-JDBC
      * @deprecated use RubyHash.op_aset instead
@@ -985,7 +1011,7 @@ public class RubyHash extends RubyObject implements Map {
         visitAll(new Visitor() {
             public void visit(IRubyObject key, IRubyObject value) {
                 // rb_assoc_new equivalent
-                block.yield(context, RubyArray.newArray(runtime, key, value), null, null, false);
+                block.yield(context, RubyArray.newArray(runtime, key, value));
             }
         });
 
@@ -1061,8 +1087,8 @@ public class RubyHash extends RubyObject implements Map {
      *
      */
     @JRubyMethod(name = "sort", frame = true)
-    public RubyArray sort(Block block) {
-        return to_a().sort_bang(block);
+    public IRubyObject sort(ThreadContext context, Block block) {
+        return to_a().sort_bang(context, block);
     }
 
     /** rb_hash_index
@@ -1514,6 +1540,7 @@ public class RubyHash extends RubyObject implements Map {
         return result;
     }
 
+    @Override
     public Class getJavaClass() {
         return Map.class;
     }
@@ -1562,6 +1589,7 @@ public class RubyHash extends RubyObject implements Map {
         rb_clear();
     }
 
+    @Override
     public boolean equals(Object other) {
         if (!(other instanceof RubyHash)) return false;
         if (this == other) return true;
@@ -1603,10 +1631,12 @@ public class RubyHash extends RubyObject implements Map {
             return new BaseIterator(view);
         }
 
+        @Override
         public boolean contains(Object o) {
             return view.contains(RubyHash.this, o);
         }
 
+        @Override
         public void clear() {
             RubyHash.this.clear();
         }
@@ -1615,6 +1645,7 @@ public class RubyHash extends RubyObject implements Map {
             return RubyHash.this.size;
         }
 
+        @Override
         public boolean remove(Object o) {
             return view.remove(RubyHash.this, o);
         }
@@ -1631,10 +1662,12 @@ public class RubyHash extends RubyObject implements Map {
             return new BaseIterator(view);
         }
 
+        @Override
         public boolean contains(Object o) {
             return view.contains(RubyHash.this, o);
         }
 
+        @Override
         public void clear() {
             RubyHash.this.clear();
         }
@@ -1643,6 +1676,7 @@ public class RubyHash extends RubyObject implements Map {
             return RubyHash.this.size;
         }
 
+        @Override
         public boolean remove(Object o) {
             return view.remove(RubyHash.this, o);
         }
@@ -1814,6 +1848,7 @@ public class RubyHash extends RubyObject implements Map {
             return entry.setValue(JavaUtil.convertJavaToRuby(runtime, o));
         }
 
+        @Override
         public boolean equals(Object o) {
             if (!(o instanceof ConvertingEntry)) {
                 return false;
@@ -1821,6 +1856,8 @@ public class RubyHash extends RubyObject implements Map {
             ConvertingEntry other = (ConvertingEntry)o;
             return entry.equals(other.entry);
         }
+        
+        @Override
         public int hashCode() {
             return entry.hashCode();
         }
