@@ -12,6 +12,7 @@
  * rights and limitations under the License.
  *
  * Copyright (C) 2007, 2008 Nick Sieger <nicksieger@gmail.com>
+ * Copyright (C) 2009 Joseph LaFata <joe@quibb.org>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -42,6 +43,8 @@ import java.util.Map;
 
 import java.util.Set;
 import org.jruby.ast.executable.Script;
+import org.jruby.compiler.ASTCompiler;
+import org.jruby.compiler.ASTCompiler19;
 import org.jruby.exceptions.MainExitException;
 import org.jruby.runtime.Constants;
 import org.jruby.runtime.load.LoadService;
@@ -166,31 +169,34 @@ public class RubyInstanceConfig {
     private boolean shouldCheckSyntax = false;
     private String inputFieldSeparator = null;
     private boolean managementEnabled = true;
+    private String inPlaceBackupExtension = null;
+    private boolean parserDebug = false;
 
     private int safeLevel = 0;
 
     private String jrubyHome;
 
-    public static final boolean FASTEST_COMPILE_ENABLED
+    public static final boolean PEEPHOLE_OPTZ
+            = SafePropertyAccessor.getBoolean("jruby.peephole.optz", true);
+    public static boolean FASTEST_COMPILE_ENABLED
             = SafePropertyAccessor.getBoolean("jruby.compile.fastest");
-    public static final boolean BOXED_COMPILE_ENABLED
-            = FASTEST_COMPILE_ENABLED
-            || SafePropertyAccessor.getBoolean("jruby.compile.boxed");
-    public static final boolean FASTOPS_COMPILE_ENABLED
+    public static boolean FASTOPS_COMPILE_ENABLED
             = FASTEST_COMPILE_ENABLED
             || SafePropertyAccessor.getBoolean("jruby.compile.fastops");
-    public static final boolean FRAMELESS_COMPILE_ENABLED
+    public static boolean FRAMELESS_COMPILE_ENABLED
             = FASTEST_COMPILE_ENABLED
             || SafePropertyAccessor.getBoolean("jruby.compile.frameless");
-    public static final boolean POSITIONLESS_COMPILE_ENABLED
+    public static boolean POSITIONLESS_COMPILE_ENABLED
             = FASTEST_COMPILE_ENABLED
             || SafePropertyAccessor.getBoolean("jruby.compile.positionless");
-    public static final boolean THREADLESS_COMPILE_ENABLED
+    public static boolean THREADLESS_COMPILE_ENABLED
             = FASTEST_COMPILE_ENABLED
             || SafePropertyAccessor.getBoolean("jruby.compile.threadless");
-    public static final boolean FASTCASE_COMPILE_ENABLED =
+    public static boolean FASTCASE_COMPILE_ENABLED =
             SafePropertyAccessor.getBoolean("jruby.compile.fastcase");
-    public static final boolean LAZYHANDLES_COMPILE = SafePropertyAccessor.getBoolean("jruby.compile.lazyHandles", false);
+    public static boolean FASTSEND_COMPILE_ENABLED =
+            SafePropertyAccessor.getBoolean("jruby.compile.fastsend");
+    public static boolean LAZYHANDLES_COMPILE = SafePropertyAccessor.getBoolean("jruby.compile.lazyHandles", false);
     public static final boolean FORK_ENABLED
             = SafePropertyAccessor.getBoolean("jruby.fork.enabled");
     public static final boolean POOLING_ENABLED
@@ -211,7 +217,6 @@ public class RubyInstanceConfig {
     public static final String COMPILE_EXCLUDE
             = SafePropertyAccessor.getProperty("jruby.jit.exclude");
     public static boolean nativeEnabled = true;
-    
 
     public static interface LoadServiceCreator {
         LoadService create(Ruby runtime);
@@ -263,8 +268,9 @@ public class RubyInstanceConfig {
             compatVersion = CompatVersion.RUBY1_8;
         } else if (compatString.equalsIgnoreCase("RUBY1_9")) {
             compatVersion = CompatVersion.RUBY1_9;
+            compileMode = compileMode.OFF;
         } else {
-            System.err.println("Compatibility version `" + compatString + "' invalid; use RUBY1_8 or RUBY1_9. Using RUBY1_8.");
+            error.println("Compatibility version `" + compatString + "' invalid; use RUBY1_8 or RUBY1_9. Using RUBY1_8.");
             compatVersion = CompatVersion.RUBY1_8;
         }
 
@@ -351,7 +357,7 @@ public class RubyInstanceConfig {
                 .append("  -d              set debugging flags (set $DEBUG to true)\n")
                 .append("  -e 'command'    one line of script. Several -e's allowed. Omit [programfile]\n")
                 .append("  -Fpattern       split() pattern for autosplit (-a)\n")
-                //.append("  -i[extension]   edit ARGV files in place (make backup if extension supplied)\n")
+                .append("  -i[extension]   edit ARGV files in place (make backup if extension supplied)\n")
                 .append("  -Idirectory     specify $LOAD_PATH directory (may be used more than once)\n")
                 .append("  -J[java option] pass an option on to the JVM (e.g. -J-Xmx512m)\n")
                 .append("                    use --properties to list JRuby properties\n")
@@ -369,6 +375,7 @@ public class RubyInstanceConfig {
                 .append("  -W[level]       set warning level; 0=silence, 1=medium, 2=verbose (default)\n")
                 //.append("  -x[directory]   strip off text before #!ruby line and perhaps cd to directory\n")
                 .append("  -X[option]      enable extended option (omit option to list)\n")
+                .append("  -y              enable parsing debug output")
                 .append("  --copyright     print the copyright\n")
                 .append("  --debug         sets the execution mode most suitable for debugger functionality\n")
                 .append("  --jdb           runs JRuby process under JDB\n")
@@ -412,8 +419,6 @@ public class RubyInstanceConfig {
                 .append("       Set compilation mode. JIT is default; FORCE compiles all, OFF disables\n")
                 .append("    jruby.compile.fastest=true|false\n")
                 .append("       (EXPERIMENTAL) Turn on all experimental compiler optimizations\n")
-                .append("    jruby.compile.boxed=true|false\n")
-                .append("       (EXPERIMENTAL) Use boxed variables; this can speed up some methods. Default is false\n")
                 .append("    jruby.compile.frameless=true|false\n")
                 .append("       (EXPERIMENTAL) Turn on frameless compilation where possible\n")
                 .append("    jruby.compile.positionless=true|false\n")
@@ -480,19 +485,22 @@ public class RubyInstanceConfig {
     }
 
     public String getVersionString() {
-        String ver = Constants.RUBY_VERSION;
+        String ver = null;
+        String patchlevel = null;
         switch (compatVersion) {
         case RUBY1_8:
             ver = Constants.RUBY_VERSION;
+            patchlevel = Constants.RUBY_PATCHLEVEL;
             break;
         case RUBY1_9:
             ver = Constants.RUBY1_9_VERSION;
+            patchlevel = Constants.RUBY1_9_PATCHLEVEL;
             break;
         }
 
         String fullVersion = String.format(
                 "jruby %s (ruby %s patchlevel %s) (%s rev %s) [%s-java]\n",
-                Constants.VERSION, ver, Constants.RUBY_PATCHLEVEL,
+                Constants.VERSION, ver, patchlevel,
                 Constants.COMPILE_DATE, Constants.REVISION,
                 SafePropertyAccessor.getProperty("os.arch", "unknown")
                 );
@@ -564,6 +572,10 @@ public class RubyInstanceConfig {
         return compatVersion;
     }
 
+    public void setCompatVersion(CompatVersion compatVersion) {
+        this.compatVersion = compatVersion;
+    }
+
     public void setOutput(PrintStream newOutput) {
         output = newOutput;
     }
@@ -634,17 +646,28 @@ public class RubyInstanceConfig {
 
     public String getJRubyHome() {
         if (jrubyHome == null) {
+            // try the normal property first
             if (!Ruby.isSecurityRestricted()) {
                 jrubyHome = SafePropertyAccessor.getProperty("jruby.home");
             }
 
             if (jrubyHome != null) {
+                // verify it if it's there
                 jrubyHome = verifyHome(jrubyHome);
             } else {
                 try {
+                    // try loading from classloader resources
                     jrubyHome = getClass().getResource("/META-INF/jruby.home")
                         .toURI().getSchemeSpecificPart();
                 } catch (Exception e) {}
+
+                if (jrubyHome != null) {
+                    // verify it if it's there
+                    jrubyHome = verifyHome(jrubyHome);
+                } else {
+                    // otherwise fall back on system temp location
+                    jrubyHome = System.getProperty("java.io.tmpdir");
+                }
             }
         }
         return jrubyHome;
@@ -664,7 +687,10 @@ public class RubyInstanceConfig {
             if (!f.isAbsolute()) {
                 home = f.getAbsolutePath();
             }
-            f.mkdirs();
+            if (!f.exists()) {
+                System.err.println("Warning: JRuby home \"" + f + "\" does not exist, using " + System.getProperty("java.io.tmpdir"));
+                return System.getProperty("java.io.tmpdir");
+            }
         }
         return home;
     }
@@ -716,7 +742,7 @@ public class RubyInstanceConfig {
         }
 
         private boolean isInterpreterArgument(String argument) {
-            return (argument.charAt(0) == '-' || argument.charAt(0) == '+') && !endOfArguments;
+            return argument.length() > 0 && (argument.charAt(0) == '-' || argument.charAt(0) == '+') && !endOfArguments;
         }
 
         private String getArgumentError(String additionalError) {
@@ -793,9 +819,10 @@ public class RubyInstanceConfig {
                     shouldPrintUsage = true;
                     shouldRunInterpreter = false;
                     break;
-                // FIXME: -i flag not supported
-//                    case 'i' :
-//                        break;
+                case 'i' :
+                    inPlaceBackupExtension = grabOptionalValue();
+                    if(inPlaceBackupExtension == null) inPlaceBackupExtension = "";
+                    break FOR;
                 case 'I':
                     String s = grabValue(getArgumentError("-I must be followed by a directory name to add to lib path"));
                     String[] ls = s.split(java.io.File.pathSeparator);
@@ -913,6 +940,9 @@ public class RubyInstanceConfig {
                         throw mee;
                     }
                     break FOR;
+                case 'y':
+                    parserDebug = true;
+                    break FOR;
                 case '-':
                     if (argument.equals("--command") || argument.equals("--bin")) {
                         characterIndex = argument.length();
@@ -921,6 +951,9 @@ public class RubyInstanceConfig {
                     } else if (argument.equals("--compat")) {
                         characterIndex = argument.length();
                         compatVersion = CompatVersion.getVersionFromString(grabValue(getArgumentError("--compat must be RUBY1_8 or RUBY1_9")));
+                        if (compatVersion == CompatVersion.RUBY1_9) {
+                            compileMode = compileMode.OFF;
+                        }
                         if (compatVersion == null) {
                             compatVersion = CompatVersion.RUBY1_8;
                         }
@@ -952,6 +985,16 @@ public class RubyInstanceConfig {
                     } else if (argument.equals("--bytecode")) {
                         setShowBytecode(true);
                         break FOR;
+                    } else if (argument.equals("--fast")) {
+                        compileMode = CompileMode.FORCE;
+                        FASTEST_COMPILE_ENABLED = true;
+                        FASTOPS_COMPILE_ENABLED = true;
+                        FRAMELESS_COMPILE_ENABLED = true;
+                        POSITIONLESS_COMPILE_ENABLED = true;
+                        FASTCASE_COMPILE_ENABLED = true;
+                        FASTSEND_COMPILE_ENABLED = true;
+                        RubyException.TRACE_TYPE = RubyException.RAW;
+                        break FOR;
                     } else {
                         if (argument.equals("--")) {
                             // ruby interpreter compatibilty
@@ -985,8 +1028,14 @@ public class RubyInstanceConfig {
         }
 
         private String resolveScript(String scriptName) {
-            File fullName = new File(getJRubyHome(), "bin/" + scriptName);
-            if (fullName.exists()) {
+            // try cwd first
+            File fullName = new File(scriptName);
+            if (fullName.exists() && fullName.isFile()) {
+                return fullName.getAbsolutePath();
+            }
+
+            fullName = new File(getJRubyHome(), "bin/" + scriptName);
+            if (fullName.exists() && fullName.isFile()) {
                 return fullName.getAbsolutePath();
             }
 
@@ -995,7 +1044,7 @@ public class RubyInstanceConfig {
                 String[] paths = path.split(System.getProperty("path.separator"));
                 for (int i = 0; i < paths.length; i++) {
                     fullName = new File(paths[i], scriptName);
-                    if (fullName.exists()) {
+                    if (fullName.exists() && fullName.isFile()) {
                         return fullName.getAbsolutePath();
                     }
                 }
@@ -1078,7 +1127,7 @@ public class RubyInstanceConfig {
                 return getInput();
             } else {
                 File file = JRubyFile.create(getCurrentDirectory(), getScriptFileName());
-                return new BufferedInputStream(new FileInputStream(file));
+                return new BufferedInputStream(new FileInputStream(file), 8192);
             }
         } catch (IOException e) {
             throw new MainExitException(1, "Error opening script file: " + e.getMessage());
@@ -1139,6 +1188,10 @@ public class RubyInstanceConfig {
         return debug;
     }
 
+    public boolean isParserDebug() {
+        return parserDebug;
+    }
+
     public boolean isShowVersion() {
         return showVersion;
     }
@@ -1207,6 +1260,10 @@ public class RubyInstanceConfig {
         return classCache;
     }
 
+    public String getInPlaceBackupExtention() {
+        return inPlaceBackupExtension;
+    }
+
     public void setClassCache(ClassCache classCache) {
         this.classCache = classCache;
     }
@@ -1222,5 +1279,13 @@ public class RubyInstanceConfig {
     public Set getExcludedMethods() {
         return excludedMethods;
     }
+
+    public ASTCompiler newCompiler() {
+        if (compatVersion == CompatVersion.RUBY1_8) {
+            return new ASTCompiler();
+        } else {
+            return new ASTCompiler19();
+        }
+    }
     
 }
