@@ -38,6 +38,7 @@ import static org.jruby.RubyEnumerator.enumeratorize;
 import static org.jruby.util.Numeric.f_gcd;
 import static org.jruby.util.Numeric.f_lcm;
 
+import org.jcodings.Encoding;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.runtime.Block;
@@ -46,6 +47,8 @@ import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
+import org.jruby.util.Numeric;
+import org.jruby.util.StringSupport;
 
 /** Implementation of the Integer class.
  *
@@ -67,7 +70,7 @@ public abstract class RubyInteger extends RubyNumeric {
 
         integer.getSingletonClass().undefineMethod("new");
 
-        if (runtime.getInstanceConfig().getCompatVersion() == CompatVersion.RUBY1_8) {
+        if (!runtime.is1_9()) {
             integer.includeModule(runtime.getPrecision());
         }
 
@@ -115,37 +118,41 @@ public abstract class RubyInteger extends RubyNumeric {
      */
     @JRubyMethod(name = "upto", frame = true)
     public IRubyObject upto(ThreadContext context, IRubyObject to, Block block) {
-        final Ruby runtime = getRuntime();
-
         if (this instanceof RubyFixnum && to instanceof RubyFixnum) {
-            RubyFixnum toFixnum = (RubyFixnum) to;
-            final long toValue = toFixnum.getLongValue();
-            final long fromValue = getLongValue();
-
-            if (block.getBody().getArgumentType() == BlockBody.ZERO_ARGS) {
-                final IRubyObject nil = runtime.getNil();
-                for (long i = fromValue; i <= toValue; i++) {
-                    block.yield(context, nil);
-                }
-            } else {
-                for (long i = fromValue; i <= toValue; i++) {
-                    block.yield(context, RubyFixnum.newFixnum(runtime, i));
-                }
-            }
+            fixnumUpto(context, ((RubyFixnum)this).getLongValue(), ((RubyFixnum)to).getLongValue(), block);
         } else {
-            RubyNumeric i = this;
-
-            while (true) {
-                if (i.callMethod(context, ">", to).isTrue()) {
-                    break;
-                }
-                block.yield(context, i);
-                i = (RubyNumeric) i.callMethod(context, "+", RubyFixnum.one(runtime));
-            }
+            duckUpto(context, this, to, block);
         }
         return this;
     }
 
+    private static void fixnumUpto(ThreadContext context, long from, long to, Block block) {
+        Ruby runtime = context.getRuntime();
+        if (block.getBody().getArgumentType() == BlockBody.ZERO_ARGS) {
+            IRubyObject nil = runtime.getNil();
+            for (long i = from; i <= to; i++) {
+                block.yield(context, nil);
+            }
+        } else {
+            for (long i = from; i <= to; i++) {
+                block.yield(context, RubyFixnum.newFixnum(runtime, i));
+            }
+        }
+    }
+
+    private static void duckUpto(ThreadContext context, IRubyObject from, IRubyObject to, Block block) {
+        Ruby runtime = context.getRuntime();
+        IRubyObject i = from;
+        RubyFixnum one = RubyFixnum.one(runtime);
+        while (true) {
+            if (i.callMethod(context, ">", to).isTrue()) {
+                break;
+            }
+            block.yield(context, i);
+            i = i.callMethod(context, "+", one);
+        }
+    }
+
     @JRubyMethod(name = "upto", frame = true, compat = CompatVersion.RUBY1_9)
     public IRubyObject upto19(final ThreadContext context, IRubyObject to, final Block block) {
         return block.isGiven() ? upto(context, to, block) : enumeratorize(context.getRuntime(), this, "upto", to);
@@ -157,35 +164,41 @@ public abstract class RubyInteger extends RubyNumeric {
     // TODO: Make callCoerced work in block context...then fix downto, step, and upto.
     @JRubyMethod(name = "downto", frame = true)
     public IRubyObject downto(ThreadContext context, IRubyObject to, Block block) {
-        final Ruby runtime = getRuntime();
-
         if (this instanceof RubyFixnum && to instanceof RubyFixnum) {
-            RubyFixnum toFixnum = (RubyFixnum) to;
-            final long toValue = toFixnum.getLongValue();
-            if (block.getBody().getArgumentType() == BlockBody.ZERO_ARGS) {
-                final IRubyObject nil = runtime.getNil();
-                for (long i = getLongValue(); i >= toValue; i--) {
-                    block.yield(context, nil);
-                }
-            } else {
-                for (long i = getLongValue(); i >= toValue; i--) {
-                    block.yield(context, RubyFixnum.newFixnum(getRuntime(), i));
-                }
-            }
+            fixnumDownto(context, ((RubyFixnum)this).getLongValue(), ((RubyFixnum)to).getLongValue(), block);
         } else {
-            RubyNumeric i = this;
-
-            while (true) {
-                if (i.callMethod(context, "<", to).isTrue()) {
-                    break;
-                }
-                block.yield(context, i);
-                i = (RubyNumeric) i.callMethod(context, "-", RubyFixnum.one(getRuntime()));
-            }
+            duckDownto(context, this, to, block);
         }
         return this;
     }
 
+    private static void fixnumDownto(ThreadContext context, long from, long to, Block block) {
+        Ruby runtime = context.getRuntime();
+        if (block.getBody().getArgumentType() == BlockBody.ZERO_ARGS) {
+            final IRubyObject nil = runtime.getNil();
+            for (long i = from; i >= to; i--) {
+                block.yield(context, nil);
+            }
+        } else {
+            for (long i = from; i >= to; i--) {
+                block.yield(context, RubyFixnum.newFixnum(runtime, i));
+            }
+        }
+    }
+
+    private static void duckDownto(ThreadContext context, IRubyObject from, IRubyObject to, Block block) {
+        Ruby runtime = context.getRuntime();
+        IRubyObject i = from;
+        RubyFixnum one = RubyFixnum.one(runtime);
+        while (true) {
+            if (i.callMethod(context, "<", to).isTrue()) {
+                break;
+            }
+            block.yield(context, i);
+            i = i.callMethod(context, "-", one);
+        }
+    }
+
     @JRubyMethod(name = "downto", frame = true, compat = CompatVersion.RUBY1_9)
     public IRubyObject downto19(final ThreadContext context, IRubyObject to, final Block block) {
         return block.isGiven() ? downto(context, to, block) : enumeratorize(context.getRuntime(), this, "downto", to);
@@ -193,31 +206,16 @@ public abstract class RubyInteger extends RubyNumeric {
 
     @JRubyMethod(name = "times", frame = true)
     public IRubyObject times(ThreadContext context, Block block) {
-        final Ruby runtime = context.getRuntime();
-
-        if (this instanceof RubyFixnum) {
-            final long value = getLongValue();
-            if (block.getBody().getArgumentType() == BlockBody.ZERO_ARGS) {
-                final IRubyObject nil = runtime.getNil();
-                for (long i = 0; i < value; i++) {
-                    block.yield(context, nil);
-                }
-            } else {
-                for (long i = 0; i < value; i++) {
-                    block.yield(context, RubyFixnum.newFixnum(runtime, i));
-                }
-            }
-        } else {
-            RubyNumeric i = RubyFixnum.zero(runtime);
-            while (true) {
-                if (!i.callMethod(context, "<", this).isTrue()) {
-                    break;
-                }
-                block.yield(context, i);
-                i = (RubyNumeric) i.callMethod(context, "+", RubyFixnum.one(runtime));
+        Ruby runtime = context.getRuntime();
+        IRubyObject i = RubyFixnum.zero(runtime);
+        RubyFixnum one = RubyFixnum.one(runtime);
+        while (true) {
+            if (!i.callMethod(context, "<", this).isTrue()) {
+                break;
             }
+            block.yield(context, i);
+            i = i.callMethod(context, "+", one);
         }
-
         return this;
     }
 
@@ -249,22 +247,94 @@ public abstract class RubyInteger extends RubyNumeric {
     /** int_chr
      * 
      */
-    @JRubyMethod(name = "chr")
-    public RubyString chr() {
-        if (getLongValue() < 0 || getLongValue() > 0xff) {
-            throw getRuntime().newRangeError(this.toString() + " out of char range");
+    @JRubyMethod(name = "chr", compat = CompatVersion.RUBY1_8)
+    public RubyString chr(ThreadContext context) {
+        Ruby runtime = context.getRuntime();
+        long value = getLongValue();
+        if (value < 0 || value > 0xff) throw runtime.newRangeError(this.toString() + " out of char range");
+        return RubyString.newStringShared(runtime, SINGLE_CHAR_BYTELISTS[(int)value]);
+    }
+
+    @JRubyMethod(name = "chr", compat = CompatVersion.RUBY1_9)
+    public RubyString chr19(ThreadContext context) {
+        Ruby runtime = context.getRuntime();
+        long value = getLongValue();
+        if (value < 0 || value > 0xff) throw runtime.newRangeError(this.toString() + " out of char range");
+        if (value < 0x80) {
+            return RubyString.newUsAsciiStringShared(runtime, SINGLE_CHAR_BYTELISTS[(int)value]);
+        } else {
+            return RubyString.newStringShared(runtime, SINGLE_CHAR_BYTELISTS[(int)value]);
         }
-        return RubyString.newString(getRuntime(), SINGLE_CHAR_BYTELISTS[(int)getLongValue()]);
+    }
+
+    @JRubyMethod(name = "chr", compat = CompatVersion.RUBY1_9)
+    public RubyString chr19(ThreadContext context, IRubyObject arg) {
+        Ruby runtime = context.getRuntime();
+        long value = getLongValue();
+        Encoding enc = arg.convertToString().toEncoding(runtime);
+        int n;
+        if (value < 0 || (n = StringSupport.codeLength(runtime, enc, (int)value)) <= 0) {
+            throw runtime.newRangeError(this.toString() + " out of char range");
+        }
+        ByteList bytes = new ByteList(n);
+        enc.codeToMbc((int)value, bytes.bytes, 0);
+        bytes.realSize = n;
+        return RubyString.newStringNoCopy(runtime, bytes, enc, 0);
+    }
+
+    /** int_ord
+     * 
+     */
+    @JRubyMethod(name = "ord", compat = CompatVersion.RUBY1_9)
+    public IRubyObject ord(ThreadContext context) {
+        return this;
     }
 
     /** int_to_i
      * 
      */
-    @JRubyMethod(name = {"to_i", "to_int", "floor", "ceil", "round", "truncate"})
-    public RubyInteger to_i() {
+    @JRubyMethod(name = {"to_i", "to_int", "floor", "ceil", "truncate"})
+    public IRubyObject to_i() {
         return this;
     }
-    
+
+    @JRubyMethod(name = "round", compat = CompatVersion.RUBY1_8)
+    public IRubyObject round() {
+        return this;
+    }
+
+    @JRubyMethod(name = "round", compat = CompatVersion.RUBY1_9)
+    public IRubyObject round19() {
+        return this;
+    }
+
+    @JRubyMethod(name = "round", compat = CompatVersion.RUBY1_9)
+    public IRubyObject round19(ThreadContext context, IRubyObject arg) {
+        int ndigits = RubyNumeric.num2int(arg);
+        if (ndigits > 0) return RubyKernel.new_float(this, this);
+        if (ndigits == 0) return this;
+        ndigits = -ndigits;
+        Ruby runtime = context.getRuntime();
+        if (ndigits < 0) throw runtime.newArgumentError("ndigits out of range");
+        IRubyObject f = Numeric.int_pow(context, 10, ndigits);
+
+        if (this instanceof RubyFixnum && f instanceof RubyFixnum) {
+            long x = ((RubyFixnum)this).getLongValue();
+            long y = ((RubyFixnum)f).getLongValue();
+            boolean neg = x < 0;
+            if (neg) x = -x;
+            x = (x + y / 2) / y * y;
+            if (neg) x = -x;
+            return RubyFixnum.newFixnum(runtime, x);
+        } else {
+            IRubyObject h = f.callMethod(context, "/", RubyFixnum.two(runtime));
+            IRubyObject r = callMethod(context, "%", f);
+            IRubyObject n = callMethod(context, "-", r);
+            if (!r.callMethod(context, "<", h).isTrue()) n = n.callMethod(context, "+", f);
+            return n;
+        }
+    }
+
     /** integer_to_r
      * 
      */
