@@ -60,6 +60,7 @@ import org.jruby.exceptions.MainExitException;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.internal.runtime.JumpTarget;
 import org.jruby.javasupport.util.RuntimeHelpers;
+import org.jruby.runtime.Binding;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.CallType;
 import org.jruby.runtime.Frame;
@@ -120,7 +121,8 @@ public class RubyKernel {
         if (!IdUtil.isValidConstantName(nonInternedName)) {
             throw runtime.newNameError("autoload must be constant name", nonInternedName);
         }
-        
+
+        if (!(file instanceof RubyString)) throw runtime.newTypeError(file, runtime.getString());
         RubyString fileString = file.convertToString();
         
         if (fileString.isEmpty()) {
@@ -317,7 +319,7 @@ public class RubyKernel {
         }
 
         IRubyObject result = runtime.getNil();
-        if (runtime.getInstanceConfig().getCompatVersion() == CompatVersion.RUBY1_9) {
+        if (runtime.is1_9()) {
             if (args.length == 1) {
                 result = args[0];
             } else if (args.length > 1) {
@@ -413,12 +415,12 @@ public class RubyKernel {
         return getLastlineString(context, context.getRuntime()).sub_bang(context, args, block);
     }
 
-    @JRubyMethod(name = "sub!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE)
+    @JRubyMethod(name = "sub!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject sub_bang(ThreadContext context, IRubyObject recv, IRubyObject arg0, Block block) {
         return getLastlineString(context, context.getRuntime()).sub_bang(context, arg0, block);
     }
 
-    @JRubyMethod(name = "sub!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE)
+    @JRubyMethod(name = "sub!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject sub_bang(ThreadContext context, IRubyObject recv, IRubyObject arg0, IRubyObject arg1, Block block) {
         return getLastlineString(context, context.getRuntime()).sub_bang(context, arg0, arg1, block);
     }
@@ -437,7 +439,7 @@ public class RubyKernel {
         return str;
     }
 
-    @JRubyMethod(name = "sub", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE)
+    @JRubyMethod(name = "sub", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject sub(ThreadContext context, IRubyObject recv, IRubyObject arg0, Block block) {
         RubyString str = (RubyString) getLastlineString(context, context.getRuntime()).dup();
 
@@ -448,7 +450,7 @@ public class RubyKernel {
         return str;
     }
 
-    @JRubyMethod(name = "sub", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE)
+    @JRubyMethod(name = "sub", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject sub(ThreadContext context, IRubyObject recv, IRubyObject arg0, IRubyObject arg1, Block block) {
         RubyString str = (RubyString) getLastlineString(context, context.getRuntime()).dup();
 
@@ -467,12 +469,12 @@ public class RubyKernel {
         return getLastlineString(context, context.getRuntime()).gsub_bang(context, args, block);
     }
 
-    @JRubyMethod(name = "gsub!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE)
+    @JRubyMethod(name = "gsub!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject gsub_bang(ThreadContext context, IRubyObject recv, IRubyObject arg0, Block block) {
         return getLastlineString(context, context.getRuntime()).gsub_bang(context, arg0, block);
     }
 
-    @JRubyMethod(name = "gsub!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE)
+    @JRubyMethod(name = "gsub!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject gsub_bang(ThreadContext context, IRubyObject recv, IRubyObject arg0, IRubyObject arg1, Block block) {
         return getLastlineString(context, context.getRuntime()).gsub_bang(context, arg0, arg1, block);
     }
@@ -491,7 +493,7 @@ public class RubyKernel {
         return str;
     }
 
-    @JRubyMethod(name = "gsub", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE)
+    @JRubyMethod(name = "gsub", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject gsub(ThreadContext context, IRubyObject recv, IRubyObject arg0, Block block) {
         RubyString str = (RubyString) getLastlineString(context, context.getRuntime()).dup();
 
@@ -502,7 +504,7 @@ public class RubyKernel {
         return str;
     }
 
-    @JRubyMethod(name = "gsub", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE)
+    @JRubyMethod(name = "gsub", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject gsub(ThreadContext context, IRubyObject recv, IRubyObject arg0, IRubyObject arg1, Block block) {
         RubyString str = (RubyString) getLastlineString(context, context.getRuntime()).dup();
 
@@ -513,18 +515,18 @@ public class RubyKernel {
         return str;
     }
 
-    @JRubyMethod(name = "chop!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE)
+    @JRubyMethod(name = "chop!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject chop_bang(ThreadContext context, IRubyObject recv, Block block) {
-        return getLastlineString(context, context.getRuntime()).chop_bang();
+        return getLastlineString(context, context.getRuntime()).chop_bang(context);
     }
 
-    @JRubyMethod(name = "chop", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE)
+    @JRubyMethod(name = "chop", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject chop(ThreadContext context, IRubyObject recv, Block block) {
         RubyString str = getLastlineString(context, context.getRuntime());
 
         if (str.getByteList().realSize > 0) {
             str = (RubyString) str.dup();
-            str.chop_bang();
+            str.chop_bang(context);
             context.getPreviousFrame().setLastLine(str);
         }
 
@@ -539,14 +541,14 @@ public class RubyKernel {
         return getLastlineString(context, context.getRuntime()).chomp_bang(args);
     }
 
-    @JRubyMethod(name = "chomp!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE)
+    @JRubyMethod(name = "chomp!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject chomp_bang(ThreadContext context, IRubyObject recv) {
-        return getLastlineString(context, context.getRuntime()).chomp_bang();
+        return getLastlineString(context, context.getRuntime()).chomp_bang(context);
     }
 
-    @JRubyMethod(name = "chomp!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE)
+    @JRubyMethod(name = "chomp!", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject chomp_bang(ThreadContext context, IRubyObject recv, IRubyObject arg0) {
-        return getLastlineString(context, context.getRuntime()).chomp_bang(arg0);
+        return getLastlineString(context, context.getRuntime()).chomp_bang(context, arg0);
     }
 
     /**
@@ -565,12 +567,12 @@ public class RubyKernel {
         return dup;
     }
 
-    @JRubyMethod(name = "chomp", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE)
+    @JRubyMethod(name = "chomp", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject chomp(ThreadContext context, IRubyObject recv) {
         RubyString str = getLastlineString(context, context.getRuntime());
         RubyString dup = (RubyString) str.dup();
 
-        if (dup.chomp_bang().isNil()) {
+        if (dup.chomp_bang(context).isNil()) {
             return str;
         } 
 
@@ -578,12 +580,12 @@ public class RubyKernel {
         return dup;
     }
 
-    @JRubyMethod(name = "chomp", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE)
+    @JRubyMethod(name = "chomp", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = LASTLINE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject chomp(ThreadContext context, IRubyObject recv, IRubyObject arg0) {
         RubyString str = getLastlineString(context, context.getRuntime());
         RubyString dup = (RubyString) str.dup();
 
-        if (dup.chomp_bang(arg0).isNil()) {
+        if (dup.chomp_bang(context, arg0).isNil()) {
             return str;
         } 
 
@@ -603,22 +605,22 @@ public class RubyKernel {
         return getLastlineString(context, context.getRuntime()).split(context, args);
     }
 
-    @JRubyMethod(name = "split", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = {LASTLINE, BACKREF})
+    @JRubyMethod(name = "split", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = {LASTLINE, BACKREF}, compat = CompatVersion.RUBY1_8)
     public static IRubyObject split(ThreadContext context, IRubyObject recv) {
         return getLastlineString(context, context.getRuntime()).split(context);
     }
 
-    @JRubyMethod(name = "split", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = {LASTLINE, BACKREF})
+    @JRubyMethod(name = "split", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = {LASTLINE, BACKREF}, compat = CompatVersion.RUBY1_8)
     public static IRubyObject split(ThreadContext context, IRubyObject recv, IRubyObject arg0) {
         return getLastlineString(context, context.getRuntime()).split(context, arg0);
     }
 
-    @JRubyMethod(name = "split", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = {LASTLINE, BACKREF})
+    @JRubyMethod(name = "split", frame = true, module = true, visibility = PRIVATE, reads = LASTLINE, writes = {LASTLINE, BACKREF}, compat = CompatVersion.RUBY1_8)
     public static IRubyObject split(ThreadContext context, IRubyObject recv, IRubyObject arg0, IRubyObject arg1) {
         return getLastlineString(context, context.getRuntime()).split(context, arg0, arg1);
     }
 
-    @JRubyMethod(name = "scan", required = 1, frame = true, module = true, visibility = PRIVATE, reads = {LASTLINE, BACKREF}, writes = {LASTLINE, BACKREF})
+    @JRubyMethod(name = "scan", required = 1, frame = true, module = true, visibility = PRIVATE, reads = {LASTLINE, BACKREF}, writes = {LASTLINE, BACKREF}, compat = CompatVersion.RUBY1_8)
     public static IRubyObject scan(ThreadContext context, IRubyObject recv, IRubyObject pattern, Block block) {
         return getLastlineString(context, context.getRuntime()).scan(context, pattern, block);
     }
@@ -731,12 +733,12 @@ public class RubyKernel {
 
     @JRubyMethod(name = "binding", module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_8)
     public static RubyBinding binding(ThreadContext context, IRubyObject recv, Block block) {
-        return RubyBinding.newBinding(context.getRuntime(), recv);
+        return RubyBinding.newBinding(context.getRuntime(), context.currentBinding(recv));
     }
 
     @JRubyMethod(name = "binding", module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
     public static RubyBinding binding_1_9(ThreadContext context, IRubyObject recv, Block block) {
-        return RubyBinding.newBinding(context.getRuntime());
+        return RubyBinding.newBinding(context.getRuntime(), context.currentBinding());
     }
 
     @JRubyMethod(name = {"block_given?", "iterator?"}, frame = true, module = true, visibility = PRIVATE)
@@ -854,43 +856,55 @@ public class RubyKernel {
     @JRubyMethod(name = "eval", required = 1, optional = 3, frame = true, module = true, visibility = PRIVATE)
     public static IRubyObject eval(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
         Ruby runtime = context.getRuntime();
-            
         // string to eval
         RubyString src = args[0].convertToString();
         runtime.checkSafeString(src);
-        
-        IRubyObject scope = args.length > 1 && !args[1].isNil() ? args[1] : null;
-        String file;
+
+        boolean bindingGiven = args.length > 1 && !args[1].isNil();
+        Binding binding = bindingGiven ? convertToBinding(args[1]) : context.previousBinding();
         if (args.length > 2) {
-            file = args[2].convertToString().toString();
-        } else if (scope == null) {
-            file = "(eval)";
+            // file given, use it and force it into binding
+            binding.setFile(args[2].convertToString().toString());
         } else {
-            file = null;
+            // file not given
+            if (bindingGiven) {
+                // binding given, use binding's file
+            } else {
+                // no binding given, use (eval)
+                binding.setFile("(eval)");
+            }
         }
-        int line;
         if (args.length > 3) {
-            line = (int) args[3].convertToInteger().getLongValue();
-        } else if (scope == null) {
-            line = 0;
+            // file given, use it and force it into binding
+            binding.setLine((int) args[3].convertToInteger().getLongValue());
         } else {
-            line = -1;
+            // no binding given, use 0 for both
+            binding.setLine(0);
         }
-        if (scope == null) scope = RubyBinding.newBindingForEval(context);
         
-        return ASTInterpreter.evalWithBinding(context, src, scope, file, line);
+        return ASTInterpreter.evalWithBinding(context, src, binding);
+    }
+
+    private static Binding convertToBinding(IRubyObject scope) {
+        if (scope instanceof RubyBinding) {
+            return ((RubyBinding)scope).getBinding().clone();
+        } else {
+            if (scope instanceof RubyProc) {
+                return ((RubyProc) scope).getBlock().getBinding().clone();
+            } else {
+                // bomb out, it's not a binding or a proc
+                throw scope.getRuntime().newTypeError("wrong argument type " + scope.getMetaClass() + " (expected Proc/Binding)");
+            }
+        }
     }
 
     @JRubyMethod(name = "callcc", frame = true, module = true, visibility = PRIVATE)
     public static IRubyObject callcc(ThreadContext context, IRubyObject recv, Block block) {
-        Ruby runtime = context.getRuntime();
-        runtime.getWarnings().warn(ID.EMPTY_IMPLEMENTATION, "Kernel#callcc: Continuations are not implemented in JRuby and will not work", "Kernel#callcc");
-        IRubyObject cc = runtime.getContinuation().callMethod(context, "new");
-        cc.dataWrapStruct(block);
-        return block.yield(context, cc);
+        RubyContinuation continuation = new RubyContinuation(context.getRuntime());
+        return continuation.enter(context, block);
     }
 
-    @JRubyMethod(name = "caller", optional = 1, frame = true, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "caller", optional = 1, frame = true, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject caller(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
         int level = args.length > 0 ? RubyNumeric.fix2int(args[0]) : 1;
 
@@ -901,6 +915,17 @@ public class RubyKernel {
         return context.createCallerBacktrace(context.getRuntime(), level);
     }
 
+    @JRubyMethod(name = "caller", optional = 1, frame = true, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject caller1_9(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
+        int level = args.length > 0 ? RubyNumeric.fix2int(args[0]) : 1;
+
+        if (level < 0) {
+            throw context.getRuntime().newArgumentError("negative level(" + level + ')');
+        }
+
+        return context.createCallerBacktrace(context.getRuntime(), level);
+    }
+
     @JRubyMethod(name = "catch", required = 1, frame = true, module = true, visibility = PRIVATE)
     public static IRubyObject rbCatch(ThreadContext context, IRubyObject recv, IRubyObject tag, Block block) {
         CatchTarget target = new CatchTarget(tag.asJavaString());
@@ -958,7 +983,7 @@ public class RubyKernel {
     public static IRubyObject warn(ThreadContext context, IRubyObject recv, IRubyObject message) {
         Ruby runtime = context.getRuntime();
         
-        if (!runtime.getVerbose().isNil()) {
+        if (runtime.warningsEnabled()) {
             IRubyObject out = runtime.getGlobalVariables().get("$stderr");
             RuntimeHelpers.invoke(context, out, "puts", message);
         }
@@ -1054,8 +1079,9 @@ public class RubyKernel {
 
     @JRubyMethod(name = "loop", frame = true, module = true, visibility = PRIVATE)
     public static IRubyObject loop(ThreadContext context, IRubyObject recv, Block block) {
+        IRubyObject nil = context.getRuntime().getNil();
         while (true) {
-            block.yield(context, context.getRuntime().getNil());
+            block.yield(context, nil);
 
             context.pollThreadEvents();
         }
@@ -1184,11 +1210,11 @@ public class RubyKernel {
 
         // Not sure how well this works, but it works much better than
         // just currentTimeMillis by itself.
+        long oldRandomSeed = runtime.getRandomSeed();
         runtime.setRandomSeed(System.currentTimeMillis() ^
                recv.hashCode() ^ runtime.incrementRandomSeedSequence() ^
                runtime.getRandom().nextInt(Math.max(1, Math.abs((int)runtime.getRandomSeed()))));
 
-        long oldRandomSeed = runtime.getRandomSeed();
         runtime.getRandom().setSeed(runtime.getRandomSeed());
         return runtime.newFixnum(oldRandomSeed);
     }
@@ -1233,12 +1259,19 @@ public class RubyKernel {
         return runtime.newFixnum(random.nextInt((int) ceil));
     }
 
+    @JRubyMethod(name = "spawn", required = 1, rest = true, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
+    public static RubyFixnum spawn(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
+        Ruby runtime = context.getRuntime();
+        long pid = ShellLauncher.runWithoutWait(runtime, args);
+        return RubyFixnum.newFixnum(runtime, pid);
+    }
+
     @JRubyMethod(name = "syscall", required = 1, optional = 9, module = true, visibility = PRIVATE)
     public static IRubyObject syscall(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         throw context.getRuntime().newNotImplementedError("Kernel#syscall is not implemented in JRuby");
     }
 
-    @JRubyMethod(name = {"system"}, required = 1, rest = true, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "system", required = 1, rest = true, module = true, visibility = PRIVATE)
     public static RubyBoolean system(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         Ruby runtime = context.getRuntime();
         int resultCode;
@@ -1254,6 +1287,12 @@ public class RubyKernel {
     @JRubyMethod(name = {"exec"}, required = 1, rest = true, module = true, visibility = PRIVATE)
     public static IRubyObject exec(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         Ruby runtime = context.getRuntime();
+
+        // This is a fairly specific hack for empty string, but it does the job
+        if (args.length == 1 && args[0].convertToString().isEmpty()) {
+            throw runtime.newErrnoENOENTError(args[0].convertToString().toString());
+        }
+        
         int resultCode;
         try {
             // TODO: exec should replace the current process.
@@ -1263,7 +1302,10 @@ public class RubyKernel {
             throw runtime.newErrnoENOENTError("cannot execute");
         }
         
-        return exit(recv, new IRubyObject[] {runtime.newFixnum(resultCode)});
+        exit(runtime, new IRubyObject[] {runtime.newFixnum(resultCode)}, true);
+
+        // not reached
+        return runtime.getNil();
     }
 
     @JRubyMethod(name = "fork", module = true, visibility = PRIVATE)
