@@ -33,8 +33,10 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby;
 
+import java.util.Arrays;
 import java.util.Iterator;
 
+import org.jcodings.Encoding;
 import org.joni.NameEntry;
 import org.joni.Regex;
 import org.joni.Region;
@@ -47,6 +49,7 @@ import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
+import org.jruby.util.StringSupport;
 
 /**
  * @author olabini
@@ -54,11 +57,13 @@ import org.jruby.util.ByteList;
 @JRubyClass(name="MatchData")
 public class RubyMatchData extends RubyObject {
     Region regs;        // captures
-    int begin;          // begin and end are used when not groups defined
-    int end;
-    RubyString str;
+    int begin, end;     // begin and end are used when not groups defined
+    RubyString str;     // source string
+    Regex pattern;
     RubyRegexp regexp;
-    
+    boolean charOffsetUpdated;
+    Region charOffsets;
+
     public static RubyClass createMatchDataClass(Ruby runtime) {
         RubyClass matchDataClass = runtime.defineClass("MatchData", runtime.getObject(), MATCH_DATA_ALLOCATOR);
         runtime.setMatchData(matchDataClass);
@@ -88,7 +93,73 @@ public class RubyMatchData extends RubyObject {
         super(runtime, metaClass);
     }
 
-    public final static int MATCH_BUSY = USER2_F;
+    private static final class Pair implements Comparable {
+        int bytePos, charPos;
+        public int compareTo(Object o) {
+            return bytePos - ((Pair)o).bytePos;
+        }
+    }
+
+    private void updateCharOffset() {
+        if (charOffsetUpdated) return;
+
+        int numRegs = regs == null ? 1 : regs.numRegs;
+        if (charOffsets == null || charOffsets.numRegs < numRegs) charOffsets = new Region(numRegs);
+
+        Pair[]pairs = new Pair[numRegs * 2];
+        for (int i = 0; i < pairs.length; i++) pairs[i] = new Pair();
+
+        int numPos = 0;
+        if (regs == null) {
+            pairs[numPos++].bytePos = begin;
+            pairs[numPos++].bytePos = end;
+        } else {
+            for (int i = 0; i < numRegs; i++) {
+                if (regs.beg[i] < 0) continue;
+                pairs[numPos++].bytePos = regs.beg[i];
+                pairs[numPos++].bytePos = regs.end[i];
+            }
+        }
+
+        Arrays.sort(pairs);
+
+        ByteList value = str.getByteList();
+        Encoding enc = value.encoding;
+        byte[]bytes = value.bytes;
+        int p = value.begin;
+        int s = p;
+
+        int c = 0;
+        for (int i = 0; i < numPos; i++) {
+            int q = s + pairs[i].bytePos;
+            c += StringSupport.strLength(enc, bytes, p, q);
+            pairs[i].charPos = c;
+            p = q;
+        }
+
+        Pair key = new Pair();
+        if (regs == null) {
+            key.bytePos = begin;
+            charOffsets.beg[0] = pairs[Arrays.binarySearch(pairs, key)].charPos;
+            key.bytePos = end;
+            charOffsets.end[0] = pairs[Arrays.binarySearch(pairs, key)].charPos;
+        } else {
+            for (int i = 0; i < numRegs; i++) {
+                if (regs.beg[i] < 0) {
+                    charOffsets.beg[i] = charOffsets.end[i] = -1;
+                    continue;
+                }
+                key.bytePos = regs.beg[i];
+                charOffsets.beg[i] = pairs[Arrays.binarySearch(pairs, key)].charPos;
+                key.bytePos = regs.end[i];
+                charOffsets.end[i] = pairs[Arrays.binarySearch(pairs, key)].charPos;
+            }
+
+        }
+        charOffsetUpdated = true;
+    }
+
+    private static final int MATCH_BUSY = USER2_F;
 
     // rb_match_busy
     public final void use() {
@@ -100,7 +171,11 @@ public class RubyMatchData extends RubyObject {
     }
 
     private void check() {
-        if (regexp == null) throw getRuntime().newTypeError("uninitialized Match");
+        if (str == null) throw getRuntime().newTypeError("uninitialized Match");
+    }
+
+    private void checkLazyRegexp() {
+        if (regexp == null) regexp = RubyRegexp.newRegexp(getRuntime(), (ByteList)pattern.getUserObject(), pattern);
     }
 
     private RubyArray match_array(Ruby runtime, int start) {
@@ -108,17 +183,17 @@ public class RubyMatchData extends RubyObject {
         if (regs == null) {
             if (start != 0) return runtime.newEmptyArray();
             if (begin == -1) {
-                return getRuntime().newArray(runtime.getNil());
+                return runtime.newArray(runtime.getNil());
             } else {
                 RubyString ss = str.makeShared(runtime, begin, end - begin);
                 if (isTaint()) ss.setTaint(true);
-                return getRuntime().newArray(ss);
+                return runtime.newArray(ss);
             }
         } else {
-            RubyArray arr = getRuntime().newArray(regs.numRegs - start);
+            RubyArray arr = runtime.newArray(regs.numRegs - start);
             for (int i=start; i<regs.numRegs; i++) {
                 if (regs.beg[i] == -1) {
-                    arr.append(getRuntime().getNil());
+                    arr.append(runtime.getNil());
                 } else {
                     RubyString ss = str.makeShared(runtime, regs.beg[i], regs.end[i] - regs.beg[i]);
                     if (isTaint()) ss.setTaint(true); 
@@ -140,15 +215,15 @@ public class RubyMatchData extends RubyObject {
 
     @JRubyMethod(name = "inspect")
     public IRubyObject inspect() {
-        if (regexp == null) return anyToString();
+        if (str == null) return anyToString();
 
-        RubyString result = getRuntime().newString();
+        Ruby runtime = getRuntime();
+        RubyString result = runtime.newString();
         result.cat((byte)'#').cat((byte)'<');
         result.append(getMetaClass().getRealClass().to_s());
 
         NameEntry[]names = new NameEntry[regs == null ? 1 : regs.numRegs];
 
-        Regex pattern = regexp.getPattern();
         if (pattern.numberOfNames() > 0) {
             for (Iterator<NameEntry> i = pattern.namedBackrefIterator(); i.hasNext();) {
                 NameEntry e = i.next();
@@ -171,7 +246,8 @@ public class RubyMatchData extends RubyObject {
             if (v.isNil()) {
                 result.cat("nil".getBytes());
             } else {
-                result.append(v.inspect());
+                RubyString str = (RubyString)v;
+                result.append(str.inspectCommon(runtime.is1_9()));
             }
         }
 
@@ -181,12 +257,14 @@ public class RubyMatchData extends RubyObject {
     @JRubyMethod(name = "regexp", compat = CompatVersion.RUBY1_9)
     public IRubyObject regexp(ThreadContext context, Block block) {
         check();
+        checkLazyRegexp();
         return regexp;
     }
 
     @JRubyMethod(name = "names", compat = CompatVersion.RUBY1_9)
     public IRubyObject names(ThreadContext context, Block block) {
         check();
+        checkLazyRegexp();
         return regexp.names(context);
     }
 
@@ -236,7 +314,7 @@ public class RubyMatchData extends RubyObject {
     private int nameToBackrefNumber(RubyString str) {
         ByteList value = str.getByteList();
         try {
-            return regexp.getPattern().nameToBackrefNumber(value.bytes, value.begin, value.begin + value.realSize, regs);
+            return pattern.nameToBackrefNumber(value.bytes, value.begin, value.begin + value.realSize, regs);
         } catch (JOniException je) {
             throw getRuntime().newIndexError(je.getMessage());
         }
@@ -336,98 +414,106 @@ public class RubyMatchData extends RubyObject {
     /** match_begin
      *
      */
-    @JRubyMethod(name = "begin")
+    @JRubyMethod(name = "begin", compat = CompatVersion.RUBY1_8)
     public IRubyObject begin(ThreadContext context, IRubyObject index) {
-        return beginCommon(context, RubyNumeric.num2int(index));
+        int i = RubyNumeric.num2int(index);
+        Ruby runtime = context.getRuntime();
+        int b = beginCommon(runtime, i);
+        return b < 0 ? runtime.getNil() : RubyFixnum.newFixnum(runtime, b);
     }
 
     @JRubyMethod(name = "begin", compat = CompatVersion.RUBY1_9)
     public IRubyObject begin19(ThreadContext context, IRubyObject index) {
-       return beginCommon(context, backrefNumber(index));
+        int i = backrefNumber(index);
+        Ruby runtime = context.getRuntime();
+        int b = beginCommon(runtime, i);
+        if (b < 0) return runtime.getNil();
+        if (!str.singleByteOptimizable()) {
+            updateCharOffset();
+            b = charOffsets.beg[i];
+        }
+        return RubyFixnum.newFixnum(runtime, b);
     }
 
-    private IRubyObject beginCommon(ThreadContext context, int i) {
+    private int beginCommon(Ruby runtime, int i) {
         check();
-        Ruby runtime = context.getRuntime();
-        if (regs == null) {
-            if (i != 0) throw runtime.newIndexError("index " + i + " out of matches");
-            if (begin < 0) return runtime.getNil();
-            return RubyFixnum.newFixnum(runtime, begin);
-        } else {
-            if (i < 0 || regs.numRegs <= i) throw runtime.newIndexError("index " + i + " out of matches");
-            if (regs.beg[i] < 0) return runtime.getNil();
-            return RubyFixnum.newFixnum(runtime, regs.beg[i]);
-        }
+        if (i < 0 || (regs == null ? 1 : regs.numRegs) <= i) throw runtime.newIndexError("index " + i + " out of matches");
+        return regs == null ? begin : regs.beg[i];
     }
 
     /** match_end
      *
      */
-    @JRubyMethod(name = "end")
+    @JRubyMethod(name = "end", compat = CompatVersion.RUBY1_8)
     public IRubyObject end(ThreadContext context, IRubyObject index) {
-        return endCommon(context, RubyNumeric.num2int(index));
+        int i = RubyNumeric.num2int(index);
+        Ruby runtime = context.getRuntime();
+        int e = endCommon(runtime, i);
+        return e < 0 ? runtime.getNil() : RubyFixnum.newFixnum(runtime, e);
     }
 
     @JRubyMethod(name = "end", compat = CompatVersion.RUBY1_9)
     public IRubyObject end19(ThreadContext context, IRubyObject index) {
-       return endCommon(context, backrefNumber(index));
+        int i = backrefNumber(index);
+        Ruby runtime = context.getRuntime();
+        int e = endCommon(runtime, i);
+        if (e < 0) return runtime.getNil();
+        if (!str.singleByteOptimizable()) {
+            updateCharOffset();
+            e = charOffsets.end[i];
+        }
+        return RubyFixnum.newFixnum(runtime, e);
     }
 
-    private IRubyObject endCommon(ThreadContext context, int i) {
+    private int endCommon(Ruby runtime, int i) {
         check();
-        Ruby runtime = context.getRuntime(); 
-        if (regs == null) {
-            if (i != 0) throw runtime.newIndexError("index " + i + " out of matches");
-            if (end < 0) return runtime.getNil();
-            return RubyFixnum.newFixnum(runtime, end);
-        } else {
-            if (i < 0 || regs.numRegs <= i) throw runtime.newIndexError("index " + i + " out of matches");
-            if (regs.end[i] < 0) return runtime.getNil();
-            return RubyFixnum.newFixnum(runtime, regs.end[i]);
-        }
+        if (i < 0 || (regs == null ? 1 : regs.numRegs) <= i) throw runtime.newIndexError("index " + i + " out of matches");
+        return regs == null ? end : regs.end[i];
     }
 
     /** match_offset
      *
      */
-    @JRubyMethod(name = "offset")
+    @JRubyMethod(name = "offset", compat = CompatVersion.RUBY1_8)
     public IRubyObject offset(ThreadContext context, IRubyObject index) {
-        return offsetCommon(context, RubyNumeric.num2int(index));
+        return offsetCommon(context, RubyNumeric.num2int(index), false);
 
     }
 
     @JRubyMethod(name = "offset", compat = CompatVersion.RUBY1_9)
     public IRubyObject offset19(ThreadContext context, IRubyObject index) {
-        return offsetCommon(context, backrefNumber(index));
+        return offsetCommon(context, backrefNumber(index), true);
     }
 
-    private IRubyObject offsetCommon(ThreadContext context, int i) {
+    private IRubyObject offsetCommon(ThreadContext context, int i, boolean is_19) {
         check();
         Ruby runtime = context.getRuntime();
-
+        if (i < 0 || (regs == null ? 1 : regs.numRegs) <= i) throw runtime.newIndexError("index " + i + " out of matches");
+        int b, e;
         if (regs == null) {
-            if (i != 0) throw runtime.newIndexError("index " + i + " out of matches");
-            if (begin < 0) return runtime.newArray(runtime.getNil(), runtime.getNil());
-            return runtime.newArray(RubyFixnum.newFixnum(runtime, begin),RubyFixnum.newFixnum(runtime, end));            
+            b = begin;
+            e = end;
         } else {
-            if (i < 0 || regs.numRegs <= i) throw runtime.newIndexError("index " + i + " out of matches");
-            if (regs.beg[i] < 0) return runtime.newArray(runtime.getNil(), runtime.getNil());
-            return runtime.newArray(RubyFixnum.newFixnum(runtime, regs.beg[i]),RubyFixnum.newFixnum(runtime, regs.end[i]));
+            b = regs.beg[i];
+            e = regs.end[i];
         }
+        if (b < 0) return runtime.newArray(runtime.getNil(), runtime.getNil());
+        if (is_19 && !str.singleByteOptimizable()) {
+            updateCharOffset();
+            b = charOffsets.beg[i];
+            e = charOffsets.end[i];
+        }
+        return runtime.newArray(RubyFixnum.newFixnum(runtime, b), RubyFixnum.newFixnum(runtime, e));
     }
+
     /** match_pre_match
      *
      */
     @JRubyMethod(name = "pre_match")
     public IRubyObject pre_match(ThreadContext context) {
         check();
-        RubyString ss;
-
         if (begin == -1) return context.getRuntime().getNil();
-        ss = str.makeShared(context.getRuntime(), 0, begin);
-
-        if (isTaint()) ss.setTaint(true);
-        return ss;
+        return str.makeShared(context.getRuntime(), 0, begin).infectBy(this);
     }
 
     /** match_post_match
@@ -436,13 +522,8 @@ public class RubyMatchData extends RubyObject {
     @JRubyMethod(name = "post_match")
     public IRubyObject post_match(ThreadContext context) {
         check();
-        RubyString ss;
-
         if (begin == -1) return context.getRuntime().getNil();
-        ss = str.makeShared(context.getRuntime(), end, str.getByteList().length() - end);
-
-        if(isTaint()) ss.setTaint(true);
-        return ss;
+        return str.makeShared(context.getRuntime(), end, str.getByteList().length() - end).infectBy(this);
     }
 
     /** match_to_s
