@@ -36,7 +36,7 @@ import org.jruby.anno.JRubyClass;
 import org.jruby.exceptions.JumpException;
 import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.runtime.Block;
-import org.jruby.runtime.CallbackFactory;
+import org.jruby.runtime.DynamicScope;
 import org.jruby.runtime.MethodBlock;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
@@ -154,9 +154,14 @@ public class RubyMethod extends RubyObject {
     @JRubyMethod(name = "to_proc", frame = true)
     public IRubyObject to_proc(ThreadContext context, Block unusedBlock) {
         Ruby runtime = context.getRuntime();
-        CallbackFactory f = runtime.callbackFactory(RubyMethod.class);
-        Block block = MethodBlock.createMethodBlock(context, context.getCurrentScope(), 
-                f.getBlockMethod("bmcall"), this, runtime.getTopSelf());
+        DynamicScope currentScope = context.getCurrentScope();
+        MethodBlock mb = new MethodBlock(this, currentScope.getStaticScope()) {
+            @Override
+            public IRubyObject callback(IRubyObject value, IRubyObject method, IRubyObject self, Block block) {
+                return bmcall(value, method, self, block);
+            }
+        };
+        Block block = MethodBlock.createMethodBlock(context, runtime.getTopSelf(), context.getCurrentScope(), mb);
         
         while (true) {
             try {
@@ -194,9 +199,11 @@ public class RubyMethod extends RubyObject {
      */
     public static IRubyObject bmcall(IRubyObject blockArg, IRubyObject arg1,
             IRubyObject self, Block unusedBlock) {
-        ThreadContext context = blockArg.getRuntime().getCurrentContext();
-        
-        if (blockArg instanceof RubyArray) {
+        ThreadContext context = arg1.getRuntime().getCurrentContext();
+
+        if (blockArg == null) {
+            return ((RubyMethod) arg1).call(context, IRubyObject.NULL_ARRAY, Block.NULL_BLOCK);
+        } else if (blockArg instanceof RubyArray) {
             // ENEBO: Very wrong
             return ((RubyMethod) arg1).call(context, ((RubyArray) blockArg).toJavaArray(), Block.NULL_BLOCK);
         }
