@@ -238,16 +238,9 @@ public class RubyNumeric extends RubyObject {
      * 
      */
     public static IRubyObject dbl_cmp(Ruby runtime, double a, double b) {
-        if (Double.isNaN(a) || Double.isNaN(b)) {
-            return runtime.getNil();
-        }
-        if (a > b) {
-            return RubyFixnum.one(runtime);
-        }
-        if (a < b) {
-            return RubyFixnum.minus_one(runtime);
-        }
-        return RubyFixnum.zero(runtime);
+        if (Double.isNaN(a) || Double.isNaN(b)) return runtime.getNil();
+        return a == b ? RubyFixnum.zero(runtime) : a > b ?
+                RubyFixnum.one(runtime) : RubyFixnum.minus_one(runtime);
     }
 
     public static long fix2long(IRubyObject arg) {
@@ -256,7 +249,12 @@ public class RubyNumeric extends RubyObject {
 
     public static int fix2int(IRubyObject arg) {
         long num = arg instanceof RubyFixnum ? fix2long(arg) : num2long(arg);
+        checkInt(arg, num);
+        return (int) num;
+    }
 
+    public static int fix2int(RubyFixnum arg) {
+        long num = arg.getLongValue();
         checkInt(arg, num);
         return (int) num;
     }
@@ -520,7 +518,7 @@ public class RubyNumeric extends RubyObject {
      */
     @JRubyMethod(name = "coerce")
     public IRubyObject coerce(IRubyObject other) {
-        if (getClass() == other.getClass()) return getRuntime().newArray(other, this);
+        if (getMetaClass() == other.getMetaClass()) return getRuntime().newArray(other, this);
 
         IRubyObject cdr = RubyKernel.new_float(this, this);
         IRubyObject car = RubyKernel.new_float(this, other);
@@ -724,8 +722,8 @@ public class RubyNumeric extends RubyObject {
     public IRubyObject truncate() {
         return convertToFloat().truncate();
     }
-    
-    @JRubyMethod(name = "step", required = 1, optional = 1, frame = true)
+
+    @Deprecated
     public IRubyObject step(ThreadContext context, IRubyObject[] args, Block block) {
         switch (args.length) {
         case 0: throw context.getRuntime().newArgumentError(0, 1);
@@ -734,81 +732,116 @@ public class RubyNumeric extends RubyObject {
         default: throw context.getRuntime().newArgumentError(args.length, 2);
         }
     }
-    
-    @JRubyMethod(name = "step", frame = true)
+
+    @JRubyMethod(name = "step", frame = true, compat = CompatVersion.RUBY1_8)
     public IRubyObject step(ThreadContext context, IRubyObject arg0, Block block) {
         return step(context, arg0, RubyFixnum.one(context.getRuntime()), block);
     }
 
-    @JRubyMethod(name = "step", frame = true, compat = CompatVersion.RUBY1_9)
-    public IRubyObject step19(ThreadContext context, IRubyObject arg0, Block block) {
-        return block.isGiven() ? step(context, arg0, block) : enumeratorize(context.getRuntime(), this, "step", arg0);
-    }
-    
-    @JRubyMethod(name = "step", frame = true)
+    @JRubyMethod(name = "step", frame = true, compat = CompatVersion.RUBY1_8)
     public IRubyObject step(ThreadContext context, IRubyObject to, IRubyObject step, Block block) {
+        Ruby runtime = context.getRuntime();
         if (this instanceof RubyFixnum && to instanceof RubyFixnum && step instanceof RubyFixnum) {
-            long value = getLongValue();
-            long end = ((RubyFixnum) to).getLongValue();
-            long diff = ((RubyFixnum) step).getLongValue();
-
-            if (diff == 0) {
-                throw getRuntime().newArgumentError("step cannot be 0");
-            }
-            if (diff > 0) {
-                for (long i = value; i <= end; i += diff) {
-                    block.yield(context, RubyFixnum.newFixnum(getRuntime(), i));
-                }
-            } else {
-                for (long i = value; i >= end; i += diff) {
-                    block.yield(context, RubyFixnum.newFixnum(getRuntime(), i));
-                }
-            }
+            fixnumStep(context, runtime, ((RubyFixnum)this).getLongValue(),
+                                         ((RubyFixnum)to).getLongValue(),
+                                         ((RubyFixnum)step).getLongValue(),
+                                          block);
         } else if (this instanceof RubyFloat || to instanceof RubyFloat || step instanceof RubyFloat) {
-            double beg = num2dbl(this);
-            double end = num2dbl(to);
-            double unit = num2dbl(step);
-
-            if (unit == 0) {
-                throw getRuntime().newArgumentError("step cannot be 0");
-            }           
-            
-            double n = (end - beg)/unit;
-            double err = (Math.abs(beg) + Math.abs(end) + Math.abs(end - beg)) / Math.abs(unit) * DBL_EPSILON;
-            
-            if (err>0.5) {
-                err=0.5;            
-            }
-            n = Math.floor(n + err) + 1;
-            
-            for(double i = 0; i < n; i++){
-                block.yield(context, RubyFloat.newFloat(getRuntime(), i * unit + beg));
-            }
-
+            floatStep(context, runtime, this, to, step, block);
         } else {
-            RubyNumeric i = this;
-            
-            String cmpString;
-            if (((RubyBoolean) step.callMethod(context, ">", RubyFixnum.zero(getRuntime()))).isTrue()) {
-                cmpString = ">";
-            } else {
-                cmpString = "<";
-            }
-
-            while (true) {
-                if (i.callMethod(context, cmpString, to).isTrue()) {
-                    break;
-                }
-                block.yield(context, i);
-                i = (RubyNumeric) i.callMethod(context, "+", step);
-            }
+            duckStep(context, runtime, this, to, step, block);
         }
         return this;
     }
 
+    @JRubyMethod(name = "step", frame = true, compat = CompatVersion.RUBY1_9)
+    public IRubyObject step19(ThreadContext context, IRubyObject arg0, Block block) {
+        return block.isGiven() ? stepCommon19(context, arg0, RubyFixnum.one(context.getRuntime()), block) : enumeratorize(context.getRuntime(), this, "step", arg0);
+    }
+
     @JRubyMethod(name = "step", frame = true, compat = CompatVersion.RUBY1_9)
     public IRubyObject step19(ThreadContext context, IRubyObject to, IRubyObject step, Block block) {
-        return block.isGiven() ? step(context, to, step, block) : enumeratorize(context.getRuntime(), this, "step", new IRubyObject[] {to, step});
+        return block.isGiven() ? stepCommon19(context, to, step, block) : enumeratorize(context.getRuntime(), this, "step", new IRubyObject[] {to, step});
+    }
+
+    private IRubyObject stepCommon19(ThreadContext context, IRubyObject to, IRubyObject step, Block block) {
+        Ruby runtime = context.getRuntime();
+        if (this instanceof RubyFixnum && to instanceof RubyFixnum && step instanceof RubyFixnum) {
+            fixnumStep(context, runtime, ((RubyFixnum)this).getLongValue(),
+                                         ((RubyFixnum)to).getLongValue(),
+                                         ((RubyFixnum)step).getLongValue(),
+                                          block);
+        } else if (this instanceof RubyFloat || to instanceof RubyFloat || step instanceof RubyFloat) {
+            floatStep19(context, runtime, this, to, step, false, block);
+        } else {
+            duckStep(context, runtime, this, to, step, block);
+        }
+        return this;
+    }
+
+    private static void fixnumStep(ThreadContext context, Ruby runtime, long value, long end, long diff, Block block) {
+        if (diff == 0) throw runtime.newArgumentError("step cannot be 0");
+        if (diff > 0) {
+            for (long i = value; i <= end; i += diff) {
+                block.yield(context, RubyFixnum.newFixnum(runtime, i));
+            }
+        } else {
+            for (long i = value; i >= end; i += diff) {
+                block.yield(context, RubyFixnum.newFixnum(runtime, i));
+            }
+        }
+    }
+
+    protected static void floatStep(ThreadContext context, Ruby runtime, IRubyObject from, IRubyObject to, IRubyObject step, Block block) { 
+        double beg = num2dbl(from);
+        double end = num2dbl(to);
+        double unit = num2dbl(step);
+
+        if (unit == 0) throw runtime.newArgumentError("step cannot be 0");
+
+        double n = (end - beg)/unit;
+        double err = (Math.abs(beg) + Math.abs(end) + Math.abs(end - beg)) / Math.abs(unit) * DBL_EPSILON;
+
+        if (err > 0.5) err = 0.5;            
+        n = Math.floor(n + err) + 1;
+
+        for (long i = 0; i < n; i++) {
+            block.yield(context, RubyFloat.newFloat(runtime, i * unit + beg));
+        }
+    }
+
+    static void floatStep19(ThreadContext context, Ruby runtime, IRubyObject from, IRubyObject to, IRubyObject step, boolean excl, Block block) { 
+        double beg = num2dbl(from);
+        double end = num2dbl(to);
+        double unit = num2dbl(step);
+
+        // TODO: remove
+        if (unit == 0) throw runtime.newArgumentError("step cannot be 0");
+
+        double n = (end - beg)/unit;
+        double err = (Math.abs(beg) + Math.abs(end) + Math.abs(end - beg)) / Math.abs(unit) * DBL_EPSILON;
+
+        if (Double.isInfinite(unit)) {
+            if (unit > 0) block.yield(context, RubyFloat.newFloat(runtime, beg));
+        } else {
+            if (err > 0.5) err = 0.5;            
+            n = Math.floor(n + err);
+            if (!excl) n++;
+            for (long i = 0; i < n; i++){
+                block.yield(context, RubyFloat.newFloat(runtime, i * unit + beg));
+            }
+        }
+    }
+
+    private static void duckStep(ThreadContext context, Ruby runtime, IRubyObject from, IRubyObject to, IRubyObject step, Block block) {
+        IRubyObject i = from;
+        String cmpString = step.callMethod(context, ">", RubyFixnum.zero(runtime)).isTrue() ? ">" : "<";
+
+        while (true) {
+            if (i.callMethod(context, cmpString, to).isTrue()) break;
+            block.yield(context, i);
+            i = i.callMethod(context, "+", step);
+        }
     }
 
     /** num_equal, doesn't override RubyObject.op_equal
