@@ -42,22 +42,23 @@ package org.jruby;
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
+import java.util.HashSet;
+import java.util.List;
 
+import java.util.Set;
+import org.jruby.anno.JRubyClass;
+import org.jruby.anno.JRubyMethod;
 import org.jruby.common.IRubyWarnings.ID;
 import org.jruby.evaluator.ASTInterpreter;
 import org.jruby.exceptions.JumpException;
+import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Block;
+import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
-import org.jruby.runtime.builtin.Variable;
 import org.jruby.util.IdUtil;
-import java.util.List;
-import org.jruby.anno.JRubyClass;
-import org.jruby.anno.JRubyMethod;
-import org.jruby.javasupport.util.RuntimeHelpers;
-import org.jruby.runtime.ClassIndex;
 
 /**
  * RubyObject is the only implementation of the
@@ -108,11 +109,6 @@ public class RubyObject extends RubyBasicObject {
         super(runtime, metaClass, useObjectSpace);
     }
 
-    private void addToObjectSpace(Ruby runtime) {
-        assert runtime.isObjectSpaceEnabled();
-        runtime.getObjectSpace().add(this);
-    }
-
     /**
      * Will create the Ruby class Object in the runtime
      * specified. This method needs to take the actual class as an
@@ -203,7 +199,8 @@ public class RubyObject extends RubyBasicObject {
      */
     @Override
     public String toString() {
-        return RuntimeHelpers.invoke(getRuntime().getCurrentContext(), this, "to_s").toString();
+        RubyString rubyString = RuntimeHelpers.invoke(getRuntime().getCurrentContext(), this, "to_s").convertToString();
+        return rubyString.getUnicodeValue();
     }
 
     /**
@@ -211,7 +208,7 @@ public class RubyObject extends RubyBasicObject {
      * If possible, call this instead of {@link #testFrozen}.
      */
    protected void checkFrozen() {
-       testFrozen("can't modify frozen ");
+       testFrozen();
    }
 
     /** init_copy
@@ -555,7 +552,7 @@ public class RubyObject extends RubyBasicObject {
      * equality, and then calling the "eql?" method.
      */
     protected static boolean eqlInternal(final ThreadContext context, final IRubyObject that, final IRubyObject other){
-        return that == other || that.callMethod(context, "eql?", other).isTrue();
+        return that.callMethod(context, "eql?", other).isTrue();
     }
 
     /** rb_obj_init_copy
@@ -796,7 +793,7 @@ public class RubyObject extends RubyBasicObject {
         context.getRuntime().secure(3);
 
         if (isTaint()) {
-            testFrozen("object");
+            testFrozen();
             setTaint(false);
         }
 
@@ -824,10 +821,9 @@ public class RubyObject extends RubyBasicObject {
      */
     @JRubyMethod(name = "freeze")
     public IRubyObject freeze(ThreadContext context) {
-        if ((flags & FROZEN_F) == 0) {
-            if (context.getRuntime().getSafeLevel() >= 4 && isTaint()) {
-                throw context.getRuntime().newSecurityError("Insecure: can't freeze object");
-            }
+        Ruby runtime = context.getRuntime();
+        if ((flags & FROZEN_F) == 0 && (runtime.is1_9() || !isImmediate())) {
+            if (runtime.getSafeLevel() >= 4 && !isTaint()) throw runtime.newSecurityError("Insecure: can't freeze object");
             flags |= FROZEN_F;
         }
         return this;
@@ -849,6 +845,47 @@ public class RubyObject extends RubyBasicObject {
         return context.getRuntime().newBoolean(isFrozen());
     }
 
+    /** rb_obj_untrusted
+     *  call-seq:
+     *     obj.untrusted?    => true or false
+     *
+     *  Returns <code>true</code> if the object is untrusted.
+     */
+    @JRubyMethod(name = "untrusted?", compat = CompatVersion.RUBY1_9)
+    public RubyBoolean untrusted_p(ThreadContext context) {
+        return context.getRuntime().newBoolean(isUntrusted());
+    }
+
+    /** rb_obj_untrust
+     *  call-seq:
+     *     obj.untrust -> obj
+     *
+     *  Marks <i>obj</i> as untrusted.
+     */
+    @JRubyMethod(compat = CompatVersion.RUBY1_9)
+    public IRubyObject untrust(ThreadContext context) {
+        if (!isUntrusted() && !isImmediate()) {
+            checkFrozen();
+            flags |= UNTRUSTED_F;
+        }
+        return this;
+    }
+
+    /** rb_obj_trust
+     *  call-seq:
+     *     obj.trust    => obj
+     *
+     *  Removes the untrusted mark from <i>obj</i>.
+     */
+    @JRubyMethod(compat = CompatVersion.RUBY1_9)
+    public IRubyObject trust(ThreadContext context) {
+        if (isUntrusted() && !isImmediate()) {
+            checkFrozen();
+            flags &= ~UNTRUSTED_F;
+        }
+        return this;
+    }
+
     /** rb_inspect
      *
      * The internal helper that ensures a RubyString instance is returned
@@ -959,19 +996,21 @@ public class RubyObject extends RubyBasicObject {
         if (args.length == 1) {
             all = args[0].isTrue();
         }
+        
+        Ruby runtime = getRuntime();
+        RubyArray singletonMethods = runtime.newArray();
+        Set<String> seen = new HashSet<String>();
 
-        RubyArray singletonMethods = null;
         if (getMetaClass().isSingleton()) {
-            singletonMethods =
-                getMetaClass().instance_methods(new IRubyObject[] {context.getRuntime().getFalse()});
+            getMetaClass().populateInstanceMethodNames(seen, singletonMethods, Visibility.PRIVATE, true, false, false);
             if (all) {
-                singletonMethods.concat(getMetaClass().getSuperClass().instance_methods(new IRubyObject[] {context.getRuntime().getTrue()}));
+                getMetaClass().getSuperClass().populateInstanceMethodNames(seen, singletonMethods, Visibility.PRIVATE, true, false, true);
             }
         } else {
             if (all) {
-                singletonMethods = getMetaClass().instance_methods(new IRubyObject[] {context.getRuntime().getTrue()});
+                getMetaClass().populateInstanceMethodNames(seen, singletonMethods, Visibility.PRIVATE, true, false, true);
             } else {
-                singletonMethods = context.getRuntime().newEmptyArray();
+                // do nothing, leave empty
             }
         }
 
@@ -1503,7 +1542,7 @@ public class RubyObject extends RubyBasicObject {
     @JRubyMethod(name = "instance_variable_set", required = 2)
     public IRubyObject instance_variable_set(IRubyObject name, IRubyObject value) {
         ensureInstanceVariablesSettable();
-        return variableTableStore(validateInstanceVariable(name.asJavaString()), value);
+        return (IRubyObject)variableTableStore(validateInstanceVariable(name.asJavaString()), value);
     }
 
     /** rb_obj_remove_instance_variable
@@ -1532,7 +1571,7 @@ public class RubyObject extends RubyBasicObject {
     public IRubyObject remove_instance_variable(ThreadContext context, IRubyObject name, Block block) {
         ensureInstanceVariablesSettable();
         IRubyObject value;
-        if ((value = variableTableRemove(validateInstanceVariable(name.asJavaString()))) != null) {
+        if ((value = (IRubyObject)variableTableRemove(validateInstanceVariable(name.asJavaString()))) != null) {
             return value;
         }
         throw context.getRuntime().newNameError("instance variable " + name.asJavaString() + " not defined", name.asJavaString());
