@@ -49,6 +49,7 @@ import org.jruby.exceptions.RaiseException;
 import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
+import org.jruby.runtime.BlockBody;
 import org.jruby.runtime.BlockCallback;
 import org.jruby.runtime.CallBlock;
 import org.jruby.runtime.ObjectAllocator;
@@ -372,19 +373,14 @@ public class RubyRange extends RubyObject {
         }
     }
 
-    @JRubyMethod(name = "each", frame = true)
+    @JRubyMethod(name = "each", frame = true, compat = CompatVersion.RUBY1_8)
     public IRubyObject each(ThreadContext context, final Block block) {
         final Ruby runtime = context.getRuntime();
 
         if (begin instanceof RubyFixnum && end instanceof RubyFixnum) {
-            long lim = ((RubyFixnum) end).getLongValue();
-            if (!isExclusive) lim++;
-
-            for (long i = ((RubyFixnum) begin).getLongValue(); i < lim; i++) {
-                block.yield(context, RubyFixnum.newFixnum(runtime, i));
-            }
+            fixnumEach(context, runtime, block);
         } else if (begin instanceof RubyString) {
-            ((RubyString) begin).upto(context, end, isExclusive, block);
+            ((RubyString) begin).uptoCommon(context, end, isExclusive, block);
         } else {
             if (!begin.respondsTo("succ")) throw getRuntime().newTypeError(
                     "can't iterate from " + begin.getMetaClass().getName());
@@ -398,17 +394,51 @@ public class RubyRange extends RubyObject {
         return this;
     }
 
-    @JRubyMethod(name = "each", frame = true, compat = CompatVersion.RUBY1_9)
-    public IRubyObject each19(final ThreadContext context, final Block block) {
-        return block.isGiven() ? each(context, block) : enumeratorize(context.getRuntime(), this, "each");
+    private void fixnumEach(ThreadContext context, Ruby runtime, Block block) {
+        long lim = ((RubyFixnum) end).getLongValue();
+        if (!isExclusive) lim++;
+
+        if (block.getBody().getArgumentType() == BlockBody.ZERO_ARGS) {
+            final IRubyObject nil = runtime.getNil();
+            for (long i = ((RubyFixnum) begin).getLongValue(); i < lim; i++) {
+                block.yield(context, nil);
+            }
+        } else {
+            for (long i = ((RubyFixnum) begin).getLongValue(); i < lim; i++) {
+                block.yield(context, RubyFixnum.newFixnum(runtime, i));
+            }
+        }
     }
 
-    @JRubyMethod(name = "step", frame = true)
+    @JRubyMethod(name = "each", frame = true, compat = CompatVersion.RUBY1_9)
+    public IRubyObject each19(final ThreadContext context, final Block block) {
+        Ruby runtime = context.getRuntime();
+        if (!block.isGiven()) return enumeratorize(runtime, this, "each");
+
+        if (!begin.respondsTo("succ")) throw getRuntime().newTypeError(
+                "can't iterate from " + begin.getMetaClass().getName());
+
+        if (begin instanceof RubyFixnum && end instanceof RubyFixnum) {
+            fixnumEach(context, runtime, block);
+        } else if (begin instanceof RubyString) {
+            ((RubyString) begin).uptoCommon19(context, end, isExclusive, block);
+        } else {
+            rangeEach(context, new RangeCallBack() {
+                @Override
+                void call(ThreadContext context, IRubyObject arg) {
+                    block.yield(context, arg);
+                }
+            });
+        }
+        return this;
+    }
+
+    @JRubyMethod(name = "step", frame = true, compat = CompatVersion.RUBY1_8)
     public IRubyObject step(ThreadContext context, IRubyObject step, Block block) {
         return stepCommon(context, step, block);
     }
 
-    @JRubyMethod(name = "step", frame = true)
+    @JRubyMethod(name = "step", frame = true, compat = CompatVersion.RUBY1_8)
     public IRubyObject step(ThreadContext context, Block block) {
         return stepCommon(context, RubyFixnum.one(context.getRuntime()), block);
     }
@@ -420,13 +450,7 @@ public class RubyRange extends RubyObject {
 
         if (begin instanceof RubyFixnum && end instanceof RubyFixnum) {
             if (unit == 0) throw runtime.newArgumentError("step can't be 0");
-
-            long e = ((RubyFixnum)end).getLongValue();
-            if (!isExclusive) e++;
-            
-            for (long i = ((RubyFixnum)begin).getLongValue(); i < e; i += unit) {
-                block.yield(context, RubyFixnum.newFixnum(runtime, i));
-            }
+            fixnumStep(context, runtime, unit, block);
         } else {
             IRubyObject tmp = begin.checkStringType();
             if (!tmp.isNil()) {
@@ -434,26 +458,13 @@ public class RubyRange extends RubyObject {
                 // rb_iterate((VALUE(*)_((VALUE)))str_step, (VALUE)args, step_i, (VALUE)iter);
                 StepBlockCallBack callback = new StepBlockCallBack(block, RubyFixnum.one(runtime), step);
                 Block blockCallback = CallBlock.newCallClosure(this, runtime.getRange(), Arity.singleArgument(), callback, context);
-                ((RubyString)tmp).upto(context, end, isExclusive, blockCallback);
+                ((RubyString)tmp).uptoCommon(context, end, isExclusive, blockCallback);
             } else if (begin instanceof RubyNumeric) {
-                if (equalInternal(context, step, RubyFixnum.zero(runtime))) {
-                    throw runtime.newArgumentError("step can't be 0");
-                }
-                final String method;
-                if (isExclusive) {
-                    method = "<";
-                } else {
-                    method = "<=";
-                }
-                IRubyObject beg = begin;
-                while (beg.callMethod(context, method, end).isTrue()) {
-                    block.yield(context, beg);
-                    beg = beg.callMethod(context, "+", step);
-                }
+                if (equalInternal(context, step, RubyFixnum.zero(runtime))) throw runtime.newArgumentError("step can't be 0");
+                numericStep(context, runtime, step, block);
             } else {
                 if (unit == 0) throw runtime.newArgumentError("step can't be 0");
-                if (!begin.respondsTo("succ")) throw runtime.newTypeError(
-                        "can't iterate from " + begin.getMetaClass().getName());
+                if (!begin.respondsTo("succ")) throw runtime.newTypeError("can't iterate from " + begin.getMetaClass().getName());
                 // range_each_func(range, step_i, b, e, args);
                 rangeEach(context, new StepBlockCallBack(block, RubyFixnum.one(runtime), step));
             }
@@ -461,14 +472,74 @@ public class RubyRange extends RubyObject {
         return this;
     }
 
+    private void fixnumStep(ThreadContext context, Ruby runtime, long unit, Block block) {
+        long e = ((RubyFixnum)end).getLongValue();
+        if (!isExclusive) e++;
+        for (long i = ((RubyFixnum)begin).getLongValue(); i < e; i += unit) {
+            block.yield(context, RubyFixnum.newFixnum(runtime, i));
+        }
+    }
+
+    private void numericStep(ThreadContext context, Ruby runtime, IRubyObject step, Block block) {
+        final String method = isExclusive ? "<" : "<=";
+        IRubyObject beg = begin;
+        while (beg.callMethod(context, method, end).isTrue()) {
+            block.yield(context, beg);
+            beg = beg.callMethod(context, "+", step);
+        }
+    }
+
     @JRubyMethod(name = "step", frame = true, compat = CompatVersion.RUBY1_9)
     public IRubyObject step19(final ThreadContext context, final Block block) {
-        return block.isGiven() ? step(context, block) : enumeratorize(context.getRuntime(), this, "step");
+        return block.isGiven() ? stepCommon19(context, RubyFixnum.zero(context.getRuntime()), block) : enumeratorize(context.getRuntime(), this, "step");
     }
 
     @JRubyMethod(name = "step", frame = true, compat = CompatVersion.RUBY1_9)
     public IRubyObject step19(final ThreadContext context, IRubyObject step, final Block block) {
-        return block.isGiven() ? step(context, step, block) : enumeratorize(context.getRuntime(), this, "step", step);
+        Ruby runtime = context.getRuntime();
+        if (!block.isGiven()) return enumeratorize(runtime, this, "step", step);
+
+        if (!(step instanceof RubyNumeric)) step = step.convertToInteger("to_int");
+        IRubyObject zero = RubyFixnum.zero(runtime);
+        if (step.callMethod(context, "<", zero).isTrue()) throw runtime.newArgumentError("step can't be negative");
+        if (!step.callMethod(context, ">", zero).isTrue()) throw runtime.newArgumentError("step can't be 0");
+        return stepCommon19(context, step, block);
+    }
+
+    private IRubyObject stepCommon19(ThreadContext context, IRubyObject step, Block block) {
+        Ruby runtime = context.getRuntime();
+        if (begin instanceof RubyFixnum && end instanceof RubyFixnum && step instanceof RubyFixnum) {
+            fixnumStep(context, runtime, ((RubyFixnum)step).getLongValue(), block);
+        } else if (begin instanceof RubyFloat || end instanceof RubyFloat || step instanceof RubyFloat) {
+            RubyNumeric.floatStep19(context, runtime, begin, end, step, isExclusive, block);
+        } else if (begin instanceof RubyNumeric ||
+                        !checkIntegerType(runtime, begin, "to_int").isNil() ||
+                        !checkIntegerType(runtime, end, "to_int").isNil()) {
+            numericStep19(context, runtime, step, block);
+        } else {
+            IRubyObject tmp = begin.checkStringType();
+            if (!tmp.isNil()) {
+                StepBlockCallBack callback = new StepBlockCallBack(block, RubyFixnum.one(runtime), step);
+                Block blockCallback = CallBlock.newCallClosure(this, runtime.getRange(), Arity.singleArgument(), callback, context);
+                ((RubyString)tmp).uptoCommon19(context, end, isExclusive, blockCallback);
+            } else {
+                if (!begin.respondsTo("succ")) throw runtime.newTypeError("can't iterate from " + begin.getMetaClass().getName());
+                // range_each_func(range, step_i, b, e, args);
+                rangeEach(context, new StepBlockCallBack(block, RubyFixnum.one(runtime), step));
+            }
+        }
+        return this;
+    }
+
+    private void numericStep19(ThreadContext context, Ruby runtime, IRubyObject step, Block block) {
+        final String method = isExclusive ? "<" : "<=";
+        IRubyObject beg = begin;
+        long i = 0;
+        while (beg.callMethod(context, method, end).isTrue()) {
+            block.yield(context, beg);
+            i++;
+            beg = begin.callMethod(context, "+", RubyFixnum.newFixnum(runtime, i).callMethod(context, "*", step));
+        }
     }
 
     @JRubyMethod(name = {"include?", "member?", "==="}, required = 1, compat = CompatVersion.RUBY1_8)
@@ -541,12 +612,13 @@ public class RubyRange extends RubyObject {
 
     @JRubyMethod(name = "max", frame = true, compat = CompatVersion.RUBY1_9)
     public IRubyObject max(ThreadContext context, Block block) {
-        if (block.isGiven() || isExclusive && !(end instanceof RubyInteger)) {
+        if (block.isGiven() || isExclusive && !(end instanceof RubyNumeric)) {
             return RuntimeHelpers.invokeSuper(context, this, block);
         } else {
             int c = RubyComparable.cmpint(context, begin.callMethod(context, "<=>", end), begin, end);
             Ruby runtime = context.getRuntime();
             if (isExclusive) {
+                if (!(end instanceof RubyInteger)) throw runtime.newTypeError("cannot exclude non Integer end value");
                 if (c == 0) return runtime.getNil();
                 if (end instanceof RubyFixnum) return RubyFixnum.newFixnum(runtime, ((RubyFixnum)end).getLongValue() - 1);
                 return end.callMethod(context, "-", RubyFixnum.one(runtime));
@@ -611,9 +683,9 @@ public class RubyRange extends RubyObject {
 
             unmarshalStream.defaultVariablesUnmarshal(range);
             
-            range.begin = range.removeInternalVariable("begin");
-            range.end = range.removeInternalVariable("end");
-            range.isExclusive = range.removeInternalVariable("excl").isTrue();
+            range.begin = (IRubyObject)range.removeInternalVariable("begin");
+            range.end = (IRubyObject)range.removeInternalVariable("end");
+            range.isExclusive = ((IRubyObject)range.removeInternalVariable("excl")).isTrue();
 
             return range;
         }
