@@ -33,7 +33,9 @@ import static org.jruby.util.Numeric.f_expt;
 import static org.jruby.util.Numeric.f_floor;
 import static org.jruby.util.Numeric.f_gcd;
 import static org.jruby.util.Numeric.f_idiv;
+import static org.jruby.util.Numeric.f_inspect;
 import static org.jruby.util.Numeric.f_integer_p;
+import static org.jruby.util.Numeric.f_lt_p;
 import static org.jruby.util.Numeric.f_mul;
 import static org.jruby.util.Numeric.f_negate;
 import static org.jruby.util.Numeric.f_negative_p;
@@ -43,6 +45,7 @@ import static org.jruby.util.Numeric.f_sub;
 import static org.jruby.util.Numeric.f_to_f;
 import static org.jruby.util.Numeric.f_to_i;
 import static org.jruby.util.Numeric.f_to_r;
+import static org.jruby.util.Numeric.f_to_s;
 import static org.jruby.util.Numeric.f_truncate;
 import static org.jruby.util.Numeric.f_xor;
 import static org.jruby.util.Numeric.f_zero_p;
@@ -55,7 +58,6 @@ import org.jcodings.specific.ASCIIEncoding;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.common.IRubyWarnings;
-import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Arity;
 import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.Frame;
@@ -365,7 +367,7 @@ public class RubyRational extends RubyNumeric {
         
         Frame frame = context.getCurrentFrame();
         IRubyObject backref = frame.getBackRef();
-        if (backref != null && backref instanceof RubyMatchData) ((RubyMatchData)backref).use();
+        if (backref instanceof RubyMatchData) ((RubyMatchData)backref).use();
         
         if (a1 instanceof RubyFloat) {
             a1 = f_to_r(context, a1);
@@ -720,6 +722,18 @@ public class RubyRational extends RubyNumeric {
         return f_negate(context, this);
     }
 
+    private IRubyObject op_roundCommonPre(ThreadContext context, IRubyObject n) {
+        Ruby runtime = context.getRuntime();
+        if (!(n instanceof RubyInteger)) throw runtime.newTypeError("not an integer");
+        return f_expt(context, RubyFixnum.newFixnum(runtime, 10), n);
+    }
+
+    private IRubyObject op_roundCommonPost(ThreadContext context, IRubyObject s, IRubyObject n, IRubyObject b) {
+        s = f_div(context, newRationalBang(context, getMetaClass(), s), b);
+        if (f_lt_p(context, n, RubyFixnum.one(context.getRuntime())).isTrue()) s = f_to_i(context, s);
+        return s;
+    }
+
     /** nurat_floor
      * 
      */
@@ -728,6 +742,12 @@ public class RubyRational extends RubyNumeric {
         return f_idiv(context, num, den);
     }
 
+    @JRubyMethod(name = "floor")
+    public IRubyObject op_floor(ThreadContext context, IRubyObject n) {
+        IRubyObject b = op_roundCommonPre(context, n);
+        return op_roundCommonPost(context, ((RubyRational)f_mul(context, this, b)).op_floor(context), n, b);
+    }
+
     /** nurat_ceil
      * 
      */
@@ -736,6 +756,12 @@ public class RubyRational extends RubyNumeric {
         return f_negate(context, f_idiv(context, f_negate(context, num), den));
     }
 
+    @JRubyMethod(name = "ceil")
+    public IRubyObject op_ceil(ThreadContext context, IRubyObject n) {
+        IRubyObject b = op_roundCommonPre(context, n);
+        return op_roundCommonPost(context, ((RubyRational)f_mul(context, this, b)).op_ceil(context), n, b);
+    }
+
     /** nurat_truncate
      * 
      */
@@ -747,22 +773,35 @@ public class RubyRational extends RubyNumeric {
         return f_idiv(context, num, den);
     }
 
+    @JRubyMethod(name = "truncate")
+    public IRubyObject op_truncate(ThreadContext context, IRubyObject n) {
+        IRubyObject b = op_roundCommonPre(context, n);
+        return op_roundCommonPost(context, ((RubyRational)f_mul(context, this, b)).op_truncate(context), n, b);
+    }
+
     /** nurat_round
      * 
      */
     @JRubyMethod(name = "round")
     public IRubyObject op_round(ThreadContext context) {
+        IRubyObject num = this.num;
+        boolean neg = f_negative_p(context, num);
+        if (neg) num = f_negate(context, num);
+
+        IRubyObject den = this.den;
         IRubyObject two = RubyFixnum.two(context.getRuntime());
-        if (f_negative_p(context, num)) {
-            IRubyObject tnum = f_negate(context, num);
-            tnum = f_add(context, f_mul(context, tnum, two), den);
-            IRubyObject tden = f_mul(context, den, two);
-            return f_negate(context, f_idiv(context, tnum, tden)); 
-        } else {
-            IRubyObject tnum = f_add(context, f_mul(context, num, two), den);
-            IRubyObject tden = f_mul(context, den, two);
-            return f_idiv(context, tnum, tden);
-        }
+        num = f_add(context, f_mul(context, num, two), den);
+        den = f_mul(context, den, two);
+        num = f_idiv(context, num, den);
+
+        if (neg) num = f_negate(context, num);
+        return num;
+    }
+
+    @JRubyMethod(name = "round")
+    public IRubyObject op_round(ThreadContext context, IRubyObject n) {
+        IRubyObject b = op_roundCommonPre(context, n);
+        return op_roundCommonPost(context, ((RubyRational)f_mul(context, this, b)).op_round(context), n, b);
     }
 
     /** nurat_to_f
@@ -799,7 +838,7 @@ public class RubyRational extends RubyNumeric {
         }
 
         long e = ne - de;
-        
+
         if (e > 1023 || e < -1022) {
             runtime.getWarnings().warn(IRubyWarnings.ID.FLOAT_OUT_OF_RANGE, "out of Float range", getMetaClass());
             return runtime.newFloat(e > 0 ? Double.MAX_VALUE : 0);
@@ -807,9 +846,7 @@ public class RubyRational extends RubyNumeric {
 
         double f = RubyNumeric.num2dbl(num) / RubyNumeric.num2dbl(den); 
 
-        if (minus) {
-            f = -f;
-        }
+        if (minus) f = -f;
 
         f = ldexp(f, e);
 
@@ -841,7 +878,11 @@ public class RubyRational extends RubyNumeric {
      */
     @JRubyMethod(name = "to_s")
     public IRubyObject to_s(ThreadContext context) {
-        return RuntimeHelpers.invoke(context, context.getRuntime().getKernel(), "format", context.getRuntime().newString("%d/%d"), num, den);
+        RubyString str = context.getRuntime().newString();
+        str.append(f_to_s(context, num));
+        str.cat((byte)'/');
+        str.append(f_to_s(context, den));
+        return str;
     }
 
     /** nurat_inspect
@@ -849,7 +890,13 @@ public class RubyRational extends RubyNumeric {
      */
     @JRubyMethod(name = "inspect")
     public IRubyObject inspect(ThreadContext context) {
-        return RuntimeHelpers.invoke(context, context.getRuntime().getKernel(), "format", context.getRuntime().newString("(%d/%d)"), num, den);
+        RubyString str = context.getRuntime().newString();
+        str.cat((byte)'(');
+        str.append(f_inspect(context, num));
+        str.cat((byte)'/');
+        str.append(f_inspect(context, den));
+        str.cat((byte)')');
+        return str;
     }
 
     /** nurat_marshal_dump
@@ -883,7 +930,7 @@ public class RubyRational extends RubyNumeric {
         Ruby runtime = context.getRuntime();
         if (bytes.realSize == 0) return runtime.newArray(runtime.getNil(), recv);
 
-        IRubyObject m = RubyRegexp.newRegexp(runtime, Numeric.RationalPatterns.rat_pat).callMethod(context, "match", s);
+        IRubyObject m = RubyRegexp.newDummyRegexp(runtime, Numeric.RationalPatterns.rat_pat).callMethod(context, "match", s);
         
         if (!m.isNil()) {
             IRubyObject si = m.callMethod(context, "[]", RubyFixnum.one(runtime));
@@ -891,11 +938,11 @@ public class RubyRational extends RubyNumeric {
             IRubyObject de = m.callMethod(context, "[]", RubyFixnum.three(runtime));
             IRubyObject re = m.callMethod(context, "post_match");
             
-            RubyArray a = nu.callMethod(context, "split", RubyRegexp.newRegexp(runtime, Numeric.RationalPatterns.an_e_pat)).convertToArray();
+            RubyArray a = nu.callMethod(context, "split", RubyRegexp.newDummyRegexp(runtime, Numeric.RationalPatterns.an_e_pat)).convertToArray();
             IRubyObject ifp = a.eltInternal(0);
             IRubyObject exp = a.size() != 2 ? runtime.getNil() : a.eltInternal(1);
             
-            a = ifp.callMethod(context, "split", RubyRegexp.newRegexp(runtime, Numeric.RationalPatterns.a_dot_pat)).convertToArray();
+            a = ifp.callMethod(context, "split", RubyRegexp.newDummyRegexp(runtime, Numeric.RationalPatterns.a_dot_pat)).convertToArray();
             IRubyObject ip = a.eltInternal(0);
             IRubyObject fp = a.size() != 2 ? runtime.getNil() : a.eltInternal(1);
             
@@ -940,7 +987,7 @@ public class RubyRational extends RubyNumeric {
         RubyArray a = str_to_r_internal(context, recv);
         if (a.eltInternal(0).isNil() || a.eltInternal(1).convertToString().getByteList().length() > 0) {
             IRubyObject s = recv.callMethod(context, "inspect");
-            throw context.getRuntime().newArgumentError("invalid value for Rational: " + s.convertToString());
+            throw context.getRuntime().newArgumentError("invalid value for convert(): " + s.convertToString());
         }
         return a.eltInternal(0);
     }    
