@@ -101,7 +101,7 @@ public class RubyStruct extends RubyObject {
         IRubyObject variable;
 
         while (type != null && type != structClass) {
-            if ((variable = type.fastGetInternalVariable(internedName)) != null) {
+            if ((variable = (IRubyObject)type.fastGetInternalVariable(internedName)) != null) {
                 return variable;
             }
 
@@ -116,7 +116,7 @@ public class RubyStruct extends RubyObject {
     }
 
     private void modify() {
-        testFrozen("Struct is frozen");
+        testFrozen();
 
         if (!isTaint() && getRuntime().getSafeLevel() >= 4) {
             throw getRuntime().newSecurityError("Insecure: can't modify struct");
@@ -229,7 +229,7 @@ public class RubyStruct extends RubyObject {
             final String memberName = args[i].asJavaString();
             // if we are storing a name as well, index is one too high for values
             final int index = (name == null && !nilName) ? i : i - 1;
-            newStruct.addMethod(memberName, new DynamicMethod(newStruct, Visibility.PUBLIC, CallConfiguration.NO_FRAME_NO_SCOPE) {
+            newStruct.addMethod(memberName, new DynamicMethod(newStruct, Visibility.PUBLIC, CallConfiguration.FrameNoneScopeNone) {
                 @Override
                 public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
                     Arity.checkArgumentCount(self.getRuntime(), args, 0, 0);
@@ -246,7 +246,7 @@ public class RubyStruct extends RubyObject {
                     return this;
                 }
             });
-            newStruct.addMethod(memberName + "=", new DynamicMethod(newStruct, Visibility.PUBLIC, CallConfiguration.NO_FRAME_NO_SCOPE) {
+            newStruct.addMethod(memberName + "=", new DynamicMethod(newStruct, Visibility.PUBLIC, CallConfiguration.FrameNoneScopeNone) {
                 @Override
                 public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
                     Arity.checkArgumentCount(self.getRuntime(), args, 1, 1);
