@@ -36,17 +36,21 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby;
 
+import static org.jruby.util.StringSupport.codeLength;
+import static org.jruby.util.StringSupport.codePoint;
+
 import java.util.concurrent.locks.ReentrantLock;
 
-import org.jruby.anno.JRubyMethod;
+import org.jcodings.Encoding;
 import org.jruby.anno.JRubyClass;
+import org.jruby.anno.JRubyMethod;
 import org.jruby.common.IRubyWarnings.ID;
 import org.jruby.javasupport.util.RuntimeHelpers;
+import org.jruby.runtime.Block;
+import org.jruby.runtime.BlockCallback;
 import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
-import org.jruby.runtime.Block;
-import org.jruby.runtime.BlockCallback;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.runtime.marshal.UnmarshalStream;
 import org.jruby.util.ByteList;
@@ -66,18 +70,21 @@ public class RubySymbol extends RubyObject {
      * @param internedSymbol the String value of the new Symbol. This <em>must</em>
      *                       have been previously interned
      */
-    private RubySymbol(Ruby runtime, String internedSymbol) {
+    private RubySymbol(Ruby runtime, String internedSymbol, ByteList symbolBytes) {
         super(runtime, runtime.getSymbol(), false, false);
         // symbol string *must* be interned
 
         //        assert internedSymbol == internedSymbol.intern() : internedSymbol + " is not interned";
 
         this.symbol = internedSymbol;
-        this.symbolBytes = ByteList.create(symbol);
-
+        this.symbolBytes = symbolBytes;
         this.id = runtime.allocSymbolId();
     }
-    
+
+    private RubySymbol(Ruby runtime, String internedSymbol) {
+        this(runtime, internedSymbol, ByteList.create(internedSymbol));
+    }
+
     public static RubyClass createSymbolClass(Ruby runtime) {
         RubyClass symbolClass = runtime.defineClass("Symbol", runtime.getObject(), ObjectAllocator.NOT_ALLOCATABLE_ALLOCATOR);
         runtime.setSymbol(symbolClass);
@@ -108,7 +115,16 @@ public class RubySymbol extends RubyObject {
     public String asJavaString() {
         return symbol;
     }
-    
+
+    @Override
+    public String toString() {
+        return symbol;
+    }
+
+    final ByteList getBytes() {
+        return symbolBytes;
+    }
+
     /** short circuit for Symbol key comparison
      * 
      */
@@ -139,36 +155,106 @@ public class RubySymbol extends RubyObject {
         return runtime.getSymbolTable().getSymbol(name);
     }
 
-    @JRubyMethod(name = "to_i")
+    @Deprecated
     public RubyFixnum to_i() {
-        return getRuntime().newFixnum(id);
+        return to_i(getRuntime());
     }
 
-    @JRubyMethod(name = "to_int")
+    @JRubyMethod(name = "to_i", compat = CompatVersion.RUBY1_8)
+    public RubyFixnum to_i(ThreadContext context) {
+        return to_i(context.getRuntime());
+    }
+
+    private final RubyFixnum to_i(Ruby runtime) {
+        return runtime.newFixnum(id);
+    }
+
+    @Deprecated
     public RubyFixnum to_int() {
-        if (getRuntime().getVerbose().isTrue()) {
-            getRuntime().getWarnings().warn(ID.SYMBOL_AS_INTEGER, "treating Symbol as an integer");
-	}
-        return to_i();
+        return to_int(getRuntime());
     }
 
-    @JRubyMethod(name = "inspect")
+    @JRubyMethod(name = "to_int", compat = CompatVersion.RUBY1_8)
+    public RubyFixnum to_int(ThreadContext context) {
+        return to_int(context.getRuntime());
+    }
+
+    private final RubyFixnum to_int(Ruby runtime) {
+        if (runtime.isVerbose()) {
+            runtime.getWarnings().warn(ID.SYMBOL_AS_INTEGER, "treating Symbol as an integer");
+        }
+        return to_i(runtime);
+    }
+
+    @Deprecated
     @Override
     public IRubyObject inspect() {
-        Ruby runtime = getRuntime();
-        return runtime.newString(":" + 
-            (isSymbolName(symbol) ? symbol : RubyString.newStringShared(runtime, symbolBytes).dump().toString())); 
+        return inspect(getRuntime());
+    }
+    @JRubyMethod(name = "inspect", compat = CompatVersion.RUBY1_8)
+    public IRubyObject inspect(ThreadContext context) {
+        return inspect(context.getRuntime());
+    }
+    private final IRubyObject inspect(Ruby runtime) {
+        
+        final ByteList bytes;
+        if (isSymbolName(symbol)) {
+            bytes = symbolBytes;
+        } else {
+            bytes = ((RubyString)RubyString.newString(runtime, symbolBytes).dump()).getByteList();
+        }
+        ByteList result = new ByteList(bytes.realSize + 1);
+        result.append((byte)':');
+        result.append(bytes);
+
+        return RubyString.newString(runtime, result);
+    }
+
+    @Deprecated
+    public IRubyObject inspect19() {
+        return inspect19(getRuntime());
+    }
+    @JRubyMethod(name = "inspect", compat = CompatVersion.RUBY1_9)
+    public IRubyObject inspect19(ThreadContext context) {
+        return inspect19(context.getRuntime());
+    }
+    private final IRubyObject inspect19(Ruby runtime) {
+        
+        ByteList result = new ByteList(symbolBytes.realSize + 1);
+        result.encoding = symbolBytes.encoding;
+        result.append((byte)':');
+        result.append(symbolBytes);
+
+        RubyString str = RubyString.newString(runtime, result); 
+        if (isPrintable() && isSymbolName(symbol)) { // TODO: 1.9 rb_enc_symname_p
+            return str;
+        } else {
+            str = (RubyString)str.inspect19();
+            ByteList bytes = str.getByteList();
+            bytes.set(0, ':');
+            bytes.set(1, '"');
+            return str;
+        }
     }
 
-    @JRubyMethod(name = "to_s")
     @Override
     public IRubyObject to_s() {
-        return RubyString.newStringShared(getRuntime(), symbolBytes);
+        return to_s(getRuntime());
+    }
+    @JRubyMethod(name = "to_s")
+    public IRubyObject to_s(ThreadContext context) {
+        return to_s(context.getRuntime());
+    }
+    private final IRubyObject to_s(Ruby runtime) {
+        return RubyString.newStringShared(runtime, symbolBytes);
     }
 
-    @JRubyMethod(name = "id2name")
     public IRubyObject id2name() {
-        return to_s();
+        return to_s(getRuntime());
+    }
+    @JRubyMethod(name = "id2name")
+    public IRubyObject id2name(ThreadContext context) {
+        return to_s(context);
     }
 
     @JRubyMethod(name = "===", required = 1)
@@ -177,10 +263,16 @@ public class RubySymbol extends RubyObject {
         return super.op_equal(context, other);
     }
 
+    @Deprecated
     @Override
     public RubyFixnum hash() {
         return getRuntime().newFixnum(hashCode());
     }
+
+    @JRubyMethod(name = "hash")
+    public RubyFixnum hash(ThreadContext context) {
+        return context.getRuntime().newFixnum(hashCode());
+    }
     
     @Override
     public int hashCode() {
@@ -201,11 +293,6 @@ public class RubySymbol extends RubyObject {
         return this;
     }
 
-    @Override
-    public IRubyObject freeze(ThreadContext context) {
-        return this;
-    }
-
     @Override
     public IRubyObject taint(ThreadContext context) {
         return this;
@@ -218,14 +305,14 @@ public class RubySymbol extends RubyObject {
     @JRubyMethod(name = {"succ", "next"}, compat = CompatVersion.RUBY1_9)
     public IRubyObject succ(ThreadContext context) {
         Ruby runtime = context.getRuntime();
-        return newSymbol(runtime, newShared(runtime).succ(context).toString());
+        return newSymbol(runtime, newShared(runtime).succ19(context).toString());
     }
 
     @JRubyMethod(name = "<=>", compat = CompatVersion.RUBY1_9)
     public IRubyObject op_cmp(ThreadContext context, IRubyObject other) {
         Ruby runtime = context.getRuntime();
         if (other instanceof RubySymbol) {
-            return (newShared(runtime).op_cmp(context, ((RubySymbol)other).newShared(runtime)));
+            return (newShared(runtime).op_cmp19(context, ((RubySymbol)other).newShared(runtime)));
         }
         return runtime.getNil();
     }
@@ -234,7 +321,7 @@ public class RubySymbol extends RubyObject {
     public IRubyObject casecmp(ThreadContext context, IRubyObject other) {
         Ruby runtime = context.getRuntime();
         if (other instanceof RubySymbol) {
-            return newShared(runtime).casecmp(((RubySymbol) other).newShared(runtime));
+            return newShared(runtime).casecmp19(context, ((RubySymbol) other).newShared(runtime));
         }
         return runtime.getNil();
     }
@@ -242,24 +329,24 @@ public class RubySymbol extends RubyObject {
     @JRubyMethod(name = {"=~", "match"}, compat = CompatVersion.RUBY1_9)
     public IRubyObject op_match19(ThreadContext context, IRubyObject other) {
         Ruby runtime = context.getRuntime();
-        return newShared(runtime).op_match(context, other);
+        return newShared(runtime).op_match19(context, other);
     }
 
     @JRubyMethod(name = {"[]", "slice"}, compat = CompatVersion.RUBY1_9)
     public IRubyObject op_aref(ThreadContext context, IRubyObject arg) {
         Ruby runtime = context.getRuntime();
-        return newShared(runtime).op_aref(context, arg);
+        return newShared(runtime).op_aref19(context, arg);
     }
 
     @JRubyMethod(name = {"[]", "slice"}, compat = CompatVersion.RUBY1_9)
     public IRubyObject op_aref(ThreadContext context, IRubyObject arg1, IRubyObject arg2) {
         Ruby runtime = context.getRuntime();
-        return newShared(runtime).op_aref(context, arg1, arg2);
+        return newShared(runtime).op_aref19(context, arg1, arg2);
     }
 
     @JRubyMethod(name = {"length", "size"}, compat = CompatVersion.RUBY1_9)
     public IRubyObject length() {
-        return newShared(getRuntime()).length();
+        return newShared(getRuntime()).length19();
     }
 
     @JRubyMethod(name = "empty?", compat = CompatVersion.RUBY1_9)
@@ -270,25 +357,25 @@ public class RubySymbol extends RubyObject {
     @JRubyMethod(name = "upcase", compat = CompatVersion.RUBY1_9)
     public IRubyObject upcase(ThreadContext context) {
         Ruby runtime = context.getRuntime();
-        return newSymbol(runtime, newShared(runtime).upcase(context).toString());
+        return newSymbol(runtime, newShared(runtime).upcase19(context).toString());
     }
 
     @JRubyMethod(name = "downcase", compat = CompatVersion.RUBY1_9)
     public IRubyObject downcase(ThreadContext context) {
         Ruby runtime = context.getRuntime();
-        return newSymbol(runtime, newShared(runtime).downcase(context).toString());
+        return newSymbol(runtime, newShared(runtime).downcase19(context).toString());
     }
 
     @JRubyMethod(name = "capitalize", compat = CompatVersion.RUBY1_9)
     public IRubyObject capitalize(ThreadContext context) {
         Ruby runtime = context.getRuntime();
-        return newSymbol(runtime, newShared(runtime).capitalize(context).toString());
+        return newSymbol(runtime, newShared(runtime).capitalize19(context).toString());
     }
 
     @JRubyMethod(name = "swapcase", compat = CompatVersion.RUBY1_9)
     public IRubyObject swapcase(ThreadContext context) {
         Ruby runtime = context.getRuntime();
-        return newSymbol(runtime, newShared(runtime).swapcase(context).toString());
+        return newSymbol(runtime, newShared(runtime).swapcase19(context).toString());
     }
 
     @JRubyMethod(name = "encoding", compat = CompatVersion.RUBY1_9)
@@ -389,11 +476,24 @@ public class RubySymbol extends RubyObject {
         }
         return true;
     }
-    
-    private static boolean isSymbolName(String s) {
-        if (s == null || s.length() < 1) {
-            return false;
+
+    private boolean isPrintable() {
+        Ruby runtime = getRuntime();
+        int p = symbolBytes.begin;
+        int end = p + symbolBytes.realSize;
+        byte[]bytes = symbolBytes.bytes;
+        Encoding enc = symbolBytes.encoding;
+
+        while (p < end) {
+            int c = codePoint(runtime, enc, bytes, p, end);
+            if (!enc.isPrint(c)) return false;
+            p += codeLength(runtime, enc, c);
         }
+        return true;
+    }
+
+    private static boolean isSymbolName(String s) {
+        if (s == null || s.length() < 1) return false;
 
         int length = s.length();
 
@@ -431,9 +531,7 @@ public class RubySymbol extends RubyObject {
             return s.equals("[]") || s.equals("[]=");
         }
         
-        if (!isIdentStart(c)) {
-            return false;
-        }
+        if (!isIdentStart(c)) return false;
 
         boolean localID = (c >= 'a' && c <= 'z');
         int last = 1;
@@ -458,6 +556,10 @@ public class RubySymbol extends RubyObject {
     }
     
     @JRubyMethod(name = "all_symbols", meta = true)
+    public static IRubyObject all_symbols(ThreadContext context, IRubyObject recv) {
+        return context.getRuntime().getSymbolTable().all_symbols();
+    }
+    @Deprecated
     public static IRubyObject all_symbols(IRubyObject recv) {
         return recv.getRuntime().getSymbolTable().all_symbols();
     }
@@ -468,7 +570,7 @@ public class RubySymbol extends RubyObject {
         return result;
     }
 
-    public static class SymbolTable {
+    public static final class SymbolTable {
         static final int DEFAULT_INITIAL_CAPACITY = 2048; // *must* be power of 2!
         static final int MAXIMUM_CAPACITY = 1 << 30;
         static final float DEFAULT_LOAD_FACTOR = 0.75f;
@@ -506,18 +608,54 @@ public class RubySymbol extends RubyObject {
 
         public RubySymbol getSymbol(String name) {
             int hash = name.hashCode();
-            SymbolEntry[] table;
-            for (SymbolEntry e = (table = symbolTable)[hash & (table.length - 1)]; e != null; e = e.next) {
-                if (hash == e.hash && name.equals(e.name)) {
-                    return e.symbol;
-                }
+            SymbolEntry[] table = symbolTable;
+            SymbolEntry e = getEntryFromTable(table, hash);
+            for (; e != null; e = e.next) {
+                if (isSymbolMatch(name, hash, e)) return e.symbol;
             }
+            return createSymbol(name, ByteList.create(name), hash, table);
+        }
+
+        public RubySymbol getSymbol(ByteList bytes) {
+            String name = bytes.toString();
+            int hash = name.hashCode();
+            SymbolEntry[] table = symbolTable;
+            SymbolEntry e = getEntryFromTable(table, hash);
+            for (; e != null; e = e.next) {
+                if (isSymbolMatch(name, hash, e)) return e.symbol;
+            }
+            return createSymbol(name, bytes, hash, table);
+        }
+
+        public RubySymbol fastGetSymbol(String internedName) {
+            //            assert internedName == internedName.intern() : internedName + " is not interned";
+            SymbolEntry[] table = symbolTable;
+            SymbolEntry e = getEntryFromTable(symbolTable, internedName.hashCode());
+            for (; e != null; e = e.next) {
+                if (isSymbolMatch(internedName, e)) return e.symbol;
+            }
+            return fastCreateSymbol(internedName, table);
+        }
+
+        private static SymbolEntry getEntryFromTable(SymbolEntry[] table, int hash) {
+            return table[hash & (table.length - 1)];
+        }
+
+        private static boolean isSymbolMatch(String name, int hash, SymbolEntry entry) {
+            return hash == entry.hash && name.equals(entry.name);
+        }
+
+        private static boolean isSymbolMatch(String internedName, SymbolEntry entry) {
+            return internedName == entry.name;
+        }
+
+        private RubySymbol createSymbol(String name, ByteList value, int hash, SymbolEntry[] table) {
             ReentrantLock lock;
             (lock = tableLock).lock();
             try {
                 int potentialNewSize;
                 if ((potentialNewSize = size + 1) > threshold) {
-                    table = rehash(); 
+                    table = rehash();
                 } else {
                     table = symbolTable;
                 }
@@ -529,7 +667,7 @@ public class RubySymbol extends RubyObject {
                     }
                 }
                 String internedName;
-                RubySymbol symbol = new RubySymbol(runtime, internedName = name.intern());
+                RubySymbol symbol = new RubySymbol(runtime, internedName = name.intern(), value);
                 table[index] = new SymbolEntry(hash, internedName, symbol, table[index]);
                 size = potentialNewSize;
                 // write-volatile
@@ -539,15 +677,8 @@ public class RubySymbol extends RubyObject {
                 lock.unlock();
             }
         }
-        
-        public RubySymbol fastGetSymbol(String internedName) {
-            //            assert internedName == internedName.intern() : internedName + " is not interned";
-            SymbolEntry[] table;
-            for (SymbolEntry e = (table = symbolTable)[internedName.hashCode() & (table.length - 1)]; e != null; e = e.next) {
-                if (internedName == e.name) {
-                    return e.symbol;
-                }
-            }
+
+        private RubySymbol fastCreateSymbol(String internedName, SymbolEntry[] table) {
             ReentrantLock lock;
             (lock = tableLock).lock();
             try {
