@@ -94,6 +94,9 @@ public class RubyThread extends RubyObject {
     public Object killLock = new Object();
     
     public final ReentrantLock lock = new ReentrantLock();
+
+    // Error info is per-thread
+    private IRubyObject errorInfo;
     
     private static final boolean DEBUG = false;
 
@@ -101,6 +104,18 @@ public class RubyThread extends RubyObject {
         super(runtime, type);
         this.threadService = runtime.getThreadService();
         finalResult = runtime.getNil();
+
+        // init errorInfo to nil
+        errorInfo = runtime.getNil();
+    }
+
+    public IRubyObject getErrorInfo() {
+        return errorInfo;
+    }
+
+    public IRubyObject setErrorInfo(IRubyObject errorInfo) {
+        this.errorInfo = errorInfo;
+        return errorInfo;
     }
     
     /**
@@ -189,12 +204,16 @@ public class RubyThread extends RubyObject {
 
         RubyRunnable runnable = new RubyRunnable(this, args, block);
         if (RubyInstanceConfig.POOLING_ENABLED) {
-            threadImpl = new FutureThread(this, runnable);
+            FutureThread futureThread = new FutureThread(this, runnable);
+            threadImpl = futureThread;
 
             // set to default thread group
             runtime.getDefaultThreadGroup().addDirectly(this);
 
             threadImpl.start();
+
+            // JRUBY-2380, associate future early so it shows up in Thread.list right away, in case it doesn't run immediately
+            runtime.getThreadService().associateThread(futureThread.getFuture(), this);
         } else {
             Thread thread = new Thread(runnable);
             thread.setDaemon(true);
@@ -229,12 +248,6 @@ public class RubyThread extends RubyObject {
         return rubyThread;
     }
     
-    private void ensureNotCurrent() {
-        if (this == getRuntime().getCurrentContext().getThread()) {
-            throw new RuntimeException("internal thread method called from another thread");
-        }
-    }
-    
     public synchronized void cleanTerminate(IRubyObject result) {
         finalResult = result;
         isStopped = true;
@@ -246,9 +259,6 @@ public class RubyThread extends RubyObject {
     }
     
     public void pollThreadEvents(ThreadContext context) {
-        // check for criticalization *before* locking ourselves
-        threadService.waitForCritical();
-
         if (killed) throwThreadKill();
         if (receivedException != null) receivedAnException(context);
     }
@@ -494,14 +504,14 @@ public class RubyThread extends RubyObject {
         return RubyArray.newArrayNoCopy(getRuntime(), getThreadLocals().keySet().toArray(keys));
     }
     
-    @JRubyMethod(name = "critical=", required = 1, meta = true)
+    @JRubyMethod(name = "critical=", required = 1, meta = true, compat = CompatVersion.RUBY1_8)
     public static IRubyObject critical_set(IRubyObject receiver, IRubyObject value) {
     	receiver.getRuntime().getThreadService().setCritical(value.isTrue());
     	
     	return value;
     }
 
-    @JRubyMethod(name = "critical", meta = true)
+    @JRubyMethod(name = "critical", meta = true, compat = CompatVersion.RUBY1_8)
     public static IRubyObject critical(IRubyObject receiver) {
     	return receiver.getRuntime().newBoolean(receiver.getRuntime().getThreadService().getCritical());
     }
@@ -514,15 +524,17 @@ public class RubyThread extends RubyObject {
         synchronized (stopLock) {
             rubyThread.pollThreadEvents();
             try {
-                rubyThread.isStopped = true;
                 // attempt to decriticalize all if we're the critical thread
                 receiver.getRuntime().getThreadService().setCritical(false);
                 
+                rubyThread.isStopped = true;
+                
                 stopLock.wait();
             } catch (InterruptedException ie) {
                 rubyThread.pollThreadEvents();
+            } finally {
+                rubyThread.isStopped = false;
             }
-            rubyThread.isStopped = false;
         }
         
         return receiver.getRuntime().getNil();
@@ -585,10 +597,13 @@ public class RubyThread extends RubyObject {
         return this.priority;
     }
 
-    @JRubyMethod(name = "raise", optional = 2, frame = true)
+    @JRubyMethod(name = "raise", optional = 3, frame = true)
     public IRubyObject raise(IRubyObject[] args, Block block) {
-        ensureNotCurrent();
         Ruby runtime = getRuntime();
+        ThreadContext context = runtime.getCurrentContext();
+        if (this == context.getThread()) {
+            return RubyKernel.raise(context, runtime.getKernel(), args, block);
+        }
         
         if (DEBUG) System.out.println("thread " + Thread.currentThread() + " before raising");
         RubyThread currentThread = getRuntime().getCurrentContext().getThread();
@@ -626,7 +641,7 @@ public class RubyThread extends RubyObject {
 
     private IRubyObject prepareRaiseException(Ruby runtime, IRubyObject[] args, Block block) {
         if(args.length == 0) {
-            IRubyObject lastException = runtime.getGlobalVariables().get("$!");
+            IRubyObject lastException = errorInfo;
             if(lastException.isNil()) {
                 return new RaiseException(runtime, runtime.getRuntimeError(), "", false).getException();
             } 
@@ -706,7 +721,7 @@ public class RubyThread extends RubyObject {
     @JRubyMethod(name = "status")
     public IRubyObject status() {
         if (threadImpl.isAlive()) {
-            if (isStopped || (currentSelector != null && currentSelector.isOpen()) || blockingIO != null) {
+            if (isSleeping()) {
             	return getRuntime().newString("sleep");
             } else if (killed) {
                 return getRuntime().newString("aborting");
@@ -720,6 +735,18 @@ public class RubyThread extends RubyObject {
         }
     }
 
+    private boolean isSleeping() {
+        return isStopped || (currentSelector != null && currentSelector.isOpen()) || blockingIO != null || currentWaitObject != null;
+    }
+
+    public void enterSleep() {
+        isStopped = true;
+    }
+
+    public void exitSleep() {
+        isStopped = false;
+    }
+
     @JRubyMethod(name = {"kill", "exit", "terminate"})
     public IRubyObject kill() {
     	// need to reexamine this
@@ -815,6 +842,7 @@ public class RubyThread extends RubyObject {
     }
     
     private volatile Selector currentSelector;
+    private volatile Object currentWaitObject;
     
     @Deprecated
     public boolean selectForAccept(RubyIO io) {
@@ -890,6 +918,12 @@ public class RubyThread extends RubyObject {
         if (iowait != null) {
             iowait.cancel();
         }
+        Object object = currentWaitObject;
+        if (object != null) {
+            synchronized (object) {
+                object.notify();
+            }
+        }
     }
     private volatile BlockingIO.Condition blockingIO = null;
     public boolean waitForIO(ThreadContext context, RubyIO io, int ops) {
@@ -934,4 +968,53 @@ public class RubyThread extends RubyObject {
         }
         kernelModule.callMethod(context, "raise", raiseException);
     }
+
+    public boolean wait_timeout(IRubyObject o, Double timeout) throws InterruptedException {
+        if ( timeout != null ) {
+            long delay_ns = (long)(timeout * 1000000000.0);
+            long start_ns = System.nanoTime();
+            if (delay_ns > 0) {
+                long delay_ms = delay_ns / 1000000;
+                int delay_ns_remainder = (int)( delay_ns % 1000000 );
+                try {
+                    currentWaitObject = o;
+                    o.wait(delay_ms, delay_ns_remainder);
+                } finally {
+                    currentWaitObject = null;
+                }
+            }
+            long end_ns = System.nanoTime();
+            return ( end_ns - start_ns ) <= delay_ns;
+        } else {
+            try {
+                currentWaitObject = o;
+                o.wait();
+            } finally {
+                currentWaitObject = null;
+            }
+            return true;
+        }
+    }
+
+    @Override
+    public boolean equals(Object obj) {
+        if (obj == null) {
+            return false;
+        }
+        if (getClass() != obj.getClass()) {
+            return false;
+        }
+        final RubyThread other = (RubyThread)obj;
+        if (this.threadImpl != other.threadImpl && (this.threadImpl == null || !this.threadImpl.equals(other.threadImpl))) {
+            return false;
+        }
+        return true;
+    }
+
+    @Override
+    public int hashCode() {
+        int hash = 3;
+        hash = 97 * hash + (this.threadImpl != null ? this.threadImpl.hashCode() : 0);
+        return hash;
+    }
 }
