@@ -21,6 +21,7 @@
  * Copyright (C) 2006 Thomas E Enebo <enebo@acm.org>
  * Copyright (C) 2006 Ola Bini <ola.bini@ki.se>
  * Copyright (C) 2006 Miguel Covarrubias <mlcovarrubias@gmail.com>
+ * Copyright (C) 2009 Joseph LaFata <joe@quibb.org>
  * 
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -83,66 +84,75 @@ public class RubyTime extends RubyObject {
             = Pattern.compile("(\\D+?)([\\+-]?)(\\d+)(:\\d+)?(:\\d+)?");
     
     private static final ByteList TZ_STRING = ByteList.create("TZ");
-     
+
     public static DateTimeZone getLocalTimeZone(Ruby runtime) {
         RubyString tzVar = runtime.newString(TZ_STRING);
         RubyHash h = ((RubyHash)runtime.getObject().fastGetConstant("ENV"));
         IRubyObject tz = h.op_aref(runtime.getCurrentContext(), tzVar);
+
         if (tz == null || ! (tz instanceof RubyString)) {
             return DateTimeZone.getDefault();
         } else {
-            String zone = tz.toString();
-            DateTimeZone cachedZone = runtime.getLocalTimezoneCache().get(zone);
+            return getTimeZone(runtime, tz.toString());
+        }
+    }
+     
+    public static DateTimeZone getTimeZone(Ruby runtime, String zone) {
+        DateTimeZone cachedZone = runtime.getTimezoneCache().get(zone);
 
-            if (cachedZone != null) return cachedZone;
+        if (cachedZone != null) return cachedZone;
 
-            String originalZone = zone;
+        String originalZone = zone;
 
-            // Value of "TZ" property is of a bit different format,
-            // which confuses the Java's TimeZone.getTimeZone(id) method,
-            // and so, we need to convert it.
+        // Value of "TZ" property is of a bit different format,
+        // which confuses the Java's TimeZone.getTimeZone(id) method,
+        // and so, we need to convert it.
 
-            Matcher tzMatcher = TZ_PATTERN.matcher(zone);
-            if (tzMatcher.matches()) {                    
-                String sign = tzMatcher.group(2);
-                String hours = tzMatcher.group(3);
-                String minutes = tzMatcher.group(4);
+        Matcher tzMatcher = TZ_PATTERN.matcher(zone);
+        if (tzMatcher.matches()) {                    
+            String sign = tzMatcher.group(2);
+            String hours = tzMatcher.group(3);
+            String minutes = tzMatcher.group(4);
                 
-                // GMT+00:00 --> Etc/GMT, see "MRI behavior"
-                // comment below.
-                if (("00".equals(hours) || "0".equals(hours))
-                        && (minutes == null || ":00".equals(minutes) || ":0".equals(minutes))) {
-                    zone = "Etc/GMT";
-                } else {
-                    // Invert the sign, since TZ format and Java format
-                    // use opposite signs, sigh... Also, Java API requires
-                    // the sign to be always present, be it "+" or "-".
-                    sign = ("-".equals(sign)? "+" : "-");
+            // GMT+00:00 --> Etc/GMT, see "MRI behavior"
+            // comment below.
+            if (("00".equals(hours) || "0".equals(hours))
+                    && (minutes == null || ":00".equals(minutes) || ":0".equals(minutes))) {
+                zone = "Etc/GMT";
+            } else {
+                // Invert the sign, since TZ format and Java format
+                // use opposite signs, sigh... Also, Java API requires
+                // the sign to be always present, be it "+" or "-".
+                sign = ("-".equals(sign)? "+" : "-");
 
-                    // Always use "GMT" since that's required by Java API.
-                    zone = "GMT" + sign + hours;
+                // Always use "GMT" since that's required by Java API.
+                zone = "GMT" + sign + hours;
 
-                    if (minutes != null) {
-                        zone += minutes;
-                    }
+                if (minutes != null) {
+                    zone += minutes;
                 }
             }
-
-            // MRI behavior: With TZ equal to "GMT" or "UTC", Time.now
-            // is *NOT* considered as a proper GMT/UTC time:
-            //   ENV['TZ']="GMT"
-            //   Time.now.gmt? ==> false
-            //   ENV['TZ']="UTC"
-            //   Time.now.utc? ==> false
-            // Hence, we need to adjust for that.
-            if ("GMT".equalsIgnoreCase(zone) || "UTC".equalsIgnoreCase(zone)) {
-                zone = "Etc/" + zone;
-            }
-
-            DateTimeZone dtz = DateTimeZone.forTimeZone(TimeZone.getTimeZone(zone));
-            runtime.getLocalTimezoneCache().put(originalZone, dtz);
-            return dtz;
         }
+
+        // MRI behavior: With TZ equal to "GMT" or "UTC", Time.now
+        // is *NOT* considered as a proper GMT/UTC time:
+        //   ENV['TZ']="GMT"
+        //   Time.now.gmt? ==> false
+        //   ENV['TZ']="UTC"
+        //   Time.now.utc? ==> false
+        // Hence, we need to adjust for that.
+        if ("GMT".equalsIgnoreCase(zone) || "UTC".equalsIgnoreCase(zone)) {
+            zone = "Etc/" + zone;
+        }
+
+        // For JRUBY-2759, when MET choose CET timezone to work around Joda
+        if ("MET".equalsIgnoreCase(zone)) {
+            zone = "CET";
+        }
+
+        DateTimeZone dtz = DateTimeZone.forTimeZone(TimeZone.getTimeZone(zone));
+        runtime.getTimezoneCache().put(originalZone, dtz);
+        return dtz;
     }
     
     public RubyTime(Ruby runtime, RubyClass rubyClass) {
@@ -154,21 +164,12 @@ public class RubyTime extends RubyObject {
         this.dt = dt;
     }
 
-    // We assume that these two time instances
-    // occurred at the same time.
-    private static final long BASE_TIME_MILLIS = System.currentTimeMillis();
-    private static final long BASE_TIME_NANOS = System.nanoTime();
-
     private static ObjectAllocator TIME_ALLOCATOR = new ObjectAllocator() {
         public IRubyObject allocate(Ruby runtime, RubyClass klass) {
-            long usecsPassed = (System.nanoTime() - BASE_TIME_NANOS) / 1000L;
-            long millisTime = BASE_TIME_MILLIS + usecsPassed / 1000L;
-            long usecs = usecsPassed % 1000L;
-
             DateTimeZone dtz = getLocalTimeZone(runtime);
-            DateTime dt = new DateTime(millisTime, dtz);
+            DateTime dt = new DateTime(System.currentTimeMillis(), dtz);
             RubyTime rt =  new RubyTime(runtime, klass, dt);
-            rt.setUSec(usecs);
+            rt.setUSec(0);
 
             return rt;
         }
@@ -335,12 +336,20 @@ public class RubyTime extends RubyObject {
         if (other instanceof RubyTime) {
             throw getRuntime().newTypeError("time + time ?");
         }
-        long adjustment = (long) (RubyNumeric.num2dbl(other) * 1000000);
-        int micro = (int) (adjustment % 1000);
+        
+        long adjustment = Math.round(RubyNumeric.num2dbl(other) * 1000000);
+        long micro = adjustment % 1000;
         adjustment = adjustment / 1000;
 
         time += adjustment;
 
+        if ((getUSec() + micro) >= 1000) {
+            time++;
+            micro = (getUSec() + micro) - 1000;
+        } else {
+            micro = getUSec() + micro;
+        }
+
         RubyTime newTime = new RubyTime(getRuntime(), getMetaClass());
         newTime.dt = new DateTime(time).withZone(dt.getZone());
         newTime.setUSec(micro);
@@ -361,12 +370,19 @@ public class RubyTime extends RubyObject {
         if (other instanceof RubyTime) return opMinus((RubyTime) other);
         
         long time = getTimeInMillis();
-        long adjustment = (long) (RubyNumeric.num2dbl(other) * 1000000);
-        int micro = (int) (adjustment % 1000);
+        long adjustment = Math.round(RubyNumeric.num2dbl(other) * 1000000);
+        long micro = adjustment % 1000;
         adjustment = adjustment / 1000;
 
         time -= adjustment;
 
+        if (getUSec() < micro) {
+            time--;
+            micro = 1000 - (micro - getUSec());
+        } else {
+            micro = getUSec() - micro;
+        }
+
         RubyTime newTime = new RubyTime(getRuntime(), getMetaClass());
         newTime.dt = new DateTime(time).withZone(dt.getZone());
         newTime.setUSec(micro);
@@ -552,10 +568,11 @@ public class RubyTime extends RubyObject {
 
     public RubyObject mdump(final IRubyObject[] args) {
         RubyTime obj = (RubyTime)args[0];
-        DateTime dateTime = obj.dt.withZone(DateTimeZone.UTC);
+        DateTime dateTime = obj.dt;
         byte dumpValue[] = new byte[8];
         int pe = 
             0x1                                 << 31 |
+            ((obj.gmt().isTrue())? 0x1 : 0x0)   << 30 |
             (dateTime.getYear()-1900)           << 14 |
             (dateTime.getMonthOfYear()-1)       << 10 |
             dateTime.getDayOfMonth()            << 5  |
@@ -682,7 +699,7 @@ public class RubyTime extends RubyObject {
     protected static RubyTime s_mload(IRubyObject recv, RubyTime time, IRubyObject from) {
         Ruby runtime = recv.getRuntime();
 
-        DateTime dt = new DateTime(DateTimeZone.UTC);
+        DateTime dt = new DateTime(getLocalTimeZone(runtime));
 
         byte[] fromAsBytes = null;
         fromAsBytes = from.convertToString().getBytes();
@@ -698,9 +715,11 @@ public class RubyTime extends RubyObject {
             s |= ((int)fromAsBytes[i] & 0xFF) << (8 * (i - 4));
         }
         if ((p & (1<<31)) == 0) {
-            dt = dt.withMillis(p * 1000L + s);
+            dt = dt.withMillis(p * 1000L);
+            time.setUSec((s & 0xFFFFF) % 1000);
         } else {
             p &= ~(1<<31);
+            if((p >>> 30 & 0x1) == 0x1) dt = dt.withZone(DateTimeZone.UTC);
             dt = dt.withYear(((p >>> 14) & 0xFFFF) + 1900);
             dt = dt.withMonthOfYear(((p >>> 10) & 0xF) + 1);
             dt = dt.withDayOfMonth(((p >>> 5)  & 0x1F));
@@ -709,7 +728,6 @@ public class RubyTime extends RubyObject {
             dt = dt.withSecondOfMinute(((s >>> 20) & 0x3F));
             // marsaling dumps usec, not msec
             dt = dt.withMillisOfSecond((s & 0xFFFFF) / 1000);
-            dt = dt.withZone(getLocalTimeZone(runtime));
             time.setUSec((s & 0xFFFFF) % 1000);
         }
         time.setDateTime(dt);
@@ -736,8 +754,21 @@ public class RubyTime extends RubyObject {
     private static RubyTime createTime(IRubyObject recv, IRubyObject[] args, boolean gmt) {
         Ruby runtime = recv.getRuntime();
         int len = ARG_SIZE;
+        Boolean isDst = null;
 
+        DateTimeZone dtz;
+        if (gmt) {
+            dtz = DateTimeZone.UTC;
+        } else if (args.length == 10 && args[9] instanceof RubyString) {
+            dtz = getTimeZone(runtime, ((RubyString) args[9]).toString());
+        } else {
+            dtz = getLocalTimeZone(runtime);
+        }
+ 
         if (args.length == 10) {
+	    if(args[8] instanceof RubyBoolean) {
+	        isDst = ((RubyBoolean)args[8]).isTrue();
+	    }
             args = new IRubyObject[] { args[5], args[4], args[3], args[2], args[1], args[0], runtime.getNil() };
         } else {
             // MRI accepts additional wday argument which appears to be ignored.
@@ -809,23 +840,40 @@ public class RubyTime extends RubyObject {
             year += 1900;
         }
 
-        DateTimeZone dtz;
-        if (gmt) {
-            dtz = DateTimeZone.UTC;
-        } else {
-            dtz = getLocalTimeZone(runtime);
-        }
-
         DateTime dt;
         // set up with min values and then add to allow rolling over
         try {
-            dt = new DateTime(year, 1, 1, 0, 0 , 0, 0, dtz);
+            dt = new DateTime(year, 1, 1, 0, 0 , 0, 0, DateTimeZone.UTC);
 
             dt = dt.plusMonths(month - 1)
                     .plusDays(int_args[0] - 1)
                     .plusHours(int_args[1])
                     .plusMinutes(int_args[2])
                     .plusSeconds(int_args[3]);
+
+	    dt = dt.withZoneRetainFields(dtz);
+
+	    // we might need to perform a DST correction
+	    if(isDst != null) {
+                // the instant at which we will ask dtz what the difference between DST and
+                // standard time is
+                long offsetCalculationInstant = dt.getMillis();
+
+                // if we might be moving this time from !DST -> DST, the offset is assumed
+                // to be the same as it was just before we last moved from DST -> !DST
+                if(dtz.isStandardOffset(dt.getMillis()))
+                    offsetCalculationInstant = dtz.previousTransition(offsetCalculationInstant);
+
+                int offset = dtz.getStandardOffset(offsetCalculationInstant) -
+                             dtz.getOffset(offsetCalculationInstant);
+
+                if (!isDst && !dtz.isStandardOffset(dt.getMillis())) {
+                    dt = dt.minusMillis(offset);
+                }
+                if (isDst &&  dtz.isStandardOffset(dt.getMillis())) {
+                    dt = dt.plusMillis(offset);
+                }
+	    }
         } catch (org.joda.time.IllegalFieldValueException e) {
             throw runtime.newArgumentError("time out of range");
         }
