@@ -36,7 +36,6 @@ import java.util.List;
 import org.jruby.Ruby;
 import org.jruby.ast.visitor.NodeVisitor;
 import org.jruby.evaluator.ASTInterpreter;
-import org.jruby.evaluator.Instruction;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.RubyEvent;
@@ -53,27 +52,36 @@ public class CaseNode extends Node {
 	 **/
     private final Node caseNode;
 	/**
-	 * the body of the case.
+	 * A list of all choices including else
 	 */
-    private final Node caseBody;
+    private final ListNode cases;
+    private Node elseNode = null;
     
-    public CaseNode(ISourcePosition position, Node caseNode, Node caseBody) {
-        super(position, NodeType.CASENODE);
+    public CaseNode(ISourcePosition position, Node caseNode, ListNode cases) {
+        super(position);
         
-        assert caseBody != null : "caseBody is not null";
+        assert cases != null : "caseBody is not null";
         // TODO: Rewriter and compiler assume case when empty expression.  In MRI this is just
         // a when.
 //        assert caseNode != null : "caseNode is not null";
         
         this.caseNode = caseNode;
-        this.caseBody = caseBody;
+        this.cases = cases;
+    }
+
+    public void setElseNode(Node elseNode) {
+        this.elseNode = elseNode;
+    }
+
+    public NodeType getNodeType() {
+        return NodeType.CASENODE;
     }
 
  	/**
      * Accept for the visitor pattern.
      * @param iVisitor the visitor
      **/
-    public Instruction accept(NodeVisitor iVisitor) {
+    public Object accept(NodeVisitor iVisitor) {
         return iVisitor.visitCaseNode(this);
     }
 
@@ -85,56 +93,44 @@ public class CaseNode extends Node {
     public Node getCaseNode() {
         return caseNode;
     }
+    
+    public ListNode getCases() {
+        return cases;
+    }
+
+    public Node getElseNode() {
+        return elseNode;
+    }
 
     /**
      * Gets the first whenNode.
 	 * the body of the case statement, the first of a list of WhenNodes
      * @return whenNode
      */
+    @Deprecated
     public Node getFirstWhenNode() {
-        return caseBody;
+        return cases;
     }
     
     public List<Node> childNodes() {
-        return Node.createList(caseNode, caseBody);
+        return Node.createList(caseNode, cases);
     }
 
     @Override
     public IRubyObject interpret(Ruby runtime, ThreadContext context, IRubyObject self, Block aBlock) {
-        IRubyObject value = caseNode == null ? null : caseNode.interpret(runtime, context, self, aBlock);
+        IRubyObject expression = caseNode == null ? null : caseNode.interpret(runtime, context, self, aBlock);
 
         context.pollThreadEvents();
 
-        for (Node current = caseBody; current != null; current = ((WhenNode) current).getNextCase()) {
-            if (!(current instanceof WhenNode)) return current.interpret(runtime, context, self, aBlock);
-            
-            IRubyObject result = interpretExpressions((WhenNode) current, value, context, runtime, self, aBlock);
-            if (result != null) return result;
-
-            context.pollThreadEvents();            
-        }
-
-        return runtime.getNil();        
-    }
-    
-    private static IRubyObject interpretExpressions(WhenNode whenNode, IRubyObject value,
-            ThreadContext context, Ruby runtime, IRubyObject self, Block aBlock) {
-        assert whenIsArray(whenNode);
-
-        for (Node expression: ((ArrayNode) whenNode.getExpressionNodes()).childNodes()) {
-            ISourcePosition position = expression.getPosition();
-            context.setFileAndLine(position.getFile(), position.getStartLine());
-
+        // We know all cases are when nodes...assert?
+        for (Node child : cases.childNodes()) {
+            WhenNode when = (WhenNode) child;
             if (runtime.hasEventHooks()) ASTInterpreter.callTraceFunction(runtime, context, RubyEvent.LINE);
-
-            IRubyObject result = expression.when(whenNode, value, context, runtime, self, aBlock);
+            IRubyObject result = when.when(expression, context, runtime, self, aBlock);
             if (result != null) return result;
+            context.pollThreadEvents();
         }
-        
-        return null;
-    }
-    
-    private static boolean whenIsArray(WhenNode whenNode) {
-        return whenNode.getExpressionNodes() instanceof ArrayNode;
+
+        return elseNode != null ? elseNode.interpret(runtime, context, self, aBlock) : runtime.getNil();
     }
 }
