@@ -37,7 +37,6 @@ import org.jruby.Ruby;
 import org.jruby.RubyModule;
 import org.jruby.ast.types.INameNode;
 import org.jruby.ast.visitor.NodeVisitor;
-import org.jruby.evaluator.Instruction;
 import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.runtime.Block;
@@ -51,16 +50,20 @@ public abstract class Colon2Node extends Colon3Node implements INameNode {
     protected final Node leftNode;
 
     public Colon2Node(ISourcePosition position, Node leftNode, String name) {
-        super(position, NodeType.COLON2NODE, name);
+        super(position, name);
         this.leftNode = leftNode;
     }
+
+    public NodeType getNodeType() {
+        return NodeType.COLON2NODE;
+    }
     
     /**
      * Accept for the visitor pattern.
      * @param iVisitor the visitor
      **/
     @Override
-    public Instruction accept(NodeVisitor iVisitor) {
+    public Object accept(NodeVisitor iVisitor) {
         return iVisitor.visitColon2Node(this);
     }
 
