@@ -37,12 +37,9 @@ import org.jruby.Ruby;
 import org.jruby.ast.types.INameNode;
 import org.jruby.ast.visitor.NodeVisitor;
 import org.jruby.evaluator.ASTInterpreter;
-import org.jruby.evaluator.Instruction;
-import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.CallSite;
-import org.jruby.runtime.InterpretedBlock;
 import org.jruby.runtime.MethodIndex;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -52,7 +49,7 @@ import org.jruby.runtime.builtin.IRubyObject;
  */
 public class FCallNode extends Node implements INameNode, IArgumentNode, BlockAcceptingNode {
     private Node argsNode;
-    private Node iterNode;
+    protected Node iterNode;
     public CallSite callAdapter;
 
     @Deprecated
@@ -61,17 +58,21 @@ public class FCallNode extends Node implements INameNode, IArgumentNode, BlockAc
     }
     
     protected FCallNode(ISourcePosition position, String name, Node argsNode, Node iterNode) {
-        super(position, NodeType.FCALLNODE);
+        super(position);
         setArgsNode(argsNode);
         this.iterNode = iterNode;
         this.callAdapter = MethodIndex.getFunctionalCallSite(name);
     }
+
+    public NodeType getNodeType() {
+        return NodeType.FCALLNODE;
+    }
     
     /**
      * Accept for the visitor pattern.
      * @param iVisitor the visitor
      **/
-    public Instruction accept(NodeVisitor iVisitor) {
+    public Object accept(NodeVisitor iVisitor) {
         return iVisitor.visitFCallNode(this);
     }
     
@@ -136,20 +137,6 @@ public class FCallNode extends Node implements INameNode, IArgumentNode, BlockAc
         return null;
     }
     
-    public Block getBlock(ThreadContext context, IRubyObject self) {
-        IterNode iter = (IterNode) iterNode;
-        
-        iter.getScope().determineModule();
-            
-        // Create block for this iter node
-        // FIXME: We shouldn't use the current scope if it's not actually from the same hierarchy of static scopes
-        return InterpretedBlock.newInterpretedClosure(context, iter.getBlockBody(), self);
-    }
-    
-    public Block getBlock(Ruby runtime, ThreadContext context, IRubyObject self, Block aBlock) {
-        return RuntimeHelpers.getBlockFromBlockPassBody(runtime, iterNode.interpret(runtime, context, self, aBlock), aBlock);
-    }
-    
     @Override
     public String definition(Ruby runtime, ThreadContext context, IRubyObject self, Block aBlock) {
         if (self.getMetaClass().isMethodBound(getName(), false)) {
