@@ -35,7 +35,6 @@ import java.util.List;
 
 import org.jruby.Ruby;
 import org.jruby.ast.visitor.NodeVisitor;
-import org.jruby.evaluator.Instruction;
 import org.jruby.exceptions.JumpException;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.runtime.Block;
@@ -61,12 +60,16 @@ public class ForNode extends IterNode {
         // 'For's are implemented as blocks in evaluation, but they have no scope so we
         // just deal with this lack of scope throughout its lifespan.  We should probably
         // change the way this works to get rid of multiple null checks.
-        super(position, varNode, null, bodyNode, NodeType.FORNODE);
+        super(position, varNode, null, bodyNode);
         
         assert iterNode != null : "iterNode is not null";
         
         this.iterNode = iterNode;
     }
+
+    public NodeType getNodeType() {
+        return NodeType.FORNODE;
+    }
     
     public Node getIterNode() {
         return iterNode;
@@ -77,7 +80,7 @@ public class ForNode extends IterNode {
      * @param iVisitor the visitor
      **/
     @Override
-    public Instruction accept(NodeVisitor iVisitor) {
+    public Object accept(NodeVisitor iVisitor) {
         return iVisitor.visitForNode(this);
     }
     
