@@ -35,7 +35,6 @@ import java.util.List;
 
 import org.jruby.Ruby;
 import org.jruby.ast.visitor.NodeVisitor;
-import org.jruby.evaluator.Instruction;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.parser.StaticScope;
 import org.jruby.runtime.Arity;
@@ -56,14 +55,9 @@ public class IterNode extends Node {
     // What static scoping relationship exists when it comes into being.
     private StaticScope scope;
     private BlockBody blockBody;
-
-    public IterNode(ISourcePosition position, Node varNode, StaticScope scope, Node bodyNode) {
-        this(position, varNode, scope, bodyNode, NodeType.ITERNODE);
-    }
     
-    public IterNode(ISourcePosition position, Node varNode, StaticScope scope, Node bodyNode, 
-            NodeType id) {
-        super(position, id);
+    public IterNode(ISourcePosition position, Node varNode, StaticScope scope, Node bodyNode) {
+        super(position);
         this.varNode = varNode;
         this.scope = scope;
         this.bodyNode = bodyNode;
@@ -72,7 +66,7 @@ public class IterNode extends Node {
     }
 
     public IterNode(ISourcePosition position, ArgsNode args, Node body, StaticScope scope) {
-        super(position, NodeType.ITERNODE);
+        super(position);
 
         this.varNode = args;
         this.bodyNode = body;
@@ -80,11 +74,15 @@ public class IterNode extends Node {
         this.blockBody = new Interpreted19Block(this);
     }
 
+    public NodeType getNodeType() {
+        return NodeType.ITERNODE;
+    }
+
     /**
      * Accept for the visitor pattern.
      * @param iVisitor the visitor
      **/
-    public Instruction accept(NodeVisitor iVisitor) {
+    public Object accept(NodeVisitor iVisitor) {
         return iVisitor.visitIterNode(this);
     }
     
