@@ -44,5 +44,61 @@ public enum NodeType {
     SELFNODE, SPLATNODE, STARNODE, STRNODE, SUPERNODE, SVALUENODE, SYMBOLNODE, TOARYNODE,
     TRUENODE, UNDEFNODE, UNTILNODE, VALIASNODE, VCALLNODE, WHENNODE, WHILENODE, XSTRNODE, YIELDNODE,
     ZARRAYNODE, ZEROARGNODE, ZSUPERNODE, COMMENTNODE, ROOTNODE, ATTRASSIGNNODE, ARGSPUSHNODE,
-    OPTARGNODE, ARGAUXILIARYNODE, LAMBDANODE, MULTIPLEASGN19NODE, RESTARG
+    OPTARGNODE, ARGAUXILIARYNODE, LAMBDANODE, MULTIPLEASGN19NODE, RESTARG;
+
+
+    /**
+     * Whether this node type would always evaluate as being true.
+     *
+     * @return whether this node type would always evaluate as being true
+     */
+    public boolean alwaysTrue() {
+        switch (this) {
+        case TRUENODE:
+        case FIXNUMNODE:
+        case SELFNODE:
+        case FLOATNODE:
+        case REGEXPNODE:
+        case STRNODE:
+        case DOTNODE:
+        case SYMBOLNODE:
+        case BIGNUMNODE:
+        case ARRAYNODE:
+        case HASHNODE:
+            return true;
+        default:
+            return false;
+        }
+    }
+
+    /**
+     * Whether this node type would always evaluate as being false.
+     * 
+     * @return whether this node type would always evaluate as being false
+     */
+    public boolean alwaysFalse() {
+        switch (this) {
+        case NILNODE:
+        case FALSENODE:
+            return true;
+        default:
+            return false;
+        }
+    }
+
+    public boolean isImmediate() {
+        switch (this) {
+        case FIXNUMNODE:
+        case STRNODE:
+        case FLOATNODE:
+        case BIGNUMNODE:
+        case SELFNODE:
+        case TRUENODE:
+        case FALSENODE:
+        case NILNODE:
+            return true;
+        default:
+            return false;
+        }
+    }
 }
