@@ -38,7 +38,6 @@ import org.jruby.Ruby;
 import org.jruby.RubyString;
 import org.jruby.ast.types.ILiteralNode;
 import org.jruby.ast.visitor.NodeVisitor;
-import org.jruby.evaluator.Instruction;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ThreadContext;
@@ -52,22 +51,26 @@ public class StrNode extends Node implements ILiteralNode {
     private final ByteList value;
 
     public StrNode(ISourcePosition position, ByteList value) {
-        super(position, NodeType.STRNODE);
+        super(position);
         this.value = value;
     }
 
     public StrNode(ISourcePosition position, StrNode head, StrNode tail) {
-        super(position, NodeType.STRNODE);
+        super(position);
         
         this.value = (ByteList) head.getValue();
         
         value.append(tail.getValue());
     }
+
+    public NodeType getNodeType() {
+        return NodeType.STRNODE;
+    }
     /**
      * Accept for the visitor pattern.
      * @param iVisitor the visitor
      **/
-    public Instruction accept(NodeVisitor iVisitor) {
+    public Object accept(NodeVisitor iVisitor) {
         return iVisitor.visitStrNode(this);
     }
 
