@@ -27,7 +27,9 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby.common;
 
+import org.joni.WarnCallback;
 import org.jruby.Ruby;
+import org.jruby.common.IRubyWarnings.ID;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -35,13 +37,17 @@ import org.jruby.runtime.builtin.IRubyObject;
 /** 
  *
  */
-public class RubyWarnings implements IRubyWarnings {
+public class RubyWarnings implements IRubyWarnings, WarnCallback {
     private final Ruby runtime;
 
     public RubyWarnings(Ruby runtime) {
         this.runtime = runtime;
     }
 
+    public void warn(String message) {
+        warn(ID.MISCELLANEOUS, message);
+    }
+
     /**
      * Prints a warning, unless $VERBOSE is nil.
      */
@@ -53,8 +59,8 @@ public class RubyWarnings implements IRubyWarnings {
      * Prints a warning, unless $VERBOSE is nil.
      */
     public void warn(ID id, String fileName, int lineNumber, String message, Object... data) {
-        if (runtime.getVerbose().isNil()) return;
-    	
+        if (!runtime.warningsEnabled()) return; // TODO make an assert here
+
         StringBuilder buffer = new StringBuilder(100);
 
         buffer.append(fileName).append(':').append(lineNumber + 1).append(' ');
@@ -64,18 +70,20 @@ public class RubyWarnings implements IRubyWarnings {
     }
 
     public boolean isVerbose() {
-        return runtime.getVerbose().isTrue();
+        return runtime.isVerbose();
     }
 
     public void warn(ID id, String message, Object... data) {
         ThreadContext context = runtime.getCurrentContext();
-        
         warn(id, context.getFile(), context.getLine(), message, data);
     }
 
+    /**
+     * Verbose mode warning methods, their contract is that consumer must explicitly check for runtime.isVerbose()
+     * before calling them
+     */
     public void warning(ID id, String message, Object... data) {
         ThreadContext context = runtime.getCurrentContext();
-
         warning(id, context.getFile(), context.getLine(), message, data);
     }
     
@@ -90,6 +98,7 @@ public class RubyWarnings implements IRubyWarnings {
      * Prints a warning, only in verbose mode.
      */
     public void warning(ID id, String fileName, int lineNumber, String message, Object... data) {
-        if (isVerbose()) warn(id, fileName, lineNumber, message, data);
+        assert isVerbose(); 
+        warn(id, fileName, lineNumber, message, data);
     }
 }
