@@ -32,6 +32,8 @@ package org.jruby.compiler;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.Set;
+import org.jruby.RubyInstanceConfig;
+import org.jruby.RubyModule;
 import org.jruby.ast.AndNode;
 import org.jruby.ast.ArgsCatNode;
 import org.jruby.ast.ArgsNode;
@@ -46,8 +48,10 @@ import org.jruby.ast.BlockPassNode;
 import org.jruby.ast.BreakNode;
 import org.jruby.ast.CallNode;
 import org.jruby.ast.CaseNode;
+import org.jruby.ast.ClassNode;
 import org.jruby.ast.Colon2Node;
 import org.jruby.ast.ConstNode;
+import org.jruby.ast.DefinedNode;
 import org.jruby.ast.DotNode;
 import org.jruby.ast.EvStrNode;
 import org.jruby.ast.FlipNode;
@@ -63,6 +67,8 @@ import org.jruby.ast.LocalAsgnNode;
 import org.jruby.ast.Match2Node;
 import org.jruby.ast.Match3Node;
 import org.jruby.ast.MatchNode;
+import org.jruby.ast.ModuleNode;
+import org.jruby.ast.MultipleAsgn19Node;
 import org.jruby.ast.MultipleAsgnNode;
 import org.jruby.ast.NewlineNode;
 import org.jruby.ast.NextNode;
@@ -70,12 +76,17 @@ import org.jruby.ast.Node;
 import org.jruby.ast.NotNode;
 import org.jruby.ast.OpAsgnAndNode;
 import org.jruby.ast.OpAsgnNode;
+import org.jruby.ast.OpAsgnOrNode;
 import org.jruby.ast.OpElementAsgnNode;
+import org.jruby.ast.OptArgNode;
 import org.jruby.ast.OrNode;
 import org.jruby.ast.PostExeNode;
 import org.jruby.ast.PreExeNode;
+import org.jruby.ast.RescueBodyNode;
+import org.jruby.ast.RescueNode;
 import org.jruby.ast.ReturnNode;
 import org.jruby.ast.RootNode;
+import org.jruby.ast.SClassNode;
 import org.jruby.ast.SValueNode;
 import org.jruby.ast.SplatNode;
 import org.jruby.ast.SuperNode;
@@ -87,6 +98,7 @@ import org.jruby.ast.WhileNode;
 import org.jruby.ast.YieldNode;
 import org.jruby.ast.ZSuperNode;
 import org.jruby.ast.types.INameNode;
+import org.jruby.internal.runtime.methods.CallConfiguration;
 import org.jruby.util.SafePropertyAccessor;
 
 /**
@@ -113,6 +125,8 @@ public class ASTInspector {
     public static final int ZSUPER = 0x10000; // makes a zero-argument super call
     public static final int CONSTANT = 0x20000; // accesses or sets constants
     public static final int CLASS_VAR = 0x40000; // accesses or sets class variables
+    public static final int SUPER = 0x80000; // makes normal super call
+    public static final int RETRY = 0x100000; // contains a retry
     
     private int flags;
     
@@ -137,12 +151,7 @@ public class ASTInspector {
         FRAME_AWARE_METHODS.add("block_given?");
         FRAME_AWARE_METHODS.add("iterator?");
         
-        SCOPE_AWARE_METHODS.add("eval");
-        SCOPE_AWARE_METHODS.add("module_eval");
-        SCOPE_AWARE_METHODS.add("class_eval");
-        SCOPE_AWARE_METHODS.add("instance_eval");
-        SCOPE_AWARE_METHODS.add("binding");
-        SCOPE_AWARE_METHODS.add("local_variables");
+        SCOPE_AWARE_METHODS.addAll(RubyModule.SCOPE_CAPTURING_METHODS);
         
         PRAGMAS.add("__NOFRAME__");
     }
@@ -150,6 +159,47 @@ public class ASTInspector {
     public void disable() {
         flags = 0xFFFFFFFF;
     }
+
+    public CallConfiguration getCallConfig() {
+        if (hasFrameAwareMethods() || hasClosure() || !(noFrame() || RubyInstanceConfig.FRAMELESS_COMPILE_ENABLED)) {
+            // We're doing normal framed compilation or the method needs a frame
+            if (hasClosure() || hasScopeAwareMethods()) {
+                // The method also needs a scope, do both
+                return CallConfiguration.FrameFullScopeFull;
+            } else {
+                if (hasConstant() || hasMethod() || hasClass() || hasClassVar()) {
+                    // The method doesn't need a scope, but has static scope needs; use a dummy scope
+                    return CallConfiguration.FrameFullScopeDummy;
+                } else {
+                    // The method doesn't need a scope or static scope; frame only
+                    return CallConfiguration.FrameFullScopeNone;
+                }
+            }
+        } else {
+            if (hasClosure() || hasScopeAwareMethods()) {
+                // TODO: call config with scope but no frame
+                if (RubyInstanceConfig.FASTEST_COMPILE_ENABLED) {
+                    return CallConfiguration.FrameNoneScopeFull;
+                } else {
+                    return CallConfiguration.FrameBacktraceScopeFull;
+                }
+            } else {
+                if (hasConstant() || hasMethod() || hasClass() || hasClassVar()) {
+                    if (RubyInstanceConfig.FASTEST_COMPILE_ENABLED || noFrame()) {
+                        return CallConfiguration.FrameNoneScopeDummy;
+                    } else {
+                        return CallConfiguration.FrameBacktraceScopeDummy;
+                    }
+                } else {
+                    if (RubyInstanceConfig.FASTEST_COMPILE_ENABLED || noFrame()) {
+                        return CallConfiguration.FrameNoneScopeNone;
+                    } else {
+                        return CallConfiguration.FrameBacktraceScopeNone;
+                    }
+                }
+            }
+        }
+    }
     
     public static final boolean ENABLED = SafePropertyAccessor.getProperty("jruby.astInspector.enabled", "true").equals("true");
     
@@ -194,7 +244,7 @@ public class ASTInspector {
 
         if (node == null) return;
         
-        switch (node.nodeId) {
+        switch (node.getNodeType()) {
         case ALIASNODE:
             setFlag(METHOD);
             break;
@@ -293,10 +343,14 @@ public class ASTInspector {
         case CASENODE:
             CaseNode caseNode = (CaseNode)node;
             inspect(caseNode.getCaseNode());
-            inspect(caseNode.getFirstWhenNode());
+            for (Node when : caseNode.getCases().childNodes()) inspect(when);
+            inspect(caseNode.getElseNode());
             break;
         case CLASSNODE:
             setFlag(CLASS);
+            ClassNode classNode = (ClassNode)node;
+            inspect(classNode.getCPath());
+            inspect(classNode.getSuperNode());
             break;
         case CLASSVARNODE:
             setFlag(CLASS_VAR);
@@ -327,7 +381,36 @@ public class ASTInspector {
             setFlag(FRAME_VISIBILITY);
             break;
         case DEFINEDNODE:
-            disable();
+            switch (((DefinedNode)node).getExpressionNode().getNodeType()) {
+            case CLASSVARASGNNODE:
+            case CLASSVARDECLNODE:
+            case CONSTDECLNODE:
+            case DASGNNODE:
+            case GLOBALASGNNODE:
+            case LOCALASGNNODE:
+            case MULTIPLEASGNNODE:
+            case OPASGNNODE:
+            case OPELEMENTASGNNODE:
+            case DVARNODE:
+            case FALSENODE:
+            case TRUENODE:
+            case LOCALVARNODE:
+            case INSTVARNODE:
+            case BACKREFNODE:
+            case SELFNODE:
+            case VCALLNODE:
+            case YIELDNODE:
+            case GLOBALVARNODE:
+            case CONSTNODE:
+            case FCALLNODE:
+            case CLASSVARNODE:
+                // ok, we have fast paths
+                inspect(((DefinedNode)node).getExpressionNode());
+                break;
+            default:
+                // long, slow way causes disabling
+                disable();
+            }
             break;
         case DOTNODE:
             DotNode dotNode = (DotNode)node;
@@ -369,6 +452,7 @@ public class ASTInspector {
             } else if (globalAsgnNode.getName().equals("$~")) {
                 setFlag(BACKREF);
             }
+            inspect(globalAsgnNode.getValueNode());
             break;
         case GLOBALVARNODE:
             if (((GlobalVarNode)node).getName().equals("$_")) {
@@ -399,6 +483,9 @@ public class ASTInspector {
         case ITERNODE:
             setFlag(CLOSURE);
             break;
+        case LAMBDANODE:
+            setFlag(CLOSURE);
+            break;
         case LOCALASGNNODE:
             LocalAsgnNode localAsgnNode = (LocalAsgnNode)node;
             if (PRAGMAS.contains(localAsgnNode.getName())) {
@@ -429,6 +516,14 @@ public class ASTInspector {
             break;
         case MODULENODE:
             setFlag(CLASS);
+            inspect(((ModuleNode)node).getCPath());
+            break;
+        case MULTIPLEASGN19NODE:
+            MultipleAsgn19Node multipleAsgn19Node = (MultipleAsgn19Node)node;
+            inspect(multipleAsgn19Node.getPre());
+            inspect(multipleAsgn19Node.getPost());
+            inspect(multipleAsgn19Node.getRest());
+            inspect(multipleAsgn19Node.getValueNode());
             break;
         case MULTIPLEASGNNODE:
             MultipleAsgnNode multipleAsgnNode = (MultipleAsgnNode)node;
@@ -460,7 +555,36 @@ public class ASTInspector {
             inspect(opAsgnNode.getValueNode());
             break;
         case OPASGNORNODE:
-            disable(); // Depends on defined
+            switch (((OpAsgnOrNode)node).getFirstNode().getNodeType()) {
+            case CLASSVARASGNNODE:
+            case CLASSVARDECLNODE:
+            case CONSTDECLNODE:
+            case DASGNNODE:
+            case GLOBALASGNNODE:
+            case LOCALASGNNODE:
+            case MULTIPLEASGNNODE:
+            case OPASGNNODE:
+            case OPELEMENTASGNNODE:
+            case DVARNODE:
+            case FALSENODE:
+            case TRUENODE:
+            case LOCALVARNODE:
+            case INSTVARNODE:
+            case BACKREFNODE:
+            case SELFNODE:
+            case VCALLNODE:
+            case YIELDNODE:
+            case GLOBALVARNODE:
+            case CONSTNODE:
+            case FCALLNODE:
+            case CLASSVARNODE:
+                // ok, we have fast paths
+                inspect(((OpAsgnOrNode)node).getSecondNode());
+                break;
+            default:
+                // long, slow way causes disabling for defined
+                disable();
+            }
             break;
         case OPELEMENTASGNNODE:
             OpElementAsgnNode opElementAsgnNode = (OpElementAsgnNode)node;
@@ -469,6 +593,9 @@ public class ASTInspector {
             inspect(opElementAsgnNode.getReceiverNode());
             inspect(opElementAsgnNode.getValueNode());
             break;
+        case OPTARGNODE:
+            inspect(((OptArgNode)node).getValue());
+            break;
         case ORNODE:
             OrNode orNode = (OrNode)node;
             inspect(orNode.getFirstNode());
@@ -504,19 +631,28 @@ public class ASTInspector {
             }
             break;
         case RESCUEBODYNODE:
-            disable();
+            RescueBodyNode rescueBody = (RescueBodyNode)node;
+            inspect(rescueBody.getExceptionNodes());
+            inspect(rescueBody.getBodyNode());
+            inspect(rescueBody.getOptRescueNode());
             break;
         case RESCUENODE:
+            RescueNode rescueNode = (RescueNode)node;
+            inspect(rescueNode.getBodyNode());
+            inspect(rescueNode.getElseNode());
+            inspect(rescueNode.getRescueNode());
             disable();
             break;
         case RETRYNODE:
-            disable();
+            setFlag(RETRY);
             break;
         case RETURNNODE:
             inspect(((ReturnNode)node).getValueNode());
             break;
         case SCLASSNODE:
             setFlag(CLASS);
+            SClassNode sclassNode = (SClassNode)node;
+            inspect(sclassNode.getReceiverNode());
             break;
         case SCOPENODE:
             break;
@@ -533,6 +669,7 @@ public class ASTInspector {
             SuperNode superNode = (SuperNode)node;
             inspect(superNode.getArgsNode());
             inspect(superNode.getIterNode());
+            setFlag(SUPER);
             break;
         case SVALUENODE:
             inspect(((SValueNode)node).getValue());
@@ -628,7 +765,7 @@ public class ASTInspector {
     public boolean hasFrameAwareMethods() {
         return getFlag(
                 FRAME_AWARE | FRAME_BLOCK | FRAME_CLASS | FRAME_NAME | FRAME_SELF | FRAME_VISIBILITY |
-                CLOSURE | EVAL | BACKREF | LASTLINE | ZSUPER);
+                CLOSURE | EVAL | BACKREF | LASTLINE | ZSUPER | SUPER);
     }
 
     public boolean hasScopeAwareMethods() {
