@@ -19,19 +19,29 @@ import org.jruby.util.ByteList;
 public interface CacheCompiler {
     public void cacheCallSite(BaseBodyCompiler method, String name, CallType callType);
     
-    public void cacheByteList(BaseBodyCompiler method, ByteList contents);
+    public void cacheString(BaseBodyCompiler method, ByteList contents);
     
     public void cacheSymbol(BaseBodyCompiler method, String symbol);
     
     public void cacheFixnum(BaseBodyCompiler method, long value);
     
     public void cacheBigInteger(BaseBodyCompiler method, BigInteger bigint);
-    
+
+    public void cacheRegexp(BaseBodyCompiler method, String pattern, int options);
+
+    public void cacheDRegexp(BaseBodyCompiler method, CompilerCallback createStringCallback, int options);
+
     public void cacheClosure(BaseBodyCompiler method, String closureMethod, int arity, StaticScope scope, boolean hasMultipleArgsHead, NodeType argsNodeId, ASTInspector inspector);
+
+    public void cacheClosure19(BaseBodyCompiler method, String closureMethod, int arity, StaticScope scope, boolean hasMultipleArgsHead, NodeType argsNodeId, ASTInspector inspector);
     
-    public void cacheClosureOld(BaseBodyCompiler method, String closureMethod);
+    public void cacheSpecialClosure(BaseBodyCompiler method, String closureMethod);
 
     public void cacheConstant(BaseBodyCompiler method, String constantName);
 
+    public void cacheConstantFrom(BaseBodyCompiler method, String constantName);
+
+    public void cacheStaticScope(BaseBodyCompiler method, StaticScope scope);
+
     public void finish();
 }
