@@ -28,7 +28,6 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby.compiler;
 
-import java.lang.ref.SoftReference;
 
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicLong;
@@ -51,8 +50,6 @@ import org.jruby.util.JavaNameMangler;
 public class JITCompiler implements JITCompilerMBean {
     public static final boolean USE_CACHE = true;
     
-    private SoftReference<Ruby> ruby;
-    
     private AtomicLong compiledCount = new AtomicLong(0);
     private AtomicLong successCount = new AtomicLong(0);
     private AtomicLong failCount = new AtomicLong(0);
@@ -64,8 +61,6 @@ public class JITCompiler implements JITCompilerMBean {
     private AtomicLong largestCodeSize = new AtomicLong(0);
     
     public JITCompiler(Ruby ruby) {
-        this.ruby = new SoftReference<Ruby>(ruby);
-        
         ruby.getBeanManager().register(this);
     }
 
@@ -88,7 +83,7 @@ public class JITCompiler implements JITCompilerMBean {
     }
 
     private DynamicMethod jitIsEnabled(final DefaultMethod method, final ThreadContext context, final String name) {
-        RubyInstanceConfig instanceConfig = ruby.get().getInstanceConfig();
+        RubyInstanceConfig instanceConfig = context.getRuntime().getInstanceConfig();
         
         if (method.incrementCallCount() >= instanceConfig.getJitThreshold()) {
             return jitThresholdReached(method, instanceConfig, context, name);
@@ -117,7 +112,7 @@ public class JITCompiler implements JITCompilerMBean {
                 return null;
             }
 
-            JITClassGenerator generator = new JITClassGenerator(name, method, context);
+            JITClassGenerator generator = new JITClassGenerator(name, context.getRuntime(), method, context);
 
             String key = SexpMaker.create(name, method.getArgsNode(), method.getBodyNode());
 
@@ -137,7 +132,7 @@ public class JITCompiler implements JITCompilerMBean {
             Script jitCompiledScript = sourceClass.newInstance();
 
             // add to the jitted methods set
-            Set<Script> jittedMethods = ruby.get().getJittedMethods();
+            Set<Script> jittedMethods = context.getRuntime().getJittedMethods();
             jittedMethods.add(jitCompiledScript);
 
             // logEvery n methods based on configuration
@@ -150,9 +145,7 @@ public class JITCompiler implements JITCompilerMBean {
 
             if (instanceConfig.isJitLogging()) log(method, name, "done jitting");
 
-            method.setJITCallConfig(generator.callConfig());
-            method.setJITCompiledScript(jitCompiledScript);
-            method.setCallCount(-1);
+            method.switchToJitted(jitCompiledScript, generator.callConfig());
             return null;
         } catch (Throwable t) {
             t.printStackTrace();
@@ -174,8 +167,9 @@ public class JITCompiler implements JITCompilerMBean {
         
         private byte[] bytecode;
         private String name;
+        private Ruby ruby;
         
-        public JITClassGenerator(String name, DefaultMethod method, ThreadContext context) {
+        public JITClassGenerator(String name, Ruby ruby, DefaultMethod method, ThreadContext context) {
             this.method = method;
             String packageName = "ruby/jit/" + JavaNameMangler.mangleFilenameForClasspath(method.getPosition().getFile());
             String cleanName = packageName + "/" + JavaNameMangler.mangleStringForCleanJavaIdentifier(name);
@@ -185,6 +179,7 @@ public class JITCompiler implements JITCompilerMBean {
             staticScope = method.getStaticScope();
             asmCompiler = new StandardASMCompiler(cleanName + 
                     method.hashCode() + "_" + context.hashCode(), filename);
+            this.ruby = ruby;
         }
         
         @SuppressWarnings("unchecked")
@@ -195,11 +190,11 @@ public class JITCompiler implements JITCompilerMBean {
             long start = System.nanoTime();
             
             asmCompiler.startScript(staticScope);
-            final ASTCompiler compiler = new ASTCompiler();
+            final ASTCompiler compiler = ruby.getInstanceConfig().newCompiler();
 
             CompilerCallback args = new CompilerCallback() {
                 public void call(BodyCompiler context) {
-                    compiler.compileArgs(argsNode, context);
+                    compiler.compileArgs(argsNode, context, true);
                 }
             };
 
@@ -212,7 +207,7 @@ public class JITCompiler implements JITCompilerMBean {
             if (bodyNode != null) {
                 // we have a body, do a full-on method
                 methodCompiler = asmCompiler.startMethod("__file__", "__file__", args, staticScope, inspector);
-                compiler.compile(bodyNode, methodCompiler);
+                compiler.compile(bodyNode, methodCompiler,true);
             } else {
                 // If we don't have a body, check for required or opt args
                 // if opt args, they could have side effects
@@ -224,7 +219,7 @@ public class JITCompiler implements JITCompilerMBean {
                 } else {
                     methodCompiler = asmCompiler.startMethod("__file__", "__file__", null, staticScope, inspector);
                     methodCompiler.loadNil();
-                    jitCallConfig = CallConfiguration.NO_FRAME_NO_SCOPE;
+                    jitCallConfig = CallConfiguration.FrameNoneScopeNone;
                 }
             }
             methodCompiler.endBody();
@@ -235,21 +230,21 @@ public class JITCompiler implements JITCompilerMBean {
                 // if we're not doing any of the operations that still need
                 // a scope, use the scopeless config
                 if (inspector.hasClosure() || inspector.hasScopeAwareMethods()) {
-                    jitCallConfig = CallConfiguration.FRAME_AND_SCOPE;
+                    jitCallConfig = CallConfiguration.FrameFullScopeFull;
                 } else {
                     // switch to a slightly faster call config
-                    jitCallConfig = CallConfiguration.FRAME_AND_DUMMY_SCOPE;
+                    jitCallConfig = CallConfiguration.FrameFullScopeDummy;
                 }
             }
             
             bytecode = asmCompiler.getClassByteArray();
             name = CodegenUtils.c(asmCompiler.getClassname());
             
-            if (bytecode.length > ruby.get().getInstanceConfig().getJitMaxSize()) {
+            if (bytecode.length > ruby.getInstanceConfig().getJitMaxSize()) {
                 bytecode = null;
                 throw new NotCompilableException(
                         "JITed method size exceeds configured max of " +
-                        ruby.get().getInstanceConfig().getJitMaxSize());
+                        ruby.getInstanceConfig().getJitMaxSize());
             }
             
             compiledCount.incrementAndGet();
