@@ -11,6 +11,7 @@ package org.jruby.compiler.impl;
 
 import java.io.PrintStream;
 import java.io.PrintWriter;
+import org.jruby.util.SafePropertyAccessor;
 import static org.jruby.util.CodegenUtils.*;
 import org.objectweb.asm.AnnotationVisitor;
 import org.objectweb.asm.Attribute;
@@ -24,7 +25,7 @@ import org.objectweb.asm.util.TraceMethodVisitor;
  * @author headius
  */
 public class SkinnyMethodAdapter implements MethodVisitor, Opcodes {
-    private final static boolean DEBUG = false;
+    private final static boolean DEBUG = SafePropertyAccessor.getBoolean("jruby.compile.dump");
     private MethodVisitor method;
     
     /** Creates a new instance of SkinnyMethodAdapter */
@@ -448,10 +449,18 @@ public class SkinnyMethodAdapter implements MethodVisitor, Opcodes {
     public void ifeq(Label arg0) {
         getMethodVisitor().visitJumpInsn(IFEQ, arg0);
     }
+
+    public void iffalse(Label arg0) {
+        ifeq(arg0);
+    }
     
     public void ifne(Label arg0) {
         getMethodVisitor().visitJumpInsn(IFNE, arg0);
     }
+
+    public void iftrue(Label arg0) {
+        ifne(arg0);
+    }
     
     public void if_acmpne(Label arg0) {
         getMethodVisitor().visitJumpInsn(IF_ACMPNE, arg0);
@@ -499,6 +508,16 @@ public class SkinnyMethodAdapter implements MethodVisitor, Opcodes {
         getMethodVisitor().visitMaxs(1, 1);
         getMethodVisitor().visitEnd();
     }
+
+    public void line(int line) {
+        Label label = new Label();
+        label(label);
+        visitLineNumber(line, label);
+    }
+
+    public void line(int line, Label label) {
+        visitLineNumber(line, label);
+    }
     
     public void ifnonnull(Label arg0) {
         getMethodVisitor().visitJumpInsn(IFNONNULL, arg0);
