@@ -79,6 +79,7 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
         sig(IRubyObject.class, new Class[]{ThreadContext.class, IRubyObject.class, IRubyObject[].class, Block.class}),
     };
     public static final String CLOSURE_SIGNATURE = sig(IRubyObject.class, new Class[]{ThreadContext.class, IRubyObject.class, IRubyObject.class});
+    public static final String CLOSURE_SIGNATURE19 = sig(IRubyObject.class, new Class[]{ThreadContext.class, IRubyObject.class, IRubyObject[].class, Block.class});
 
     public static final int THIS = 0;
     public static final int THREADCONTEXT_INDEX = 1;
@@ -93,6 +94,8 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
     public static final int EXCEPTION_OFFSET = 5;
     public static final int PREVIOUS_EXCEPTION_OFFSET = 6;
     public static final int FIRST_TEMP_OFFSET = 7;
+
+    public static final int STARTING_DSTR_SIZE = 20;
     
     private String classname;
     private String sourcename;
@@ -150,11 +153,10 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
     
     public void dumpClass(PrintStream out) {
         PrintWriter pw = new PrintWriter(out);
-        TraceClassVisitor tcv = new TraceClassVisitor(pw);
-        new ClassReader(classWriter.toByteArray()).accept(tcv, 0);
 
         try {
-            tcv.print(pw);
+            TraceClassVisitor tcv = new TraceClassVisitor(pw);
+            new ClassReader(classWriter.toByteArray()).accept(tcv, 0);
         } finally {
             pw.close();
         }
@@ -207,11 +209,21 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
 
         // Create the class with the appropriate class name and source file
         classWriter.visit(RubyInstanceConfig.JAVA_VERSION, ACC_PUBLIC + ACC_SUPER,getClassname(), null, p(AbstractScript.class), null);
+
+        // add setPosition impl, which stores filename as constant to speed updates
+        SkinnyMethodAdapter method = new SkinnyMethodAdapter(getClassVisitor().visitMethod(ACC_PRIVATE | ACC_STATIC | ACC_SYNTHETIC, "setPosition", sig(Void.TYPE, params(ThreadContext.class, int.class)), null, null));
+        method.start();
+
+        method.aload(0); // thread context
+        method.ldc(sourcename);
+        method.iload(1); // line number
+        method.invokevirtual(p(ThreadContext.class), "setFileAndLine", sig(void.class, String.class, int.class));
+        method.voidreturn();
+        method.end();
         
         topLevelScope = scope;
 
         beginInit();
-        beginClassInit();
         
         cacheCompiler = new InheritedCacheCompiler(this);
 
@@ -225,21 +237,9 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
         } else {
             sourceNoPath = sourcename;
         }
-        
-        StringBuffer smap = new StringBuffer();
-        smap.append("SMAP\n")
-                .append(sourceNoPath).append("\n")
-                .append("Ruby\n")
-                .append("*S Ruby\n")
-                .append("*F\n")
-                .append("+ 1 ").append(sourceNoPath).append("\n")
-                .append(sourcename).append("\n")
-                .append("*L\n")
-                .append("1#1,999999:1,1\n")
-                .append("*E\n");
 
         
-        classWriter.visitSource(sourceNoPath, smap.toString());
+        classWriter.visitSource(sourceNoPath, null);
     }
 
     public void endScript(boolean generateLoad, boolean generateMain) {
@@ -318,17 +318,6 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
             method.voidreturn();
             method.end();
         }
-        
-        // add setPosition impl, which stores filename as constant to speed updates
-        SkinnyMethodAdapter method = new SkinnyMethodAdapter(getClassVisitor().visitMethod(ACC_PRIVATE | ACC_STATIC | ACC_SYNTHETIC, "setPosition", sig(Void.TYPE, params(ThreadContext.class, int.class)), null, null));
-        method.start();
-
-        method.aload(0); // thread context
-        method.ldc(sourcename);
-        method.iload(1); // line number
-        method.invokevirtual(p(ThreadContext.class), "setFileAndLine", sig(void.class, String.class, int.class));
-        method.voidreturn();
-        method.end();
 
         getCacheCompiler().finish();
         
@@ -415,8 +404,10 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
     }
 
     private void endClassInit() {
-        clinitMethod.voidreturn();
-        clinitMethod.end();
+        if (clinitMethod != null) {
+            clinitMethod.voidreturn();
+            clinitMethod.end();
+        }
     }
     
     public SkinnyMethodAdapter getInitMethod() {
@@ -424,6 +415,10 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
     }
     
     public SkinnyMethodAdapter getClassInitMethod() {
+        // lazily create class init only if necessary
+        if (clinitMethod == null) {
+            beginClassInit();
+        }
         return clinitMethod;
     }
     
@@ -436,9 +431,14 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
         
         methodCompiler.beginMethod(args, scope);
         
-        // Emite a nop, to mark the end of the method preamble
-        methodCompiler.method.nop();
-        
+        return methodCompiler;
+    }
+
+    public BodyCompiler startRoot(String rubyName, String javaName, StaticScope scope, ASTInspector inspector) {
+        RootScopedBodyCompiler methodCompiler = new MethodBodyCompiler(this, rubyName, javaName, inspector, scope);
+
+        methodCompiler.beginMethod(null, scope);
+
         return methodCompiler;
     }
 
@@ -480,13 +480,14 @@ public class StandardASMCompiler implements ScriptCompiler, Opcodes {
         return getNewConstant(type, name_prefix, null);
     }
 
+    public synchronized String getNewConstantName() {
+        return "_" + constants++;
+    }
+
     public String getNewConstant(String type, String name_prefix, Object init) {
         ClassVisitor cv = getClassVisitor();
 
-        String realName;
-        synchronized (this) {
-            realName = "_" + constants++;
-        }
+        String realName = getNewConstantName();
 
         // declare the field
         cv.visitField(ACC_PRIVATE, realName, type, null, null).visitEnd();
