@@ -27,6 +27,7 @@
 
 package org.jruby.compiler.impl;
 
+import org.jruby.RubyArray;
 import org.jruby.compiler.ArgumentsCallback;
 import org.jruby.compiler.CompilerCallback;
 import org.jruby.compiler.InvocationCompiler;
@@ -44,8 +45,8 @@ import org.objectweb.asm.Label;
  * @author headius
  */
 public class StandardInvocationCompiler implements InvocationCompiler {
-    private BaseBodyCompiler methodCompiler;
-    private SkinnyMethodAdapter method;
+    protected BaseBodyCompiler methodCompiler;
+    protected SkinnyMethodAdapter method;
 
     public StandardInvocationCompiler(BaseBodyCompiler methodCompiler, SkinnyMethodAdapter method) {
         this.methodCompiler = methodCompiler;
@@ -136,22 +137,15 @@ public class StandardInvocationCompiler implements InvocationCompiler {
     }
 
     public void invokeAttrAssign(String name, CompilerCallback receiverCallback, ArgumentsCallback argsCallback) {
-        Label variableCallType = new Label();
-        Label readyForCall = new Label();
-        
         // receiver first, so we know which call site to use
         receiverCallback.call(methodCompiler);
         
         // select appropriate call site
         method.dup(); // dup receiver
         methodCompiler.loadSelf(); // load self
-        method.if_acmpeq(variableCallType); // compare
-        
         methodCompiler.getScriptCompiler().getCacheCompiler().cacheCallSite(methodCompiler, name, CallType.NORMAL);
-        method.go_to(readyForCall);
-        method.label(variableCallType);
         methodCompiler.getScriptCompiler().getCacheCompiler().cacheCallSite(methodCompiler, name, CallType.VARIABLE);
-        method.label(readyForCall);
+        methodCompiler.invokeUtilityMethod("selectAttrAsgnCallSite", sig(CallSite.class, IRubyObject.class, IRubyObject.class, CallSite.class, CallSite.class));
         
         String signature = null;
         switch (argsCallback.getArity()) {
@@ -591,18 +585,109 @@ public class StandardInvocationCompiler implements InvocationCompiler {
         method.invokevirtual(p(Block.class), "yield", signature);
     }
 
-    public void invokeEqq() {
-        // receiver and args already present on the stack
-        
-        methodCompiler.getScriptCompiler().getCacheCompiler().cacheCallSite(methodCompiler, "===", CallType.NORMAL);
+    public void yieldSpecific(ArgumentsCallback argsCallback) {
+        methodCompiler.loadBlock();
         methodCompiler.loadThreadContext();
-        methodCompiler.loadSelf();
-        
-        methodCompiler.invokeUtilityMethod("invokeEqqForCaseWhen", sig(IRubyObject.class,
-                IRubyObject.class, /*receiver*/
-                IRubyObject.class, /*arg*/
-                CallSite.class,
-                ThreadContext.class,
-                IRubyObject.class /*self*/));
+
+        String signature;
+        if (argsCallback == null) {
+            signature = sig(IRubyObject.class, ThreadContext.class);
+        } else {
+            argsCallback.call(methodCompiler);
+            switch (argsCallback.getArity()) {
+            case 1:
+                signature = sig(IRubyObject.class, ThreadContext.class, IRubyObject.class);
+                break;
+            case 2:
+                signature = sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, IRubyObject.class);
+                break;
+            case 3:
+                signature = sig(IRubyObject.class, ThreadContext.class, IRubyObject.class, IRubyObject.class, IRubyObject.class);
+                break;
+            default:
+                throw new NotCompilableException("Can't do specific-arity call for > 3 args yet");
+            }
+        }
+
+        method.invokevirtual(p(Block.class), "yieldSpecific", signature);
+    }
+
+    public void invokeEqq(ArgumentsCallback receivers, CompilerCallback argument) {
+        if (argument == null) {
+            receivers.call(methodCompiler);
+
+            switch (receivers.getArity()) {
+            case 1:
+                methodCompiler.invokeUtilityMethod("invokeEqqForCaselessWhen", sig(boolean.class,
+                        IRubyObject.class /*receiver*/
+                        ));
+                break;
+            case 2:
+                methodCompiler.invokeUtilityMethod("invokeEqqForCaselessWhen", sig(boolean.class,
+                        IRubyObject.class, /*receiver*/
+                        IRubyObject.class
+                        ));
+                break;
+            case 3:
+                methodCompiler.invokeUtilityMethod("invokeEqqForCaselessWhen", sig(boolean.class,
+                        IRubyObject.class, /*receiver*/
+                        IRubyObject.class,
+                        IRubyObject.class
+                        ));
+                break;
+            default:
+                methodCompiler.invokeUtilityMethod("invokeEqqForCaselessWhen", sig(boolean.class,
+                        IRubyObject[].class /*receiver*/
+                        ));
+            }
+        } else {
+            // arg and receiver already present on the stack
+            methodCompiler.getScriptCompiler().getCacheCompiler().cacheCallSite(methodCompiler, "===", CallType.NORMAL);
+            methodCompiler.loadThreadContext();
+            methodCompiler.loadSelf();
+            argument.call(methodCompiler);
+            receivers.call(methodCompiler);
+
+            switch (receivers.getArity()) {
+            case 1:
+                methodCompiler.invokeUtilityMethod("invokeEqqForCaseWhen", sig(boolean.class,
+                        CallSite.class,
+                        ThreadContext.class,
+                        IRubyObject.class /*self*/,
+                        IRubyObject.class, /*arg*/
+                        IRubyObject.class /*receiver*/
+                        ));
+                break;
+            case 2:
+                methodCompiler.invokeUtilityMethod("invokeEqqForCaseWhen", sig(boolean.class,
+                        CallSite.class,
+                        ThreadContext.class,
+                        IRubyObject.class /*self*/,
+                        IRubyObject.class, /*arg*/
+                        IRubyObject.class, /*receiver*/
+                        IRubyObject.class
+                        ));
+                break;
+            case 3:
+                methodCompiler.invokeUtilityMethod("invokeEqqForCaseWhen", sig(boolean.class,
+                        CallSite.class,
+                        ThreadContext.class,
+                        IRubyObject.class /*self*/,
+                        IRubyObject.class, /*arg*/
+                        IRubyObject.class, /*receiver*/
+                        IRubyObject.class,
+                        IRubyObject.class
+                        ));
+                break;
+            default:
+                methodCompiler.invokeUtilityMethod("invokeEqqForCaseWhen", sig(boolean.class,
+                        CallSite.class,
+                        ThreadContext.class,
+                        IRubyObject.class /*self*/,
+                        IRubyObject.class, /*arg*/
+                        IRubyObject[].class /*receiver*/
+                        ));
+            }
+        }
     }
 }
