@@ -29,8 +29,12 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby.ext;
 
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
 import java.io.IOException;
+import java.io.PrintWriter;
 import java.lang.reflect.Field;
+import java.nio.CharBuffer;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Iterator;
@@ -50,6 +54,7 @@ import jline.Completor;
 import jline.FileNameCompletor;
 import jline.CandidateListCompletionHandler;
 import jline.History;
+import org.jruby.RubyIO;
 import org.jruby.RubyNumeric;
 import org.jruby.RubyString;
 import org.jruby.anno.JRubyMethod;
@@ -59,9 +64,12 @@ import org.jruby.runtime.Visibility;
 /**
  * @author <a href="mailto:ola.bini@ki.se">Ola Bini</a>
  * @author <a href="mailto:pldms@mac.com">Damian Steer</a>
+ * @author <a href="mailto:koichiro@meadowy.org">Koichiro Ohba</a>
  */
-@JRubyModule(name = "Readline", include = "Enumerable")
+@JRubyModule(name = "Readline")
 public class Readline {
+    public static final char ESC_KEY_CODE = (char)27;
+    private final static boolean DEBUG = false;
 
     public static class Service implements Library {
 
@@ -143,17 +151,30 @@ public class Readline {
     }
 
     // We lazily initialize this in case Readline.readline has been overridden in ruby (s_readline)
-    protected static void initReadline(Ruby runtime, ConsoleHolder holder) throws IOException {
+    protected static void initReadline(Ruby runtime, final ConsoleHolder holder) throws IOException {
         holder.readline = new ConsoleReader();
         holder.readline.setUseHistory(false);
         holder.readline.setUsePagination(true);
-        holder.readline.setBellEnabled(false);
+        holder.readline.setBellEnabled(true);
         ((CandidateListCompletionHandler) holder.readline.getCompletionHandler()).setAlwaysIncludeNewline(false);
         if (holder.currentCompletor == null) {
             holder.currentCompletor = new RubyFileNameCompletor();
         }
         holder.readline.addCompletor(holder.currentCompletor);
         holder.readline.setHistory(holder.history);
+
+        // JRUBY-852, ignore escape key (it causes IRB to quit if we pass it out through readline)
+        holder.readline.addTriggeredAction(ESC_KEY_CODE, new ActionListener() {
+            public void actionPerformed(ActionEvent e) {
+                try {
+                    holder.readline.beep();
+                } catch (IOException ioe) {
+                    // ignore
+                }
+            }
+        });
+
+        if (DEBUG) holder.readline.setDebug(new PrintWriter(System.err));
     }
 
     public static History getHistory(ConsoleHolder holder) {
@@ -178,17 +199,40 @@ public class Readline {
         return holder.currentCompletor;
     }
 
-    @JRubyMethod(name = "readline", module = true, visibility = Visibility.PRIVATE)
     public static IRubyObject s_readline(IRubyObject recv, IRubyObject prompt, IRubyObject add_to_hist) throws IOException {
-        ConsoleHolder holder = getHolder(recv.getRuntime());
-        if (holder.readline == null) {
-            initReadline(recv.getRuntime(), holder); // not overridden, let's go
+        return s_readline(recv.getRuntime().getCurrentContext(), recv, prompt, add_to_hist);
+    }
 
+    @JRubyMethod(name = "readline", module = true, visibility = Visibility.PRIVATE)
+    public static IRubyObject s_readline(ThreadContext context, IRubyObject recv, IRubyObject prompt, IRubyObject add_to_hist) throws IOException {
+        Ruby runtime = context.getRuntime();
+        ConsoleHolder holder = getHolder(runtime);
+        if (holder.readline == null) {
+            initReadline(runtime, holder); // not overridden, let's go
         }
-        IRubyObject line = recv.getRuntime().getNil();
-        holder.readline.getTerminal().disableEcho();
-        String v = holder.readline.readLine(prompt.toString());
-        holder.readline.getTerminal().enableEcho();
+        
+        IRubyObject line = runtime.getNil();
+        String v = null;
+        while (true) {
+            try {
+                holder.readline.getTerminal().disableEcho();
+                v = holder.readline.readLine(prompt.toString());
+                break;
+            } catch (IOException ioe) {
+                if (RubyIO.restartSystemCall(ioe)) {
+                    // This is for JRUBY-2988, since after a suspend the terminal seems
+                    // to need to be reinitialized. Since we can't easily detect suspension,
+                    // initialize after every readline. Probably not fast, but this is for
+                    // interactive terminals anyway...so who cares?
+                    try {holder.readline.getTerminal().initializeTerminal();} catch (Exception e) {}
+                    continue;
+                }
+                throw runtime.newIOErrorFromException(ioe);
+            } finally {
+                holder.readline.getTerminal().enableEcho();
+            }
+        }
+        
         if (null != v) {
             if (add_to_hist.isTrue()) {
                 holder.readline.getHistory().addToHistory(v);
@@ -210,6 +254,21 @@ public class Readline {
         return s_readline(recv, RubyString.newEmptyString(recv.getRuntime()), recv.getRuntime().getFalse());
     }
 
+    @JRubyMethod(name = "basic_word_break_characters=", module = true, visibility = Visibility.PRIVATE)
+    public static IRubyObject s_set_basic_word_break_character(IRubyObject recv, IRubyObject achar) throws Exception {
+        Ruby runtime = recv.getRuntime();
+        if (!achar.respondsTo("to_str")) {
+            throw runtime.newTypeError("can't convert " + achar.getMetaClass() + " into String");
+        }
+        ProcCompletor.setDelimiter(achar.convertToString().toString());
+        return achar;
+    }
+
+    @JRubyMethod(name = "basic_word_break_characters", module = true, visibility = Visibility.PRIVATE)
+    public static IRubyObject s_get_basic_word_break_character(IRubyObject recv) throws Exception {
+        return recv.getRuntime().newString(ProcCompletor.getDelimiter());
+    }
+
     @JRubyMethod(name = "completion_append_character=", module = true, visibility = Visibility.PRIVATE)
     public static IRubyObject s_set_completion_append_character(IRubyObject recv, IRubyObject achar) throws Exception {
         return recv.getRuntime().getNil();
@@ -355,14 +414,42 @@ public class Readline {
     public static class ProcCompletor implements Completor {
 
         IRubyObject procCompletor;
+        //\t\n\"\\'`@$><=;|&{(
+        static private String[] delimiters = {" ", "\t", "\n", "\"", "\\", "'", "`", "@", "$", ">", "<", "=", ";", "|", "&", "{", "("};
 
         public ProcCompletor(IRubyObject procCompletor) {
             this.procCompletor = procCompletor;
         }
 
+        public static String getDelimiter() {
+            StringBuilder result = new StringBuilder(delimiters.length);
+            for (String delimiter : delimiters) {
+                result.append(delimiter);
+            }
+            return result.toString();
+        }
+
+        public static void setDelimiter(String delimiter) {
+            List<String> l = new ArrayList<String>();
+            CharBuffer buf = CharBuffer.wrap(delimiter);
+            while (buf.hasRemaining()) {
+                l.add(String.valueOf(buf.get()));
+            }
+            delimiters = l.toArray(new String[l.size()]);
+        }
+
+        private int wordIndexOf(String buffer) {
+            int index = 0;
+            for (String c : delimiters) {
+                index = buffer.lastIndexOf(c);
+                if (index != -1) return index;
+            }
+            return index;
+        }
+
         public int complete(String buffer, int cursor, List candidates) {
             buffer = buffer.substring(0, cursor);
-            int index = buffer.lastIndexOf(" ");
+            int index = wordIndexOf(buffer);
             if (index != -1) {
                 buffer = buffer.substring(index + 1);
             }
@@ -386,7 +473,7 @@ public class Readline {
 
     // Fix FileNameCompletor to work mid-line
     public static class RubyFileNameCompletor extends FileNameCompletor {
-
+        @Override
         public int complete(String buffer, int cursor, List candidates) {
             buffer = buffer.substring(0, cursor);
             int index = buffer.lastIndexOf(" ");
