@@ -33,9 +33,12 @@ import org.jruby.RubyArray;
 import org.jruby.RubyClass;
 import org.jruby.RubyFixnum;
 import org.jruby.RubyFloat;
+import org.jruby.RubyHash;
 import org.jruby.RubyModule;
+import org.jruby.RubyNumeric;
 import org.jruby.RubyObject;
 import org.jruby.RubyString;
+import org.jruby.RubySymbol;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.runtime.ObjectAllocator;
@@ -51,8 +54,6 @@ import org.jruby.util.ByteList;
 abstract public class AbstractMemory extends RubyObject {
     public final static String ABSTRACT_MEMORY_RUBY_CLASS = "AbstractMemory";
 
-    /** The offset from the base memory pointer */
-    protected final long offset;
     /** The total size of the memory area */
     protected final long size;
     /** The Memory I/O object */
@@ -69,10 +70,27 @@ abstract public class AbstractMemory extends RubyObject {
         return result;
     }
     
-    protected AbstractMemory(Ruby runtime, RubyClass klass, MemoryIO io, long offset, long size) {
+    protected static final int calculateSize(ThreadContext context, IRubyObject sizeArg) {
+        if (sizeArg instanceof RubyFixnum) {
+            return (int) ((RubyFixnum) sizeArg).getLongValue();
+
+        } else if (sizeArg instanceof RubySymbol) {
+            return TypeSizeMapper.getTypeSize(context, sizeArg);
+
+        } else if (sizeArg instanceof RubyClass && Struct.isStruct(context.getRuntime(), (RubyClass) sizeArg)) {
+            return Struct.getStructSize(context.getRuntime(), sizeArg);
+
+        } else if (sizeArg.respondsTo("size")) {
+            return (int) RubyFixnum.num2long(sizeArg.callMethod(context, "size"));
+
+        } else {
+            throw context.getRuntime().newArgumentError("Invalid size argument");
+        }
+    }
+    
+    protected AbstractMemory(Ruby runtime, RubyClass klass, MemoryIO io, long size) {
         super(runtime, klass);
         this.io = io;
-        this.offset = offset;
         this.size = size;
     }
     /**
@@ -80,7 +98,7 @@ abstract public class AbstractMemory extends RubyObject {
      *
      * @return A memory accessor.
      */
-    protected final MemoryIO getMemoryIO() {
+    public final MemoryIO getMemoryIO() {
         return io;
     }
 
@@ -92,18 +110,9 @@ abstract public class AbstractMemory extends RubyObject {
      * @return The total offset from the base memory pointer.
      */
     protected final long getOffset(IRubyObject offset) {
-        return getOffset() + Util.longValue(offset);
+        return Util.longValue(offset);
     }
     
-    /**
-     * Gets the offset within the memory area.
-     *
-     * @return The offset within the original memory area.
-     */
-    protected final long getOffset() {
-        return this.offset;
-    }
-
     /**
      * Gets the size of the memory area.
      *
@@ -141,7 +150,7 @@ abstract public class AbstractMemory extends RubyObject {
             return false;
         }
         final AbstractMemory other = (AbstractMemory) obj;
-        return other.getMemoryIO().equals(getMemoryIO()) && other.offset == offset;
+        return other.getMemoryIO().equals(getMemoryIO());
     }
     
     @JRubyMethod(name = "==", required = 1)
@@ -160,7 +169,7 @@ abstract public class AbstractMemory extends RubyObject {
      */
     @Override
     public int hashCode() {
-        return 67 * getMemoryIO().hashCode() + (int) (this.offset ^ (this.offset >>> 32));
+        return 67 * getMemoryIO().hashCode();
     }
 
     /**
@@ -168,7 +177,7 @@ abstract public class AbstractMemory extends RubyObject {
      */
     @JRubyMethod(name = "clear")
     public IRubyObject clear(ThreadContext context) {
-        getMemoryIO().setMemory(offset, size, (byte) 0);
+        getMemoryIO().setMemory(0, size, (byte) 0);
         return this;
     }
     /**
@@ -193,7 +202,7 @@ abstract public class AbstractMemory extends RubyObject {
     protected final long checkBounds(ThreadContext context, IRubyObject offArg, long len) {
         long off = Util.longValue(offArg);
         checkBounds(context, size, off, len);
-        return this.offset + off;
+        return off;
     }
     /**
      * Writes a 8 bit signed integer value to the memory area.
@@ -362,7 +371,7 @@ abstract public class AbstractMemory extends RubyObject {
      * @param value The value to write.
      * @return The value written.
      */
-    @JRubyMethod(name = { "put_int64", "put_uint64", "put_long_long",  "put_ulong_long" }, required = 2)
+    @JRubyMethod(name = { "put_int64", "put_long_long" }, required = 2)
     public IRubyObject put_int64(ThreadContext context, IRubyObject offset, IRubyObject value) {
         checkBounds(context, offset, 8);
         getMemoryIO().putLong(getOffset(offset), Util.int64Value(value));
@@ -375,12 +384,38 @@ abstract public class AbstractMemory extends RubyObject {
      * @param offset The offset from the base pointer address to read the value.
      * @return The value read from the address.
      */
-    @JRubyMethod(name = { "get_int64", "get_uint64", "get_long_long", "get_ulong_long" }, required = 1)
+    @JRubyMethod(name = { "get_int64", "get_long_long" }, required = 1)
     public IRubyObject get_int64(ThreadContext context, IRubyObject offset) {
         checkBounds(context, offset, 8);
-        return RubyFixnum.newFixnum(context.getRuntime(), getMemoryIO().getLong(getOffset(offset)));
+        return Util.newSigned64(context.getRuntime(), getMemoryIO().getLong(getOffset(offset)));
     }
-    
+
+    /**
+     * Writes a 64 bit unsigned integer value to the memory area.
+     *
+     * @param offset The offset from the base pointer address to write the value.
+     * @param value The value to write.
+     * @return The value written.
+     */
+    @JRubyMethod(name = { "put_uint64", "put_ulong_long" }, required = 2)
+    public IRubyObject put_uint64(ThreadContext context, IRubyObject offset, IRubyObject value) {
+        checkBounds(context, offset, 8);
+        getMemoryIO().putLong(getOffset(offset), Util.uint64Value(value));
+        return this;
+    }
+
+    /**
+     * Reads a 64 bit unsigned integer value from the memory address.
+     *
+     * @param offset The offset from the base pointer address to read the value.
+     * @return The value read from the address.
+     */
+    @JRubyMethod(name = { "get_uint64", "get_ulong_long" }, required = 1)
+    public IRubyObject get_uint64(ThreadContext context, IRubyObject offset) {
+        checkBounds(context, offset, 8);
+        return Util.newUnsigned64(context.getRuntime(), getMemoryIO().getLong(getOffset(offset)));
+    }
+
     /**
      * Writes a C long integer value to the memory area.
      *
@@ -390,9 +425,9 @@ abstract public class AbstractMemory extends RubyObject {
      */
     @JRubyMethod(name = "put_long", required = 2)
     public IRubyObject put_long(ThreadContext context, IRubyObject offset, IRubyObject value) {
-        checkBounds(context, offset, Platform.getPlatform().longSize() / 8);
-        getMemoryIO().putNativeLong(getOffset(offset), Util.longValue(value));
-        return this;
+        return Platform.getPlatform().longSize() == 32
+                ? put_int32(context, offset, value)
+                : put_int64(context, offset, value);
     }
     
     /**
@@ -403,8 +438,9 @@ abstract public class AbstractMemory extends RubyObject {
      */
     @JRubyMethod(name = "get_long", required = 1)
     public IRubyObject get_long(ThreadContext context, IRubyObject offset) {
-        checkBounds(context, offset, Platform.getPlatform().longSize() / 8);
-        return RubyFixnum.newFixnum(context.getRuntime(), getMemoryIO().getNativeLong(getOffset(offset)));
+        return Platform.getPlatform().longSize() == 32
+                ? get_int32(context, offset)
+                : get_int64(context, offset);
     }
     
     /**
@@ -416,9 +452,9 @@ abstract public class AbstractMemory extends RubyObject {
      */
     @JRubyMethod(name = "put_ulong", required = 2)
     public IRubyObject put_ulong(ThreadContext context, IRubyObject offset, IRubyObject value) {
-        checkBounds(context, offset, Platform.getPlatform().longSize() / 8);
-        getMemoryIO().putNativeLong(getOffset(offset), Util.longValue(value));
-        return this;
+        return Platform.getPlatform().longSize() == 32
+                ? put_uint32(context, offset, value)
+                : put_uint64(context, offset, value);
     }
     
     /**
@@ -429,10 +465,9 @@ abstract public class AbstractMemory extends RubyObject {
      */
     @JRubyMethod(name = "get_ulong", required = 1)
     public IRubyObject get_ulong(ThreadContext context, IRubyObject offset) {
-        checkBounds(context, offset, Platform.getPlatform().longSize() / 8);
-        long value = getMemoryIO().getNativeLong(getOffset(offset));
-        return RubyFixnum.newFixnum(context.getRuntime(),
-                value < 0 ? (long)((value & 0x7FFFFFFFL) + 0x80000000L) : value);
+        return Platform.getPlatform().longSize() == 32
+                ? get_uint32(context, offset)
+                : get_uint64(context, offset);
     }
     /**
      * Writes an 32 bit floating point value to the memory area.
@@ -712,7 +747,36 @@ abstract public class AbstractMemory extends RubyObject {
     }
     @JRubyMethod(name = "get_pointer", required = 1)
     public IRubyObject get_pointer(ThreadContext context, IRubyObject offset) {
-        return getPointer(context.getRuntime(), Util.int64Value(offset));
+        return getPointer(context.getRuntime(), getOffset(offset));
+    }
+
+    @JRubyMethod(name = "put_pointer", required = 2)
+    public IRubyObject put_pointer(ThreadContext context, IRubyObject offset, IRubyObject value) {
+        if (value instanceof Pointer) {
+            MemoryIO ptr = ((Pointer) value).getMemoryIO();
+            if (ptr.isDirect()) {
+                getMemoryIO().putMemoryIO(getOffset(offset), ptr);
+            } else if (ptr.isNull()) {
+                getMemoryIO().putAddress(getOffset(offset), 0L);
+            } else {
+                throw context.getRuntime().newArgumentError("Cannot convert argument to pointer");
+            }
+        } else if (value.isNil()) {
+            getMemoryIO().putAddress(getOffset(offset), 0L);
+        } else {
+            throw context.getRuntime().newArgumentError("Cannot convert argument to pointer");
+        }
+        return this;
+    }
+
+    @JRubyMethod(name = "put_callback", required = 3)
+    public IRubyObject put_callback(ThreadContext context, IRubyObject offset, IRubyObject proc, IRubyObject cbInfo) {
+        if (!(cbInfo instanceof CallbackInfo)) {
+            throw context.getRuntime().newArgumentError("invalid CallbackInfo");
+        }
+        Pointer ptr = Factory.getInstance().getCallbackManager().getCallback(context.getRuntime(), (CallbackInfo) cbInfo, proc);
+        getMemoryIO().putMemoryIO(getOffset(offset), ((AbstractMemory) ptr).getMemoryIO());
+        return this;
     }
     @JRubyMethod(name = "__get_buffer", required = 2)
     public IRubyObject get_buffer(ThreadContext context, IRubyObject off, IRubyObject len_) {
@@ -729,5 +793,10 @@ abstract public class AbstractMemory extends RubyObject {
         getMemoryIO().put(getOffset(off), bl.unsafeBytes(), bl.begin(), len);
         return context.getRuntime().newFixnum(len);
     }
+    @JRubyMethod(name = "+", required = 1)
+    public IRubyObject op_plus(ThreadContext context, IRubyObject value) {
+        return slice(context.getRuntime(), RubyNumeric.fix2long(value));
+    }
+    abstract protected AbstractMemory slice(Ruby runtime, long offset);
     abstract protected Pointer getPointer(Ruby runtime, long offset);
 }
