@@ -88,7 +88,8 @@ public abstract class FFIProvider extends RubyObject {
             }
         }
         try {
-            return createInvoker(context.getRuntime(), args[0].toString(), 
+            return createInvoker(context.getRuntime(), 
+                    args[0].isNil() ? null : args[0].toString(), 
                     args[1].toString(), NativeType.valueOf(Util.int32Value(args[2])), 
                     nativeParamTypes, args[4].toString());
         } catch (UnsatisfiedLinkError ex) {
@@ -118,7 +119,7 @@ public abstract class FFIProvider extends RubyObject {
      * @param parameterTypes The parameter types the function takes.
      * @return a new <tt>Invoker</tt> instance.
      */
-    public abstract Invoker createInvoker(Ruby runtime, String libraryName, String functionName, NativeType returnType,
+    public abstract AbstractInvoker createInvoker(Ruby runtime, String libraryName, String functionName, NativeType returnType,
             NativeParam[] parameterTypes, String convention);
 
     /**
