@@ -30,6 +30,8 @@ package org.jruby.ext.ffi;
 
 import java.io.IOException;
 import java.nio.channels.ByteChannel;
+import java.util.ArrayList;
+import java.util.List;
 import org.jruby.Ruby;
 import org.jruby.RubyInstanceConfig;
 import org.jruby.RubyModule;
@@ -46,21 +48,25 @@ public abstract class Factory {
         private static final Factory INSTANCE = getInstance();
 
         private static final Factory getInstance() {
-            final boolean useJNA = Boolean.getBoolean("jruby.ffi.usejna");
-            String prefix = FFIProvider.class.getPackage().getName();
+            final String providerName = System.getProperty("ffi.factory");
             Factory factory = null;
-            if (!useJNA) {
+            List<String> providerNames = new ArrayList<String>();
+            if (providerName != null) {
+                providerNames.add(providerName);
+            }
+            final String prefix = FFIProvider.class.getPackage().getName();
+            providerNames.add(prefix + ".jffi.Factory");
+            providerNames.add(prefix + ".jna.Factory");
+            for (String className : providerNames) {
                 try {
-                    factory = (Factory) Class.forName(prefix + ".jffi.JFFIFactory", true, Ruby.getClassLoader()).newInstance();
+                    factory = (Factory) Class.forName(className, true, Ruby.getClassLoader()).newInstance();
+                    break;
                 } catch (Throwable ex) {
                 }
             }
+
             if (factory == null) {
-                try {
-                    factory = (Factory) Class.forName(prefix + ".jna.JNAFactory", true, Ruby.getClassLoader()).newInstance();
-                } catch (Throwable ex) {
-                    throw new RuntimeException("Could not load FFI provider", ex);
-                }
+                throw new RuntimeException("Could not load FFI provider");
             }
             return factory;
         }
@@ -75,6 +81,10 @@ public abstract class Factory {
             if (!RubyInstanceConfig.nativeEnabled) {
                 throw runtime.newLoadError("Native API access is disabled");
             }
+            if (!Platform.getPlatform().isSupported()) {
+                throw runtime.newLoadError("Unsupported platform: " + Platform.getPlatform().getName());
+            }
+
             RubyModule ffi = runtime.defineModule("FFI");
             Factory factory = Factory.getInstance();
             factory.init(runtime, ffi);
@@ -100,23 +110,29 @@ public abstract class Factory {
             if (ffi.fastGetClass(FFIProvider.CLASS_NAME) == null) {
                 FFIProvider.createProviderClass(runtime, ffi);
             }
-            if (ffi.fastGetClass(Invoker.CLASS_NAME) == null) {
-                Invoker.createInvokerClass(runtime, ffi);
+            if (ffi.fastGetClass(AbstractInvoker.CLASS_NAME) == null) {
+                AbstractInvoker.createAbstractInvokerClass(runtime, ffi);
             }
             if (ffi.fastGetClass(AbstractMemory.ABSTRACT_MEMORY_RUBY_CLASS) == null) {
                 AbstractMemory.createAbstractMemoryClass(runtime, ffi);
             }
+            if (ffi.fastGetClass("Buffer") == null) {
+                Buffer.createBufferClass(runtime, ffi);
+            }
             if (ffi.fastGetClass("Pointer") == null) {
                 Pointer.createPointerClass(runtime, ffi);
             }
             if (ffi.fastGetClass("AutoPointer") == null) {
                 AutoPointer.createAutoPointerClass(runtime, ffi);
             }
-            if (ffi.fastGetClass(AbstractMemoryPointer.className) == null) {
-                AbstractMemoryPointer.createMemoryPointerClass(runtime, ffi);
+            if (ffi.fastGetClass("BasePointer") == null) {
+                BasePointer.createBasePointerClass(runtime, ffi);
             }
-            if (ffi.fastGetClass(AbstractBuffer.ABSTRACT_BUFFER_RUBY_CLASS) == null) {
-                AbstractBuffer.createBufferClass(runtime, ffi);
+            if (ffi.fastGetClass("MemoryPointer") == null) {
+                MemoryPointer.createMemoryPointerClass(runtime, ffi);
+            }
+            if (ffi.fastGetClass("Struct") == null) {
+                Struct.createStructClass(runtime, ffi);
             }
             if (ffi.fastGetClass(CallbackInfo.CLASS_NAME) == null) {
                 CallbackInfo.createCallbackInfoClass(runtime, ffi);
@@ -137,7 +153,7 @@ public abstract class Factory {
             for (NativeType type : NativeType.values()) {
                 nativeType.defineConstant(type.name(), runtime.newFixnum(type.ordinal()));
             }
-            Platform.getPlatform().init(runtime, ffi);
+            Platform.createPlatformModule(runtime, ffi);
         }
     }
     
@@ -153,17 +169,29 @@ public abstract class Factory {
      */
     public abstract <T> T loadLibrary(String libraryName, Class<T> libraryClass);
 
-    /**
-     * Gets the platform info for this <tt>FFIProvider</tt>.
-     *
-     * @return A platform information instance.
-     */
-    public abstract Platform getPlatform();
-
     /**
      * Wraps a {@link java.nio.ByteChannel} around a native file descriptor
      */
     public abstract ByteChannel newByteChannel(int fd);
     
-    
+    /**
+     * Allocates memory on the native C heap and wraps it in a <tt>MemoryIO</tt> accessor.
+     *
+     * @param size The number of bytes to allocate.
+     * @param clear If the memory should be cleared.
+     * @return A new <tt>MemoryIO</tt>.
+     */
+    public abstract AllocatedDirectMemoryIO allocateDirectMemory(int size, boolean clear);
+
+    /**
+     * Wraps a  native C memory address in a <tt>MemoryIO</tt> accessor.
+     *
+     * @param address The native address to wrap.
+     * 
+     * @return A new <tt>MemoryIO</tt>.
+     */
+    public abstract DirectMemoryIO wrapDirectMemory(long address);
+
+
+    public abstract CallbackManager getCallbackManager();
 }
