@@ -28,9 +28,11 @@ public abstract class Pointer extends AbstractMemory {
 
         return result;
     }
-
-    protected Pointer(Ruby runtime, RubyClass klass, MemoryIO io, long offset, long size) {
-        super(runtime, klass, io, offset, size);
+    protected Pointer(Ruby runtime, RubyClass klass, MemoryIO io) {
+        super(runtime, klass, io, Long.MAX_VALUE);
+    }
+    protected Pointer(Ruby runtime, RubyClass klass, MemoryIO io, long size) {
+        super(runtime, klass, io, size);
     }
     
    /**
