@@ -37,6 +37,7 @@ import org.jruby.RubyString;
 import org.jruby.javasupport.JavaObject;
 import org.jruby.javasupport.JavaUtil;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.util.ByteList;
 
 /**
  *
@@ -102,12 +103,7 @@ public final class Util {
         return value;
     }
     public static final float floatValue(IRubyObject parameter) {
-        final double value = RubyNumeric.num2dbl(parameter);
-        if (value < Float.MIN_VALUE || value > Float.MAX_VALUE) {
-            throw parameter.getRuntime().newRangeError("Value "
-                    + value + " outside float range");
-        }
-        return (float) value;
+        return (float) RubyNumeric.num2dbl(parameter);
     }
     public static final double doubleValue(IRubyObject parameter) {
         return RubyNumeric.num2dbl(parameter);
@@ -138,15 +134,29 @@ public final class Util {
         throw parameter.getRuntime().newRangeError("Value "
                     + parameter + " is not an integer");
     }
+    public static final IRubyObject newSigned8(Ruby runtime, int value) {
+        value &= 0xff;
+        return runtime.newFixnum(value < 0x80 ? value : -0x80 + (value - 0x80));
+    }
     public static final IRubyObject newUnsigned8(Ruby runtime, int value) {
         return runtime.newFixnum(value < 0 ? (long)((value & 0x7FL) + 0x80L) : value);
     }
+    public static final IRubyObject newSigned16(Ruby runtime, int value) {
+        value &= 0xffff;
+        return runtime.newFixnum(value < 0x8000 ? value : -0x8000 + (value - 0x8000));
+    }
     public static final IRubyObject newUnsigned16(Ruby runtime, int value) {
         return runtime.newFixnum(value < 0 ? (long)((value & 0x7FFFL) + 0x8000L) : value);
     }
+    public static final IRubyObject newSigned32(Ruby runtime, int value) {
+        return runtime.newFixnum(value);
+    }
     public static final IRubyObject newUnsigned32(Ruby runtime, int value) {
         return runtime.newFixnum(value < 0 ? (long)((value & 0x7FFFFFFFL) + 0x80000000L) : value);
     }
+    public static final IRubyObject newSigned64(Ruby runtime, long value) {
+        return runtime.newFixnum(value);
+    }
     private static final BigInteger UINT64_BASE = BigInteger.valueOf(Long.MAX_VALUE).add(BigInteger.ONE);
     public static final IRubyObject newUnsigned64(Ruby runtime, long value) {
         return value < 0
@@ -163,4 +173,18 @@ public final class Util {
         tmp.position((int) offset);
         return tmp.slice();
     }
+    public static final void checkStringSafety(Ruby runtime, IRubyObject value) {
+        RubyString s = value.asString();
+        if (runtime.getSafeLevel() > 0 && s.isTaint()) {
+            throw runtime.newSecurityError("Unsafe string parameter");
+        }
+        ByteList bl = s.getByteList();
+        final byte[] array = bl.unsafeBytes();
+        final int end = bl.length();
+        for (int i = bl.begin(); i < end; ++i) {
+            if (array[i] == (byte) 0) {
+                throw runtime.newSecurityError("string contains null byte");
+            }
+        }
+    }
 }
