@@ -50,7 +50,7 @@ final class CallbackMethodWithBlock extends DynamicMethod {
     
     public CallbackMethodWithBlock(RubyModule implementationClass, Function function, 
             FunctionInvoker functionInvoker, Marshaller[] marshallers, int cbindex) {
-        super(implementationClass, Visibility.PUBLIC, CallConfiguration.FRAME_AND_SCOPE);
+        super(implementationClass, Visibility.PUBLIC, CallConfiguration.FrameFullScopeFull);
         this.function = function;
         this.functionInvoker = functionInvoker;
         this.marshallers = marshallers;
