@@ -1,14 +1,18 @@
 package org.jruby.ext.ffi.jna;
 
 import java.util.ArrayList;
+import org.jruby.runtime.ThreadContext;
 
 /**
  * An invocation session.
  * This provides post-invoke cleanup.
  */
 final class Invocation {
+    private final ThreadContext context;
     private ArrayList<Runnable> postInvokeList;
-
+    Invocation(ThreadContext context) {
+        this.context = context;
+    }
     void finish() {
         if (postInvokeList != null) {
             for (Runnable r : postInvokeList) {
@@ -23,4 +27,8 @@ final class Invocation {
         }
         postInvokeList.add(postInvoke);
     }
+
+    ThreadContext getThreadContext() {
+        return context;
+    }
 }
