@@ -33,7 +33,6 @@ import com.sun.jna.Function;
 import com.sun.jna.Memory;
 import com.sun.jna.Native;
 import com.sun.jna.NativeLibrary;
-import com.sun.jna.Pointer;
 import java.nio.ByteBuffer;
 import org.jruby.Ruby;
 import org.jruby.RubyString;
@@ -50,9 +49,12 @@ public final class JNAProvider extends FFIProvider {
     }
     
     @Override
-    public final Invoker createInvoker(Ruby runtime, String libraryName, String functionName,
+    public final AbstractInvoker createInvoker(Ruby runtime, String libraryName, String functionName,
             NativeType returnType, NativeParam[] parameterTypes, String convention) {
         int conv = "stdcall".equals(convention) ? Function.ALT_CONVENTION : Function.C_CONVENTION;
+        if (libraryName == null) {
+            libraryName = Platform.LIBC;
+        }
         Function function = NativeLibrary.getInstance(libraryName).getFunction(functionName, conv);
         FunctionInvoker functionInvoker = getFunctionInvoker(returnType);
         Marshaller[] marshallers = new Marshaller[parameterTypes.length];
@@ -60,7 +62,7 @@ public final class JNAProvider extends FFIProvider {
             marshallers[i] = getMarshaller(parameterTypes[i], conv);
         }
 
-        return new JNAInvoker(runtime, function, functionInvoker, marshallers);
+        return new JNAInvoker(runtime, FFIProvider.getModule(runtime).fastGetClass("Invoker"), function, functionInvoker, marshallers);
     }
     
     public int getLastError() {
@@ -106,7 +108,7 @@ public final class JNAProvider extends FFIProvider {
             case ULONG:
                 return Platform.getPlatform().addressSize() == 32
                         ? Unsigned32Invoker.INSTANCE
-                        : Signed64Invoker.INSTANCE;
+                        : Unsigned64Invoker.INSTANCE;
             case FLOAT32:
                 return Float32Invoker.INSTANCE;
             case FLOAT64:
@@ -200,8 +202,7 @@ public final class JNAProvider extends FFIProvider {
      */
     private static final class Signed8Invoker implements FunctionInvoker {
         public final IRubyObject invoke(Ruby runtime, Function function, Object[] args) {
-            int value = function.invokeInt(args) & 0xff;
-            return runtime.newFixnum(value < 0x80 ? value : -0x80 + (value - 0x80));
+            return Util.newSigned8(runtime, function.invokeInt(args));
         }
         public static final FunctionInvoker INSTANCE = new Signed8Invoker();
     }
@@ -223,8 +224,7 @@ public final class JNAProvider extends FFIProvider {
      */
     private static final class Signed16Invoker implements FunctionInvoker {
         public final IRubyObject invoke(Ruby runtime, Function function, Object[] args) {
-            int value = function.invokeInt(args) & 0xffff;
-            return runtime.newFixnum(value < 0x8000 ? value : -0x8000 + (value - 0x8000));
+            return Util.newSigned16(runtime, function.invokeInt(args));
         }
         public static final FunctionInvoker INSTANCE = new Signed16Invoker();
     }
@@ -245,7 +245,7 @@ public final class JNAProvider extends FFIProvider {
      */
     private static final class Signed32Invoker implements FunctionInvoker {
         public final IRubyObject invoke(Ruby runtime, Function function, Object[] args) {   
-            return runtime.newFixnum(function.invokeInt(args));
+            return Util.newSigned32(runtime, function.invokeInt(args));
         }
         public static final FunctionInvoker INSTANCE = new Signed32Invoker();
     }
@@ -267,7 +267,7 @@ public final class JNAProvider extends FFIProvider {
      */
     private static final class Signed64Invoker implements FunctionInvoker {
         public final IRubyObject invoke(Ruby runtime, Function function, Object[] args) {
-            return runtime.newFixnum(function.invokeLong(args));
+            return Util.newSigned64(runtime, function.invokeLong(args));
         }
         public static final FunctionInvoker INSTANCE = new Signed64Invoker();
     }
@@ -311,7 +311,8 @@ public final class JNAProvider extends FFIProvider {
      */
     private static final class PointerInvoker implements FunctionInvoker {
         public final IRubyObject invoke(Ruby runtime, Function function, Object[] args) {
-            return new JNABasePointer(runtime, function.invokePointer(args));
+            com.sun.jna.Pointer ptr = function.invokePointer(args);
+            return new BasePointer(runtime, ptr != null ? new NativeMemoryIO(ptr) : new NullMemoryIO(runtime));
         }
         public static final FunctionInvoker INSTANCE = new PointerInvoker();
     }
@@ -322,7 +323,7 @@ public final class JNAProvider extends FFIProvider {
      */
     private static final class StringInvoker implements FunctionInvoker {
         public final IRubyObject invoke(Ruby runtime, Function function, Object[] args) {
-            Pointer address = function.invokePointer(args);
+            com.sun.jna.Pointer address = function.invokePointer(args);
             if (address == null) {
                 return runtime.getNil();
             }
@@ -440,16 +441,36 @@ public final class JNAProvider extends FFIProvider {
         }
         public static final Marshaller INSTANCE = new Float64Marshaller();
     }
-    
+    private static final Object getNativeMemory(Pointer memory) {
+        MemoryIO io = memory.getMemoryIO();
+        return io instanceof NativeMemoryIO ? ((NativeMemoryIO) io).getPointer() : null;
+    }
+    private static final Object getNativeMemory(Ruby runtime, IRubyObject memory) {
+        if (memory instanceof Pointer) {
+            return getNativeMemory((Pointer) memory);
+        } else if (memory instanceof Buffer) {
+            ArrayMemoryIO io = (ArrayMemoryIO) ((Buffer) memory).getMemoryIO();
+            return ByteBuffer.wrap(io.array(), io.arrayOffset(), io.arrayLength());
+        } else if (memory == null || memory.isNil()) {
+            return com.sun.jna.Pointer.NULL;
+        } else {
+            throw runtime.newArgumentError("Invalid memory object");
+        }
+    }
     /**
      * Converts a ruby MemoryPointer into a native pointer.
      */
     static final class PointerMarshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
-            if (parameter instanceof JNAMemory) {
-                return (((JNAMemory) parameter).getNativeMemory());
+            if (parameter instanceof Pointer) {
+                return getNativeMemory((Pointer) parameter);
+            } else if (parameter instanceof Buffer) {
+                ArrayMemoryIO io = (ArrayMemoryIO) ((Buffer) parameter).getMemoryIO();
+                return ByteBuffer.wrap(io.array(), io.arrayOffset(), io.arrayLength());
+            } else if (parameter instanceof Struct) {
+                return getNativeMemory(invocation.getThreadContext().getRuntime(), ((Struct) parameter).getMemory());
             } else if (parameter.isNil()) {
-                return Pointer.NULL;
+                return com.sun.jna.Pointer.NULL;
             } else if (parameter instanceof RubyString) {
                 // Handle a string being used as a inout buffer
                 final ByteList bl = ((RubyString) parameter).getByteList();
@@ -467,10 +488,8 @@ public final class JNAProvider extends FFIProvider {
                 });
                 return memory;
             } else if (parameter.respondsTo("to_ptr")) {
-                IRubyObject ptr = parameter.callMethod(invocation.getThreadContext(), "to_ptr");
-                if (ptr instanceof JNAMemory) {
-                    return (((JNAMemory) ptr).getNativeMemory());
-                }
+                return getNativeMemory(invocation.getThreadContext().getRuntime(), 
+                        parameter.callMethod(invocation.getThreadContext(), "to_ptr"));
             }
             throw invocation.getThreadContext().getRuntime().newArgumentError("Invalid Pointer argument");
         }
@@ -486,16 +505,20 @@ public final class JNAProvider extends FFIProvider {
      */
     private static final class StringMarshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
-            // Ruby strings are UTF-8, so should be able to just copy directly
-            RubyString s = parameter.asString();
-            if (invocation.getThreadContext().getRuntime().getSafeLevel() > 0 && s.isTaint()) {
-                throw invocation.getThreadContext().getRuntime().newSecurityError("Unsafe string parameter");
+            if (parameter instanceof RubyString) {
+                Util.checkStringSafety(invocation.getThreadContext().getRuntime(), parameter);
+                // Ruby strings are UTF-8, so should be able to just copy directly
+                RubyString s = parameter.asString();
+                ByteList bl = s.getByteList();
+                final Memory memory = new Memory(bl.length() + 1);
+                memory.write(0, bl.unsafeBytes(), bl.begin(), bl.length());
+                memory.setByte(bl.length(), (byte) 0);
+                return memory;
+            } else if (parameter.isNil()) {
+                return null;
+            } else {
+                throw invocation.getThreadContext().getRuntime().newArgumentError("Invalid string parameter");
             }
-            ByteList bl = s.getByteList();
-            final Memory memory = new Memory(bl.length() + 1);
-            memory.write(0, bl.unsafeBytes(), bl.begin(), bl.length());
-            memory.setByte(bl.length(), (byte) 0);
-            return memory;
         }
         public static final Marshaller INSTANCE = new StringMarshaller();
     }
@@ -534,18 +557,21 @@ public final class JNAProvider extends FFIProvider {
      */
     private static final class BufferMarshaller implements Marshaller {
         public final Object marshal(Invocation invocation, IRubyObject parameter) {
-            if (parameter instanceof JNAMemory) {
-                return (((JNAMemory) parameter).getNativeMemory());
+            if (parameter instanceof Pointer) {
+                return getNativeMemory((Pointer) parameter);
+            } else if (parameter instanceof Buffer) {
+                ArrayMemoryIO io = (ArrayMemoryIO) ((Buffer) parameter).getMemoryIO();
+                return ByteBuffer.wrap(io.array(), io.arrayOffset(), io.arrayLength());
+            } else if (parameter instanceof Struct) {
+                return getNativeMemory(invocation.getThreadContext().getRuntime(), ((Struct) parameter).getMemory());
             } else if (parameter.isNil()) {
-                return Pointer.NULL;
+                return com.sun.jna.Pointer.NULL;
             } else if (parameter instanceof RubyString) {
                 ByteList bl = ((RubyString) parameter).getByteList();
                 return ByteBuffer.wrap(bl.unsafeBytes(), bl.begin(), bl.realSize);
             } else if (parameter.respondsTo("to_ptr")) {
-                IRubyObject ptr = parameter.callMethod(invocation.getThreadContext(), "to_ptr");
-                if (ptr instanceof JNAMemory) {
-                    return (((JNAMemory) ptr).getNativeMemory());
-                }
+                return getNativeMemory(invocation.getThreadContext().getRuntime(),
+                        parameter.callMethod(invocation.getThreadContext(), "to_ptr"));
             }
             throw invocation.getThreadContext().getRuntime().newArgumentError("Invalid Buffer argument");
         }
