@@ -71,12 +71,13 @@ public class RubyIPSocket extends RubyBasicSocket {
         IRubyObject[] ret = new IRubyObject[4];
         ret[0] = r.newString("AF_INET");
         ret[1] = r.newFixnum(addr.getPort());
+        String hostAddress = addr.getAddress().getHostAddress();
         if(r.isDoNotReverseLookupEnabled()) {
-            ret[2] = r.newString(addr.getAddress().getHostAddress());
+            ret[2] = r.newString(hostAddress);
         } else {
             ret[2] = r.newString(addr.getHostName());
         }
-        ret[3] = r.newString(addr.getAddress().getHostAddress());
+        ret[3] = r.newString(hostAddress);
         return r.newArrayNoCopy(ret);
     }
     @Deprecated
