@@ -76,7 +76,7 @@ import org.jruby.runtime.load.Library;
 import org.jruby.util.ClassProvider;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.anno.JRubyModule;
-import org.jruby.internal.runtime.methods.JavaMethod.JavaMethodNoBlock;
+import org.jruby.internal.runtime.methods.JavaMethod.JavaMethodN;
 import org.jruby.internal.runtime.methods.JavaMethod.JavaMethodZero;
 import org.jruby.java.MiniJava;
 import org.jruby.java.addons.ArrayJavaAddons;
@@ -141,9 +141,6 @@ public class Java implements Library {
         });
 
         JavaArrayUtilities.createJavaArrayUtilitiesModule(runtime);
-
-        RubyClass javaProxy = runtime.defineClass("JavaProxy", runtime.getObject(), runtime.getObject().getAllocator());
-        javaProxy.defineAnnotatedMethods(JavaProxy.class);
         
         // Now attach Java-related extras to core classes
         runtime.getArray().defineAnnotatedMethods(ArrayJavaAddons.class);
@@ -691,7 +688,7 @@ public class Java implements Library {
             }
         });
 
-        rubySubclass.addMethod("__jcreate!", new JavaMethodNoBlock(subclassSingleton, Visibility.PUBLIC) {
+        rubySubclass.addMethod("__jcreate!", new JavaMethodN(subclassSingleton, Visibility.PUBLIC) {
             private final Map<Integer, ParameterTypes> methodCache = new HashMap<Integer, ParameterTypes>();
             @Override
             public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args) {
@@ -835,14 +832,16 @@ public class Java implements Library {
             throw runtime.newArgumentError("empty class or package name");
         }
         String fullName = packageName + name;
-        if (Character.isLowerCase(name.charAt(0))) {
+        if (!Character.isUpperCase(name.charAt(0))) {
+            // filter out any Java primitive names
             // TODO: should check against all Java reserved names here, not just primitives
             if (JAVA_PRIMITIVES.containsKey(name)) {
                 throw runtime.newArgumentError("illegal package name component: " + name);
+            }
+            
             // this covers the rare case of lower-case class names (and thus will
             // fail 99.999% of the time). fortunately, we'll only do this once per
             // package name. (and seriously, folks, look into best practices...)
-            }
             try {
                 return getProxyClass(runtime, JavaClass.forNameQuiet(runtime, fullName));
             } catch (RaiseException re) { /* expected */
@@ -852,13 +851,15 @@ public class Java implements Library {
                 }
             } catch (Exception e) { /* expected */ }
 
+            // Haven't found a class, continue on as though it were a package
             RubyModule packageModule;
             // TODO: decompose getJavaPackageModule so we don't parse fullName
             if ((packageModule = getJavaPackageModule(runtime, fullName)) == null) {
                 return null;
+            }
+            
             // save package module as ivar in parent, and add method to parent so
             // we don't have to come back here.
-            }
             final String ivarName = ("@__pkg__" + name).intern();
             parentPackage.fastSetInstanceVariable(ivarName, packageModule);
             RubyClass singleton = parentPackage.getSingletonClass();
