@@ -51,6 +51,7 @@ import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -247,7 +248,7 @@ public class JavaClass extends JavaObject {
         // called only by initializing thread; no synchronization required
         void addMethod(Method method, Class<?> javaClass) {
             if (methods == null) {
-                methods = new ArrayList<Method>();
+                methods = new ArrayList<Method>(4);
             }
             if (!Ruby.isSecurityRestricted()) {
                 method.setAccessible(true);
@@ -259,7 +260,7 @@ public class JavaClass extends JavaObject {
         // called only by initializing thread; no synchronization required
         void addAlias(String alias) {
             if (aliases == null) {
-                aliases = new ArrayList<String>();
+                aliases = new ArrayList<String>(4);
             }
             if (!aliases.contains(alias))
                 aliases.add(alias);
@@ -283,7 +284,7 @@ public class JavaClass extends JavaObject {
         // called only by initializing thread; no synchronization required
         void addConstructor(Constructor ctor, Class<?> javaClass) {
             if (constructors == null) {
-                constructors = new ArrayList<Constructor>();
+                constructors = new ArrayList<Constructor>(4);
             }
             if (!Ruby.isSecurityRestricted()) {
                 ctor.setAccessible(true);
@@ -349,7 +350,7 @@ public class JavaClass extends JavaObject {
             this.field = field;
         }
         void install(final RubyModule proxy) {
-            if (proxy.fastGetConstantAt(field.getName()) == null) {
+            if (proxy.getConstantAt(field.getName()) == null) {
                 // TODO: catch exception if constant is already set by other
                 // thread
                 if (!Ruby.isSecurityRestricted()) {
@@ -473,6 +474,14 @@ public class JavaClass extends JavaObject {
             int modifiers = field.getModifiers();
             if (Modifier.isStatic(modifiers)) addField(staticCallbacks, staticNames, field, Modifier.isFinal(modifiers), true);
         }
+
+        // Now add all aliases for the static methods (fields) as appropriate
+        for (Map.Entry<String, NamedInstaller> entry : staticCallbacks.entrySet()) {
+            if (entry.getValue().type == NamedInstaller.STATIC_METHOD && entry.getValue().hasLocalMethod()) {
+                assignAliases((MethodInstaller)entry.getValue(), staticNames);
+            }
+        }
+
         this.staticAssignedNames = staticNames;
         this.staticInstallers = staticCallbacks;        
         this.constantFields = constants;
@@ -500,11 +509,11 @@ public class JavaClass extends JavaObject {
         setupClassMethods(javaClass, staticNames, staticCallbacks, instanceNames, instanceCallbacks);
         setupClassConstructors(javaClass);
         
-        this.staticAssignedNames = staticNames;
-        this.instanceAssignedNames = instanceNames;
-        this.staticInstallers = staticCallbacks;
-        this.instanceInstallers = instanceCallbacks;
-        this.constantFields = constantFields;
+        this.staticAssignedNames = Collections.unmodifiableMap(staticNames);
+        this.instanceAssignedNames = Collections.unmodifiableMap(instanceNames);
+        this.staticInstallers = Collections.unmodifiableMap(staticCallbacks);
+        this.instanceInstallers = Collections.unmodifiableMap(instanceCallbacks);
+        this.constantFields = Collections.unmodifiableList(constantFields);
     }
     
     public void setupProxy(final RubyClass proxy) {
@@ -624,31 +633,30 @@ public class JavaClass extends JavaObject {
         }
     }
 
-    private void installClassConstructors(final RubyClass proxy) {
+    private synchronized void installClassConstructors(final RubyClass proxy) {
         if (constructorInstaller != null) {
             constructorInstaller.install(proxy);
+            constructorInstaller = null;
         }
     }
 
-    private void installClassFields(final RubyClass proxy) {
+    private synchronized void installClassFields(final RubyClass proxy) {
         for (ConstantField field : constantFields) {
             field.install(proxy);
         }
+        constantFields = null;
     }
 
-    private void installClassMethods(final RubyClass proxy) {
+    private synchronized void installClassMethods(final RubyClass proxy) {
         for (NamedInstaller installer : staticInstallers.values()) {
-            if (installer.type == NamedInstaller.STATIC_METHOD && installer.hasLocalMethod()) {
-                assignAliases((MethodInstaller) installer, staticAssignedNames);
-            }
             installer.install(proxy);            
         }
+        staticInstallers = null;
+        
         for (NamedInstaller installer : instanceInstallers.values()) {
-            if (installer.type == NamedInstaller.INSTANCE_METHOD && installer.hasLocalMethod()) {
-                assignAliases((MethodInstaller) installer, instanceAssignedNames);
-            }
             installer.install(proxy);
         }
+        instanceInstallers = null;
     }
 
     private void setupClassConstructors(Class<?> javaClass) {
@@ -748,6 +756,18 @@ public class JavaClass extends JavaObject {
                 installInstanceMethods(instanceCallbacks, javaClass, method, name);
             }
         }
+
+        // now iterate over all installers and make sure they also have appropriate aliases
+        for (Map.Entry<String, NamedInstaller> entry : staticCallbacks.entrySet()) {
+            if (entry.getValue().type == NamedInstaller.STATIC_METHOD && entry.getValue().hasLocalMethod()) {
+                assignAliases((MethodInstaller) entry.getValue(), staticNames);
+            }
+        }
+        for (Map.Entry<String, NamedInstaller> entry : instanceCallbacks.entrySet()) {
+            if (entry.getValue().type == NamedInstaller.INSTANCE_METHOD && entry.getValue().hasLocalMethod()) {
+                assignAliases((MethodInstaller) entry.getValue(), instanceNames);
+            }
+        }
     }
 
     private void installInstanceMethods(Map<String, NamedInstaller> methodCallbacks, Class<?> javaClass, Method method, String name) {
@@ -788,11 +808,8 @@ public class JavaClass extends JavaObject {
             field.install(module);
         }
         for (NamedInstaller installer : staticInstallers.values()) {
-            if (installer.type == NamedInstaller.STATIC_METHOD && installer.hasLocalMethod()) {
-                assignAliases((MethodInstaller)installer,staticAssignedNames);
-            }
             installer.install(module);
-        }        
+        }
         // setup constants for public inner classes
         Class<?>[] classes = getClasses(javaClass);
 
