@@ -63,7 +63,6 @@ import org.jruby.internal.runtime.methods.CallConfiguration;
 import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.java.proxies.JavaProxy;
 import org.jruby.javasupport.proxy.InternalJavaProxy;
-import org.jruby.javasupport.proxy.JavaProxyInvocationHandler;
 import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ClassIndex;
@@ -851,7 +850,7 @@ public class JavaUtil {
             // no matter what method is called on the interface
             RubyClass singletonClass = rubyObject.getSingletonClass();
 
-            singletonClass.addMethod("method_missing", new DynamicMethod(singletonClass, Visibility.PUBLIC, CallConfiguration.NO_FRAME_NO_SCOPE) {
+            singletonClass.addMethod("method_missing", new DynamicMethod(singletonClass, Visibility.PUBLIC, CallConfiguration.FrameNoneScopeNone) {
 
                 @Override
                 public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
@@ -941,6 +940,9 @@ public class JavaUtil {
     }
         
     public static Object coerceBignumToType(RubyBignum bignum, Class target) {
+        if (target == BigInteger.class) {
+            return bignum.getValue();
+        }
         return coerceNumericToType(bignum, target);
     }
     
@@ -983,16 +985,33 @@ public class JavaUtil {
         } else if (target == Object.class) {
             // for Object, default to natural wrapper type
             if (numeric instanceof RubyFixnum) {
-                return Long.valueOf(numeric.getLongValue());
+                long value = numeric.getLongValue();
+                if (value >= Byte.MIN_VALUE && value <= Byte.MAX_VALUE) {
+                    return Byte.valueOf((byte)value);
+                } else if (value >= Short.MIN_VALUE && value <= Short.MAX_VALUE) {
+                    return Short.valueOf((short)value);
+                } else if (value >= Integer.MIN_VALUE && value <= Integer.MAX_VALUE) {
+                    return Integer.valueOf((int)value);
+                } 
+
+                return Long.valueOf(value);
             } else if (numeric instanceof RubyFloat) {
-                return Double.valueOf(numeric.getDoubleValue());
+                double value = numeric.getDoubleValue();
+                if (value == 0.0 || value >= Float.MIN_VALUE && value <= Float.MAX_VALUE || 
+                        value >= -Float.MAX_VALUE && value <= -Float.MIN_VALUE ||
+                        Double.isNaN(value) || value == Double.POSITIVE_INFINITY || value == Double.NEGATIVE_INFINITY) {
+                    return Float.valueOf((float)value);
+                } 
+
+                return Double.valueOf(value);
             } else if (numeric instanceof RubyBignum) {
                 return ((RubyBignum)numeric).getValue();
             }
         }
         throw numeric.getRuntime().newTypeError("could not coerce " + numeric.getMetaClass() + " to " + target);
     }
-    
+
+    // FIXME: This doesn't actually support anything but String
     public static Object coerceStringToType(RubyString string, Class target) {
         try {
             ByteList bytes = string.getByteList();
