@@ -79,8 +79,8 @@ public class JRubyEngine extends BSFEngineImpl {
             }
 
             return JavaEmbedUtils.rubyToJava(evaler.parse(runtime, funcBody.toString(), file, line).run());
-        } catch (StackOverflowError e) {
-            throw runtime.newSystemStackError("stack level too deep");
+        } catch (StackOverflowError soe) {
+            throw context.getRuntime().newSystemStackError("stack level too deep", soe);
         } finally {
             context.postBsfApply();
         }
