@@ -17,6 +17,8 @@ import org.jruby.RubyProc;
 import org.jruby.RubyRegexp;
 import org.jruby.RubyString;
 import org.jruby.RubySymbol;
+import org.jruby.ast.IterNode;
+import org.jruby.ast.Node;
 import org.jruby.ast.util.ArgsUtil;
 import org.jruby.common.IRubyWarnings.ID;
 import org.jruby.evaluator.ASTInterpreter;
@@ -24,6 +26,7 @@ import org.jruby.exceptions.JumpException;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.internal.runtime.methods.CallConfiguration;
 import org.jruby.internal.runtime.methods.DynamicMethod;
+import org.jruby.internal.runtime.methods.UndefinedMethod;
 import org.jruby.internal.runtime.methods.WrapperMethod;
 import org.jruby.javasupport.JavaClass;
 import org.jruby.javasupport.JavaUtil;
@@ -37,14 +40,20 @@ import org.jruby.runtime.CallSite;
 import org.jruby.runtime.CallType;
 import org.jruby.runtime.CallbackFactory;
 import org.jruby.runtime.CompiledBlock;
+import org.jruby.runtime.CompiledBlock19;
 import org.jruby.runtime.CompiledBlockCallback;
+import org.jruby.runtime.CompiledBlockCallback19;
 import org.jruby.runtime.CompiledBlockLight;
+import org.jruby.runtime.CompiledBlockLight19;
 import org.jruby.runtime.CompiledSharedScopeBlock;
 import org.jruby.runtime.DynamicScope;
+import org.jruby.runtime.Interpreted19Block;
+import org.jruby.runtime.InterpretedBlock;
 import org.jruby.runtime.MethodFactory;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.util.ByteList;
 import org.jruby.util.TypeConverter;
 
 /**
@@ -53,6 +62,10 @@ import org.jruby.util.TypeConverter;
  *
  */
 public class RuntimeHelpers {
+    public static CallSite selectAttrAsgnCallSite(IRubyObject receiver, IRubyObject self, CallSite normalSite, CallSite variableSite) {
+        if (receiver == self) return variableSite;
+        return normalSite;
+    }
     public static IRubyObject doAttrAsgn(IRubyObject value, IRubyObject receiver, ThreadContext context, IRubyObject caller, CallSite callSite) {
         callSite.call(context, caller, receiver, value);
         return value;
@@ -90,8 +103,49 @@ public class RuntimeHelpers {
         return args[args.length - 1];
     }
 
-    public static IRubyObject invokeEqqForCaseWhen(IRubyObject receiver, IRubyObject arg, CallSite callSite, ThreadContext context, IRubyObject caller) {
-        return callSite.call(context, caller, receiver, arg);
+    public static boolean invokeEqqForCaseWhen(CallSite callSite, ThreadContext context, IRubyObject caller, IRubyObject arg, IRubyObject[] receivers) {
+        for (int i = 0; i < receivers.length; i++) {
+            IRubyObject receiver = receivers[i];
+            if (invokeEqqForCaseWhen(callSite, context, caller, arg, receiver)) return true;
+        }
+        return false;
+    }
+
+    public static boolean invokeEqqForCaseWhen(CallSite callSite, ThreadContext context, IRubyObject caller, IRubyObject arg, IRubyObject receiver) {
+        IRubyObject result = callSite.call(context, caller, receiver, arg);
+        if (result.isTrue()) return true;
+        return false;
+    }
+
+    public static boolean invokeEqqForCaseWhen(CallSite callSite, ThreadContext context, IRubyObject caller, IRubyObject arg, IRubyObject receiver0, IRubyObject receiver1) {
+        IRubyObject result = callSite.call(context, caller, receiver0, arg);
+        if (result.isTrue()) return true;
+        return invokeEqqForCaseWhen(callSite, context, caller, arg, receiver1);
+    }
+
+    public static boolean invokeEqqForCaseWhen(CallSite callSite, ThreadContext context, IRubyObject caller, IRubyObject arg, IRubyObject receiver0, IRubyObject receiver1, IRubyObject receiver2) {
+        IRubyObject result = callSite.call(context, caller, receiver0, arg);
+        if (result.isTrue()) return true;
+        return invokeEqqForCaseWhen(callSite, context, caller, arg, receiver1, receiver2);
+    }
+
+    public static boolean areAnyTrueForCaselessWhen(IRubyObject[] receivers) {
+        for (int i = 0; i < receivers.length; i++) {
+            if (receivers[i].isTrue()) return true;
+        }
+        return false;
+    }
+
+    public static boolean invokeEqqForCaselessWhen(IRubyObject receiver) {
+        return receiver.isTrue();
+    }
+
+    public static boolean invokeEqqForCaselessWhen(IRubyObject receiver0, IRubyObject receiver1) {
+        return receiver0.isTrue() || receiver1.isTrue();
+    }
+
+    public static boolean invokeEqqForCaselessWhen(IRubyObject receiver0, IRubyObject receiver1, IRubyObject receiver2) {
+        return receiver0.isTrue() || receiver1.isTrue() || receiver2.isTrue();
     }
     
     public static CompiledBlockCallback createBlockCallback(Ruby runtime, Object scriptObject, String closureMethod) {
@@ -101,6 +155,14 @@ public class RuntimeHelpers {
         
         return factory.getBlockCallback(closureMethod, scriptObject);
     }
+
+    public static CompiledBlockCallback19 createBlockCallback19(Ruby runtime, Object scriptObject, String closureMethod) {
+        Class scriptClass = scriptObject.getClass();
+        ClassLoader scriptClassLoader = scriptClass.getClassLoader();
+        CallbackFactory factory = CallbackFactory.createFactory(runtime, scriptClass, scriptClassLoader);
+
+        return factory.getBlockCallback19(closureMethod, scriptObject);
+    }
     
     public static BlockBody createCompiledBlockBody(ThreadContext context, Object scriptObject, String closureMethod, int arity, 
             String[] staticScopeNames, boolean hasMultipleArgsHead, int argsNodeType, boolean light) {
@@ -120,6 +182,25 @@ public class RuntimeHelpers {
                     hasMultipleArgsHead, argsNodeType);
         }
     }
+
+    public static BlockBody createCompiledBlockBody19(ThreadContext context, Object scriptObject, String closureMethod, int arity,
+            String[] staticScopeNames, boolean hasMultipleArgsHead, int argsNodeType, boolean light) {
+        StaticScope staticScope =
+            new BlockStaticScope(context.getCurrentScope().getStaticScope(), staticScopeNames);
+        staticScope.determineModule();
+
+        if (light) {
+            return CompiledBlockLight19.newCompiledBlockLight(
+                    Arity.createArity(arity), staticScope,
+                    createBlockCallback19(context.getRuntime(), scriptObject, closureMethod),
+                    hasMultipleArgsHead, argsNodeType);
+        } else {
+            return CompiledBlock19.newCompiledBlock(
+                    Arity.createArity(arity), staticScope,
+                    createBlockCallback19(context.getRuntime(), scriptObject, closureMethod),
+                    hasMultipleArgsHead, argsNodeType);
+        }
+    }
     
     public static Block createBlock(ThreadContext context, IRubyObject self, BlockBody body) {
         return CompiledBlock.newCompiledClosure(
@@ -127,6 +208,13 @@ public class RuntimeHelpers {
                 self,
                 body);
     }
+
+    public static Block createBlock19(ThreadContext context, IRubyObject self, BlockBody body) {
+        return CompiledBlock19.newCompiledClosure(
+                context,
+                self,
+                body);
+    }
     
     public static IRubyObject runBeginBlock(ThreadContext context, IRubyObject self, String[] staticScopeNames, CompiledBlockCallback callback) {
         StaticScope staticScope = 
@@ -353,63 +441,73 @@ public class RuntimeHelpers {
      * invoking.
      */
     public static IRubyObject invokeSuper(ThreadContext context, IRubyObject self, IRubyObject[] args, Block block) {
+        checkSuperDisabledOrOutOfMethod(context);
         RubyModule klazz = context.getFrameKlazz();
+        String name = context.getFrameName();
 
         RubyClass superClass = findImplementerIfNecessary(self.getMetaClass(), klazz).getSuperClass();
+        DynamicMethod method = superClass != null ? superClass.searchMethod(name) : UndefinedMethod.INSTANCE;
         
-        if (superClass == null) {
-            String name = context.getFrameName(); 
-            return callMethodMissing(context, self, klazz.searchMethod(name), name, args, CallType.SUPER, block);
+        if (method.isUndefined()) {
+            return callMethodMissing(context, self, method, name, args, CallType.SUPER, block);
         }
-        return invokeAs(context, superClass, self, context.getFrameName(), args, block);
+        return method.call(context, self, superClass, name, args, block);
     }
     
     public static IRubyObject invokeSuper(ThreadContext context, IRubyObject self, Block block) {
+        checkSuperDisabledOrOutOfMethod(context);
         RubyModule klazz = context.getFrameKlazz();
+        String name = context.getFrameName();
 
         RubyClass superClass = findImplementerIfNecessary(self.getMetaClass(), klazz).getSuperClass();
-        
-        if (superClass == null) {
-            String name = context.getFrameName(); 
-            return callMethodMissing(context, self, klazz.searchMethod(name), name, CallType.SUPER, block);
+        DynamicMethod method = superClass != null ? superClass.searchMethod(name) : UndefinedMethod.INSTANCE;
+
+        if (method.isUndefined()) {
+            return callMethodMissing(context, self, method, name, CallType.SUPER, block);
         }
-        return invokeAs(context, superClass, self, context.getFrameName(), block);
+        return method.call(context, self, superClass, name, block);
     }
     
     public static IRubyObject invokeSuper(ThreadContext context, IRubyObject self, IRubyObject arg0, Block block) {
+        checkSuperDisabledOrOutOfMethod(context);
         RubyModule klazz = context.getFrameKlazz();
+        String name = context.getFrameName();
 
         RubyClass superClass = findImplementerIfNecessary(self.getMetaClass(), klazz).getSuperClass();
-        
-        if (superClass == null) {
-            String name = context.getFrameName(); 
-            return callMethodMissing(context, self, klazz.searchMethod(name), name, arg0, CallType.SUPER, block);
+        DynamicMethod method = superClass != null ? superClass.searchMethod(name) : UndefinedMethod.INSTANCE;
+
+        if (method.isUndefined()) {
+            return callMethodMissing(context, self, method, name, arg0, CallType.SUPER, block);
         }
-        return invokeAs(context, superClass, self, context.getFrameName(), arg0, block);
+        return method.call(context, self, superClass, name, arg0, block);
     }
     
     public static IRubyObject invokeSuper(ThreadContext context, IRubyObject self, IRubyObject arg0, IRubyObject arg1, Block block) {
+        checkSuperDisabledOrOutOfMethod(context);
         RubyModule klazz = context.getFrameKlazz();
+        String name = context.getFrameName();
 
         RubyClass superClass = findImplementerIfNecessary(self.getMetaClass(), klazz).getSuperClass();
-        
-        if (superClass == null) {
-            String name = context.getFrameName(); 
-            return callMethodMissing(context, self, klazz.searchMethod(name), name, arg0, arg1, CallType.SUPER, block);
+        DynamicMethod method = superClass != null ? superClass.searchMethod(name) : UndefinedMethod.INSTANCE;
+
+        if (method.isUndefined()) {
+            return callMethodMissing(context, self, method, name, arg0, arg1, CallType.SUPER, block);
         }
-        return invokeAs(context, superClass, self, context.getFrameName(), arg0, arg1, block);
+        return method.call(context, self, superClass, name, arg0, arg1, block);
     }
     
     public static IRubyObject invokeSuper(ThreadContext context, IRubyObject self, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Block block) {
+        checkSuperDisabledOrOutOfMethod(context);
         RubyModule klazz = context.getFrameKlazz();
+        String name = context.getFrameName();
 
         RubyClass superClass = findImplementerIfNecessary(self.getMetaClass(), klazz).getSuperClass();
-        
-        if (superClass == null) {
-            String name = context.getFrameName(); 
-            return callMethodMissing(context, self, klazz.searchMethod(name), name, arg0, arg1, arg2, CallType.SUPER, block);
+        DynamicMethod method = superClass != null ? superClass.searchMethod(name) : UndefinedMethod.INSTANCE;
+
+        if (method.isUndefined()) {
+            return callMethodMissing(context, self, method, name, arg0, arg1, arg2, CallType.SUPER, block);
         }
-        return invokeAs(context, superClass, self, context.getFrameName(), arg0, arg1, arg2, block);
+        return method.call(context, self, superClass, name, arg0, arg1, arg2, block);
     }
 
     public static RubyArray ensureRubyArray(IRubyObject value) {
@@ -682,8 +780,6 @@ public class RuntimeHelpers {
     }
     
     public static IRubyObject callZSuper(Ruby runtime, ThreadContext context, Block block, IRubyObject self) {
-        checkSuperDisabledOrOutOfMethod(context);
-
         // Has the method that is calling super received a block argument
         if (!block.isGiven()) block = context.getCurrentFrame().getBlock(); 
         
@@ -724,36 +820,107 @@ public class RuntimeHelpers {
         System.arraycopy(add, 0, newArray, array.length, add.length);
         return newArray;
     }
-    
-    public static IRubyObject isExceptionHandled(RubyException currentException, IRubyObject[] exceptions, Ruby runtime, ThreadContext context, IRubyObject self) {
+
+    public static IRubyObject isExceptionHandled(RubyException currentException, IRubyObject[] exceptions, ThreadContext context) {
         for (int i = 0; i < exceptions.length; i++) {
-            if (!runtime.getModule().isInstance(exceptions[i])) {
-                throw runtime.newTypeError("class or module required for rescue clause");
-            }
-            IRubyObject result = exceptions[i].callMethod(context, "===", currentException);
+            IRubyObject result = isExceptionHandled(currentException, exceptions[i], context);
             if (result.isTrue()) return result;
         }
+        return context.getRuntime().getFalse();
+    }
+
+    public static IRubyObject isExceptionHandled(RubyException currentException, IRubyObject exception, ThreadContext context) {
+        Ruby runtime = context.getRuntime();
+        if (!runtime.getModule().isInstance(exception)) {
+            throw runtime.newTypeError("class or module required for rescue clause");
+        }
+        IRubyObject result = invoke(context, exception, "===", currentException);
+        if (result.isTrue()) return result;
         return runtime.getFalse();
     }
     
-    public static IRubyObject isJavaExceptionHandled(Exception currentException, IRubyObject[] exceptions, Ruby runtime, ThreadContext context, IRubyObject self) {
+    public static IRubyObject isExceptionHandled(RubyException currentException, IRubyObject exception0, IRubyObject exception1, ThreadContext context) {
+        IRubyObject result = isExceptionHandled(currentException, exception0, context);
+        if (result.isTrue()) return result;
+        return isExceptionHandled(currentException, exception1, context);
+    }
+
+    public static IRubyObject isExceptionHandled(RubyException currentException, IRubyObject exception0, IRubyObject exception1, IRubyObject exception2, ThreadContext context) {
+        IRubyObject result = isExceptionHandled(currentException, exception0, context);
+        if (result.isTrue()) return result;
+        return isExceptionHandled(currentException, exception1, exception2, context);
+    }
+
+    private static boolean checkJavaException(Exception currentException, IRubyObject exception) {
+        if (exception instanceof RubyClass) {
+            RubyClass rubyClass = (RubyClass)exception;
+            JavaClass javaClass = (JavaClass)rubyClass.fastGetInstanceVariable("@java_class");
+            if (javaClass != null) {
+                Class cls = javaClass.javaClass();
+                if (cls.isInstance(currentException)) {
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+    
+    public static IRubyObject isJavaExceptionHandled(Exception currentException, IRubyObject[] exceptions, ThreadContext context) {
         if (currentException instanceof RaiseException) {
-            return isExceptionHandled(((RaiseException)currentException).getException(), exceptions, runtime, context, self);
+            return isExceptionHandled(((RaiseException)currentException).getException(), exceptions, context);
         } else {
             for (int i = 0; i < exceptions.length; i++) {
-                if (exceptions[i] instanceof RubyClass) {
-                    RubyClass rubyClass = (RubyClass)exceptions[i];
-                    JavaClass javaClass = (JavaClass)rubyClass.fastGetInstanceVariable("@java_class");
-                    if (javaClass != null) {
-                        Class cls = javaClass.javaClass();
-                        if (cls.isInstance(currentException)) {
-                            return runtime.getTrue();
-                        }
-                    }
+                if (checkJavaException(currentException, exceptions[0])) {
+                    return context.getRuntime().getTrue();
                 }
             }
 
-            return runtime.getFalse();
+            return context.getRuntime().getFalse();
+        }
+    }
+
+    public static IRubyObject isJavaExceptionHandled(Exception currentException, IRubyObject exception, ThreadContext context) {
+        if (currentException instanceof RaiseException) {
+            return isExceptionHandled(((RaiseException)currentException).getException(), exception, context);
+        } else {
+            if (checkJavaException(currentException, exception)) {
+                return context.getRuntime().getTrue();
+            }
+
+            return context.getRuntime().getFalse();
+        }
+    }
+
+    public static IRubyObject isJavaExceptionHandled(Exception currentException, IRubyObject exception0, IRubyObject exception1, ThreadContext context) {
+        if (currentException instanceof RaiseException) {
+            return isExceptionHandled(((RaiseException)currentException).getException(), exception0, exception1, context);
+        } else {
+            if (checkJavaException(currentException, exception0)) {
+                return context.getRuntime().getTrue();
+            }
+            if (checkJavaException(currentException, exception1)) {
+                return context.getRuntime().getTrue();
+            }
+
+            return context.getRuntime().getFalse();
+        }
+    }
+
+    public static IRubyObject isJavaExceptionHandled(Exception currentException, IRubyObject exception0, IRubyObject exception1, IRubyObject exception2, ThreadContext context) {
+        if (currentException instanceof RaiseException) {
+            return isExceptionHandled(((RaiseException)currentException).getException(), exception0, exception1, exception2, context);
+        } else {
+            if (checkJavaException(currentException, exception0)) {
+                return context.getRuntime().getTrue();
+            }
+            if (checkJavaException(currentException, exception1)) {
+                return context.getRuntime().getTrue();
+            }
+            if (checkJavaException(currentException, exception2)) {
+                return context.getRuntime().getTrue();
+            }
+
+            return context.getRuntime().getFalse();
         }
     }
 
@@ -804,7 +971,36 @@ public class RuntimeHelpers {
     }
     
     public static RubyArray createSubarray(IRubyObject[] input, Ruby runtime, int start) {
-        return RubyArray.newArrayNoCopy(runtime, input, start);
+        if (start >= input.length) {
+            return RubyArray.newEmptyArray(runtime);
+        } else {
+            return RubyArray.newArrayNoCopy(runtime, input, start);
+        }
+    }
+
+    public static RubyArray createSubarray(IRubyObject[] input, Ruby runtime, int start, int exclude) {
+        int length = input.length - exclude - start;
+        if (length <= 0) {
+            return RubyArray.newEmptyArray(runtime);
+        } else {
+            return RubyArray.newArrayNoCopy(runtime, input, start, length);
+        }
+    }
+
+    public static IRubyObject elementOrNull(IRubyObject[] input, int element) {
+        if (element >= input.length) {
+            return null;
+        } else {
+            return input[element];
+        }
+    }
+
+    public static IRubyObject elementOrNil(IRubyObject[] input, int element, IRubyObject nil) {
+        if (element >= input.length) {
+            return nil;
+        } else {
+            return input[element];
+        }
     }
     
     public static RubyBoolean isWhenTriggered(IRubyObject expression, IRubyObject expressionsObject, ThreadContext context) {
@@ -825,6 +1021,10 @@ public class RuntimeHelpers {
     public static IRubyObject setConstantInModule(IRubyObject module, IRubyObject value, String name, ThreadContext context) {
         return context.setConstantInModule(name, module, value);
     }
+
+    public static IRubyObject setConstantInCurrent(IRubyObject value, ThreadContext context, String name) {
+        return context.setConstantInCurrent(name, value);
+    }
     
     public static IRubyObject retryJump() {
         throw JumpException.RETRY_JUMP;
@@ -867,6 +1067,26 @@ public class RuntimeHelpers {
     public static IRubyObject[] constructObjectArray(IRubyObject one, IRubyObject two, IRubyObject three, IRubyObject four, IRubyObject five) {
         return new IRubyObject[] {one, two, three, four, five};
     }
+
+    public static RubyArray constructRubyArray(Ruby runtime, IRubyObject one) {
+        return RubyArray.newArrayLight(runtime, one);
+    }
+
+    public static RubyArray constructRubyArray(Ruby runtime, IRubyObject one, IRubyObject two) {
+        return RubyArray.newArrayLight(runtime, one, two);
+    }
+
+    public static RubyArray constructRubyArray(Ruby runtime, IRubyObject one, IRubyObject two, IRubyObject three) {
+        return RubyArray.newArrayLight(runtime, one, two, three);
+    }
+
+    public static RubyArray constructRubyArray(Ruby runtime, IRubyObject one, IRubyObject two, IRubyObject three, IRubyObject four) {
+        return RubyArray.newArrayLight(runtime, one, two, three, four);
+    }
+
+    public static RubyArray constructRubyArray(Ruby runtime, IRubyObject one, IRubyObject two, IRubyObject three, IRubyObject four, IRubyObject five) {
+        return RubyArray.newArrayLight(runtime, one, two, three, four, five);
+    }
     
     public static String[] constructStringArray(String one) {
         return new String[] {one};
@@ -1229,11 +1449,35 @@ public class RuntimeHelpers {
         return rubyClass;
     }
     
-    public static IRubyObject arrayEntryOrNil(RubyArray array, IRubyObject nil, int index) {
+    public static IRubyObject arrayEntryOrNil(RubyArray array, int index) {
         if (index < array.getLength()) {
-            return array.entry(index);
+            return array.eltInternal(index);
         } else {
-            return nil;
+            return array.getRuntime().getNil();
+        }
+    }
+
+    public static IRubyObject arrayEntryOrNilZero(RubyArray array) {
+        if (0 < array.getLength()) {
+            return array.eltInternal(0);
+        } else {
+            return array.getRuntime().getNil();
+        }
+    }
+
+    public static IRubyObject arrayEntryOrNilOne(RubyArray array) {
+        if (1 < array.getLength()) {
+            return array.eltInternal(1);
+        } else {
+            return array.getRuntime().getNil();
+        }
+    }
+
+    public static IRubyObject arrayEntryOrNilTwo(RubyArray array) {
+        if (2 < array.getLength()) {
+            return array.eltInternal(2);
+        } else {
+            return array.getRuntime().getNil();
         }
     }
     
@@ -1258,19 +1502,90 @@ public class RuntimeHelpers {
     public static IRubyObject setGlobalVariable(IRubyObject value, Ruby runtime, String name) {
         return runtime.getGlobalVariables().set(name, value);
     }
-    
+
     public static IRubyObject getInstanceVariable(IRubyObject self, Ruby runtime, String internedName) {
-        IRubyObject result;
-        
-        if ((result = self.getInstanceVariables().fastGetInstanceVariable(internedName)) == null) {
-            runtime.getWarnings().warning(ID.IVAR_NOT_INITIALIZED, "instance variable " + internedName + " not initialized");
-            return runtime.getNil();
-        }
-        
-        return result;
+        IRubyObject result = self.getInstanceVariables().fastGetInstanceVariable(internedName);
+        if (result != null) return result;
+        if (runtime.isVerbose()) warnAboutUninitializedIvar(runtime, internedName);
+        return runtime.getNil();
     }
-    
+
+    private static void warnAboutUninitializedIvar(Ruby runtime, String internedName) {
+        runtime.getWarnings().warning(ID.IVAR_NOT_INITIALIZED, "instance variable " + internedName + " not initialized");
+    }
+
     public static IRubyObject setInstanceVariable(IRubyObject value, IRubyObject self, String name) {
         return self.getInstanceVariables().fastSetInstanceVariable(name, value);
     }
+
+    public static RubyProc newLiteralLambda(ThreadContext context, Block block, IRubyObject self) {
+        return RubyProc.newProc(context.getRuntime(), block, Block.Type.LAMBDA);
+    }
+
+    public static void fillNil(IRubyObject[]arr, int from, int to, Ruby runtime) {
+        IRubyObject nils[] = runtime.getNilPrefilledArray();
+        int i;
+
+        for (i = from; i + Ruby.NIL_PREFILLED_ARRAY_SIZE < to; i += Ruby.NIL_PREFILLED_ARRAY_SIZE) {
+            System.arraycopy(nils, 0, arr, i, Ruby.NIL_PREFILLED_ARRAY_SIZE);
+        }
+        System.arraycopy(nils, 0, arr, i, to - i);
+    }
+
+    public static void fillNil(IRubyObject[]arr, Ruby runtime) {
+        fillNil(arr, 0, arr.length, runtime);
+    }
+
+    public static boolean isFastSwitchableString(IRubyObject str) {
+        return str instanceof RubyString;
+    }
+
+    public static boolean isFastSwitchableSingleCharString(IRubyObject str) {
+        return str instanceof RubyString && ((RubyString)str).getByteList().length() == 1;
+    }
+
+    public static int getFastSwitchString(IRubyObject str) {
+        ByteList byteList = ((RubyString)str).getByteList();
+        return byteList.hashCode();
+    }
+
+    public static int getFastSwitchSingleCharString(IRubyObject str) {
+        ByteList byteList = ((RubyString)str).getByteList();
+        return byteList.get(0);
+    }
+
+    public static boolean isFastSwitchableSymbol(IRubyObject sym) {
+        return sym instanceof RubySymbol;
+    }
+
+    public static boolean isFastSwitchableSingleCharSymbol(IRubyObject sym) {
+        return sym instanceof RubySymbol && ((RubySymbol)sym).asJavaString().length() == 1;
+    }
+
+    public static int getFastSwitchSymbol(IRubyObject sym) {
+        String str = ((RubySymbol)sym).asJavaString();
+        return str.hashCode();
+    }
+
+    public static int getFastSwitchSingleCharSymbol(IRubyObject sym) {
+        String str = ((RubySymbol)sym).asJavaString();
+        return (int)str.charAt(0);
+    }
+
+    public static Block getBlock(ThreadContext context, IRubyObject self, Node node) {
+        IterNode iter = (IterNode)node;
+        iter.getScope().determineModule();
+
+        // Create block for this iter node
+        // FIXME: We shouldn't use the current scope if it's not actually from the same hierarchy of static scopes
+        if (iter.getBlockBody() instanceof InterpretedBlock) {
+            return InterpretedBlock.newInterpretedClosure(context, iter.getBlockBody(), self);
+        } else {
+            return Interpreted19Block.newInterpretedClosure(context, iter.getBlockBody(), self);
+        }
+    }
+
+    public static Block getBlock(Ruby runtime, ThreadContext context, IRubyObject self, Node node, Block aBlock) {
+        return RuntimeHelpers.getBlockFromBlockPassBody(runtime, node.interpret(runtime, context, self, aBlock), aBlock);
+    }
 }
