@@ -407,7 +407,7 @@ public class RubyYaccLexer {
 
     private boolean isBEG() {
         return lex_state == LexState.EXPR_BEG || lex_state == LexState.EXPR_MID ||
-                (!isOneEight && (lex_state == LexState.EXPR_VALUE || lex_state == LexState.EXPR_CLASS));
+                lex_state == LexState.EXPR_CLASS || (!isOneEight && lex_state == LexState.EXPR_VALUE);
     }
 
     private boolean isARG() {
@@ -618,7 +618,36 @@ public class RubyYaccLexer {
     }
     
     private void arg_ambiguous() {
-        warnings.warning(ID.AMBIGUOUS_ARGUMENT, getPosition(), "Ambiguous first argument; make sure.");
+        if (warnings.isVerbose()) warnings.warning(ID.AMBIGUOUS_ARGUMENT, getPosition(), "Ambiguous first argument; make sure.");
+    }
+
+    private int magicCommentMarker(String str, int begin) {
+        int i = begin;
+        int len = str.length();
+
+        while (i < len) {
+            switch (str.charAt(i)) {
+                case '-':
+                    if (str.charAt(i - 1) == '*' && str.charAt(i - 2) == '-') return i + 1;
+                    i += 2;
+                    break;
+                case '*':
+                    if (i + 1 >= len) return 0;
+
+                    if (str.charAt(i + 1) != '-') {
+                        i += 4;
+                    } else if (str.charAt(i - 1) != '-') {
+                        i += 2;
+                    } else {
+                        return i + 2;
+                    }
+                    break;
+                default:
+                    i += 3;
+                    break;
+            }
+        }
+        return 0;
     }
 
     /**
@@ -880,6 +909,8 @@ public class RubyYaccLexer {
                     }
                 }
 
+                if (c == -1) return EOF;
+
                 src.unread(c);
                 getPosition();
 
@@ -1100,7 +1131,7 @@ public class RubyYaccLexer {
         //a wrong position if the "inclusive" flag is not set.
         ISourcePosition tmpPosition = getPosition();
         if (isARG() && spaceSeen && !Character.isWhitespace(c)) {
-            warnings.warning(ID.ARGUMENT_AS_PREFIX, tmpPosition, "`&' interpreted as argument prefix", "&");
+            if (warnings.isVerbose()) warnings.warning(ID.ARGUMENT_AS_PREFIX, tmpPosition, "`&' interpreted as argument prefix", "&");
             c = Tokens.tAMPER;
         } else if (isBEG()) {
             c = Tokens.tAMPER;
@@ -1518,7 +1549,7 @@ public class RubyYaccLexer {
                         if (state != LexState.EXPR_CMDARG && cmdArgumentState.isInState()) {
                             return Tokens.kDO_BLOCK;
                         }
-                        if (state == LexState.EXPR_ENDARG || state == LexState.EXPR_BEG) return Tokens.kDO_BLOCK;
+                        if (state == LexState.EXPR_ENDARG || (!isOneEight && state == LexState.EXPR_BEG)) return Tokens.kDO_BLOCK;
                         return Tokens.kDO;
                     }
                 }
@@ -1532,7 +1563,7 @@ public class RubyYaccLexer {
         }
 
         if (isBEG() || lex_state == LexState.EXPR_DOT || isARG()) {
-            lex_state = commandState ? LexState.EXPR_CMDARG : LexState.EXPR_ARG; 
+            lex_state = commandState ? LexState.EXPR_CMDARG : LexState.EXPR_ARG;
         } else {
             lex_state = LexState.EXPR_END;
         }
@@ -1847,7 +1878,15 @@ public class RubyYaccLexer {
         
         c &= 0xff;
         lex_state = LexState.EXPR_END;
-        yaccValue = new FixnumNode(getPosition(), c);
+        if (isOneEight) {
+            yaccValue = new FixnumNode(getPosition(), c);
+        } else {
+            // TODO: this isn't handling multibyte yet
+            ByteList oneCharBL = new ByteList(1);
+            oneCharBL.append(c);
+            yaccValue = new StrNode(getPosition(), oneCharBL);
+        }
+        // TODO: This should be something else like a tCHAR
         return Tokens.tINTEGER;
     }
     
@@ -1935,7 +1974,7 @@ public class RubyYaccLexer {
         default:
             src.unread(c);
             if (isARG() && spaceSeen && !Character.isWhitespace(c)) {
-                warnings.warning(ID.ARGUMENT_AS_PREFIX, getPosition(), "`*' interpreted as argument prefix", "*");
+                if (warnings.isVerbose()) warnings.warning(ID.ARGUMENT_AS_PREFIX, getPosition(), "`*' interpreted as argument prefix", "*");
                 c = Tokens.tSTAR;
             } else if (isBEG()) {
                 c = Tokens.tSTAR;
