@@ -51,7 +51,8 @@ public class SyntaxException extends RuntimeException {
         STRING_MARKER_MISSING("STRING_MARKER_MISSING"), 
         STRING_UNKNOWN_TYPE("STRING_UNKNOWN_TYPE"), 
         TRAILING_UNDERSCORE_IN_NUMBER("TRAILING_UNDERSCORE_IN_NUMBER"),
-        DUBY_EXTENSIONS_OFF("DUBY_EXTENSIONS_OFF");
+        DUBY_EXTENSIONS_OFF("DUBY_EXTENSIONS_OFF"),
+        BLOCK_GIVEN_TO_YIELD("BLOCK_GIVEN_TO_YIELD");
         
         private String id;
         
