@@ -60,23 +60,6 @@ public class ThreadLibrary implements Library {
         SizedQueue.setup(runtime);
     }
 
-    static boolean wait_timeout(IRubyObject o, Double timeout) throws InterruptedException {
-        if ( timeout != null ) {
-            long delay_ns = (long)(timeout * 1000000000.0);
-            long start_ns = System.nanoTime();
-            if (delay_ns > 0) {
-                long delay_ms = delay_ns / 1000000;
-                int delay_ns_remainder = (int)( delay_ns % 1000000 );
-                o.wait(delay_ms, delay_ns_remainder);
-            }
-            long end_ns = System.nanoTime();
-            return ( end_ns - start_ns ) <= delay_ns;
-        } else {
-            o.wait();
-            return true;
-        }
-    }
-
     @JRubyClass(name="Mutex")
     public static class Mutex extends RubyObject {
         private RubyThread owner = null;
@@ -120,23 +103,31 @@ public class ThreadLibrary implements Library {
             return context.getRuntime().getTrue();
         }
 
-        @JRubyMethod
+        @JRubyMethod(frame = true)
         public IRubyObject lock(ThreadContext context) throws InterruptedException {
             //if (Thread.interrupted()) {
             //    throw new InterruptedException();
             //}
-            synchronized (this) {
-                try {
-                    while ( owner != null ) {
-                        wait();
+            try {
+                context.getThread().enterSleep();
+                synchronized (this) {
+                    try {
+                        if (owner == context.getThread()) {
+                            throw context.getRuntime().newThreadError("Mutex relocking by same thread");
+                        }
+                        while ( owner != null ) {
+                            wait();
+                        }
+                        owner = context.getThread();
+                    } catch (InterruptedException ex) {
+                        if ( owner == null ) {
+                            notify();
+                        }
+                        throw ex;
                     }
-                    owner = context.getThread();
-                } catch (InterruptedException ex) {
-                    if ( owner == null ) {
-                        notify();
-                    }
-                    throw ex;
                 }
+            } finally {
+                context.getThread().exitSleep();
             }
             return this;
         }
@@ -213,7 +204,7 @@ public class ThreadLibrary implements Library {
                 synchronized (this) {
                     mutex.unlock(context);
                     try {
-                        success = ThreadLibrary.wait_timeout(this, timeout);
+                        success = context.getThread().wait_timeout(this, timeout);
                     } finally {
                         // An interrupt or timeout may have caused us to miss
                         // a notify that we consumed, so do another notify in
@@ -237,13 +228,13 @@ public class ThreadLibrary implements Library {
         @JRubyMethod
         public synchronized IRubyObject broadcast(ThreadContext context) {
             notifyAll();
-            return context.getRuntime().getNil();
+            return this;
         }
 
         @JRubyMethod
         public synchronized IRubyObject signal(ThreadContext context) {
             notify();
-            return context.getRuntime().getNil();
+            return this;
         }
     }
 
