@@ -75,6 +75,7 @@ import org.jruby.ast.CallThreeArgNode;
 import org.jruby.ast.CallTwoArgBlockNode;
 import org.jruby.ast.CallTwoArgBlockPassNode;
 import org.jruby.ast.CallTwoArgNode;
+import org.jruby.ast.CaseNode;
 import org.jruby.ast.ClassVarAsgnNode;
 import org.jruby.ast.ClassVarDeclNode;
 import org.jruby.ast.ClassVarNode;
@@ -147,6 +148,7 @@ import org.jruby.ast.SuperNode;
 import org.jruby.ast.SymbolNode;
 import org.jruby.ast.TrueNode;
 import org.jruby.ast.WhenNode;
+import org.jruby.ast.WhenOneArgNode;
 import org.jruby.ast.YieldNode;
 import org.jruby.ast.types.ILiteralNode;
 import org.jruby.ast.types.INameNode;
@@ -230,7 +232,7 @@ public class ParserSupport {
      * @return an AST node representing this new variable
      */
     public Node gettable2(Node node) {
-        switch (node.nodeId) {
+        switch (node.getNodeType()) {
         case DASGNNODE: // LOCALVAR
         case LOCALASGNNODE:
             return currentScope.declare(node.getPosition(), ((INameNode) node).getName());
@@ -536,7 +538,7 @@ public class ParserSupport {
         breakLoop: do {
             if (node == null) return false;
 
-            switch (node.nodeId) {
+            switch (node.getNodeType()) {
             case NEWLINENODE:
                 node = ((NewlineNode) node).getNextNode();
                 continue breakLoop;
@@ -556,7 +558,7 @@ public class ParserSupport {
     }
 
     public void warningUnlessEOption(ID id, Node node, String message) {
-        if (!configuration.isInlineSource()) {
+        if (warnings.isVerbose() && !configuration.isInlineSource()) {
             warnings.warning(id, node.getPosition(), message);
         }
     }
@@ -567,7 +569,7 @@ public class ParserSupport {
      * @return true if an expression, false otherwise
      */
     public void checkExpression(Node node) {
-        if (!isExpression(node)) {
+        if (warnings.isVerbose() && !isExpression(node)) {
             warnings.warning(ID.VOID_VALUE_EXPRESSION, node.getPosition(), "void value expression");
         }
     }
@@ -588,7 +590,7 @@ public class ParserSupport {
         do {
             if (node == null) return true;
             
-            switch (node.nodeId) {
+            switch (node.getNodeType()) {
             case BEGINNODE:
                 node = ((BeginNode) node).getBodyNode();
                 break;
@@ -643,7 +645,7 @@ public class ParserSupport {
         uselessLoop: do {
             if (node == null) return;
             
-            switch (node.nodeId) {
+            switch (node.getNodeType()) {
             case NEWLINENODE:
                 node = ((NewlineNode) node).getNextNode();
                 continue uselessLoop;
@@ -737,7 +739,7 @@ public class ParserSupport {
         Node rightNode = null;
 
         // FIXME: DSTR,EVSTR,STR: warning "string literal in condition"
-        switch(node.nodeId) {
+        switch(node.getNodeType()) {
         case DREGEXPNODE: {
             ISourcePosition position = node.getPosition();
 
@@ -824,10 +826,87 @@ public class ParserSupport {
         
         return new OrNode(union(left, right), makeNullNil(left), makeNullNil(right));
     }
+
+    /**
+     * Ok I admit that this is somewhat ugly.  We post-process a chain of when nodes and analyze
+     * them to re-insert them back into our new CaseNode the way we want.  The grammar is being
+     * difficult and until I go back into the depths of that this is where things are.
+     *
+     * @param expression of the case node (e.g. case foo)
+     * @param firstWhenNode first when (which could also be the else)
+     * @return a new case node
+     */
+    public CaseNode newCaseNode(ISourcePosition position, Node expression, Node firstWhenNode) {
+        ArrayNode cases = new ArrayNode(firstWhenNode != null ? firstWhenNode.getPosition() : position);
+        CaseNode caseNode = new CaseNode(position, expression, cases);
+
+        for (Node current = firstWhenNode; current != null; current = ((WhenNode) current).getNextCase()) {
+            if (current instanceof WhenOneArgNode) {
+                cases.add(current);
+            } else if (current instanceof WhenNode) {
+                simplifyMultipleArgumentWhenNodes((WhenNode) current, cases);
+            } else {
+                caseNode.setElseNode(current);
+                break;
+            }
+        }
+
+        return caseNode;
+    }
+
+    /*
+     * This method exists for us to break up multiple expression when nodes (e.g. when 1,2,3:)
+     * into individual whenNodes.  The primary reason for this is to ensure lazy evaluation of
+     * the arguments (when foo,bar,gar:) to prevent side-effects.  In the old code this was done
+     * using nested when statements, which was awful for interpreter and compilation.
+     *
+     * Notes: This has semantic equivalence but will not be lexically equivalent.  Compiler
+     * needs to detect same bodies to simplify bytecode generated.
+     */
+    private void simplifyMultipleArgumentWhenNodes(WhenNode sourceWhen, ArrayNode cases) {
+        Node expressionNodes = sourceWhen.getExpressionNodes();
+
+        if (expressionNodes instanceof SplatNode || expressionNodes instanceof ArgsCatNode) {
+            cases.add(sourceWhen);
+            return;
+        }
+
+        if (expressionNodes instanceof ListNode) {
+            ListNode list = (ListNode) expressionNodes;
+            ISourcePosition position = sourceWhen.getPosition();
+            Node bodyNode = sourceWhen.getBodyNode();
+
+            for (int i = 0; i < list.size(); i++) {
+                Node expression = list.get(i);
+
+                if (expression instanceof SplatNode || expression instanceof ArgsCatNode) {
+                    cases.add(new WhenNode(position, expression, bodyNode, null));
+                } else {
+                    cases.add(new WhenOneArgNode(position, expression, bodyNode, null));
+                }
+            }
+        } else {
+            cases.add(sourceWhen);
+        }
+    }
     
     public WhenNode newWhenNode(ISourcePosition position, Node expressionNodes, Node bodyNode, Node nextCase) {
         if (bodyNode == null) bodyNode = NilImplicitNode.NIL;
 
+        if (expressionNodes instanceof SplatNode || expressionNodes instanceof ArgsCatNode) {
+            return new WhenNode(position, expressionNodes, bodyNode, nextCase);
+        }
+
+        ListNode list = (ListNode) expressionNodes;
+
+        if (list.size() == 1) {
+            Node element = list.get(0);
+            
+            if (!(element instanceof SplatNode)) {
+                return new WhenOneArgNode(position, element, bodyNode, nextCase);
+            }
+        }
+
         return new WhenNode(position, expressionNodes, bodyNode, nextCase);
     }
 
