@@ -33,6 +33,7 @@
 package org.jruby.runtime;
 
 import org.jruby.RubyArray;
+import org.jruby.RubyLocalJumpError;
 import org.jruby.RubyModule;
 import org.jruby.ast.IterNode;
 import org.jruby.ast.MultipleAsgnNode;
@@ -61,7 +62,7 @@ public abstract class BlockBody implements JumpTarget {
     public IRubyObject call(ThreadContext context, IRubyObject[] args, Binding binding, Block.Type type) {
         args = prepareArgumentsForCall(context, args, type);
 
-        return yield(context, context.getRuntime().newArrayNoCopy(args), null, null, true, binding, type);
+        return yield(context, RubyArray.newArrayNoCopy(context.getRuntime(), args), null, null, true, binding, type);
     }
 
     // This should only be called by 1.8 (1.9 subclasses this to handle unusedBlock).
@@ -73,6 +74,11 @@ public abstract class BlockBody implements JumpTarget {
     public int getArgumentType() {
         return argumentType;
     }
+
+    public abstract IRubyObject yieldSpecific(ThreadContext context, Binding binding, Block.Type type);
+    public abstract IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, Binding binding, Block.Type type);
+    public abstract IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, IRubyObject arg1, Binding binding, Block.Type type);
+    public abstract IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Binding binding, Block.Type type);
     
     public abstract IRubyObject yield(ThreadContext context, IRubyObject value, Binding binding, Block.Type type);
     
@@ -139,7 +145,7 @@ public abstract class BlockBody implements JumpTarget {
             if (argumentType == ARRAY && args.length != 1) {
                 context.getRuntime().getWarnings().warn(ID.MULTIPLE_VALUES_FOR_BLOCK, "multiple values for a block parameter (" + args.length + " for " + arity().getValue() + ")");
                 if (args.length == 0) {
-                    args = new IRubyObject[] {context.getRuntime().getNil()};
+                    args = context.getRuntime().getSingleNilArray();
                 } else {
                     args = new IRubyObject[] {context.getRuntime().newArrayNoCopy(args)};
                 }
@@ -154,9 +160,9 @@ public abstract class BlockBody implements JumpTarget {
     
     public static NodeType getArgumentTypeWackyHack(IterNode iterNode) {
         NodeType argsNodeId = null;
-        if (iterNode.getVarNode() != null && iterNode.getVarNode().nodeId != NodeType.ZEROARGNODE) {
+        if (iterNode.getVarNode() != null && iterNode.getVarNode().getNodeType() != NodeType.ZEROARGNODE) {
             // if we have multiple asgn with just *args, need a special type for that
-            argsNodeId = iterNode.getVarNode().nodeId;
+            argsNodeId = iterNode.getVarNode().getNodeType();
             if (argsNodeId == NodeType.MULTIPLEASGNNODE) {
                 MultipleAsgnNode multipleAsgnNode = (MultipleAsgnNode)iterNode.getVarNode();
                 if (multipleAsgnNode.getHeadNode() == null && multipleAsgnNode.getArgsNode() != null) {
@@ -172,17 +178,44 @@ public abstract class BlockBody implements JumpTarget {
     public static final BlockBody NULL_BODY = new BlockBody(ZERO_ARGS) {
         @Override
         public IRubyObject call(ThreadContext context, IRubyObject[] args, Binding binding, Type type) {
-            return null;
+            throw context.getRuntime().newLocalJumpError(
+                    RubyLocalJumpError.Reason.NOREASON, context.getRuntime().newArrayNoCopy(args), "yield called out of block");
+        }
+
+        @Override
+        public IRubyObject yieldSpecific(ThreadContext context, Binding binding, Type type) {
+            throw context.getRuntime().newLocalJumpError(
+                    RubyLocalJumpError.Reason.NOREASON, context.getRuntime().getNil(), "yield called out of block");
+        }
+
+        @Override
+        public IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, Binding binding, Type type) {
+            throw context.getRuntime().newLocalJumpError(
+                    RubyLocalJumpError.Reason.NOREASON, context.getRuntime().newArrayNoCopyLight(arg0), "yield called out of block");
+        }
+
+        @Override
+        public IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, IRubyObject arg1, Binding binding, Type type) {
+            throw context.getRuntime().newLocalJumpError(
+                    RubyLocalJumpError.Reason.NOREASON, context.getRuntime().newArrayNoCopyLight(arg0, arg1), "yield called out of block");
+        }
+
+        @Override
+        public IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Binding binding, Type type) {
+            throw context.getRuntime().newLocalJumpError(
+                    RubyLocalJumpError.Reason.NOREASON, context.getRuntime().newArrayNoCopyLight(arg0, arg1, arg2), "yield called out of block");
         }
 
         @Override
         public IRubyObject yield(ThreadContext context, IRubyObject value, Binding binding, Type type) {
-            return null;
+            throw context.getRuntime().newLocalJumpError(
+                    RubyLocalJumpError.Reason.NOREASON, value, "yield called out of block");
         }
 
         @Override
         public IRubyObject yield(ThreadContext context, IRubyObject value, IRubyObject self, RubyModule klass, boolean aValue, Binding binding, Type type) {
-            return null;
+            throw context.getRuntime().newLocalJumpError(
+                    RubyLocalJumpError.Reason.NOREASON, value, "yield called out of block");
         }
         
         @Override
