@@ -32,11 +32,12 @@
 package org.jruby.runtime;
 
 import org.jruby.Ruby;
+import org.jruby.RubyClass;
 import org.jruby.runtime.callback.Callback;
 import org.jruby.runtime.callback.ReflectionCallbackFactory;
 import org.jruby.runtime.callback.InvocationCallbackFactory;
 import org.jruby.runtime.callback.DumpingInvocationCallbackFactory;
-import org.jruby.util.JRubyClassLoader;
+import org.jruby.util.SafePropertyAccessor;
 
 /**
  * Helper class to build Callback method.
@@ -45,14 +46,23 @@ import org.jruby.util.JRubyClassLoader;
  * will need to be explicitly created.
  **/
 public abstract class CallbackFactory {
+
     public static final Class[] NULL_CLASS_ARRAY = new Class[0];
-    
+
     /**
      * gets an instance method with no arguments.
      * @param method name of the method
      * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
      **/
     public abstract Callback getMethod(String method);
+
+    /**
+     * gets a fast instance method with no arguments.
+     * @param method name of the method
+     * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
+     **/
     public abstract Callback getFastMethod(String method);
 
     /**
@@ -60,8 +70,17 @@ public abstract class CallbackFactory {
      * @param method name of the method
      * @param arg1 the class of the only argument for this method
      * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
      **/
     public abstract Callback getMethod(String method, Class arg1);
+
+    /**
+     * gets a fast instance method with 1 argument.
+     * @param method name of the method
+     * @param arg1 the class of the only argument for this method
+     * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
+     **/
     public abstract Callback getFastMethod(String method, Class arg1);
 
     /**
@@ -70,10 +89,20 @@ public abstract class CallbackFactory {
      * @param arg1 the java class of the first argument for this method
      * @param arg2 the java class of the second argument for this method
      * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
      **/
     public abstract Callback getMethod(String method, Class arg1, Class arg2);
+
+    /**
+     * gets a fast instance method with two arguments.
+     * @param method name of the method
+     * @param arg1 the java class of the first argument for this method
+     * @param arg2 the java class of the second argument for this method
+     * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
+     **/
     public abstract Callback getFastMethod(String method, Class arg1, Class arg2);
-    
+
     /**
      * gets an instance method with two arguments.
      * @param method name of the method
@@ -81,16 +110,35 @@ public abstract class CallbackFactory {
      * @param arg2 the java class of the second argument for this method
      * @param arg3 the java class of the second argument for this method
      * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
      **/
     public abstract Callback getMethod(String method, Class arg1, Class arg2, Class arg3);
+
+    /**
+     * gets a fast instance method with two arguments.
+     * @param method name of the method
+     * @param arg1 the java class of the first argument for this method
+     * @param arg2 the java class of the second argument for this method
+     * @param arg3 the java class of the second argument for this method
+     * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
+     **/
     public abstract Callback getFastMethod(String method, Class arg1, Class arg2, Class arg3);
 
     /**
      * gets a singleton (class) method without arguments.
      * @param method name of the method
      * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
      **/
     public abstract Callback getSingletonMethod(String method);
+
+    /**
+     * gets a fast singleton (class) method without arguments.
+     * @param method name of the method
+     * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
+     */
     public abstract Callback getFastSingletonMethod(String method);
 
     /**
@@ -98,81 +146,124 @@ public abstract class CallbackFactory {
      * @param method name of the method
      * @param arg1 the class of the only argument for this method
      * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
      **/
     public abstract Callback getSingletonMethod(String method, Class arg1);
+
+    /**
+     * gets a fast singleton (class) method with 1 argument.
+     * @param method name of the method
+     * @param arg1 the class of the only argument for this method
+     * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
+     **/
     public abstract Callback getFastSingletonMethod(String method, Class arg1);
 
     /**
      * gets a singleton (class) method with 2 arguments.
      * @param method name of the method
      * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
      **/
     public abstract Callback getSingletonMethod(String method, Class arg1, Class arg2);
+
+    /**
+     * gets a fast singleton (class) method with 2 arguments.
+     * @param method name of the method
+     * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
+     **/
     public abstract Callback getFastSingletonMethod(String method, Class arg1, Class arg2);
 
     /**
      * gets a singleton (class) method with 3 arguments.
      * @param method name of the method
      * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
      **/
     public abstract Callback getSingletonMethod(String method, Class arg1, Class arg2, Class arg3);
+
+    /**
+     * gets a fast singleton (class) method with 3 arguments.
+     * @param method name of the method
+     * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
+     **/
     public abstract Callback getFastSingletonMethod(String method, Class arg1, Class arg2, Class arg3);
 
     public abstract Callback getBlockMethod(String method);
-    public abstract CompiledBlockCallback getBlockCallback(String method);
+
+    public abstract CompiledBlockCallback getBlockCallback(String method, Object scriptObject);
+
+    public abstract CompiledBlockCallback19 getBlockCallback19(String method, Object scriptObject);
 
     /**
-    * gets a singleton (class) method with no mandatory argument and some optional arguments.
+     * gets a singleton (class) method with no mandatory argument and some optional arguments.
      * @param method name of the method
-    * @return a CallBack object corresponding to the appropriate method
-    **/
+     * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
+     **/
     public abstract Callback getOptSingletonMethod(String method);
+
+
+    /**
+     * gets a fast singleton (class) method with no mandatory argument and some optional arguments.
+     * @param method name of the method
+     * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
+     **/
     public abstract Callback getFastOptSingletonMethod(String method);
 
     /**
-    * gets an instance method with no mandatory argument and some optional arguments.
+     * gets an instance method with no mandatory argument and some optional arguments.
      * @param method name of the method
-    * @return a CallBack object corresponding to the appropriate method
-    **/
+     * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
+     **/
     public abstract Callback getOptMethod(String method);
+
+    /**
+     * gets a fast instance method with no mandatory argument and some optional arguments.
+     * @param method name of the method
+     * @return a CallBack object corresponding to the appropriate method
+     * @deprecated Callbacks are inefficient; use MethodFactory.
+     **/
     public abstract Callback getFastOptMethod(String method);
 
-    private static boolean reflection = false;
-    private static boolean dumping = false;
-    private static String dumpingPath = null;
+    private static final boolean reflection;
+    private static final boolean dumping;
+    
 
     static {
-       if (Ruby.isSecurityRestricted())
-           reflection = true;
-       else {
-           if(System.getProperty("jruby.reflection") != null && Boolean.getBoolean("jruby.reflection")) {
-               reflection = true;
-           }
-           if(System.getProperty("jruby.dump_invocations") != null) {
-               dumping = true;
-               dumpingPath = System.getProperty("jruby.dump_invocations").toString();
-           }
-       }
+        boolean reflection_ = false;
+        boolean dumping_ = false;
+        if (Ruby.isSecurityRestricted()) {
+            reflection_ = true;
+        } else {
+            if (SafePropertyAccessor.getProperty("jruby.reflection") != null && SafePropertyAccessor.getBoolean("jruby.reflection")) {
+                reflection_ = true;
+            }
+            if (SafePropertyAccessor.getProperty("jruby.dump_invocations") != null) {
+                dumping_ = true;
+            }
+        }
+        reflection = reflection_;
+        dumping = dumping_;
     }
 
     public static CallbackFactory createFactory(Ruby runtime, Class type) {
-        if(reflection) {
-            return new ReflectionCallbackFactory(type);
-        } else if(dumping) {
-            return new DumpingInvocationCallbackFactory(runtime, type, dumpingPath);
-        } else {
-            return new InvocationCallbackFactory(runtime, type, runtime.getJRubyClassLoader());
-        }
+        return createFactory(runtime, type, runtime.getJRubyClassLoader());
     }
 
+    // used by compiler
     public static CallbackFactory createFactory(Ruby runtime, Class type, ClassLoader classLoader) {
-        if(reflection) {
+        if (reflection) {
             return new ReflectionCallbackFactory(type);
-        } else if(dumping) {
-            return new DumpingInvocationCallbackFactory(runtime, type, dumpingPath);
+        } else if (dumping) {
+            return new DumpingInvocationCallbackFactory(runtime, type, classLoader);
         } else {
             // FIXME: No, I don't like it.
-            return new InvocationCallbackFactory(runtime, type, (JRubyClassLoader)classLoader);
+            return new InvocationCallbackFactory(runtime, type, classLoader);
         }
     }
 }
