@@ -206,7 +206,6 @@ public final class Frame implements JumpTarget {
         this.block = block;
         this.visibility = Visibility.PUBLIC;
         this.isBindingFrame = false;
-        this.backrefAndLastline = null;
     }
 
     /**
@@ -227,7 +226,6 @@ public final class Frame implements JumpTarget {
         this.line = line;
         this.visibility = Visibility.PRIVATE;
         this.isBindingFrame = false;
-        this.backrefAndLastline = null;
     }
 
     /**
@@ -247,11 +245,7 @@ public final class Frame implements JumpTarget {
      * @return A new frame with duplicate information to the target frame
      */
     public Frame duplicate() {
-        Frame newFrame = new Frame(this);
-        
-        newFrame.updateFrame(this);
-        
-        return newFrame;
+        return new Frame(this);
     }
 
     /**
@@ -377,6 +371,14 @@ public final class Frame implements JumpTarget {
         this.line = line;
     }
 
+    /**
+     * Set both the file and line
+     */
+    public void setFileAndLine(String file, int line) {
+        this.fileName = file;
+        this.line = line;
+    }
+
     /** 
      * Return class that we are calling against
      * 
