@@ -75,44 +75,18 @@ public class InterpretedBlock extends BlockBody {
     private final Arity arity;
 
     public static Block newInterpretedClosure(ThreadContext context, IterNode iterNode, IRubyObject self) {
-        Frame f = context.getCurrentFrame();
+        Binding binding = context.currentBinding(self);
+        NodeType argsNodeId = getArgumentTypeWackyHack(iterNode);
 
-        return newInterpretedClosure(iterNode,
-                         self,
-                         Arity.procArityOf(iterNode.getVarNode()),
-                         f,
-                         f.getVisibility(),
-                         context.getRubyClass(),
-                         context.getCurrentScope());
+        BlockBody body = new InterpretedBlock(
+                iterNode,
+                Arity.procArityOf(iterNode.getVarNode()),
+                asArgumentType(argsNodeId));
+        return new Block(body, binding);
     }
 
     public static Block newInterpretedClosure(ThreadContext context, BlockBody body, IRubyObject self) {
-        Frame f = context.getCurrentFrame();
-
-        Binding binding = new Binding(self,
-                         f,
-                         f.getVisibility(),
-                         context.getRubyClass(),
-                         context.getCurrentScope());
-        return new Block(body, binding);
-    }
-    
-    public static Block newInterpretedClosure(IterNode iterNode, IRubyObject self, Arity arity, Frame frame,
-            Visibility visibility, RubyModule klass, DynamicScope dynamicScope) {
-        NodeType argsNodeId = getArgumentTypeWackyHack(iterNode);
-        
-        BlockBody body = new InterpretedBlock(
-                iterNode,
-                arity,
-                asArgumentType(argsNodeId));
-        
-        Binding binding = new Binding(
-                self, 
-                frame,
-                visibility,
-                klass,
-                dynamicScope);
-        
+        Binding binding = context.currentBinding(self);
         return new Block(body, binding);
     }
 
@@ -138,6 +112,22 @@ public class InterpretedBlock extends BlockBody {
         binding.getFrame().setVisibility(vis);
         context.postYield(binding, lastFrame);
     }
+
+    public IRubyObject yieldSpecific(ThreadContext context, Binding binding, Block.Type type) {
+        return yield(context, null, binding, type);
+    }
+
+    public IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, Binding binding, Block.Type type) {
+        return yield(context, arg0, binding, type);
+    }
+
+    public IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, IRubyObject arg1, Binding binding, Block.Type type) {
+        return yield(context, context.getRuntime().newArrayNoCopyLight(arg0, arg1), null, null, true, binding, type);
+    }
+
+    public IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Binding binding, Block.Type type) {
+        return yield(context, context.getRuntime().newArrayNoCopyLight(arg0, arg1, arg2), null, null, true, binding, type);
+    }
     
     public IRubyObject yield(ThreadContext context, IRubyObject value, Binding binding, Block.Type type) {
         IRubyObject self = prepareSelf(binding);
@@ -203,8 +193,8 @@ public class InterpretedBlock extends BlockBody {
             } catch (JumpException.RedoJump rj) {
                 context.pollThreadEvents();
                 // do nothing, allow loop to redo
-            } catch (StackOverflowError sfe) {
-                throw context.getRuntime().newSystemStackError("stack level too deep");
+            } catch (StackOverflowError soe) {
+                throw context.getRuntime().newSystemStackError("stack level too deep", soe);
             }
         }
     }
@@ -217,13 +207,13 @@ public class InterpretedBlock extends BlockBody {
     }
     
     private IRubyObject handleNextJump(ThreadContext context, JumpException.NextJump nj, Block.Type type) {
-        return type == Block.Type.LAMBDA ? context.getRuntime().getNil() : (IRubyObject)nj.getValue();
+        return nj.getValue() == null ? context.getRuntime().getNil() : (IRubyObject)nj.getValue();
     }
 
     private void setupBlockArgs(ThreadContext context, Node varNode, IRubyObject value, IRubyObject self) {
         Ruby runtime = context.getRuntime();
         
-        switch (varNode.nodeId) {
+        switch (varNode.getNodeType()) {
         case ZEROARGNODE:
             break;
         case MULTIPLEASGNNODE:
@@ -237,7 +227,7 @@ public class InterpretedBlock extends BlockBody {
     private void setupBlockArg(ThreadContext context, Node varNode, IRubyObject value, IRubyObject self) {
         Ruby runtime = context.getRuntime();
         
-        switch (varNode.nodeId) {
+        switch (varNode.getNodeType()) {
         case ZEROARGNODE:
             return;
         case MULTIPLEASGNNODE:
@@ -281,13 +271,7 @@ public class InterpretedBlock extends BlockBody {
         // captured instances of this block may still be around and we do not want to start
         // overwriting those values when we create a new one.
         // ENEBO: Once we make self, lastClass, and lastMethod immutable we can remove duplicate
-        binding = new Binding(
-                binding.getSelf(),
-                binding.getFrame().duplicate(),
-                binding.getVisibility(),
-                binding.getKlass(), 
-                binding.getDynamicScope());
-        
+        binding = binding.clone();
         return new Block(this, binding);
     }
 
