@@ -36,40 +36,32 @@ import org.jruby.RubyModule;
 import org.jruby.exceptions.JumpException;
 import org.jruby.parser.StaticScope;
 import org.jruby.runtime.builtin.IRubyObject;
-import org.jruby.runtime.callback.Callback;
 
 /**
  *  Internal live representation of a block ({...} or do ... end).
  */
-public class MethodBlock extends BlockBody {
-    
+public abstract class MethodBlock extends BlockBody {
     private final RubyMethod method;
-    private final Callback callback;
     
     private final Arity arity;
      
     // This is a dummy scope; we should find a way to make that more explicit
     private final StaticScope staticScope;
 
-    public static Block createMethodBlock(ThreadContext context, DynamicScope dynamicScope, Callback callback, RubyMethod method, IRubyObject self) {
-        Binding binding = new Binding(self,
-                               context.getCurrentFrame().duplicate(),
-                         context.getCurrentFrame().getVisibility(),
-                         context.getRubyClass(),
-                         dynamicScope);
-        BlockBody body = new MethodBlock(callback, method, dynamicScope.getStaticScope());
-        
+    public static Block createMethodBlock(ThreadContext context, IRubyObject self, DynamicScope dynamicScope, MethodBlock body) {
+        Binding binding = context.currentBinding(self, dynamicScope);
         return new Block(body, binding);
     }
 
-    public MethodBlock(Callback callback, RubyMethod method, StaticScope staticScope) {
+    public MethodBlock(RubyMethod method, StaticScope staticScope) {
         super(BlockBody.SINGLE_RESTARG);
-        this.callback = callback;
         this.method = method;
         this.arity = Arity.createArity((int) method.arity().getLongValue());
         this.staticScope = staticScope;
     }
 
+    public abstract IRubyObject callback(IRubyObject value, IRubyObject method, IRubyObject self, Block block);
+
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject[] args, Binding binding, Block.Type type) {
         return yield(context, context.getRuntime().newArrayNoCopy(args), null, null, true, binding, type);
@@ -82,6 +74,22 @@ public class MethodBlock extends BlockBody {
     protected void post(ThreadContext context, Binding binding, Frame lastFrame) {
         context.postYieldNoScope(lastFrame);
     }
+
+    public IRubyObject yieldSpecific(ThreadContext context, Binding binding, Block.Type type) {
+        return yield(context, null, binding, type);
+    }
+
+    public IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, Binding binding, Block.Type type) {
+        return yield(context, arg0, binding, type);
+    }
+
+    public IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, IRubyObject arg1, Binding binding, Block.Type type) {
+        return yield(context, context.getRuntime().newArrayNoCopyLight(arg0, arg1), null, null, true, binding, type);
+    }
+
+    public IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Binding binding, Block.Type type) {
+        return yield(context, context.getRuntime().newArrayNoCopyLight(arg0, arg1, arg2), null, null, true, binding, type);
+    }
     
     public IRubyObject yield(ThreadContext context, IRubyObject value, Binding binding, Block.Type type) {
         return yield(context, value, null, null, false, binding, type);
@@ -110,7 +118,7 @@ public class MethodBlock extends BlockBody {
             // This while loop is for restarting the block call in case a 'redo' fires.
             while (true) {
                 try {
-                    return callback.execute(value, new IRubyObject[] { method, self }, Block.NULL_BLOCK);
+                    return callback(value, method, self, Block.NULL_BLOCK);
                 } catch (JumpException.RedoJump rj) {
                     context.pollThreadEvents();
                     // do nothing, allow loop to redo
@@ -137,12 +145,7 @@ public class MethodBlock extends BlockBody {
     }
 
     public Block cloneBlock(Binding binding) {
-        // We clone dynamic scope because this will be a new instance of a block.  Any previously
-        // captured instances of this block may still be around and we do not want to start
-        // overwriting those values when we create a new one.
-        // ENEBO: Once we make self, lastClass, and lastMethod immutable we can remove duplicate
-        binding = new Binding(binding.getSelf(), binding.getFrame().duplicate(), binding.getVisibility(), binding.getKlass(), 
-                binding.getDynamicScope().cloneScope());
+        binding = binding.clone();
 
         return new Block(this, binding);
     }
