@@ -42,7 +42,6 @@ import org.jruby.runtime.scope.ManyVarsDynamicScope;
 import org.jruby.Ruby;
 import org.jruby.RubyArray;
 import org.jruby.RubyClass;
-import org.jruby.RubyObject;
 import org.jruby.RubyKernel.CatchTarget;
 import org.jruby.RubyModule;
 import org.jruby.RubyString;
@@ -51,6 +50,7 @@ import org.jruby.evaluator.ASTInterpreter;
 import org.jruby.exceptions.JumpException.ReturnJump;
 import org.jruby.internal.runtime.JumpTarget;
 import org.jruby.internal.runtime.methods.DefaultMethod;
+import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.libraries.FiberLibrary.Fiber;
 import org.jruby.parser.BlockStaticScope;
 import org.jruby.parser.LocalStaticScope;
@@ -81,9 +81,6 @@ public final class ThreadContext {
     private RubyThread thread;
     private Fiber fiber;
     
-    // Error info is per-thread
-    private IRubyObject errorInfo;
-    
     //private UnsynchronizedStack parentStack;
     private RubyModule[] parentStack = new RubyModule[INITIAL_SIZE];
     private int parentIndex = -1;
@@ -119,9 +116,6 @@ public final class ThreadContext {
     private ThreadContext(Ruby runtime) {
         this.runtime = runtime;
         
-        // init errorInfo to nil
-        errorInfo = runtime.getNil();
-        
         // TOPLEVEL self and a few others want a top-level scope.  We create this one right
         // away and then pass it into top-level parse so it ends up being the top level.
         StaticScope topStaticScope = new LocalStaticScope(null);
@@ -148,11 +142,11 @@ public final class ThreadContext {
     }
     
     public IRubyObject getErrorInfo() {
-        return errorInfo;
+        return thread.getErrorInfo();
     }
     
     public IRubyObject setErrorInfo(IRubyObject errorInfo) {
-        this.errorInfo = errorInfo;
+        thread.setErrorInfo(errorInfo);
         return errorInfo;
     }
     
@@ -376,8 +370,7 @@ public final class ThreadContext {
     
     private void popFrame() {
         Frame frame = frameStack[frameIndex--];
-        setFile(frame.getFile());
-        setLine(frame.getLine());
+        setFileAndLine(frame);
         
         frame.clear();
     }
@@ -473,6 +466,11 @@ public final class ThreadContext {
         this.file = frame.getFile();
         this.line = frame.getLine();
     }
+
+    public void setFileAndLine(ISourcePosition position) {
+        this.file = position.getFile();
+        this.line = position.getStartLine();
+    }
     
     public Visibility getCurrentVisibility() {
         return getCurrentFrame().getVisibility();
@@ -487,7 +485,7 @@ public final class ThreadContext {
     }
     
     public void pollThreadEvents() {
-        getThread().pollThreadEvents(this);
+        thread.pollThreadEvents(this);
     }
     
     int calls = 0;
@@ -523,44 +521,21 @@ public final class ThreadContext {
     }
     
     public RubyModule getRubyClass() {
-        assert !(parentIndex == -1) : "Trying to get RubyClass from empty stack";
-        
+        assert parentIndex != -1 : "Trying to get RubyClass from empty stack";
         RubyModule parentModule = parentStack[parentIndex];
-        
-        return parentModule.getNonIncludedClass();
-    }
-    
-    public RubyModule getImmediateBindingRubyClass() {
-        int index = parentIndex;
-        RubyModule parentModule = null;
-        parentModule = parentStack[index];
         return parentModule.getNonIncludedClass();
     }
 
-    public RubyModule getEvalBindingRubyClass() {
-        int index = parentIndex;
-        RubyModule parentModule = null;
-        if(index == 0) {
-            parentModule = parentStack[index];
-        } else {
-            parentModule = parentStack[index-1];
-        }
+    public RubyModule getPreviousRubyClass() {
+        assert parentIndex != 0 : "Trying to get RubyClass from too-shallow stack";
+        RubyModule parentModule = parentStack[parentIndex - 1];
         return parentModule.getNonIncludedClass();
     }
     
     public boolean getConstantDefined(String internedName) {
-        IRubyObject result;
-        
-        // flipped from while to do to search current class first
-        for (StaticScope scope = getCurrentScope().getStaticScope(); scope != null; scope = scope.getPreviousCRefScope()) {
-            RubyModule module = scope.getModule();
-            if ((result = module.fastFetchConstant(internedName)) != null) {
-                if (result != RubyObject.UNDEF) return true;
-                return runtime.getLoadService().autoloadFor(module.getName() + "::" + internedName) != null;
-            }
-        }
-        
-        return getCurrentScope().getStaticScope().getModule().fastIsConstantDefined(internedName);
+        IRubyObject value = getConstant(internedName);
+
+        return value != null;
     }
     
     /**
@@ -627,6 +602,10 @@ public final class ThreadContext {
         RubyString traceLine;
         if (previousFrame.getName() != null) {
             traceLine = RubyString.newString(runtime, frame.getFile() + ':' + (frame.getLine() + 1) + ":in `" + previousFrame.getName() + '\'');
+        } else if (runtime.is1_9()) {
+            // TODO: This probably isn't the best hack, but it works until we can have different
+            // root frame setup for 1.9 easily.
+            traceLine = RubyString.newString(runtime, frame.getFile() + ':' + (frame.getLine() + 1) + ":in `<main>'");
         } else {
             traceLine = RubyString.newString(runtime, frame.getFile() + ':' + (frame.getLine() + 1));
         }
@@ -639,15 +618,18 @@ public final class ThreadContext {
                 frame.getLineNumber() == previousFrame.getLineNumber() &&
                 frame.getMethodName() != null &&
                 frame.getMethodName().equals(previousFrame.getMethodName()) &&
+                frame.getFileName() != null &&
                 frame.getFileName().equals(previousFrame.getFileName())) {
             return;
         }
         
         RubyString traceLine;
+        String fileName = frame.getFileName();
+        if (fileName == null) fileName = "";
         if (previousFrame.getMethodName() == UNKNOWN_NAME) {
-            traceLine = RubyString.newString(runtime, frame.getFileName() + ':' + (frame.getLineNumber()));
+            traceLine = RubyString.newString(runtime, fileName + ':' + (frame.getLineNumber()));
         } else {
-            traceLine = RubyString.newString(runtime, frame.getFileName() + ':' + (frame.getLineNumber()) + ":in `" + previousFrame.getMethodName() + '\'');
+            traceLine = RubyString.newString(runtime, fileName + ':' + (frame.getLineNumber()) + ":in `" + previousFrame.getMethodName() + '\'');
         }
         
         backtrace.append(traceLine);
@@ -865,7 +847,7 @@ public final class ThreadContext {
     }
     
     private static String createRubyBacktraceString(StackTraceElement element) {
-        return element.getClassName() + ":" + element.getLineNumber() + ":in `" + element.getMethodName() + "'";
+        return element.getFileName() + ":" + element.getLineNumber() + ":in `" + element.getMethodName() + "'";
     }
     
     public static String createRawBacktraceStringFromThrowable(Throwable t) {
@@ -895,10 +877,15 @@ public final class ThreadContext {
     
     public static IRubyObject createRawBacktrace(Ruby runtime, StackTraceElement[] stackTrace, boolean filter) {
         RubyArray traceArray = RubyArray.newArray(runtime);
-        for (int i = 17; i < stackTrace.length; i++) {
+        for (int i = 0; i < stackTrace.length; i++) {
             StackTraceElement element = stackTrace[i];
             
-            if (filter && element.getClassName().startsWith("org.jruby")) continue;
+            if (filter) {
+                if (element.getClassName().startsWith("org.jruby") ||
+                        element.getLineNumber() < 0) {
+                    continue;
+                }
+            }
             RubyString str = RubyString.newString(runtime, createRubyBacktraceString(element));
             traceArray.append(str);
         }
@@ -923,8 +910,21 @@ public final class ThreadContext {
     private Frame pushFrameForBlock(Binding binding) {
         Frame lastFrame = getNextFrame();
         Frame f = pushFrame(binding.getFrame());
-        f.setFile(file);
-        f.setLine(line);
+
+        // set the binding's frame's "previous" file and line to current, so
+        // trace will show who called the block
+        f.setFileAndLine(file, line);
+        
+        setFileAndLine(binding.getFile(), binding.getLine());
+        f.setVisibility(binding.getVisibility());
+        
+        return lastFrame;
+    }
+
+    private Frame pushFrameForEval(Binding binding) {
+        Frame lastFrame = getNextFrame();
+        Frame f = pushFrame(binding.getFrame());
+        setFileAndLine(binding.getFile(), binding.getLine());
         f.setVisibility(binding.getVisibility());
         return lastFrame;
     }
@@ -1027,16 +1027,24 @@ public final class ThreadContext {
         getCurrentFrame().setSelf(runtime.getTopSelf());
     }
     
-    public void preCompiledClass(RubyModule type, String[] scopeNames) {
+    public void preCompiledClass(RubyModule type, StaticScope staticScope) {
         pushRubyClass(type);
         pushFrameCopy();
         getCurrentFrame().setSelf(type);
         getCurrentFrame().setVisibility(Visibility.PUBLIC);
-        StaticScope staticScope = new LocalStaticScope(getCurrentScope().getStaticScope(), scopeNames);
         staticScope.setModule(type);
-        pushScope(new ManyVarsDynamicScope(staticScope, null));
+        pushScope(DynamicScope.newDynamicScope(staticScope));
     }
-    
+
+    public void preCompiledClassDummyScope(RubyModule type, StaticScope staticScope) {
+        pushRubyClass(type);
+        pushFrameCopy();
+        getCurrentFrame().setSelf(type);
+        getCurrentFrame().setVisibility(Visibility.PUBLIC);
+        staticScope.setModule(type);
+        pushScope(staticScope.getDummyScope());
+    }
+
     public void postCompiledClass() {
         popScope();
         popRubyClass();
@@ -1099,6 +1107,16 @@ public final class ThreadContext {
         pushScope(staticScope.getDummyScope());
         pushRubyClass(implementationClass);
     }
+
+    public void preMethodNoFrameAndDummyScope(RubyModule clazz, StaticScope staticScope) {
+        RubyModule implementationClass = staticScope.getModule();
+        // FIXME: This is currently only here because of some problems with IOOutputStream writing to a "bare" runtime without a proper scope
+        if (implementationClass == null) {
+            implementationClass = clazz;
+        }
+        pushScope(staticScope.getDummyScope());
+        pushRubyClass(implementationClass);
+    }
     
     public void postMethodFrameAndScope() {
         popRubyClass();
@@ -1144,10 +1162,27 @@ public final class ThreadContext {
     public void preMethodBacktraceOnly(String name) {
         pushBacktraceFrame(name);
     }
+
+    public void preMethodBacktraceDummyScope(RubyModule clazz, String name, StaticScope staticScope) {
+        pushBacktraceFrame(name);
+        RubyModule implementationClass = staticScope.getModule();
+        // FIXME: This is currently only here because of some problems with IOOutputStream writing to a "bare" runtime without a proper scope
+        if (implementationClass == null) {
+            implementationClass = clazz;
+        }
+        pushScope(staticScope.getDummyScope());
+        pushRubyClass(implementationClass);
+    }
     
     public void postMethodBacktraceOnly() {
         popFrame();
     }
+
+    public void postMethodBacktraceDummyScope() {
+        popFrame();
+        popRubyClass();
+        popScope();
+    }
     
     public void prepareTopLevel(RubyClass objectClass, IRubyObject topSelf) {
         pushFrame();
@@ -1251,11 +1286,8 @@ public final class ThreadContext {
     }
     
     public Frame preEvalWithBinding(Binding binding) {
-        Frame lastFrame = getNextFrame();
-        Frame frame = binding.getFrame();
-        frame.setIsBindingFrame(true);
-        pushFrame(frame);
-        getCurrentFrame().setVisibility(binding.getVisibility());
+        binding.getFrame().setIsBindingFrame(true);
+        Frame lastFrame = pushFrameForEval(binding);
         pushRubyClass(binding.getKlass());
         return lastFrame;
     }
@@ -1328,4 +1360,83 @@ public final class ThreadContext {
     public void setWithinDefined(boolean isWithinDefined) {
         this.isWithinDefined = isWithinDefined;
     }
+
+    /**
+     * Return a binding representing the current call's state
+     * @return the current binding
+     */
+    public Binding currentBinding() {
+        Frame frame = getCurrentFrame();
+        return new Binding(frame, getRubyClass(), getCurrentScope(), file, line);
+    }
+
+    /**
+     * Return a binding representing the current call's state but with a specified self
+     * @param self the self object to use
+     * @return the current binding, using the specified self
+     */
+    public Binding currentBinding(IRubyObject self) {
+        Frame frame = getCurrentFrame();
+        return new Binding(self, frame, frame.getVisibility(), getRubyClass(), getCurrentScope(), file, line);
+    }
+
+    /**
+     * Return a binding representing the current call's state but with the
+     * specified visibility and self.
+     * @param self the self object to use
+     * @param visibility the visibility to use
+     * @return the current binding using the specified self and visibility
+     */
+    public Binding currentBinding(IRubyObject self, Visibility visibility) {
+        Frame frame = getCurrentFrame();
+        return new Binding(self, frame, visibility, getRubyClass(), getCurrentScope(), file, line);
+    }
+
+    /**
+     * Return a binding representing the current call's state but with the
+     * specified scope and self.
+     * @param self the self object to use
+     * @param visibility the scope to use
+     * @return the current binding using the specified self and scope
+     */
+    public Binding currentBinding(IRubyObject self, DynamicScope scope) {
+        Frame frame = getCurrentFrame();
+        return new Binding(self, frame, frame.getVisibility(), getRubyClass(), scope, file, line);
+    }
+
+    /**
+     * Return a binding representing the current call's state but with the
+     * specified visibility, scope, and self. For shared-scope binding
+     * consumers like for loops.
+     * 
+     * @param self the self object to use
+     * @param visibility the visibility to use
+     * @param scope the scope to use
+     * @return the current binding using the specified self, scope, and visibility
+     */
+    public Binding currentBinding(IRubyObject self, Visibility visibility, DynamicScope scope) {
+        Frame frame = getCurrentFrame();
+        return new Binding(self, frame, visibility, getRubyClass(), scope, file, line);
+    }
+
+    /**
+     * Return a binding representing the previous call's state
+     * @return the current binding
+     */
+    public Binding previousBinding() {
+        Frame frame = getPreviousFrame();
+        Frame current = getCurrentFrame();
+        return new Binding(frame, getPreviousRubyClass(), getCurrentScope(), current.getFile(), current.getLine());
+    }
+
+    /**
+     * Return a binding representing the previous call's state but with a specified self
+     * @param self the self object to use
+     * @return the current binding, using the specified self
+     */
+    public Binding previousBinding(IRubyObject self) {
+        Frame frame = getPreviousFrame();
+        Frame current = getCurrentFrame();
+        return new Binding(self, frame, frame.getVisibility(), getPreviousRubyClass(), getCurrentScope(), current.getFile(), current.getLine());
+    }
 }
