@@ -55,6 +55,7 @@ import org.jruby.RubyFile;
 import org.jruby.RubyHash;
 import org.jruby.RubyString;
 import org.jruby.ast.executable.Script;
+import org.jruby.exceptions.MainExitException;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.runtime.Constants;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -190,7 +191,7 @@ public class LoadService {
               String rubyDir = jrubyHome + sep + "lib" + sep + "ruby" + sep;
 
               // If we're running in 1.9 compat mode, add Ruby 1.9 libs to path before 1.8 libs
-              if (runtime.getInstanceConfig().getCompatVersion() == CompatVersion.RUBY1_9) {
+              if (runtime.is1_9()) {
                   addPath(rubyDir + "site_ruby" + sep + Constants.RUBY1_9_MAJOR_VERSION);
                   addPath(rubyDir + "site_ruby");
                   addPath(rubyDir + Constants.RUBY1_9_MAJOR_VERSION);
@@ -253,26 +254,31 @@ public class LoadService {
         }
     }
 
-    public boolean smartLoad(final String file) {
-        checkEmptyLoad(file);
-
+    public SearchState findFileForLoad(String file) throws AlreadyLoaded {
         SearchState state = new SearchState(file);
         state.prepareRequireSearch(file);
 
-        try {
-            for (LoadSearcher searcher : searchers) {
-                if (searcher.shouldTrySearch(state)) {
-                    searcher.trySearch(state);
-                } else {
-                    continue;
-                }
+        for (LoadSearcher searcher : searchers) {
+            if (searcher.shouldTrySearch(state)) {
+                searcher.trySearch(state);
+            } else {
+                continue;
             }
+        }
+
+        return state;
+    }
+
+    public boolean smartLoad(final String file) {
+        checkEmptyLoad(file);
+
+        try {
+            SearchState state = findFileForLoad(file);
+            return tryLoadingLibraryOrScript(runtime, state);
         } catch (AlreadyLoaded al) {
             // Library has already been loaded in some form, bail out
             return false;
         }
-
-        return tryLoadingLibraryOrScript(runtime, state);
     }
 
     public boolean require(String file) {
@@ -563,6 +569,9 @@ public class LoadService {
             // otherwise load the library we've found
             state.library.load(runtime, false);
             return true;
+        } catch (MainExitException mee) {
+            // allow MainExitException to propagate out for exec and friends
+            throw mee;
         } catch (Throwable e) {
             if(isJarfileLibrary(state, state.searchFile)) {
                 return true;
@@ -699,17 +708,24 @@ public class LoadService {
     }
     
     private LoadServiceResource tryResourceFromJarURL(SearchState state, String baseName, SuffixType suffixType) {
-        // if a jar URL, return load service resource directly without further searching
+        // if a jar or file URL, return load service resource directly without further searching
         LoadServiceResource foundResource = null;
-        for (String suffix : suffixType.getSuffixes()) {
-            String namePlusSuffix = baseName + suffix;
-            if (namePlusSuffix.startsWith("jar:")) {
+        if (baseName.startsWith("jar:")) {
+            for (String suffix : suffixType.getSuffixes()) {
+                String namePlusSuffix = baseName + suffix;
                 try {
                     foundResource = new LoadServiceResource(new URL(namePlusSuffix), namePlusSuffix);
                 } catch (MalformedURLException e) {
                     throw runtime.newIOErrorFromException(e);
                 }
-            } else if(namePlusSuffix.startsWith("file:") && namePlusSuffix.indexOf("!/") != -1) {
+                if (foundResource != null) {
+                    state.loadName = namePlusSuffix;
+                    break; // end suffix iteration
+                }
+            }
+        } else if(baseName.startsWith("file:") && baseName.indexOf("!/") != -1) {
+            for (String suffix : suffixType.getSuffixes()) {
+                String namePlusSuffix = baseName + suffix;
                 try {
                     JarFile file = new JarFile(namePlusSuffix.substring(5, namePlusSuffix.indexOf("!/")));
                     String filename = namePlusSuffix.substring(namePlusSuffix.indexOf("!/") + 2);
@@ -717,11 +733,11 @@ public class LoadService {
                         foundResource = new LoadServiceResource(new URL("jar:" + namePlusSuffix), namePlusSuffix);
                     }
                 } catch(Exception e) {}
+                if (foundResource != null) {
+                    state.loadName = namePlusSuffix;
+                    break; // end suffix iteration
+                }
             }    
-            if (foundResource != null) {
-                state.loadName = namePlusSuffix;
-                break; // end suffix iteration
-            }
         }
         
         return foundResource;
