@@ -32,7 +32,6 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby.runtime.marshal;
 
-import java.io.BufferedInputStream;
 import java.io.EOFException;
 import java.io.IOException;
 import java.io.InputStream;
@@ -54,29 +53,39 @@ import org.jruby.RubyStruct;
 import org.jruby.RubySymbol;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.javasupport.util.RuntimeHelpers;
+import org.jruby.runtime.Constants;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.runtime.builtin.Variable;
 import org.jruby.runtime.component.VariableEntry;
 import org.jruby.util.ByteList;
 
 /**
- * Unmarshals objects from strings or streams in Ruby's marsal format.
+ * Unmarshals objects from strings or streams in Ruby's marshal format.
  *
  * @author Anders
  */
-public class UnmarshalStream extends BufferedInputStream {
+public class UnmarshalStream extends InputStream {
     protected final Ruby runtime;
     private final UnmarshalCache cache;
     private final IRubyObject proc;
+    private final InputStream inputStream;
 
     public UnmarshalStream(Ruby runtime, InputStream in, IRubyObject proc) throws IOException {
-        super(in);
         this.runtime = runtime;
         this.cache = new UnmarshalCache(runtime);
         this.proc = proc;
+        this.inputStream = in;
 
-        in.read(); // Major
-        in.read(); // Minor
+        int major = in.read(); // Major
+        int minor = in.read(); // Minor
+
+        if(major == -1 || minor == -1) {
+            throw new EOFException("Unexpected end of stream");
+        }
+        
+        if(major != Constants.MARSHAL_MAJOR || minor > Constants.MARSHAL_MINOR) {
+            throw runtime.newTypeError(String.format("incompatible marshal file format (can't be read)\n\tformat version %d.%d required; %d.%d given", Constants.MARSHAL_MAJOR, Constants.MARSHAL_MINOR, major, minor));
+        }
     }
 
     public IRubyObject unmarshalObject() throws IOException {
@@ -118,6 +127,7 @@ public class UnmarshalStream extends BufferedInputStream {
                 rubyObj = unmarshalObject();
                 if (ivarsWaiting) {
                     defaultVariablesUnmarshal(rubyObj);
+                    ivarsWaiting = false;
                 }
                 break;
             case '0' :
@@ -361,4 +371,8 @@ public class UnmarshalStream extends BufferedInputStream {
         }
         return (RubyClass) classInstance;
     }
+
+    public int read() throws IOException {
+        return inputStream.read();
+    }
 }
