@@ -400,7 +400,7 @@ public class RubyDateFormat extends DateFormat {
                     break;
                 case FORMAT_ZONE_OFF:
                     value = dt.getZone().getOffset(dt.getMillis());
-                    if (value <= 0) {
+                    if (value < 0) {
                         toAppendTo.append('-');
                     } else {
                         toAppendTo.append('+');
