@@ -39,10 +39,10 @@ import java.io.OutputStream;
 import java.io.PipedInputStream;
 import java.io.PipedOutputStream;
 import java.io.PrintStream;
+import java.lang.reflect.Field;
 import static java.lang.System.*;
 import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
-import java.util.Arrays;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
@@ -107,6 +107,7 @@ public class ShellLauncher {
             } finally {
                 this.config.getOutput().close();
                 this.config.getError().close();
+                try {this.config.getInput().close();} catch (IOException ioe) {}
             }
         }
 
@@ -191,6 +192,10 @@ public class ShellLauncher {
         return runAndWait(runtime, rawArgs, runtime.getOutputStream());
     }
 
+    public static long runWithoutWait(Ruby runtime, IRubyObject[] rawArgs) {
+        return runWithoutWait(runtime, rawArgs, runtime.getOutputStream());
+    }
+
     public static int execAndWait(Ruby runtime, IRubyObject[] rawArgs) {
         String[] args = parseCommandLine(runtime.getCurrentContext(), runtime, rawArgs);
         if (shouldRunInProcess(runtime, args)) {
@@ -238,6 +243,125 @@ public class ShellLauncher {
         }
     }
 
+    public static long runWithoutWait(Ruby runtime, IRubyObject[] rawArgs, OutputStream output) {
+        try {
+            POpenProcess aProcess = new POpenProcess(popenShared(runtime, rawArgs), runtime);
+            return getPidFromProcess(aProcess);
+        } catch (IOException e) {
+            throw runtime.newIOErrorFromException(e);
+        }
+    }
+
+    public static long getPidFromProcess(Process process) {
+        if (process instanceof ScriptThreadProcess) {
+            return process.hashCode();
+        } else if (process instanceof POpenProcess) {
+            return reflectPidFromProcess(((POpenProcess)process).getChild());
+        } else {
+            return reflectPidFromProcess(process);
+        }
+    }
+    
+    private static final Class UNIXProcess;
+    private static final Field UNIXProcess_pid;
+    private static final Class ProcessImpl;
+    private static final Field ProcessImpl_handle;
+    private interface PidGetter { public long getPid(Process process); }
+    private static final PidGetter PID_GETTER;
+    
+    static {
+        // default PidGetter
+        PidGetter pg = new PidGetter() {
+            public long getPid(Process process) {
+                return process.hashCode();
+            }
+        };
+        
+        Class up = null;
+        Field pid = null;
+        try {
+            up = Class.forName("java.lang.UNIXProcess");
+            pid = up.getDeclaredField("pid");
+            pid.setAccessible(true);
+        } catch (Exception e) {
+            // ignore and try windows version
+        }
+        UNIXProcess = up;
+        UNIXProcess_pid = pid;
+
+        Class pi = null;
+        Field handle = null;
+        try {
+            pi = Class.forName("java.lang.ProcessImpl");
+            handle = pi.getDeclaredField("handle");
+            handle.setAccessible(true);
+        } catch (Exception e) {
+            // ignore and use hashcode
+        }
+        ProcessImpl = pi;
+        ProcessImpl_handle = handle;
+
+        if (UNIXProcess_pid != null) {
+            if (ProcessImpl_handle != null) {
+                // try both
+                pg = new PidGetter() {
+                    public long getPid(Process process) {
+                        try {
+                            if (UNIXProcess.isInstance(process)) {
+                                return (Integer)UNIXProcess_pid.get(process);
+                            } else if (ProcessImpl.isInstance(process)) {
+                                return (Long)ProcessImpl_handle.get(process);
+                            }
+                        } catch (Exception e) {
+                            // ignore and use hashcode
+                        }
+                        return process.hashCode();
+                    }
+                };
+            } else {
+                // just unix
+                pg = new PidGetter() {
+                    public long getPid(Process process) {
+                        try {
+                            if (UNIXProcess.isInstance(process)) {
+                                return (Integer)UNIXProcess_pid.get(process);
+                            }
+                        } catch (Exception e) {
+                            // ignore and use hashcode
+                        }
+                        return process.hashCode();
+                    }
+                };
+            }
+        } else if (ProcessImpl_handle != null) {
+            // just windows
+            pg = new PidGetter() {
+                public long getPid(Process process) {
+                    try {
+                        if (ProcessImpl.isInstance(process)) {
+                            return (Long)ProcessImpl_handle.get(process);
+                        }
+                    } catch (Exception e) {
+                        // ignore and use hashcode
+                    }
+                    return process.hashCode();
+                }
+            };
+        } else {
+            // neither
+            pg = new PidGetter() {
+                public long getPid(Process process) {
+                    return process.hashCode();
+                }
+            };
+        }
+        PID_GETTER = pg;
+    }
+
+    public static long reflectPidFromProcess(Process process) {
+        return PID_GETTER.getPid(process);
+    }
+
     public static Process run(Ruby runtime, IRubyObject string) throws IOException {
         return run(runtime, new IRubyObject[] {string});
     }
@@ -337,7 +461,10 @@ public class ShellLauncher {
             if (modes.isWritable()) {
                 prepareOutput(child);
             } else {
-                pumpOutput(child, runtime);
+                // close process output
+                // See JRUBY-3405; hooking up to parent process stdin caused
+                // problems for IRB etc using stdin.
+                try {child.getOutputStream().close();} catch (IOException ioe) {}
             }
             
             if (modes.isReadable()) {
@@ -386,11 +513,19 @@ public class ShellLauncher {
             return inerrChannel;
         }
 
+        public boolean hasOutput() {
+            return output != null || outputChannel != null;
+        }
+
+        public Process getChild() {
+            return child;
+        }
+
         @Override
         public int waitFor() throws InterruptedException {
             if (outputPumper == null) {
                 try {
-                    output.close();
+                    if (output != null) output.close();
                 } catch (IOException ioe) {
                     // ignore, we're on the way out
                 }
@@ -411,9 +546,16 @@ public class ShellLauncher {
         @Override
         public void destroy() {
             try {
-                input.close();
-                inerr.close();
-                output.close();
+                child.destroy();
+                if (input != null) input.close();
+                if (inerr != null) inerr.close();
+                if (output != null) output.close();
+                if (inputChannel != null) inputChannel.close();
+                if (inerrChannel != null) inerrChannel.close();
+                if (outputChannel != null) outputChannel.close();
+                if (inputPumper != null) inputPumper.quit();
+                if (inerrPumper != null) inerrPumper.quit();
+                if (outputPumper != null) outputPumper.quit();
             } catch (IOException ioe) {
                 throw new RuntimeException(ioe);
             }
