@@ -33,8 +33,8 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby.util.io;
 
-import java.io.BufferedInputStream;
-import java.io.BufferedOutputStream;
+import static java.util.logging.Logger.getLogger;
+
 import java.io.EOFException;
 import java.io.FileDescriptor;
 import java.io.FileNotFoundException;
@@ -43,15 +43,14 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.RandomAccessFile;
 import java.nio.ByteBuffer;
+import java.nio.channels.Channel;
 import java.nio.channels.Channels;
 import java.nio.channels.FileChannel;
-
-import java.nio.channels.Channel;
 import java.nio.channels.IllegalBlockingModeException;
 import java.nio.channels.ReadableByteChannel;
 import java.nio.channels.SelectableChannel;
 import java.nio.channels.WritableByteChannel;
-import static java.util.logging.Logger.getLogger;
+
 import org.jruby.Finalizable;
 import org.jruby.Ruby;
 import org.jruby.util.ByteList;
@@ -98,7 +97,7 @@ public class ChannelStream implements Stream, Finalizable {
     protected int ungotc = -1;
     private volatile boolean closedExplicitly = false;
 
-    private boolean eof = false;
+    private volatile boolean eof = false;
 
     public ChannelStream(Ruby runtime, ChannelDescriptor descriptor, ModeFlags modes, FileDescriptor fileDescriptor) throws InvalidValueException {
         descriptor.checkNewModes(modes);
@@ -401,7 +400,11 @@ public class ChannelStream implements Stream, Finalizable {
             if (DEBUG) getLogger("ChannelStream").info("Descriptor for fileno "
                     + descriptor.getFileno() + " closed by stream");
         } finally {
-            if (!finalizing) getRuntime().removeInternalFinalizer(this);
+            Ruby localRuntime = getRuntime();
+            if (!finalizing && localRuntime != null) localRuntime.removeInternalFinalizer(this);
+            
+            // clear runtime so it doesn't get stuck in memory (JRUBY-2933)
+            runtime = null;
         }
     }
 
@@ -496,7 +499,7 @@ public class ChannelStream implements Stream, Finalizable {
     public InputStream newInputStream() {
         InputStream in = descriptor.getBaseInputStream();
         if (in == null) {
-            return new BufferedInputStream(Channels.newInputStream((ReadableByteChannel)descriptor.getChannel()));
+            return Channels.newInputStream((ReadableByteChannel)descriptor.getChannel());
         } else {
             return in;
         }
@@ -506,7 +509,7 @@ public class ChannelStream implements Stream, Finalizable {
      * @see org.jruby.util.IOHandler#getOutputStream()
      */
     public OutputStream newOutputStream() {
-        return new BufferedOutputStream(Channels.newOutputStream((WritableByteChannel)descriptor.getChannel()));
+        return Channels.newOutputStream((WritableByteChannel)descriptor.getChannel());
     }
     
     public void clearerr() {
@@ -626,7 +629,12 @@ public class ChannelStream implements Stream, Finalizable {
     private void ensureReadNonBuffered() throws IOException, BadDescriptorException {
         if (reading) {
             if (buffer.hasRemaining()) {
-                throw getRuntime().newIOError("sysread for buffered IO");
+                Ruby localRuntime = getRuntime();
+                if (localRuntime != null) {
+                    throw localRuntime.newIOError("sysread for buffered IO");
+                } else {
+                    throw new IOException("sysread for buffered IO");
+                }
             }
         } else {
             // libc flushes writes on any read from the actual file, so we flush here
@@ -753,7 +761,6 @@ public class ChannelStream implements Stream, Finalizable {
      * @see org.jruby.util.IOHandler#syswrite(String buf)
      */
     private int bufferedWrite(ByteList buf) throws IOException, BadDescriptorException {
-        getRuntime().secure(4);
         checkWritable();
         ensureWrite();
         
@@ -785,7 +792,6 @@ public class ChannelStream implements Stream, Finalizable {
      * @see org.jruby.util.IOHandler#syswrite(String buf)
      */
     private int bufferedWrite(int c) throws IOException, BadDescriptorException {
-        getRuntime().secure(4);
         checkWritable();
         ensureWrite();
 
@@ -851,7 +857,33 @@ public class ChannelStream implements Stream, Finalizable {
     }
 
     public int ready() throws IOException {
-        return newInputStream().available();
+        if (descriptor.getChannel() instanceof SelectableChannel) {
+            int ready_stat = 0;
+            java.nio.channels.Selector sel = java.nio.channels.Selector.open();
+            SelectableChannel selchan = (SelectableChannel)descriptor.getChannel();
+            synchronized (selchan.blockingLock()) {
+                boolean is_block = selchan.isBlocking();
+                try {
+                    selchan.configureBlocking(false);
+                    selchan.register(sel, java.nio.channels.SelectionKey.OP_READ);
+                    ready_stat = sel.selectNow();
+                    sel.close();
+                } catch (Throwable ex) {
+                    ex.printStackTrace();
+                } finally {
+                    if (sel != null) {
+                        try {
+                            sel.close();
+                        } catch (Exception e) {
+                        }
+                    }
+                    selchan.configureBlocking(is_block);
+                }
+            }
+            return ready_stat;
+        } else {
+            return newInputStream().available();
+        }
     }
 
     public synchronized void fputc(int c) throws IOException, BadDescriptorException {
@@ -893,7 +925,6 @@ public class ChannelStream implements Stream, Finalizable {
         return bufferedWrite(string);
     }
     public synchronized int writenonblock(ByteList buf) throws IOException, BadDescriptorException {
-        getRuntime().secure(4);
         checkWritable();
         ensureWrite();
         
@@ -1044,7 +1075,7 @@ public class ChannelStream implements Stream, Finalizable {
         return blocking;
     }
 
-    public synchronized void freopen(String path, ModeFlags modes) throws DirectoryAsFileException, IOException, InvalidValueException, PipeException, BadDescriptorException {
+    public synchronized void freopen(Ruby runtime, String path, ModeFlags modes) throws DirectoryAsFileException, IOException, InvalidValueException, PipeException, BadDescriptorException {
         // flush first
         flushWrite();
 
@@ -1063,19 +1094,19 @@ public class ChannelStream implements Stream, Finalizable {
         if (path.equals("/dev/null") || path.equalsIgnoreCase("nul:") || path.equalsIgnoreCase("nul")) {
             descriptor = new ChannelDescriptor(new NullChannel(), descriptor.getFileno(), modes, new FileDescriptor());
         } else {
-            String cwd = getRuntime().getCurrentDirectory();
+            String cwd = runtime.getCurrentDirectory();
             JRubyFile theFile = JRubyFile.create(cwd,path);
 
             if (theFile.isDirectory() && modes.isWritable()) throw new DirectoryAsFileException();
 
             if (modes.isCreate()) {
                 if (theFile.exists() && modes.isExclusive()) {
-                    throw getRuntime().newErrnoEEXISTError("File exists - " + path);
+                    throw runtime.newErrnoEEXISTError("File exists - " + path);
                 }
                 theFile.createNewFile();
             } else {
                 if (!theFile.exists()) {
-                    throw getRuntime().newErrnoENOENTError("file not found - " + path);
+                    throw runtime.newErrnoENOENTError("file not found - " + path);
                 }
             }
 
