@@ -135,7 +135,7 @@ class MethodFixture
   private
     def method5; end
 end
-test_equal(2, MethodFixture.singleton_methods(false).length)
+test_equal(3, MethodFixture.singleton_methods(false).length)
 m = MethodFixture.new
 test_equal(3 + Object.new.methods.length, m.methods.length)
 test_equal(2, m.public_methods(false).length)
@@ -148,6 +148,8 @@ class Class1
     @one = 123
     @two = nil
   end
+  def initialize_copy
+  end
 end
 
 c1 = Class1.new
@@ -159,3 +161,11 @@ end
 test_ok !c1.instance_variable_defined?(:@three)
 test_ok c1.instance_variable_defined?(:@one)
 test_ok c1.instance_variable_defined?(:@two)
+
+# JRUBY-2624: Object#initialize_copy should always be private
+test_ok c1.private_methods.include?("initialize")
+test_ok c1.private_methods.include?("initialize_copy")
+
+# JRUBY-2247
+test_equal ['now'], Time.methods.grep('now')
+test_equal ["_load", "at", "gm", "local", "mktime", "now", "utc"], Time.methods(false).sort
