@@ -1,3 +1,4 @@
+# -*- coding: iso-8859-1 -*-
 require 'test/unit'
 require 'rbconfig'
 require 'fileutils'
@@ -516,27 +517,34 @@ class TestFile < Test::Unit::TestCase
     File.utime(time, time, filename)
     # File mtime resolution may not be sub-second on all platforms (e.g., windows)
     # allow for some slop
+    assert((time.to_i - File.atime(filename).to_i).abs < 5)
     assert((time.to_i - File.mtime(filename).to_i).abs < 5)
     File.unlink(filename)
   end
   
-  def test_file_utime_nil_mtime
+  def test_file_utime_nil
     filename = '__test__file'
     File.open(filename, 'w') {|f| }
     time = File.mtime(filename)
     sleep 2
     File.utime(nil, nil, filename)
+    assert((File.atime(filename).to_i - time.to_i) >= 2)
     assert((File.mtime(filename).to_i - time.to_i) >= 2)
     File.unlink(filename)
   end
   
-  def test_file_utime_bad_mtime_raises_typeerror
+  def test_file_utime_bad_time_raises_typeerror
     args = [ [], {}, '4000' ]
     filename = '__test__file'
     File.open(filename, 'w') {|f| }
     args.each do |arg|
+      assert_raises(TypeError) {  File.utime(arg, nil, filename) }
+      assert_raises(TypeError) {  File.utime(nil, arg, filename) }
       assert_raises(TypeError) {  File.utime(arg, arg, filename) }
     end
+    time = Time.now
+    assert_raises(TypeError) {  File.utime(time, nil, filename) }
+    assert_raises(TypeError) {  File.utime(nil, time, filename) }
     File.unlink(filename)
   end
   
@@ -807,7 +815,27 @@ class TestFile < Test::Unit::TestCase
     File.umask(orig_mask)
   end
 
-  # JRUBY-2694
+  def test_allow_override_of_make_tmpname
+    require 'tempfile'
+    # mimics behavior of attachment_fu, which overrides private #make_tmpname
+    Tempfile.class_eval do
+      def make_tmpname(basename, n)
+        ext = nil
+        sprintf("%s%d-%d%s", basename.to_s.gsub(/\.\w+$/) { |s| ext = s; '' }, $$, n, ext)
+      end
+    end
+
+    t = Tempfile.new "tcttac.jpg", File.dirname(__FILE__)
+    assert t.path =~ /\.jpg$/
+  end
+
+  def test_mode_of_tempfile_is_600
+    t = Tempfile.new "tcttac.jpg"
+    assert_equal 0100600, File.stat(t.path).mode
+  end
+
+  # See JRUBY-2694; we don't have 1.8.7 support yet
+=begin
   def test_tempfile_with_suffix
     require 'tempfile'
     Tempfile.open(['prefix', 'suffix']) { |f|
@@ -815,4 +843,11 @@ class TestFile < Test::Unit::TestCase
       assert_match(/suffix$/, f.path)
     }
   end
+=end
+
+  def test_file_size
+    size = File.size('build.xml')
+    assert(size > 0)
+    assert_equal(size, File.size(File.new('build.xml')))
+  end
 end
